// AlignBehavior.cpp: implementation of the AlignBehavior class.
//
//////////////////////////////////////////////////////////////////////

#include "AlignBehavior.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AlignBehavior::AlignBehavior()
{
  alignrollflag = false;
  matchspeedflag = false;
  type = ALIGN_HEADING;
}

AlignBehavior::~AlignBehavior()
{

}

Coordinate AlignBehavior::getSteeringVector(Targetable * body, Targetable * target)
{
  Coordinate alignwith;
  Coordinate zero;

  switch(type)
  {
    case ALIGN_UP :
      alignwith = target->getUp();
      break;
    case ALIGN_DOWN :
      alignwith = target->getUp();
      alignwith = zero - alignwith;
      break;
    case ALIGN_LEFT :
      alignwith = target->getleft();
      break;
    case ALIGN_RIGHT :
      alignwith = target->getleft();
      alignwith = zero - alignwith;
      break;
    case ALIGN_HEADING :
      alignwith = target->getHeading();
      break;
  };
  Coordinate difference = alignwith  - body->getHeading() ;
  if(alignrollflag)
    alignRoll(body, target);
  if(matchspeedflag)
    matchSpeed(body, target);

  return difference;
}

void AlignBehavior::setAlignWith(int newtype)
{
  type = newtype;
}


void  AlignBehavior::move(Targetable * body, Targetable * target)
{
  steerTo(body, getSteeringVector(body, target) );
}

void AlignBehavior::setAlignRoll(bool newalignroll)
{
  alignrollflag = newalignroll;
}

void AlignBehavior::alignRoll(Targetable * body, Targetable * target)
{
  Coordinate updifference = body->getUp();

  updifference  = updifference - target->getUp() ;
  if(updifference.getlength() > .05f)
  {
     if(updifference.dot(body->getleft() ) > 0.0f)
     {
       body->bankleft();
     }
     else
     {
       body->bankright();
     }
  }
}

void AlignBehavior::matchSpeed(Targetable * body, Targetable * target)
{
  if(body->getVelocity() < target->getVelocity() )
    body->incrementVelocity();
  else if(body->getVelocity() > target->getVelocity() )
  {
    body->decrementVelocity();
  }
}

void AlignBehavior::setMatchSpeed(bool match)
{
  matchspeedflag = match;
}
