// AutoPilotCamera.cpp: implementation of the AutoPilotCamera class.
//
//////////////////////////////////////////////////////////////////////

#include "AutoPilotCamera.h"
#include "glerr.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
AutoPilotCamera AutoPilotCamera::me;


AutoPilotCamera::AutoPilotCamera()
{

}

AutoPilotCamera::~AutoPilotCamera()
{

}


Coordinate AutoPilotCamera::getHeading()
{
  Coordinate newheading = Camera::getFocusObject()->getLocation() - autopilotcam ;
  newheading.normalize();

  return  newheading;
}


void AutoPilotCamera::render()
{
  Object3d * myfocus = Camera::getFocusObject();

  Coordinate lookat = myfocus->getLocation();
  Coordinate camera = autopilotcam;

  myfocus->setrenderingenabled(true);
  advanceCamera();  

  gluLookAt((GLdouble) camera.x, (GLdouble) camera.y, (GLdouble) camera.z, 
            (GLdouble) lookat.x, (GLdouble) lookat.y, (GLdouble) lookat.z,             
            (GLdouble) newup.x, (GLdouble) newup.y, (GLdouble) newup.z);
 GLERRORCHECK
}

void AutoPilotCamera::onCameraChange()
{
  autopilotcam = getInitialPosition(Camera::getFocusObject() );
}

Coordinate AutoPilotCamera::getInitialPosition(Object3d * player)
{
  const float forwardradius = 35.0f;
  float sideradius = - 8.0f;
  const float downradius = 0.7f;

  Coordinate returncoord;

  float xextents = player->getExtents().x / 2.0f;
  if(xextents > 8.0f )
  {
    sideradius = -(xextents + 3.0f);
  
  }
  returncoord = player->getHeading() * forwardradius + 
                player->getleft() * sideradius + 
                player->getUp() * downradius;
  initialup = player->getUp();
  return returncoord + player->getLocation();
}

bool AutoPilotCamera::showMovingStars()
{
  return false;
}

Camera * AutoPilotCamera::getInstance()
{
  return & me;
}

Coordinate AutoPilotCamera::getLocation()
{
  return autopilotcam;
}

Coordinate AutoPilotCamera::getUp()
{
  advanceCamera();
  return newup;
}

void AutoPilotCamera::advanceCamera()
{
  Coordinate heading = Camera::getFocusObject()->getLocation() - autopilotcam ;
  heading.normalize();

  Coordinate right = heading.cross(initialup);
  right.normalize();
  newup = right.cross(heading);
  newup.normalize();

}
