// AutoPilotCutScene.cpp: implementation of the AutoPilotCutScene class.
//
//////////////////////////////////////////////////////////////////////

#include "AutoPilotCutScene.h"
#include "Camera.h"
#include "AutoPilotCamera.h"
#include <string.h>
#include "SoundFactory.h"
#include "SoundEffect.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AutoPilotCutScene::AutoPilotCutScene(Camera * newcamera)
  : camera(newcamera)
{
  duration = 9000;
  player = NULL;
  wingmen = NULL;
  numwingmen = 0;
  starttime = 0;
}

AutoPilotCutScene::~AutoPilotCutScene()
{

}

void AutoPilotCutScene::start()
{
  int i;
  if(player == NULL)
    return;
  Object3d * left, * right;
  Object3d * currentwingman;
  left = player;
  right = player;
  Coordinate leftoffset, rightoffset;

  SoundEffect * autopilotsound = SoundFactory::getSoundFactory()->getAutoPilotSound();
  autopilotsound->play();
  Camera::setCamera(AutoPilotCamera::getInstance() );
  starttime = Time::getCurrentTime();

  player->setMaxVelocity();
  
  for(i=0; i < numwingmen; i++)
  {
    currentwingman = wingmen[i];
    currentwingman->setMaxVelocity();
    currentwingman->setOrientation(player->getHeading(), player->getUp(), player->getleft() );
    if(i % 2)
    {
      //left
      Coordinate newlocation;
      float sideradius = (left->getExtents().x + currentwingman->getExtents().x) /2.0f + 5.0f;

      newlocation = left->getLocation() + player->getleft() * sideradius;
      
      currentwingman->setLocation(newlocation);
      left = currentwingman;
    }
    else
    {//right
      Coordinate newlocation;
      float sideradius = (right->getExtents().x + currentwingman->getExtents().x) /2.0f + 5.0f;

      newlocation = right->getLocation() - player->getleft() * sideradius ;

      currentwingman->setLocation(newlocation);
      right = currentwingman;
    }
  }
  //camera->setCameraType(Camera::cameratype::autopilot);
   
}

bool AutoPilotCutScene::isDone()
{
  
  TimeValue currenttime = Time::getCurrentTime();
  int i;

  if(starttime == 0)
  {
    player->setMaxVelocity();
    for(i=0 ; i < numwingmen; i++)
    {
      wingmen[i]->setMaxVelocity();
    }    
    return true;
  }
  return currenttime - starttime > duration;
}

void AutoPilotCutScene::setPlayer(Object3d * newplayer)
{
  player = newplayer;
}

void AutoPilotCutScene::setWing(Object3d * newwingmen [ ], int newnumwingmen)
{
  wingmen = newwingmen;
  numwingmen = newnumwingmen;
}


bool AutoPilotCutScene::instanceof(char classname [ ])
{
  return  strcmp(classname, "AutoPilotCutScene") == 0;
}
