// AviMovie.cpp: implementation of the AviMovie class.
//
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#include <windows.h>
#include <vfw.h>
#endif
#include "AviMovie.h"
#include "FileNotFoundException.h"
#ifdef  WIN32
#include "BmpImage.h"
#include "BMPFile.h"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AviMovie::AviMovie(char filename[])
{
#ifdef WIN32
  if (AVIStreamOpenFromFile(&pavi, filename, streamtypeVIDEO, 0, OF_READ, NULL) !=0)
  {
     throw FileNotFoundException(filename);
  }
  AVIStreamInfo(pavi, &psi, sizeof(psi));		
  lastimage = NULL;
  currentframe = 0;
  lastframe=AVIStreamLength(pavi);	

  mpf= (float) AVIStreamSampleToTime(pavi,lastframe)/ (float)lastframe;	
  playing = false;
#endif
}

AviMovie::~AviMovie()
{
   if(playing)
   {
	  stop();
   }
#ifdef WIN32
   AVIStreamRelease(pavi);
#endif
}

Image * AviMovie::getCurrentFrame()
{
#ifdef WIN32
  BITMAPINFOHEADER * bmpheader;
  char * data;
  TimeValue now;
  BMPFile * bmpfile;

  if(! playing)
  {
	return NULL;
  }

  if(currentframe == 0)
  {
    data = grabFrame(&bmpheader, 0);
	bmpfile = new BMPFile((BITMAPINFO *)bmpheader, data);
    lastimage = new BmpImage( bmpfile->getData(), bmpfile->getWidth(), bmpfile->getHeight()  );
	delete bmpfile;
  }

  now = Time::getCurrentTime();
  TimeValue elapsed = now - starttime;
  int frame = (int) ((float) (elapsed) / mpf);
  if(frame == currentframe)
  {
	return lastimage;
  }
  else if(frame >= lastframe) // loop
  {
    currentframe = 0;
	starttime = now;
    data = grabFrame(&bmpheader, 0);
	bmpfile = new BMPFile((BITMAPINFO *)bmpheader, data);
    lastimage->recycle(bmpfile->getData(), bmpfile->getWidth(), bmpfile->getHeight()  );
	delete bmpfile;
	return lastimage;
  }
  else
  {
    data = grabFrame(&bmpheader, frame);
	bmpfile = new BMPFile((BITMAPINFO *)bmpheader, data);
    lastimage->recycle(bmpfile->getData(), bmpfile->getWidth(), bmpfile->getHeight()  );
	delete bmpfile;
	currentframe = frame;
	return lastimage;
  } 
#else
  return NULL;
#endif
}

void AviMovie::start()
{
#ifdef WIN32
	starttime = Time::getCurrentTime();
    pgf=AVIStreamGetFrameOpen(pavi, NULL);	
	playing = true;
#endif
}

#ifdef WIN32
char * AviMovie::grabFrame(BITMAPINFOHEADER * * bipp, int frame)
{  

	char * pdata;

    *bipp = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pgf, frame);	
	BITMAPINFOHEADER * lpbi = * bipp;
	pdata=(char *)lpbi+lpbi->biSize+lpbi->biClrUsed * sizeof(RGBQUAD);
	return pdata;

}
#endif

void AviMovie::stop()
{
#ifdef WIN32
  playing = false;
  AVIStreamGetFrameClose(pgf);
#endif
}

void AviMovie::staticInit()
{
#ifdef WIN32
  AVIFileInit();	
#endif
}

void AviMovie::staticClear()
{
#ifdef WIN32
  AVIFileExit();	
#endif
}

bool AviMovie::isPlaying()
{
  return playing;
}
