// Billboard.cpp: implementation of the Billboard class.
//
//////////////////////////////////////////////////////////////////////

#include "Billboard.h"
#include "Camera.h"
#include "TextureFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Billboard::Billboard(char filename[], Coordinate relative_location)
  : location(relative_location)
{
/*  BMPFile bmp(filename);


  width = bmp.getWidth();
  height = bmp.getHeight();
  texture = new Texture(bmp.getData(), bmp.getWidth(), bmp.getHeight() );
*/
  texture = TextureFactory::getTransparentTexture(filename, Color(0,0,0) );
  
}

Billboard::~Billboard()
{

}

void Billboard::render()
{
  Camera * camera = Camera::getCamera();

  Coordinate center = location + camera->getLocation() ;
  Coordinate up = camera->getUp() * 2.0f;
  Coordinate left = camera->getLeft() * 2.0f;

  Coordinate zero;
  Coordinate normal = zero - camera->getHeading();

  Coordinate upper_left = center + up - left;
  Coordinate upper_right = center + up + left;
  Coordinate lower_left = center - up - left;
  Coordinate lower_right = center - up + left;

  glColor3f(1.0f, 1.0f, 1.0f);
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_COLOR_MATERIAL);
  glColor3f(1.0f, 1.0f, 1.0f);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);


  texture->render();
  glNormal3f(0.0f, 0.0f, -1.0f);

  glEnable(GL_TEXTURE_2D);
  glDisable(GL_COLOR_MATERIAL);
  glColor3f(1.0f, 1.0f, 1.0f);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

  Coordinate newlocation;
  glNormal3f(normal.x, normal.y, normal.z);


  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);

  texture->render();
  glBegin(GL_QUADS);  
           newlocation = upper_left;
           glTexCoord2f(0.0f, 0.0f);

           glVertex3f(newlocation.x , newlocation.y , newlocation.z);
           newlocation = upper_right;
           glTexCoord2f(1.0f, 0.0f);
//  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(newlocation.x , newlocation.y , newlocation.z);


           newlocation = lower_right;
           glTexCoord2f(1.0f, 1.0f);
//  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(newlocation.x , newlocation.y , newlocation.z);

           newlocation = lower_left;
           glTexCoord2f(0.0f, 1.0f);
//  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(newlocation.x , newlocation.y , newlocation.z);

  glEnd();
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_COLOR_MATERIAL);
  glDisable(GL_BLEND);


}
