// BriefingNavPoint.cpp: implementation of the BriefingNavPoint class.
//
//////////////////////////////////////////////////////////////////////
#include "GL/glew.h"
#include "BriefingNavPoint.h"
#include "TextToSpeech.h"

using std::string;

Color BriefingNavPoint::green(0.0f, 1.0f, 0.0f);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BriefingNavPoint::BriefingNavPoint(int id, Coordinate newlocation, float newradius,
								   std::vector<BriefingShip *> * newfixedships,
								   std::string newattext, std::string newleavingtext)
  : location(newlocation),
    radius(newradius),
	fixedships(newfixedships),
	at_text(newattext),
	leaving_text(newleavingtext)
{
  int i;
  char buffer[100];

  mycircle = new Circle(location, radius);
  for(i=0; i < fixedships->size(); i++)
  {
    (*fixedships)[i]->setFacing(location);
  }
 
  sprintf(buffer, "nav%d_at.wav", id);
  at_speech = TextToSpeech::getInstance()->
	  synthesize((char*) at_text.c_str(), buffer);

  sprintf(buffer, "nav%d_leaving.wav", id);
  leaving_speech = TextToSpeech::getInstance()->
	  synthesize((char*) leaving_text.c_str(), buffer);

}

BriefingNavPoint::~BriefingNavPoint()
{

}

Coordinate BriefingNavPoint::getLocation()
{
  return location;
}

void BriefingNavPoint::render()
{
  int i;

  green.render();
  glLineWidth(2.0f);
    mycircle->render();
  glLineWidth(1.0f);
  for(i=0; i < fixedships->size(); i++)
  {
    (*fixedships)[i]->render();
  }

}

std::string BriefingNavPoint::getAtText()
{
  return at_text;
}

std::string BriefingNavPoint::getLeavingText()
{
  return leaving_text;
}


SoundEffect * BriefingNavPoint::getAtSpeech()
{
  return at_speech;
}

SoundEffect * BriefingNavPoint::getLeavingSpeech()
{
  return leaving_speech;
}
