// BriefingShip.cpp: implementation of the BriefingShip class.
//
//////////////////////////////////////////////////////////////////////

#include "BriefingShip.h"
#include "Color.h"
#include "mesh.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BriefingShip::BriefingShip(char filename[], bool enemy, bool capship, 
						   Coordinate newrelativepos, int newlastnav)

: Object3d(new mesh(filename, getColor(enemy, capship), true) ) ,
  relativeposition(newrelativepos),
  lastnav(newlastnav)

{

}

BriefingShip::~BriefingShip()
{

}

Color BriefingShip::getColor(bool enemy, bool capship)
{
  if(enemy)
	  return Color(1.0, 0.0, 0.0);
  else
	  return Color(0.0, 0.0, 1.0);
}

Coordinate BriefingShip::getRelativePosition()
{
  return relativeposition;
}

void BriefingShip::setFacing(Coordinate center)
{
  Coordinate heading;
  Coordinate up(0.0f, 1.0f, 0.0f);
  Coordinate side;

  setPositionRelative(center);
  heading = center - getLocation();
  heading.normalize();

  side = heading.cross(up); 
  setOrientation(heading, up, side);
}

void BriefingShip::setPositionRelative(Coordinate center)
{
  setLocation(center + relativeposition);
}

void BriefingShip::setPositionRelative(Object3d * relativeto)
{
  Coordinate newplace;

  newplace = (relativeto->getUp() * relativeposition.y)
	  + 
      relativeto->getleft() * relativeposition.x
	  + 
	  relativeto->getHeading() * relativeposition.z;
  setLocation(newplace + relativeto->getLocation());
}

int BriefingShip::getLastAutopilotNav()
{
  return lastnav;
}
