// BriefingXmlAdapter.cpp: implementation of the BriefingXmlAdapter class.
//
//////////////////////////////////////////////////////////////////////

#include "BriefingXmlAdapter.h"
#include "AttributeList.h"
#include "saxexpat.h"
#include "InputSource.h"
#include <ctype.h>
#include "ErrorHandlerE.h"
#include "BriefingNavPoint.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BriefingXmlAdapter::BriefingXmlAdapter(char filename[])
{
	navpoints = new std::vector<BriefingNavPoint *>;
    autopilotships = new std::vector<BriefingShip *>;
	    scale = 1.0f;
	expat myParser;
   ErrorHandlerE::checkFile(filename);
 	myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(*(new SAX::InputSource(filename)));

}

BriefingXmlAdapter::~BriefingXmlAdapter()
{

}

void BriefingXmlAdapter::startDocument()
{

}

void BriefingXmlAdapter::endDocument()
{

}

void BriefingXmlAdapter::startElement(const std::string & name, const SAX::AttributeList * atts)
{
   int i;

   if(name == "point")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "x")
        {
            currentpoint.x =  atof( currentvalue.c_str() );        
        }
        else if(currentname == "y")
        {
            currentpoint.y =  atof( currentvalue.c_str() );        
        }
        else if(currentname == "z")
        {
            currentpoint.z =  atof( currentvalue.c_str() );        
        }
      }
   }
   else if(name == "navpoint")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "radius")
        {
            radius =  atof( currentvalue.c_str() );        
        }
      }
      fixedships = new std::vector<BriefingShip *>;
   }
   else if(name == "ship")
   {
      lastautopilotnav = -1;
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "name")
        {
            shipname =   currentvalue;        
        }
		else if(currentname == "autopilot")
		{
           autopilot = (currentvalue == "true");		   
		}
		else if(currentname == "alignment")
		{
           enemy = (currentvalue == "enemy");		   
		}		
		else if(currentname == "capship")
		{
           capship = (currentvalue == "true");		   
		}		
		else if(currentname == "scale")
		{
            scale =  atof( currentvalue.c_str() );        
		}		
		else if(currentname == "lastautopilotnav")
		{
			lastautopilotnav = atoi(currentvalue.c_str() );   
		}

      }
   }
   else if(name == "briefing")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "playership")
        {
       	  player = 
	    	  new BriefingShip(
		         (char *) currentvalue.c_str(),
 		         false,
			     false,
			     Coordinate(0.0f, 0.0f, 0.0f),
				 lastautopilotnav);

        }
      }
   }

}

void BriefingXmlAdapter::endElement(const std::string& name)
{
   if(name == "atmessage")
   {
      at_text = lasttext;
	  lasttext = "";
   }
   else if(name == "leavingmessage")
   {
      leaving_text = lasttext;
	  lasttext = "";
   }
   else if(name == "ship")
   {
	  BriefingShip * newship = 
		  new BriefingShip(
		    (char *) shipname.c_str(),
			enemy,
			capship,
			currentpoint,
			lastautopilotnav);
	  if(autopilot)
	  {
		autopilotships->push_back(newship);
	  }
	  else
	  {
		fixedships->push_back(newship);
	  }
	  if(scale != 1.0f)
	  {
	    newship->setScale(Coordinate(scale, scale, scale));
	    scale = 1.0f;
	  }

   }
   else if(name == "navpoint")
   {
	   int id = navpoints->size();
	   BriefingNavPoint * newnav = 
		   new BriefingNavPoint(
			  id, 
			  currentpoint, 
			  radius,
			  fixedships,
			  at_text,
			  leaving_text
		   );
		navpoints->push_back(newnav);
		at_text = "";
		leaving_text = "";
   }
}

void BriefingXmlAdapter::characters(const std::string& ch)
{
  char buffer[100];
  int result;;

	// check for only whitespace
  result = sscanf ((char *) ch.c_str(), " %s", buffer);
  if(result <= 0)
  {
    return;
  }

  lasttext += "\n" + ch;
}

BriefingShip * BriefingXmlAdapter::getPlayer()
{
  return player;
}


std::vector<BriefingNavPoint *> * BriefingXmlAdapter::getNavPoints()
{
  return navpoints;
}

std::vector<BriefingShip *> * BriefingXmlAdapter::getAutopilotShips()
{
  return autopilotships;
}
