// CampaignPanel.cpp: implementation of the CampaignPanel class.
//
//////////////////////////////////////////////////////////////////////

#include "CampaignPanel.h"
#include "Campaign.h"
#include "TextToSpeech.h"
#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#include <sys/stat.h>
#include <dirent.h>
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CampaignPanel::CampaignPanel()
{
  int i;

  getCampaignNames();
  setTitle("Choose a Campaign");
  setConfirmText("OK");
  setNumLines(numcampaigns);
  for(i=0; i < numcampaigns; i++)
  {
     setLine(campaigntitles[i], i);
  }  
  campaignchanged = false;
}

CampaignPanel::~CampaignPanel()
{
  delete [] campaignfiles;
  delete [] campaigntitles;
}

void CampaignPanel::onLineClick(int linenumber)
{
   setStatusText(campaigntitles[linenumber] );

}

#ifndef WIN32
  int myfilter(const struct dirent * entry)
  {
    return strstr(entry->d_name, ".campaign.xml") != NULL;
  }
#endif

void CampaignPanel::getCampaignNames()
{
  int count=0;
  int i;  
  Campaign * campaign;  

#ifdef WIN32
  WIN32_FIND_DATA finddata;
  HANDLE findhandle;

  /* first count how many we need */
  count++;
  findhandle = FindFirstFile("*.campaign.xml", &finddata);
  while( FindNextFile(findhandle, &finddata) )
  {

    count++;
  }
  FindClose(findhandle);
  campaignfiles = new string[count];
  campaigntitles = new string[count];  


  /* now get the names */
  findhandle = FindFirstFile("*.campaign.xml", &finddata);
  campaignfiles[0] =  finddata.cFileName;
  count = 1;
  while( FindNextFile(findhandle, &finddata) )
  {
    campaignfiles[count] =  finddata.cFileName;
    count++;
  }
  FindClose(findhandle);
#else

  struct dirent **namelist;
  int n;

  n = scandir(".", &namelist, myfilter, alphasort);
  if (n < 0)
  {
     perror("scandir");
  }
  else 
  {
      count = n;
      campaignfiles = new string[count];
      campaigntitles = new string[count];  
      
      while(n--) 
      {
        campaignfiles[n] = namelist[n]->d_name;
        free(namelist[n]);
      }
      free(namelist);
  }
      
#endif


  numcampaigns = count;
  for(i=0; i < numcampaigns; i++)
  {
     campaign = new Campaign((char *) campaignfiles[i].c_str() );
     campaigntitles[i] = campaign->getTitle();
     delete campaign;
  }
}


void CampaignPanel::onCancel()
{
   setStatusText(string("") );
   campaignchanged = false;
}

void CampaignPanel::onConfirm()
{
   int i;

   string title;
   title = getStatusText();
   if(title == "")
     return;
   for(i=0; i < numcampaigns ; i++)
   {
      if(campaigntitles[i] == title)
         newcampaign = campaignfiles[i];

   }   

   TextToSpeech::getInstance()->synthesizeAndPlay((char *) title.c_str() );
   setStatusText(string("") );
   campaignchanged = true;
}

bool CampaignPanel::campaignChanged()
{
  return campaignchanged;
}


string CampaignPanel::getCampaignName()
{
  return newcampaign;
}
