// CapitalShip.cpp: implementation of the CapitalShip class.
//
//////////////////////////////////////////////////////////////////////

#include "CapitalShip.h"
#include "Mission.h"
#include "SoundFactory.h"
#include "World.h"
#include "Explosion.h"
#include "Component.h"
#include "OutOfBoundsException.h"
#include "SoundEffect3d.h"
#include <iostream.h>
#include "glerr.h"
#include "Turret.h"
#include "LightManager.h"
#include "Math.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

   


CapitalShip::CapitalShip(mesh * newmesh, int nummounts, mount * mounts[], 
        Coordinate cockpit, shipareas newshields, shipareas newarmor,
        float newvelocity, float newroll, float newpitch, float newyaw, 
        float newacceleration, float newangularacceleration,
		float newafterburnervelocity, float newfterburneracceleration, float newfuel,
        int numthrusters, Thruster * thrusters[], Image * newimage, string newname, int totalenergy, int energyrecharge,
        float mass, 
		std::vector<Turret *> * turrets,
		std::vector<Coordinate> newenginelocations,
		Component ** new_components, int new_num_components,
		std::vector<Coordinate> newexplosionlocations
		)
        : ship(newmesh, nummounts, mounts, cockpit, newshields, newarmor, 
		       newvelocity, newroll,
               newpitch, newyaw, 
		       newacceleration, newangularacceleration,
		       newafterburnervelocity, newfterburneracceleration, newfuel,			   
			   numthrusters, thrusters, newimage, newname, totalenergy,
               energyrecharge, mass, "", 0, turrets, newenginelocations, 0),
          land(false),		  
          components(new_components),
          num_components( new_num_components),
		  fullradius(0.0f),
		  explosionlocations(newexplosionlocations)
		  

{
  computeFullExtents();
}


CapitalShip::CapitalShip(CapitalShip & newship)
   : ship(newship),
     land(false),
     num_components(newship.num_components),
	 fullextents(newship.fullextents),
	 fullcenter(newship.fullcenter),
	 fullradius(newship.fullradius),
	 explosionlocations(newship.explosionlocations)
{
  int i;
 
  components = new Component * [num_components];
  for(i=0; i < num_components; i++)
  {
    components[i] = new Component(*newship.components[i]);
  }
}

CapitalShip::~CapitalShip()
{
  int i;

  for(i=0 ; i < num_components; i++)
  {
     delete components[i];
  }
  delete [] components;
}


Color CapitalShip::getRadarColor(ship * player)
{
  if(isdestroyed() )
  {
	  return Color(.6f, 0.6f, 0.6f);
  }
  else if(iffcode == FRIEND)
  {
    return Color(0.0f,  0.70196f, 1.0f);
  }
  else if(iffcode == ENEMY)
  {
    return Color(1.0f, 0.5f, 0.0f);
  }
  else if(iffcode == NEUTRAL )
  {
    return Color(.6f, 0.6f, 0.6f);
  }
  return Color(1.0f, 0.0f, 0.0f);
}

void CapitalShip::setLandOn(bool value)
{
  land = value;
}


bool CapitalShip::instanceof(char classname [ ])
{
  if(0 == strcmp(classname, "CapitalShip")) 
    return true;
  return ship::instanceof(classname);
}	


void CapitalShip::onCollisionWith(Object3d * otherobject)
{
  Mission * mission = Mission::getCurrentMission();


  if(land && (otherobject == (Object3d *) mission->getPlayer()) && mission->isComplete() )
  {
     mission->onLanding();
  }
  else
  {
    ship::onCollisionWith(otherobject);
  }
}

/* Damage components only  */
void CapitalShip::damage(int amount, Coordinate fromlocation)
{

}

void CapitalShip::destroy()
{
  int i;

  Mission::getCurrentMission()->onDestroyed(this);
  setDestroyed();
  for(i=0; i < explosionlocations.size() ; i++)
  {
	Coordinate explocation = explosionlocations[i];
	Coordinate worldlocation = toWorldLocation(explocation);
    World::getWorld().addExplosion(new Explosion(worldlocation));
  }

  SoundFactory::getSoundFactory()->getExplosionSound()->play();
  getComm()->dead();
}

void CapitalShip::render()
{
   if(!shouldRender() )
   {
	return;
   }

   ship::render();
   Coordinate location = getLocation();
   Coordinate heading = getHeading();
   Coordinate up = getUp();


   int i;
   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); 
   for(i=0; i < num_components; i++)
   {
///
	   // render local lights

	   LightManager * lightmanager = LightManager::getInstance();
/*
       glPushMatrix();
       glLoadIdentity();
	   lightmanager->renderLocalLights( getComponentLocation(i)  );
	   glPopMatrix();
*/
       glPushMatrix();
       lightmanager->renderGlobalLights();
	   lightmanager->renderLocalLights(getComponentLocation(i) );
	   glPopMatrix();


// end lighting stuff      
       if(components[i]->getName() == "Engine")
	   {
//		 glDisable(GL_LIGHTING);
	   }


      glPushMatrix();
          location = getLocation();
          glTranslatef(location.x, location.y, location.z);
          matchDOF(heading, up);    
		  if(isdestroyed() )
		  {
            components[i]->renderDestroyed();
		  }
		  else
		  {
            components[i]->render();
		  }
      glPopMatrix();
       if(components[i]->getName() == "Engine")
	   {
//		 glEnable(GL_LIGHTING);
	   }

   }
   for(i=0; i < turrets->size(); i++)
   {
///
	   // render local lights

	   LightManager * lightmanager = LightManager::getInstance();

/*       glPushMatrix();
       glLoadIdentity();
	   lightmanager->renderLocalLights( getComponentLocation(i + num_components)  );
	   glPopMatrix();
*/
       glPushMatrix();
       lightmanager->renderGlobalLights();
	   lightmanager->renderLocalLights(getComponentLocation(i + num_components)  );
	   glPopMatrix();


// end lighting stuff      

      glPushMatrix();
          location = getLocation();
          glTranslatef(location.x, location.y, location.z);
          matchDOF(heading, up);    
		  if(isdestroyed() )
		  {

            (*turrets)[i]->renderDestroyed();
		  }
		  else
		  {
            (*turrets)[i]->render();
		  }
      glPopMatrix();
   }
   // calculate turret matrix for collision detection 
   glPushMatrix();
     glLoadIdentity();
     location = getLocation();
     glTranslatef(location.x, location.y, location.z);
     matchDOF(heading, up);    

     for(i=0; i < turrets->size(); i++)
	 {
        glPushMatrix();
		    Turret * myturret = (*turrets)[i];
            myturret->pushLocalTransform();
			float matrix[16];
			glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
			myturret->setMatrix(matrix);
			// now do turret barel
			myturret->pushBarrelTransform();
			glGetFloatv(GL_MODELVIEW_MATRIX, matrix);
			myturret->setBarrelMatrix(matrix);
        glPopMatrix();
	 }
	 for(i=0; i < num_components; i++)
	 {
        Component * mycomp  = components[i];
        glPushMatrix();
			mycomp->pushLocalTransform();
			float matrix2[16];
			glGetFloatv(GL_MODELVIEW_MATRIX, matrix2);
			mycomp->setMatrix(matrix2);
        glPopMatrix();
	 }

   glPopMatrix();
   GLERRORCHECK
}

int CapitalShip::getNumComponents()
{
   return num_components + (*turrets).size() ;
}

Component * CapitalShip::getComponent(int index)
{
  if(index >=  getNumComponents() )
  {
     throw OutOfBoundsException("CapitalShip.cpp");
  } 
  if(index < num_components)
  {
    return components[index];
  }
  else
  {
     return (*turrets)[index - num_components];
  }
}

void CapitalShip::calculateRectangle()
{
	Targetable::calculateRectangle();

	ship * player = Mission::getCurrentMission()->getPlayer();
    Component * component = getComponent(player->getTargetedCapshipComponentIndex() );
	if(true /*component->instanceof(INST_TURRET)*/ )
	{
		float matrix[16];

		component->getMatrix(matrix);
		matrix[12] = 0.0f; 
		matrix[13] = 0.0f;
		matrix[14] = 0.0f;

		Coordinate center;

		center = component->getMeshCenter();
        center = center.multiplybyglmatrix(matrix);
		center = center + component->getLocation();

	
    componentsquare = Targetable::calculateRectangle(
		 center
		, component->getExtents() );

	}
	else
	{
/*    componentsquare = Targetable::calculateRectangle(
		component->getMeshCenter() + component->getLocation()
		, component->getExtents() );
*/
	}
}

ReticleSquare CapitalShip::getComponentSquare()
{
  return componentsquare;
}

Coordinate CapitalShip::getComponentLocation(int index)
{
  GLfloat matrix[16];
  Coordinate worldpoint;
  Component * comp;

  comp = getComponent(index);
  if(comp->instanceof(INST_TURRET) )
  {
    Turret * turr = (Turret *) comp;

	turr->getMatrix(matrix);
    worldpoint = comp->getMeshCenter().multiplybyglmatrix(matrix);
  }
  else
  {
    matchDOF(getHeading(), getUp(), matrix);
  //  worldpoint = components[index]->getMeshCenter().multiplybyglmatrix(matrix);
    worldpoint = comp->getMeshCenter().multiplybyglmatrix(matrix);
    worldpoint = worldpoint  + getLocation();
  }
  return worldpoint;
}

Coordinate CapitalShip::getExtents()
{
//	return Object3d::getExtents();
  return fullextents;
}

void CapitalShip::computeFullExtents()
{
  Coordinate currentmax, currentmin;
  Coordinate delta_extents;
  Component * current_component;
  int i;


  currentmax = getMeshMaxs();
  currentmin = getMeshMins();

  for(i=0; i < num_components; i++)
  {
	current_component = components[i];
    currentmax = currentmax.maxPairwise(current_component->getMeshMaxs() );
    currentmin = currentmin.minPairwise(current_component->getMeshMins() );
  }

  fullextents = currentmax - currentmin;
  fullcenter = (currentmax + currentmin) * .5f;
  delta_extents = fullextents * .5f;
  fullradius = fastsqrt( fabs_f(delta_extents.x * delta_extents.x) + 
					 fabs_f(delta_extents.y * delta_extents.y) +
					 fabs_f(delta_extents.z * delta_extents.z) );
  
}

Coordinate CapitalShip::getMeshCenter()
{
//  return Object3d::getMeshCenter();
  return fullcenter;
}

float CapitalShip::getradius()
{
//  return Object3d::getradius();  
  return fullradius;
//	return 999999.f;
}

void CapitalShip::advance()
{
  if(isdestroyed() )
	  return;

  ship::advance();
  int i;
  bool alldestroyed = true;
  Component * comp;
  Coordinate worldlocation;

  for(i=0; i < getNumComponents() ; i++)
  {
	  comp = getComponent(i);
	  if((! comp->isDestroyed())
		  && (!(comp->instanceof(INST_TURRET)))
		)
	  {
		alldestroyed = false;
	  }

	  worldlocation = getComponentLocation(i ) ;
	  if(comp->justDestroyed() )
	  {
		  // do an explosion
          World::getWorld().addExplosion(new Explosion (worldlocation ) );
          SoundEffect3d * effect = SoundFactory::getSoundFactory()->getExplosionSound();
          effect->setPosition( worldlocation  );
          effect->play();
	  }
	  comp->setWorldLocation(worldlocation);
	  // don't move this
	  comp->advance();
  }
  if(alldestroyed)
  {
     destroy();
  }
}


bool CapitalShip::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_TARGETABLE | INST_SHIP|INST_CAPITALSHIP);
}


bool CapitalShip::getLandOn()
{
  return land;
}

Coordinate CapitalShip::getVelocityVector()
{
  return getHeading() * getVelocity() * Time::getTimeScale();
}

int CapitalShip::getNumComponentsOnly()
{
  return num_components;
}
