// CircularBuffer.cpp: implementation of the CircularBuffer class.
//
//////////////////////////////////////////////////////////////////////

#include "CircularBuffer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#ifndef CIRCBUFF_CPP
#define CIRCBUFF_CPP


template <class T>
CircularBuffer<T>::CircularBuffer(int maxsize)
{
  array = new T[maxsize];
  maxelements = maxsize;
  front = 0;
  numelements = 0;
  back = maxsize - 1;
}

template <class T>
CircularBuffer<T>::~CircularBuffer()
{
  delete [] array;
}

template <class T>
int CircularBuffer<T>::getNumElements()
{
  return numelements;
}

template <class T>
T CircularBuffer<T>::get(int index)
{
  if((index > numelements - 1) || index < 0)
  {
	throw IllegalArgumentException();
  }
  int trueindex = (front + index);
  if(trueindex >= maxelements)
  {
	trueindex -= maxelements;
  }
  return array[trueindex];
}

template <class T>
void CircularBuffer<T>::addBack(T newelement)
{
  int index;

  if(numelements + 1 > maxelements)
  {
	throw IllegalArgumentException();
  }
  numelements++;
  index = (back + 1);
  if(index == maxelements)
  {
	index = 0;
  }
  array[index] = newelement;
  back = index;
}

template <class T>
void CircularBuffer<T>::removeFront()
{
  if(numelements == 0)
  {
	throw IllegalArgumentException();
  }

  if(front == maxelements - 1)
  {
	front = 0;
  }
  else
  {
    front = front + 1;
  }
  numelements--;
}

template <class T>
bool CircularBuffer<T>::isFull()
{
  return numelements == maxelements;
}

#endif