// Cockpit.cpp: implementation of the Cockpit class.
//
//////////////////////////////////////////////////////////////////////

#include "GL/glew.h"
#include "Cockpit.h"
#include <glut.h>
#include "Font.h"
#include "Circle.h"
#include "coord.h"
#include "World.h"
#include "shipareas.h"
#include "Targetable.h"
#include "Mission.h"
#include <iostream.h>
#include "TransparentImage.h"
#include "Constants.h"
#include "ReticleFactory.h"
#include "Reticle.h"
#include "FontFactory.h"
#include "CapitalShip.h"
#include "ImageFactory.h"
#include "Component.h"
#include "Settings.h"
#include "glerr.h"
#include "FixedCamera.h"
#include "CockpitCamera.h"
#include "Ellipse.h"
#include "Comm.h"
#include "LightManager.h"
#include "Math.h"
#include "DefaultColor.h"
#include "SoundFactory.h"

//#include "Shader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Cockpit::Cockpit(ship * newplayer)
  : player(newplayer),
    viewcone(640.0f, 480.0f, 45.0f)
  
{
  DisplayMode displaymode;
  const float defaultwidth = 800.0f;

  displaymode = Settings::getSettings()->getDisplayMode();
  sizemultiplier = ((float)displaymode.width) / defaultwidth;

  simplecockpit = ! Settings::getSettings()->getCockpitDetails();
  fpsenables = false;
//  POINT p;
  float winWidth = glutGet(GLUT_WINDOW_WIDTH);
  float winHeight = glutGet(GLUT_WINDOW_HEIGHT);
/*  p.x = 10;
  p.y = 10;
*/

//  HWND window = WindowFromPoint(p);
  projectionx = winWidth / 2.0f;
  projectiony = winHeight / 5.0f;
  circle1 = new Circle(projectionx, projectiony, 40.0f * sizemultiplier);
  circle2 = new Circle(projectionx, projectiony, 30.0f * sizemultiplier);
  circle3 = new Circle(projectionx, projectiony, 10.0f * sizemultiplier);

		float rsizex = winHeight * .25f;
		float rsizey = rsizex * .75;
		float rstartx, rstarty;

		rstartx = .9f * winWidth - rsizex ;
	    rstarty = .9f * winHeight - rsizey;

  ellipse = new EllipseE(rsizex * .6f, 
	          rsizey * .6f,
			  rstartx + rsizex *.5f, 
			  rstarty + rsizey *.5f);

  innerprojectionradius = 30.0f * sizemultiplier;
  outerprojectionradius = 40.0f * sizemultiplier;
  autopilot = NULL;
  eject = NULL;
  lock = NULL;

  linesdisplaylist = glGenLists(1);
  if(linesdisplaylist == GL_INVALID_VALUE  || linesdisplaylist == GL_INVALID_OPERATION)
     cout << "Display list error" << endl;
  glNewList(linesdisplaylist, GL_COMPILE);

   glColor3f(0.04f, 0.0f, .52f);
//  	 DefaultColor::textcolor.render();
//  DefaultColor::green.render();
   // divider lines on the circles
   float sqrt2 =  0.707106f;

   const float crosswidth= 20.0f;
   float centerx;
   float centery;

   centerx = winWidth /2.0f;
   centery = winHeight /2.0f;

   float linesize = 10.0f * sizemultiplier;


   glBegin(GL_LINES);
      glVertex2f(projectionx - sqrt2 * linesize , projectiony- sqrt2 * linesize);
      glVertex2f(projectionx- sqrt2 * outerprojectionradius, projectiony - sqrt2 * outerprojectionradius);
      glVertex2f(projectionx + sqrt2 * linesize , projectiony + sqrt2 * linesize);
      glVertex2f(projectionx + sqrt2 * outerprojectionradius, projectiony + sqrt2 * outerprojectionradius);
      glVertex2f(projectionx - sqrt2 * linesize , projectiony + sqrt2 * linesize);
      glVertex2f(projectionx - sqrt2 * outerprojectionradius, projectiony + sqrt2 * outerprojectionradius);
      glVertex2f(projectionx + sqrt2 * linesize , projectiony - sqrt2 * linesize);
      glVertex2f(projectionx + sqrt2 * outerprojectionradius, projectiony - sqrt2 * outerprojectionradius);
/*
      glColor3f(1.0f, 1.0f, 1.0f);
 old crosshair
      glVertex2f(centerx - crosswidth, centery);
      glVertex2f(centerx + crosswidth, centery);

      glVertex2f(centerx , centery - crosswidth);
      glVertex2f(centerx , centery + crosswidth);
*/

   glEnd();

  glEndList();
  show_comm = false;
  show_damage = false;
  greenarrow = new TransparentImage("greenarrow.png");;
  ittsarrow = new TransparentImage("ittsarrow.png");
  decoy2dpic = new TransparentImage("decoy2d.png");
  crosshair_fullguns = new TransparentImage("crosshair3.png");;
  crosshair_energy = new TransparentImage("crosshair1.png");;
  crosshair_ammo = new TransparentImage("crosshair2.png");;
  crosshair_charging = new TransparentImage("crosshair4.png");
  if( simplecockpit)
  {
    oval = NULL;
  }
  else
  {
    oval = new TransparentImage("oval.bmp", Color(0.0f, 0.0f, 0.0f));
  }
  if(winWidth > 800.0f)
  {
    font = FontFactory::getInstance().getLargeFont();
  }
  else
  {
    font = FontFactory::getInstance().getFont();
  }

  if(simplecockpit)
  {
    struts = NULL;
	guns2dpic = NULL;
  }
  else
  {
    struts = new Object3d( new mesh((char *)player->getCockpitMesh().c_str()  ), 0.0f, 0.0f, 0.0f , 0.0f, 0.0f, 0.0f);
    guns2dpic = ImageFactory::getTransparentImage("fullguns2d.bmp", Color(1.0f, 0.0f, 1.0f));	   

  }

  // create comm circle

  float bsizex, bsizey;
  float bstartx, bstarty;

  bsizex = 128.f;
  bsizey = 128.f;

  float boffset = .125f * bsizex;
  bstartx = .1f * winWidth ;
  bstarty = .9f * winHeight - bsizey;
	
  commcircle = new EllipseE(.5f * bsizey + .75 * boffset,
	.5f * bsizey + .75f * boffset , 
	bstartx + .5f*bsizex, 
	bstarty + .5f * bsizey);


  GLERRORCHECK

}

Cockpit::~Cockpit()
{
  delete circle1;
  delete circle2;
  delete circle3;
  delete autopilot;
  delete lock;
  delete crosshair_fullguns;
  delete crosshair_energy;
  delete crosshair_ammo;
  delete crosshair_charging;
  delete oval;
  delete guns2dpic;
  delete ellipse;
  delete commcircle;
  delete greenarrow;
  delete ittsarrow;
  delete decoy2dpic;
  glDeleteLists(linesdisplaylist, 1);
  GLERRORCHECK
	    SoundEffect * locksound = SoundFactory::getSoundFactory()->getMissileAlertSound();
		locksound->stop();
	    SoundEffect * ejectsound = SoundFactory::getSoundFactory()->getEjectAlertSound();
		ejectsound->stop();

}

//extern Shader * shader;

void Cockpit::render()
{
   float distancefactor = Constants::getReciprocalDistanceFactor();
//   static Reticle * reticle = ReticleFactory::getInstance()->getReticle("center", "ar_reticle");
   float centerx, centery, x, y;

   //////////////////////////
   
   ///////////////////////
  	winWidth = glutGet(GLUT_WINDOW_WIDTH);
	winHeight = glutGet(GLUT_WINDOW_HEIGHT);

   // render 3d struts

   if(! simplecockpit)
   {
      Coordinate location = player->getCockpitLocation() + player->getHeading() * 1.f;
	  struts->setLocation(location);
	  struts->setOrientation(player->getHeading(), player->getUp(), player->getleft() );
//	  shader->begin();
	  struts->render();
//	  shader->end();
   }




   glClear(GL_DEPTH_BUFFER_BIT );

   glViewport(0, 0, winWidth, winHeight);

   glDisable(GL_TEXTURE_2D);
   glDisable(GL_DEPTH_TEST);
   glDepthMask(GL_FALSE);
   glColorMask( GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
   glColorMask( GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
 

   glDisable(GL_CULL_FACE);
   glDisable(GL_LIGHTING);

//	float radarWidth;
//	float radarHeight;

//   glClear (GL_DEPTH_BUFFER_BIT);
   float winHeightOverFive = winHeight / 5.0f;
   float winWidthOverThree = winWidth / 3.0f;
   float winWidthOverFour = winWidth / 4.0f;


    glMatrixMode (GL_PROJECTION);
    glPushMatrix();//1
    glLoadIdentity ();

    gluOrtho2D(-0.01f, winWidth + 0.01f, 0.0f, winHeight);

    glMatrixMode (GL_MODELVIEW);
	 glPushMatrix();//2
    glLoadIdentity ();
   glLineWidth(2.0f * sizemultiplier);   
   glCallList(linesdisplaylist);
   glLineWidth(1.0f);   
   drawCrossHair();
   



//   glColor3f(0.0f, 1.0f, 0.0f);
//   DefaultColor::green.render();
   glColor3f(0.04f, 0.0f, .52f);
//	 DefaultColor::textcolor.render();

   glLineWidth(2.0f * sizemultiplier);
   circle1->render();
   circle2->render();
   circle3->render();
   glLineWidth(1.0f);
   glColor3f(0.0f,  0.0f, 1.0f);
   glPointSize(2.0f * sizemultiplier);
   glBegin(GL_POINTS);
   World::getWorld().enumerateShips(this);
   glEnd();
   glPointSize(1.0f);
   Targetable * target = player->getTarget();
   if(target != NULL)
   {
     float xlocation = winWidth/16.0f;
      drawShipStatus(target, winWidthOverThree, winHeightOverFive);
//     target->getImage()->render(20.0f, 20.0f);
     Color color = target->getRadarColor(player);
     color.render();
     font->print(xlocation, winHeightOverFive + 20.0f, "Target");


/// added to test targeting
	 if(isTargetVisible(target) ) // may have to be more general for capships
	 {
		ReticleSquare location = target->getReticleSquare();

		// if all rectangle points are behind, don't render
		if(location.isValid() )
		{	 
	//       location.render();
			location.render(winWidthOverThree, winHeight / 50, 
				player->isTargetLockSelected() );
			if(player->lockRequired() && player->hasLock() && (! target->instanceof(INST_CAPITALSHIP)) )
			{
				Reticle * reticle = player->getCurrentReticle();
				if(reticle != NULL)
				{
					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
					glEnable(GL_BLEND);
					Rect temprect = location.toRect();
					reticle->render(temprect, .5f);
					glDisable(GL_BLEND);
				}
			}
		}

		if(target->instanceof(INST_CAPITALSHIP) )
		{
			CapitalShip * capship = (CapitalShip *) target;
			ReticleSquare componentlocation = capship->getComponentSquare();
			if(componentlocation.isValid() && ! capship->isdestroyed() )
			{
				Color(1.0f, 1.0, 0.1f).render();

      			componentlocation.render(winWidthOverThree, winHeight / 50, 
	     			player->isTargetLockSelected() );
    			Reticle * reticle = player->getCurrentReticle();
	    		if(reticle != NULL && player->lockRequired() && player->hasLock() )
				{
					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
					glEnable(GL_BLEND);
                                        Rect temprect2 = componentlocation.toRect();
					reticle->render(temprect2, .5f);
					glDisable(GL_BLEND);
				}
				color.render();
			}
		}
		else
		{
			// itts
			Coordinate projecteditts = player->getProjectedItts();
	  	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable(GL_BLEND);

			ittsarrow->render(
				projecteditts.x - ittsarrow->getWidth(), 
				projecteditts.y - ittsarrow->getHeight()
			);
			glDisable(GL_BLEND);
			

		}
	 } // is target visible
////
	 else // target not visible
	 {
		// render an arrow showing direction to target
        Coordinate vectortotarget = target->getLocation() - player->getLocation() ;
        float side = vectortotarget.dot( player->getleft() );
		float up = vectortotarget.dot( player->getUp() );
		Coordinate newcoord(side, up, 0.0f);
		newcoord.normalize();

		float theta = atan2( newcoord.y, newcoord.x);
		newcoord = newcoord * .06f * winWidth;
  	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		greenarrow->setRotation(-Math::radtodeg(theta) );
		float gwidth = greenarrow->getWidth() * .5f;;
		float gheight = greenarrow->getHeight() *.5f;
        greenarrow->render(winWidth * .5f - newcoord.x- gwidth, winHeight * .5f + newcoord.y - gheight);
		glDisable(GL_BLEND);
	 }

     if(target->instanceof(INST_SHIP) )
     {
        ship * shp = (ship *) target;
        if(shp->instanceof(INST_CAPITALSHIP) && ! shp->isdestroyed() )
        {
			Component * comp = player->getTargetedCapshipComponent();
			if(comp->isDestroyed() )
			{
				// grey out destroyed component
				Color greycolor(1.0f, 0.0f, 0.0f) ;
				greycolor.render();
				font->printf(xlocation, winHeightOverFive, "%s destroyed", comp->getName().c_str() );
			}
			else
			{
				Color color = target->getRadarColor(player);
				color.render();

				font->printf(xlocation, winHeightOverFive, "%s", comp->getName().c_str() );
				float capshippercent = comp->getDamagedPercent();
				barHorizontalGraph(capshippercent, 1.0f, 10, xlocation + 70, 
					winHeightOverFive, xlocation + 120, winHeightOverFive + 10, 
					Color(1.0f, 0.0f, 0.0f), Color(1.0f, 1.0f, 1.0f), false);
				color.render();
			}
        }
        else 
        {
 	      Color tcolor = target->getRadarColor(player);
          tcolor.render();

          font->printf(xlocation, winHeightOverFive, "%s", shp->getPilot().c_str() );
        }
     }
     target->getRadarColor(player).render();
     font->print(xlocation, winHeightOverFive - 20.0f, (char *) target->getName().c_str() );

     float distance = target->getLocation().distance(player->getLocation() ) * distancefactor;
     target->getRadarColor(player).render();
     font->printf(xlocation, winHeightOverFive - 40.0f, "Distance:%5.0f", distance );
   }
//   glColor3f(0.0f, 1.0f, 0.0f);
   DefaultColor::textcolor.render();
   float newxlocation = winWidth*4.0f/5.0f;
   if(player->isFullGuns() )
   {
      {
        font->print(newxlocation, winHeightOverFive, "Full Guns");
      }
   }
   else
   {
      if(player->isCurrentGunUnlimitedAmmo() )
      {
         font->print(newxlocation, winHeightOverFive, (char *) player->getCurrentGunName().c_str() );
      }
      else
      {
         font->printf(newxlocation, winHeightOverFive, "%s : %d", player->getCurrentGunName().c_str(),
          player->getCurrentGunAmmoRemaining() );
      }
   }

   font->printf(newxlocation, winHeightOverFive - 60.0f, "%d %s", player->getNumCurrentMissile(),
         player->getCurrentMissileName().c_str() );


   font->printf(newxlocation + decoy2dpic->getWidth(), winHeightOverFive - 85.0f, "%d Decoys", player->getNumDecoys());
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
     decoy2dpic->render(newxlocation, winHeightOverFive - 90.0f);
   glDisable(GL_BLEND);


   if(! simplecockpit)
   {

     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     if(player->isFullGuns() )
	 {
       guns2dpic->render(newxlocation, winHeightOverFive + 20.0f);
	 }
     else if(player->getCurrentGunIcon() != NULL)
	 {
  	   player->getCurrentGunIcon()->render(newxlocation, winHeightOverFive + 20.0f );
	 }
   
     if(player->getCurrentMissileIcon() != NULL)
	 {
       player->getCurrentMissileIcon()->render(newxlocation, winHeightOverFive - 40.0f);
	 }

     glDisable(GL_BLEND);
   }

//

   drawShipStatus(player, 2.0f * winWidthOverThree, winHeightOverFive);

   drawEnergy(player->getMaxEnergy(),player->getCurrentEnergy() );
   drawFuel(player->getMaximumFuel(),player->getCurrentFuel() );
   doAutoPilot();
   doEject();
   doLock();
   if(show_comm)
   {
     renderComm();
   }
   if(show_damage)
   {
	 renderDamageReport();
   }
   // frames per second

   
//    glColor3f(0.0f, 1.0f, 0.0f);
	DefaultColor::textcolor.render();
	if(fpsenables)
	{
      float yfpslocation = 4 * winHeight/5.0;
      float xfpslocation = winWidth/16.0f;
	  font->printf(xfpslocation, yfpslocation, "%d frames per second", Time::getFramesPerSecond() );
	}

   // speeds
	 if(winWidth <= 800)
	 {
       centery = winHeight * .95f;
	 }
	 else
	 {
       centery = winHeight * .9f;
	 }
     centerx = winWidth  * .6f;

	 // old oval draw

	 if( ! simplecockpit )
	 {
         centerx = winWidth  * .4f;
	 }
	 else
	 {
       centerx = winWidth  * .4f;
	 }

	 int charwidth = font->getWidth("Vel:999");
	 int charheight = font->getWidth("Q");
     x = centerx - charwidth * .5f;
     y = centery  - charheight * .5f;
//     glColor3f(.8f, .8f, 1.0f);
     DefaultColor::textcolor.render();
	 font->printf(x, y, "Set:%3.f", player->getDesiredVelocity() * distancefactor );
     centerx = winWidth  * .6f;
     x = centerx - charwidth * .5f;
     y = centery  - charheight * .5f;
	 font->printf(x, y, "Vel:%3.f", player->getVelocity() * distancefactor );

	 if(winWidth <= 800)
	 {
		centery = winHeight * .95f;
	 }
	 else
	 {
		centery = winHeight * .9f;
	 }
     centerx = winWidth  * .6f;

	 if( ! simplecockpit )
	 {
       x = centerx - ((float) oval->getWidth() ) * .5f;
       y = centery - ((float) oval->getHeight() ) * .5f;

	 
       glBlendFunc(GL_SRC_ALPHA, GL_ONE);
       glEnable(GL_BLEND);
	     oval->render(x, y);
         centerx = winWidth  * .4f;
         x = centerx - ((float) oval->getWidth() ) * .5f;
	     oval->render(x, y);
       glDisable(GL_BLEND);
	 }
	 else
	 {
       centerx = winWidth  * .4f;
       x = centerx ;
       y = centery ;       
	 }

	 // end speeds




// backdrop for comm
 	Movie * comm_movie = Comm::getCurrentMovie();
	 if(comm_movie != NULL)
	 {
		   if(Comm::isCurrentFriendly() )
		   {

			glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
		   }
		   else
		   {
			glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
		   }
	
			float bsizex, bsizey;
			float bstartx, bstarty;

			bsizex = 128.f;
			bsizey = 128.f;

			float boffset = .125f * bsizex;

			bstartx = .1f * winWidth ;
			bstarty = .9f * winHeight - bsizey;
	
/*			EllipseE commcircle(.5f * bsizey + .75 * boffset,
				.5f * bsizey + .75f * boffset , 
				bstartx + .5f*bsizex, 
				bstarty + .5f * bsizey);
*/

// video
			Image * videoimage = comm_movie->getCurrentFrame();
			if(videoimage != NULL)
			{			
				videoimage->render(bstartx, bstarty);
			}

			Rect brect(bstartx, bstarty, bsizex, bsizey);
			brect.render();
			float bxa = bstartx - boffset;
			float bxb = bstartx;
			float bxc = bstartx + bsizex;
			float bxd = bxc + boffset;
			float bya = bstarty + boffset;
			float byb = bstarty + bsizey - boffset;

	        glBegin(GL_LINE_STRIP);
				glVertex2f(bxb, bya);
				glVertex2f(bxa, bya);
				glVertex2f(bxa, byb);
				glVertex2f(bxb, byb);
		    glEnd();
	
			glBegin(GL_LINE_STRIP);
				glVertex2f(bxc, bya);
				glVertex2f(bxd, bya);
				glVertex2f(bxd, byb);
				glVertex2f(bxc, byb);
			glEnd();

            commcircle->render();
			// label for comm backdrop
			string commtext = Comm::getCurrentLabel();
			commtext = "[" + commtext + "]" ;
			float fontwidth = (float) font->getWidth((char *) commtext.c_str() );
			float fontheight = (float) font->getHeight((char *) commtext.c_str() );

            float fontcenterx = bstartx + .5f * bsizex;
			float fontcentery = bstarty ;
            font-> print(fontcenterx - .5f * fontwidth, 
				fontcentery - 2.0f * boffset, (char *) commtext.c_str() );
            
	 }



// backdrop for rear view
	 if(target != NULL)
	 {
		Color targcolor = target->getRadarColor(player);
		glColor4f(targcolor.r, targcolor.g, targcolor.b, .4f);
		float rsizex = winHeight * .25f;
		float rsizey = rsizex * .75;
		float rstartx, rstarty;

		rstartx = .9f * winWidth - rsizex ;
	    rstarty = .9f * winHeight - rsizey;

        float offset = .025f * winWidth;

		if(! simplecockpit)
		{
		  glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		  glEnable(GL_BLEND);
		}
		
		  glRectf(rstartx, rstarty, rstartx + rsizex, rstarty + rsizey);
		  glColor4f(targcolor.r, targcolor.g, targcolor.b, 1.0f);
		  Rect r(rstartx, rstarty, rsizex, rsizey);
		  r.render();

          float xa = rstartx - offset;
		  float xb = rstartx;
		  float xc = rstartx + rsizex;
		  float xd = xc + offset;
		  float ya = rstarty + offset;
		  float yb = rstarty + rsizey - offset;

          glBegin(GL_LINE_STRIP);
			glVertex2f(xb, ya);
			glVertex2f(xa, ya);
			glVertex2f(xa, yb);
			glVertex2f(xb, yb);
		  glEnd();

          glBegin(GL_LINE_STRIP);
			glVertex2f(xc, ya);
			glVertex2f(xd, ya);
			glVertex2f(xd, yb);
			glVertex2f(xc, yb);
		  glEnd();

		 

		  ellipse->render();
	    if(! simplecockpit)
		{
	      glDisable(GL_BLEND);
		  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		}
	 }


// end backdrop

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);

	glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);
    renderRearView(winWidth, winHeight);

// now we're back to normal model view, render the rearview
   glEnable(GL_TEXTURE_2D);
//   glEnable(GL_DEPTH_TEST);
   glEnable(GL_CULL_FACE);
   glEnable(GL_LIGHTING);

   GLERRORCHECK
}

void Cockpit::barVerticalGraph(float value, float max, int numdivisions, float x1, float y1, float x2, float y2, Color basecolor, Color endcolor,  bool updown, bool taper)
{
    Color color1, color2;

	float percent = value/max;
    float top = y1 + (y2 - y1) * percent;
	float topoffset;
	float bottomoffset;
	const float tapersize = 10.0f;

	if(updown)
	{
		color1 = basecolor;
		color2 = basecolor.interpolate(endcolor, 1.0f - percent);
        topoffset = 0.0f;
		if(taper)
		{
		  bottomoffset = tapersize;
		}
		else
		{
          bottomoffset = 0.0f;
		}
	}
	else
	{
		color2 = basecolor;
		color1 = basecolor.interpolate(endcolor, 1.0f - percent);
		if(taper)
		{
          topoffset = tapersize;
		}
		else
		{
          topoffset = 0.0f;
		}
		bottomoffset = 0.0f;

	}


    if(percent != 0.0f)
	{
		glBegin(GL_QUADS );
   		    color1.render();
			glVertex2f(x1 + bottomoffset, y1);
			color2.render();
			// upper left
			glVertex2f(x1 + topoffset, top);
            color2.render();
			glVertex2f(x2 - topoffset, top);
            color1.render();
			glVertex2f(x2 - bottomoffset, y1);
		glEnd();
	}

}

void Cockpit::barHorizontalGraph(float value, float max, int numdivisions, float x1, float y1, float x2, float y2, Color basecolor, Color endcolor, bool righttoleft, bool taper)
{
	float percent = value/max;
    float right = x1 + (x2 - x1) * percent;
	Color first, second;

	float rightoffset=0.0f;
	float leftoffset=0.0f;
	const float tapersize = 10.0f;

	if(righttoleft)
	{

		first = basecolor.interpolate(endcolor, 1.0f - percent);
		second = basecolor;
		if(taper)
		{
           rightoffset = tapersize;
		   leftoffset = 0.0f;
		}
	}
	else
	{	first = basecolor;
		second = basecolor.interpolate(endcolor, 1.0f - percent);
		if(taper)
		{
           rightoffset = 0.0f;
		   leftoffset = tapersize;
		}

	}


    if(percent != 0.0f)
	{
		glBegin(GL_QUADS );
		    // lower left
		    first.render();
			glVertex2f(x1, y1 - leftoffset);
			// upper left
			glVertex2f(x1, y2 + leftoffset);
	        second.render();
			glVertex2f(right, y2 + rightoffset);
			glVertex2f(right, y1 - rightoffset);
		glEnd();
	}
}

void Cockpit::drawCrossHair()
{
   float centerx;
   float centery;
   float x, y;

   centerx = winWidth /2.0f;
   centery = winHeight /2.0f;
   Image * currentimage;
   if(player->isFullGuns() )
   {
     currentimage = crosshair_fullguns;
   }
   else if(player->isCurrentGunChargeable() )
   {
     currentimage = crosshair_charging;
   }
   else if (player->isCurrentGunUnlimitedAmmo() )
   {
     currentimage = crosshair_energy;
   }
   else
   {
     currentimage = crosshair_ammo;
   }
   x = centerx - ((float) currentimage->getWidth() ) * .5f;
   y = centery - ((float) currentimage->getHeight() ) * .5f;
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   currentimage->render(x, y);
   glDisable(GL_BLEND);
   GLERRORCHECK
}


void Cockpit::renderShipProjection(Targetable * newship)
{
  Coordinate playerlocation = player->getLocation();
  Coordinate distancevector;
  float x, y; // x and y between -1 and 1
  float radius; //      radius between -1 and 1

  if(newship == player)
    return;

  if(newship == player->getTarget() )
  {
    glEnd();
    glPointSize(4.0f * sizemultiplier);
    glBegin(GL_POINTS);
  }
  distancevector = newship->getLocation() - playerlocation;
  distancevector.normalize();

  radius = distancevector.dot(player->getHeading() );
  x = distancevector.dot(player->getleft() ) * -1.0f;
  y = distancevector.dot(player->getUp() );

  float drawnx, drawny;
  if(radius > 0.0f)
  {
     drawnx = x * innerprojectionradius + projectionx;
     drawny = y * innerprojectionradius + projectiony;

  }
  else
  {
     float xbordercircle, ybordercircle; // the point on the circle separating front from back
     float radiusoffrontpoint = fastsqrt(x * x + y * y);
     xbordercircle = x * outerprojectionradius / radiusoffrontpoint;
     ybordercircle = y * outerprojectionradius / radiusoffrontpoint;

     float differenceradius = outerprojectionradius - innerprojectionradius;

     drawnx = xbordercircle - x * differenceradius + projectionx;
     drawny = ybordercircle - y * differenceradius + projectiony;
  }
  Color color = newship->getRadarColor(player);
  glColor3f(color.r, color.g, color.b);
  glVertex2f(drawnx, drawny);
  if(newship == player->getTarget() )
  {
    glEnd();
    glPointSize(2.0f * sizemultiplier);
    glBegin(GL_POINTS);
  }
}

void Cockpit::drawShipStatus(Targetable * target, float centerx, float centery)
{
  Image * image =  target->getImage();
  float imagex, imagey, imageheight, imagewidth;
  winWidth = glutGet(GLUT_WINDOW_WIDTH);
  winHeight = glutGet(GLUT_WINDOW_HEIGHT);
  shipareas armor = target->getArmor();
  shipareas shields = target->getShields();


  imageheight = image->getHeight();
  imagewidth = image->getWidth();

  imagex = centerx - imagewidth/2.0f;
  imagey = centery - imageheight/2.0f;

  if(! simplecockpit)
  {
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);                       // Set The Blending Function For Translucency
    glEnable(GL_BLEND);                                     // Enable Blending
  }
  
  if(target->instanceof(INST_SHIP) )
  {
    ship * starget = (ship *) target;
    float hullpercent = starget->getCoreDamagePercent();
	Color damagecolor = getDamageColor(hullpercent);
	image->setColor(damagecolor);
  }
  else
  {
    image->setColor(DefaultColor::white );
  }

  
  image->render(imagex, imagey);
  if(! simplecockpit)
  {
    glDisable(GL_BLEND);
  }

  // don't draw the rest if this is a capship
  
  if(target->instanceof(INST_CAPITALSHIP | INST_MISSILE | INST_EJECTEDPOD ))
  {
    return;
  
  }
  
  // ll = lower left, ul = upperleft, etc.
  float imagellx, imagelly, imagelrx, imagelry;
  float imageulx, imageuly, imageurx, imageury;
  imagellx = imagex;
  imagelly = imagey;
  imagelrx = centerx + imagewidth/2.0f;
  imagelry = centery - imageheight/2.0f;
  imageurx = centerx + imagewidth/2.0f;
  imageury = centery + imageheight/2.0f;
  imageulx = centerx - imagewidth/2.0f;
  imageuly = centery + imageheight/2.0f;


  Color armorcolor2(.27f, .5f, .77);
  Color armorcolor1(.16f, .33f, .29);
  Color shieldcolor1 = armorcolor2;
  Color shieldcolor2(1.0f, 1.0f, 1.0f);
  float barwidth = 12.0f;

// armor
  // back
  barVerticalGraph(armor.getCurrentValue(BACK),
                   armor.getMaximum(BACK),
                   5,
                   imagellx, imagelly, imagelrx , imagelry - barwidth,
                   armorcolor1, armorcolor2, true, true);
  // front
  barVerticalGraph(armor.getCurrentValue(FRONT),
                   armor.getMaximum(FRONT),
                   5,
                   imageulx, imageuly+ barwidth, imageurx , imageury,
                   armorcolor1, armorcolor2, false, true);
  // right
  barHorizontalGraph(armor.getCurrentValue(RIGHT),
                   armor.getMaximum(RIGHT),
                   5,
                   imageurx, imageury, imagelrx + barwidth , imagelry,
                   armorcolor1, armorcolor2, false, true);
  // left
  barHorizontalGraph(armor.getCurrentValue(LEFT),
                   armor.getMaximum(LEFT),
                   5,
                   imageulx - barwidth, imageuly, imagellx , imagelly,
                   armorcolor1, armorcolor2, true, true);
// shields
  barVerticalGraph(shields.getCurrentValue(BACK),
                   shields.getMaximum(BACK),
                   5,
                   imagellx, imagelly - barwidth, imagelrx , imagelry - barwidth - barwidth,
                   shieldcolor1 ,shieldcolor2,  true);
  barVerticalGraph(shields.getCurrentValue(FRONT),
                   shields.getMaximum(FRONT),
                   5,
                   imageulx, imageuly+ barwidth , imageurx , imageury + barwidth + barwidth,
                   shieldcolor1, shieldcolor2, true);
  barHorizontalGraph(shields.getCurrentValue(RIGHT),
                   shields.getMaximum(RIGHT),
                   5,
                   imageurx + barwidth+ barwidth, imageury, imagelrx + barwidth , imagelry,
                   shieldcolor1, shieldcolor2,  true);
  barHorizontalGraph(shields.getCurrentValue(LEFT),
                   shields.getMaximum(LEFT),
                   5,
                   imageulx - barwidth- barwidth, imageuly, imagellx - barwidth, imagelly,
                  shieldcolor1, shieldcolor2, true);
  GLERRORCHECK
}


void Cockpit::drawEnergy(int max, int current)
{
  int x0, x1, y0, y1;

  x0 = projectionx - 30;
  x1 = projectionx + 30;
  y0 = winHeight * .29f;
  y1 = y0 + 10;


//  glColor3f(.8f, .8f, 1.0f);
  DefaultColor::textcolor.render();
  float energyx;

  energyx = projectionx - font->getWidth("Energy") * .5f;

  font->print(energyx, y1 + font->getHeight("Energy"),  "Energy");


  barHorizontalGraph(current, max, 10, x0, y0, x1, y1, Color(0.0f, 0.0f, .8f), Color(1.0f, 1.0f, 1.0f), false);
  GLERRORCHECK
}


void Cockpit::doAutoPilot()
{
  if(Mission::getCurrentMission()->canAutoPilot() )
  {
     if(autopilot == NULL)
     {
       autopilot = new TransparentImage("autopilot.png");

     }
     float centerx, centery;
     float x, y;

     centery = winHeight * .8f;
     centerx = winWidth  * .5f;
     x = centerx - ((float) autopilot->getWidth() ) * .5f;
     y = centery - ((float) autopilot->getHeight() ) * .5f;
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     autopilot->render(x, y);
     glDisable(GL_BLEND);
  }
  GLERRORCHECK
}

void Cockpit::setShowComm(bool showcomm)
{
  show_comm = showcomm;
}

void Cockpit::renderComm()
{
   /*
      Friendly Comm Channel
      1. Break and attack
      2. Attack my target
      3. Help me out here
      4. Defend my target
      5. Request landing
      Enenmy Comm Channel
      6. Insult
   */


  /*
     Comms
     Target: targetname
	 1) Taunt or Status or Request Landing
	 Alpha Wing:
	 2) Form or Break
	 3) Attack my target
	 4) Status
	 Broadcast:
	 5) Help me out here.
 */

	 float spacing = font->getHeight("H");
	 float widthpad = font->getWidth("WW");
	 float targetwidth = font->getWidth("Target:  ");
//     glColor3f(0.0f, 1.0f, 0.0f);
	 
     float ylocation = winHeight/2.0;
     float xlocation = winWidth/16.0f;
	 Mission * mission = Mission::getCurrentMission();


     Targetable * target = player->getTarget();
	 if(target != NULL)
	 {


       if(target->instanceof(INST_SHIP) )
	   {
		 ship * targetship = (ship *) target;
         string targetname;

         targetname = targetship->getPilot();
		 if(targetname.length() == 0)
		 {
            targetname = targetship->getName();
		 }

	     DefaultColor::green.render(); 
         font->print(xlocation, ylocation,  "Target: ");
		 IffCode iff = targetship->getIffCode() ;
	     if(iff == ENEMY)
		 {
	       DefaultColor::red.render(); 
	       font->print(xlocation + targetwidth, ylocation, 
			   (char *) targetname.c_str() );
	       DefaultColor::green.render(); 
           font->print(xlocation, ylocation - spacing,  "1)");
	       DefaultColor::red.render(); 
	       font->print(xlocation + widthpad, ylocation - spacing,  "Taunt");
		 }
		 else 
		 {

  		   DefaultColor::blue.render(); 
	       font->print(xlocation + targetwidth, ylocation, 
			   (char *) targetname.c_str() );
           DefaultColor::green.render(); 
           font->print(xlocation, ylocation - spacing,  "1)");
           string c1text = "Status";

		   if(targetship ==  mission->getLandingShip() )
		   {
				c1text = "Request landing";
		   }// capship
           DefaultColor::blue.render(); 
           font->print(xlocation + widthpad, ylocation - spacing,  (char *) c1text.c_str() );
		 }// enemy
	   }// isship
	 }// target not null


	 bool areenemies = mission->enemiesAtCurrentNav();
	 bool hasfriendly = mission->hasFriendsAtCurrentNav();

	 if(hasfriendly)
	 {
       DefaultColor::green.render(); 
       font->print( xlocation, ylocation - 2.0f * spacing, "Alpha Wing:");
	   if(areenemies)
	   {
         DefaultColor::green.render(); 
         font->print(xlocation, ylocation - 3.0f * spacing,  "2)");
         DefaultColor::blue.render(); 
	     font->print(xlocation + widthpad, ylocation - 3.0f * spacing,  "Break and attack");
	     DefaultColor::green.render(); 
         font->print(xlocation, ylocation - 4.0f * spacing,  "3)");
	     DefaultColor::blue.render(); 
	     font->print(xlocation + widthpad, ylocation - 4.0f * spacing,  "Attack my target");
	   }
	   DefaultColor::green.render(); 
       font->print(xlocation, ylocation - 5.0f * spacing, "4)");
	   DefaultColor::blue.render(); 
	   font->print(xlocation + widthpad, ylocation - 5.0f * spacing, "Status");
	 }
	 if(areenemies && hasfriendly)
	 {
	   DefaultColor::green.render(); 
       font->print(xlocation, ylocation - 6.0f * spacing, "Broadcast:");
       font->print(xlocation, ylocation - 7.0f * spacing, "5)");
	   DefaultColor::blue.render(); 
       font->print(xlocation + widthpad, ylocation - 7.0f * spacing, "Help");
	 }
	 GLERRORCHECK
}

void Cockpit::drawFuel(int max, int current)
{
  int x0, x1, y0, y1;

  x0 = projectionx - 30;
  x1 = projectionx + 30;
  y0 = winHeight /20.f;
  y1 = y0 + 10;

  float   fuelx = projectionx - font->getWidth("Fuel") * .5f;

//  glColor3f(.8f, .8f, 1.0f);
  DefaultColor::textcolor.render();
  font->print(fuelx, y1 + font->getHeight("Fuel"),  "Fuel");

  barHorizontalGraph(current, max, 10, x0, y0, x1, y1, Color(0.0f, 0.0f, 0.8f),Color(1.0f, 1.0f, 1.0f),false);
  GLERRORCHECK

}

void Cockpit::toggleFPSEnabled()
{
  fpsenables = !fpsenables;
}

void Cockpit::renderRearView(float winWidth, float winHeight)
{
  FixedCamera * fixedcam;
  static Targetable * lasttarget = NULL;

   Targetable * target= player->getTarget();
  if(target == NULL)
  {
	  lasttarget = NULL;
	  return;
  }
  float sizex = winHeight * .25f;
  float sizey = sizex*  .75f;
  float startx, starty;

  startx = .9f * winWidth - sizex ;
  starty = .9f * winHeight - sizey;

  fixedcam = (FixedCamera *) FixedCamera::getInstance();
 
//  if(target != lasttarget)
  {
	lasttarget = target;
    Coordinate offset = Coordinate(0.0f, 0.0, -2.0f * target->getradius() );
	fixedcam->setLocation( target->getLocation() + offset);   
    fixedcam->setUp(Coordinate(0.0, 1.0f, 0.0f));
	fixedcam->setLookAt(target->getLocation() );
  }
 
  glPushMatrix();

  GLERRORCHECK
    glViewport(startx, starty, sizex, sizey);
    glLoadIdentity();
	fixedcam->setFocusObject(target);	

  GLERRORCHECK
    Camera::setCamera(fixedcam);	
	fixedcam->render();
    

	target->render();
	Camera::setCamera(CockpitCamera::getInstance() );
	Camera::setFocusObject(player );
  GLERRORCHECK
  glPopMatrix();
}


void Cockpit::doLock()
{
  SoundEffect * locksound = SoundFactory::getSoundFactory()->getMissileAlertSound();

  if(player->isMissileAttackingMe() )
  {
     if(lock == NULL)
     {
       lock = new TransparentImage("lock.png");
     }
     float centerx, centery;
     float x, y;

     centery = winHeight * .55f;
     centerx = winWidth  * .12f;
     x = centerx - ((float) lock->getWidth() ) * .5f;
     y = centery - ((float) lock->getHeight() ) * .5f;
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     lock->render(x, y);
     centerx = winWidth  * (1.0f - .12f);
     x = centerx - ((float) lock->getWidth() ) * .5f;
     y = centery - ((float) lock->getHeight() ) * .5f;
     lock->render(x, y);
     glDisable(GL_BLEND);
	 if(! locksound->isPlaying() )
	 {
		locksound->setLoop(true);
		locksound->play();
	 }
  }
  else if(locksound->isPlaying() )
  {
	locksound->stop();
  }
  GLERRORCHECK
}

bool Cockpit::isTargetVisible(Targetable * targetable)
{
  Camera * camera = Camera::getCamera();

  viewcone.update(camera->getLocation(), camera->getHeading() );

  return viewcone.isSphereVisible( 
		  targetable->getLocation(),
		  targetable->getradius(),
		  targetable->getRadiusSquared() );


}

void Cockpit::renderDamageReport()
{
   /*
      Friendly Comm Channel
      1. Break and attack
      2. Attack my target
      3. Help me out here
      4. Defend my target
      5. Request landing
      Enenmy Comm Channel
      6. Insult
   */
	 float spacing = font->getHeight("H");
//     glColor3f(0.0f, 1.0f, 0.0f);

	 DefaultColor::textcolor.render();
     float ylocation = winHeight/2.0;
     float xlocation = winWidth*4.0f/5.0f;;
     font->print(xlocation, ylocation,  "Damage report");


    float hullpercent = player->getCoreDamagePercent();
	Color damagecolor = getDamageColor(hullpercent);
     damagecolor.render();
	 int value;
     font->printf(xlocation, ylocation - spacing,  
		 "Core: %d", (int) (player->getCoreDamagePercent() * 100) );
	 GLERRORCHECK
}

void Cockpit::setShowDamage(bool damagereport)
{
  show_damage = damagereport;
}

Color Cockpit::getDamageColor(float hullpercent)
{
	if(hullpercent > .5f)
	{
		if(hullpercent > .75f)
		{
			return DefaultColor::fcred ;
		}
		else
		{
          return DefaultColor::fcyellow ;
		}
	}
	else
	{
      return DefaultColor::fcgreen ;
	}    

}

void Cockpit::doEject()
{

  SoundEffect * ejectsound = SoundFactory::getSoundFactory()->getEjectAlertSound();
  if(player->shouldEject() )
  {

     if(eject == NULL)
     {
       eject = new TransparentImage("eject.png");

     }
     float centerx, centery;
     float x, y;

     centery = winHeight * .8f + 40;
     centerx = winWidth  * .5f;
     x = centerx - ((float) eject->getWidth() ) * .5f;
     y = centery - ((float) eject->getHeight() ) * .5f;
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     eject->render(x, y);
     glDisable(GL_BLEND);

     GLERRORCHECK
	 if(! ejectsound->isPlaying() )
	 {
		ejectsound->setLoop(true);
		ejectsound->play();
	 }

  }
  else if(ejectsound->isPlaying() )
  {
	ejectsound->stop();

  }

}
