// Comm.cpp: implementation of the Comm class.
//
//////////////////////////////////////////////////////////////////////

#include "Comm.h"
#include <string>
#include "fstream.h"
#include "Time.h"
#include "MovieFactory.h"
#include "Movie.h"
#include "Cockpit.h"

using std::string;
using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


std::queue<CommState * > Comm::commqueue;
TimeValue Comm::pausetime = 0;

Comm::Comm(char soundname[], char videoname[], 
		   std::string newname, bool newalignment)
{
  string moviename;

  deadsound   = getSound((char *) (string(soundname) + "_dead.wav").c_str());
  insultsound = getSound((char *) (string(soundname) + "_insult.wav").c_str());
  if(strlen(videoname) == 0)
  {
	mymovie = NULL;
	deadmovie = NULL;
  }
  else
  {
	moviename = string(videoname) + ".avi";
	mymovie = getMovie((char *) moviename.c_str() );
    moviename = string(videoname) + "_dead.avi";
	deadmovie = getMovie((char *) moviename.c_str() );
  }
  name = newname;
  friendly = newalignment;
}

Comm::~Comm()
{

}

void Comm::dead()
{
  playNow(deadsound, deadmovie, name, friendly);
}

/*void Comm::gotKill()
{

}
*/

void Comm::insult()
{
	playNow(insultsound, mymovie, name, friendly);
}

SoundEffect * Comm::getSound(char name [ ])
{
  /* first check to see if the file exists */

  if(fopen(name, "r" ) == NULL )
    return NULL;
  return new SoundEffect(name);
}

void Comm::update()
{
    if(commqueue.empty() )
	{
  	  TimeValue now = Time::getCurrentTime();
	  if(now > pausetime + 250)
	  {
		pausetime = 0;
	  }
	  return;
	}
    CommState * newstate = commqueue.front();

	if(newstate->sound->isPlaying() )
	{
		return;
	}
	else
	{
		if(pausetime == 0) // we are not waiting yet
		{
		   if(newstate->movie != NULL)
		   {
		      newstate->movie->stop();
		   }
  		    delete newstate;
		    commqueue.pop();
			pausetime = Time::getCurrentTime();
		}
		if(! commqueue.empty() )	
		{	
			TimeValue now = Time::getCurrentTime();
			if(now > pausetime + 250)
			{
				newstate= commqueue.front();
				// start up the next one
				newstate->sound->play();
				if(newstate->movie != NULL)
				{
					newstate->movie->start();
				}
				pausetime = 0;
			}
		}
	}
}

void Comm::playNow(SoundEffect * effect, Movie * newmovie, string newlabel, bool friendly)
{
  if(effect != NULL)
  {

    CommState * newstate = new CommState();
	newstate->sound = effect;
	newstate->movie = newmovie;
	newstate->label = newlabel;
	newstate->friendly = friendly;
	if(commqueue.empty() )
	{
		newstate->sound->play();
		if(newstate->movie != NULL)
		{
			newstate->movie->start();
		}
	}
	commqueue.push(newstate);
  }
}

Movie * Comm::getCurrentMovie()
{

    if(commqueue.empty() )
	{
	  return NULL;
	}
	if(pausetime != 0) // we're paused between movies
	{
		return NULL;
	}

	return  commqueue.front()->movie;
}

std::string Comm::getCurrentLabel()
{
    if(commqueue.empty() )
	{
	  return NULL;
	}
	return  commqueue.front()->label;
}

void Comm::clear()
{
  pausetime = 0;
  while(! commqueue.empty() )
  {
	commqueue.pop();
  }
}

Comm::Comm(Comm * that, std::string newname, bool newalignment)
{
  mymovie = that->mymovie;
  deadmovie = that->deadmovie;
  deadsound = that->deadsound;
  insultsound = that->insultsound;
  name = newname;
  friendly = newalignment;
}

bool Comm::isCurrentFriendly()
{
    if(commqueue.empty() )
	{
	  return false;
	}
	return  commqueue.front()->friendly;
}

Movie * Comm::getMovie(char moviename [ ])
{
  MovieFactory * moviefactory;
  /* first check to see if the file exists */

    if(fopen(moviename, "r" ) == NULL )
       return NULL;

	Movie * newmovie;
	moviefactory = MovieFactory::getInstance();
    newmovie = moviefactory->getMovie(moviename );
    return newmovie;
}
