// CommFactory.cpp: implementation of the CommFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "CommFactory.h"
#include "Comm.h"
#include "PlayerComm.h"
#include "WingmanComm.h"
#include "LandingComm.h"
#include "StatusComm.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CommFactory CommFactory::instance;

CommFactory::CommFactory()
: commtable(10)
{
  
}

CommFactory::~CommFactory()
{

}

Comm * CommFactory::getComm(char sndname [ ], char videoname[], char label[], bool friendly)
{
	std::string newlabel;

  if(strlen(label) == 0)
  {
	if(friendly)
	{
		newlabel = "Friendly vessel";
	}
	else
	{
		newlabel = "Enemy vessel";
	}
  }
  else
  {
	newlabel = label;
  }

  if(commtable.hasKey(sndname) )
  {
    Comm * previous = (Comm *) commtable.getItem(sndname);
	Comm * newcomm = new Comm(previous, newlabel, friendly);
	comms.push_back(newcomm);
    return newcomm;
  }
  else
  {
     Comm * newitem;
	 if(friendly)
	 {
        newitem = new StatusComm(sndname, videoname, (char *) newlabel.c_str());
	 }
	 else
	 {
       newitem = new Comm(sndname, videoname, newlabel, friendly);
	 }
     commtable.addItem(sndname, newitem);
	 comms.push_back(newitem);
     return newitem;
  }
}

PlayerComm * CommFactory::getPlayerComm(char sndname [ ])
{
  return new PlayerComm(sndname);
}

WingmanComm * CommFactory::getWingmanComm(char sndname [ ], char videoname[], char label[])
{
  /* if we reuse wingmen, this should be hashed as well later */
  return new WingmanComm(sndname, videoname, label);
}

CommFactory * CommFactory::getInstance()
{
  return &instance;
}

LandingComm * CommFactory::getLandingComm(char sndname [ ], char videoname [ ], char label[])
{
  return new LandingComm(sndname, videoname, label);

}

void CommFactory::clear()
{
  int i;
  for(i=0; i < comms.size(); i++)
  {
	delete comms[i];
  }
  comms.clear();
  commtable.clear();
  Comm::clear();
}
