// Component.cpp: implementation of the Component class.
//
//////////////////////////////////////////////////////////////////////

#include "Component.h"
#include "mesh.h"
#include "SoundEffect3d.h"
#include "SoundFactory.h"

using std::string;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Component::Component(string new_name, mesh * new_undamaged, mesh * new_damaged, int new_hitpoints)
  : name(new_name),
    undamaged(new_undamaged),
    damaged(new_damaged),
    current_hitpoints(new_hitpoints),
    max_hitpoints(new_hitpoints)    
{
	justdestroyed = false;
    float tmatrix[] = {1.0f, 0.0, 0.0f, 0.0f,
	        0.0f, 1.0f, 0.0f, 0.0f,
			0.0f, 0.0f, 1.0f, 0.0f,
			0.0f, 0.0f, 0.0f, 1.0f};
    memcpy(matrix, tmatrix, sizeof(tmatrix) );


}

Component::~Component()
{

}

void Component::render()
{

   pushLocalTransform();

   if(current_hitpoints <= 0)
   {  
      if(damaged != NULL)
	  {
        damaged->render();
	  }
   }
   else
   {
      undamaged->render();
   }
}

bool Component::isDestroyed()
{
  return current_hitpoints <= 0;
}

string Component::getName()
{
   return name;
}

Component::Component(Component & copy)
  : name(copy.name),
    undamaged(copy.undamaged),
    damaged(copy.damaged),
    current_hitpoints(copy.max_hitpoints),
    max_hitpoints(copy.max_hitpoints)
{
  justdestroyed = false;
//  debuglocation = copy.debuglocation;
  setLocation(copy.getLocation() );
  float tmatrix[] = {1.0f, 0.0, 0.0f, 0.0f,
	        0.0f, 1.0f, 0.0f, 0.0f,
			0.0f, 0.0f, 1.0f, 0.0f,
			0.0f, 0.0f, 0.0f, 1.0f};
  memcpy(matrix, tmatrix, sizeof(tmatrix) );
}

CollisionModel3D * Component::getCollisionModel()
{
   if(current_hitpoints <= 0)
   {  
      if(damaged != NULL)
	  {
        return damaged->getCollisionModel();
	  }
	  else
	  {
		return NULL;
	  }
   }
   else
   {
      return undamaged->getCollisionModel();
   }
}

Coordinate Component::getMeshCenter()
{
  return undamaged->getcenter();
}

Coordinate Component::getExtents()
{
  return undamaged->getExtents();
}

void Component::damage(int amount, Coordinate fromlocation)
{
  if(isDestroyed() )
  {
	return;
  }

  current_hitpoints -= amount;
  if((! justdestroyed) && isDestroyed()  )
  {
     justdestroyed = true;
  }
  SoundEffect3d * effect = SoundFactory::getSoundFactory()->getArmorSound();
  effect->setPosition(worldlocation );
  effect->play();

}

float Component::getDamagedPercent()
{
  return (float) current_hitpoints / (float)max_hitpoints;
}

Coordinate Component::getMeshMaxs()
{
  return undamaged->getMaxs();
}

Coordinate Component::getMeshMins()
{
  return undamaged->getMins();
}

bool Component::justDestroyed()
{
  return justdestroyed;
}

void Component::advance()
{
  if(justdestroyed)
	  justdestroyed = false;
}

bool Component::instanceof(char classname [ ])
{
  if(strcmp(classname, "Component") == 0)
	  return true;
  return Object3d::instanceof(classname);
}

bool Component::hasDamagedMesh()
{
  return damaged != NULL;
}

void Component::setWorldLocation(Coordinate newlocation)
{
  worldlocation = newlocation;
}

bool Component::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_COMPONENT);
}


void Component::renderDestroyed()
{
  if(damaged != NULL)
  {
        damaged->render();
  }
}


void Component::pushLocalTransform()
{
//  Coordinate location = debuglocation ;
	  Coordinate location = getLocation() ;

  glTranslatef(location.x, location.y, location.z);

}

void Component::getMatrix(float newmatrix [ 16 ])
{
  int i;
   
  for(i=0; i < 16; i++)
  {
    newmatrix[i] = matrix[i];
  }

}


void Component::setMatrix(float newmatrix [ 16 ])
{
  int i;
   
  for(i=0; i < 16; i++)
  {
    matrix[i] = newmatrix[i];
  }
}

Coordinate Component::getWorldLocation()
{
  return worldlocation;
}
