// CubeMap.cpp: implementation of the CubeMap class.
//
//////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include "CubeMap.h"
#include "ImageFile.h"
#include "TextureImpl.h"
#include <stdio.h>
#include "Settings.h"
#include "CubeMapTexture.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CubeMap::CubeMap(char bmpfilename[])
{
  int i;

//  BMPFile bmp(bmpfilename);
  ImageFile * bmp;
  TextureImpl  * impl;
  char buffer[100];

  glGenTextures(1, &mydisplaylist);
  glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, mydisplaylist);

  for(i=0; i < 6; i++)
  {
	sprintf(buffer, "screenshot%d.bmp", i);

    bmp = new ImageFile(buffer, false);

	impl = new CubeMapTexture(bmp->getData(), bmp->getWidth(), bmp->getHeight(), 
		GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT + i);
	delete bmp;

  }

    if(Settings::getSettings()->getMipMap() )
    {
       glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
       glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
	}
	else
	{
  	  glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	  glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	}
  
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);


}

CubeMap::~CubeMap()
{
    glDeleteTextures(1, &mydisplaylist);
}

void CubeMap::enable()
{
  glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
  glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
  glEnable(GL_TEXTURE_CUBE_MAP_EXT);
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);
  glEnable(GL_TEXTURE_GEN_R);
}

void CubeMap::disable()
{
  glDisable(GL_TEXTURE_CUBE_MAP_EXT);
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
  glDisable(GL_TEXTURE_GEN_R);

}

CubeMap::CubeMap()
{
    glGenTextures(1, &mydisplaylist);
    glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, mydisplaylist);
}

void CubeMap::copyFromScreen(int facenumber, int width, int height)
{
    glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, mydisplaylist);
    glCopyTexImage2D( GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT + facenumber, 0, GL_RGB , 0, 0, width, height, 0);
 
}

void CubeMap::render()
{
   glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, mydisplaylist);
}

CubeMap * CubeMap::env = NULL;

void CubeMap::setEnvironmentMap(CubeMap * newenv)
{
  env = newenv;
}

CubeMap * CubeMap::getEnvironmentMap()
{
  return env;
}
