// Decoy.cpp: implementation of the Decoy class.
//
//////////////////////////////////////////////////////////////////////

#include "Decoy.h"
#include "Time.h"
#include "GL/glew.h"
#include "glut.h"
#include "glerr.h"
#include "TextureFactory.h"
#include "Camera.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

static GLfloat constant[3] = { 1.0, 0.0, 0.0 };
static GLfloat linear[3] = { 0.0, 1/5.0, 0.0 };
static GLfloat quadratic[3] = { 0.25, 0.0, 1/60.0 };

Texture * Decoy::texture[NUMDECOYTEX];
bool Decoy::initialized = false;

int Decoy::indices[NUMDECOYFRAMES] = 
{
       6,
       5, 
       4,
       3, 
       2,
       1,
       0,
       1,
       2,
       1,
       0,
       1,
       2,
       1,
       0,
       1,
       2,
       1,
       0, 
       0,
       1,
       1,
       2,
       2,
       3,
       3,
       4,
       4,
       5,
       5,
       6,
       6
};


static bool initialized = false;

Decoy::Decoy()
{
	if(! initialized)
	{
	  texture[0] = TextureFactory::getTransparentTexture("decoy0.png");
	  texture[1] = TextureFactory::getTransparentTexture("decoy1.png");
	  texture[2] = TextureFactory::getTransparentTexture("decoy2.png");
	  texture[3] = TextureFactory::getTransparentTexture("decoy3.png");
	  texture[4] = TextureFactory::getTransparentTexture("decoy4.png");
	  texture[5] = TextureFactory::getTransparentTexture("decoy5.png");
	  texture[6] = TextureFactory::getTransparentTexture("decoy6.png");
	  initialized = true;
	}

	duration = NUMDECOYFRAMES * 1000.0f/ 12.0f ;
	starttime = Time::getCurrentTime();
	endtime = starttime + duration;
}

Decoy::~Decoy()
{

}

void Decoy::render()
{
  Coordinate location = getLocation();
  float width = 2.5f;
  int index;
  int i;
  int numoffsets = 10;
  Coordinate offsets[10];

  TimeValue now = Time::getCurrentTime();
  float percentage = ((float)(now - starttime)) / duration;
  index = percentage * NUMDECOYFRAMES;
  if(index > NUMDECOYFRAMES - 1)
  {
	index = NUMDECOYFRAMES - 1;
  }

  Coordinate up = Camera::getCamera()->getUp();
  Coordinate side = Camera::getCamera()->getLeft();

  offsets[1] = up * 5.0f + side * 13.0f;
  offsets[2] = up * -3.0f + side * 5.0f;
  offsets[3] = up * 7.0f - side * 4.0f;
  offsets[4] = up * -9.f - side * 4.4f;
  offsets[5] = up * 12.0f - side * 14.0f;
  offsets[6] = up * 7.0f - side * 4.0f;
  offsets[7] = up * -9.f - side * 12.9f;
  offsets[8] = up * -9.f + side * 16.4f;
  offsets[9] = up * -14.f - side * 10.4f;


  Coordinate p0 = location - up * width - side * width;
  Coordinate p1 = location + up * width - side * width;
  Coordinate p2 = location + up * width + side * width;
  Coordinate p3 = location - up * width + side * width;

  glColor3f(1.0f, 1.0f, 1.0f);
  glEnable(GL_TEXTURE_2D);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);
  glDisable(GL_LIGHTING);


  texture[indices[index] ]->render();
  for(i=0 ; i < numoffsets; i++)
  {
	  glPushMatrix();

	  Coordinate myoffset = offsets[i];
	  glTranslatef(myoffset.x, myoffset.y, myoffset.z);

  glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f);
    glVertex3f(p0.x, p0.y, p0.z);
    glTexCoord2f(1.0f, 0.0f);
	glVertex3f(p1.x, p1.y, p1.z);
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f(p2.x, p2.y, p2.z);
    glTexCoord2f(0.0f, 1.0f);
	glVertex3f(p3.x, p3.y, p3.z);
  glEnd();

      glPopMatrix();
  }

  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
  glEnable(GL_LIGHTING);
}

void Decoy::advance()
{
  TimeValue now = Time::getCurrentTime();
  if(now > endtime)
  {

	setDestroyed();
  }
}

bool Decoy::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_EXPLOSION);
}

void Decoy::clear()
{
  initialized = false;
}
