// DiffuseLight.cpp: implementation of the DiffuseLight class.
//
//////////////////////////////////////////////////////////////////////

#include "DiffuseLight.h"
#include <glut.h>
#include "glerr.h"
#include <string.h>
#include "Camera.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DiffuseLight::DiffuseLight(int newnumber, Color newcolor, Coordinate newlocation)
  : Light(),
	number(newnumber),
	diffusecolor(newcolor),
    location(newlocation)
{
  attenuation = false;
  constant_attenuation = 1.0f;
  linear_attenuation = 0.0f;
  quadratic_attenuation = 0.0f;
  priority = 1;
}

DiffuseLight::~DiffuseLight()
{

}

void DiffuseLight::render()
{
  int lightnumber = number + GL_LIGHT0;
  GLfloat diffuse_color[] = { diffusecolor.r, diffusecolor.g, diffusecolor.b, 1.0f};
  GLfloat diffuse_location[] = {location.x, location.y, location.z, 1.0f };
//  GLfloat spec_color[] = {.5f, .5f, .5f, 1.0f };
  GLfloat amb_color[] = {ambientcolor.r, ambientcolor.g, ambientcolor.b, 1.0f };

  glLightfv(lightnumber, GL_AMBIENT, amb_color);
  GLERRORCHECK
  glLightfv(lightnumber, GL_DIFFUSE, diffuse_color);
  GLERRORCHECK
  glLightfv(lightnumber, GL_POSITION, diffuse_location);
  GLERRORCHECK

/*  glPushMatrix();
  
  glDisable(GL_LIGHTING);
  glEnable(GL_COLOR_MATERIAL);
  glDisable(GL_TEXTURE_2D);
  glColor3f(1.0f, 1.0f, 1.0f);
  glTranslatef(location.x, location.y, location.z);
 
    glutWireSphere(.5f, 39, 39);
  glPopMatrix();
  glEnable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);
*/

/*  glLightfv(lightnumber, GL_SPECULAR, spec_color);
    glLightf(lightnumber, GL_SPOT_EXPONENT , 1.0f);
*/
 // if(attenuation)
   if(true)
  {
    glLightf(lightnumber, GL_CONSTANT_ATTENUATION, constant_attenuation);
	GLERRORCHECK
    glLightf(lightnumber, GL_LINEAR_ATTENUATION, linear_attenuation);
	GLERRORCHECK
    glLightf(lightnumber, GL_QUADRATIC_ATTENUATION, quadratic_attenuation);
	GLERRORCHECK
  }
//  glEnable(lightnumber);
  GLERRORCHECK
}

DiffuseLight::DiffuseLight()
{
  attenuation = false;
  constant_attenuation = 1.0f;
  linear_attenuation = 0.0f;
  quadratic_attenuation = 0.0f;
  priority = 0;
}


void DiffuseLight::setAmbient(Color newcolor)
{
	ambientcolor = newcolor;
}

void DiffuseLight::setDiffuse(Color newcolor)
{
	diffusecolor = newcolor;
}

void DiffuseLight::setAttenuation(float constant, float linear, float quadratic)
{
  constant_attenuation = constant;
  linear_attenuation = linear;
  quadratic_attenuation = quadratic;
  attenuation = true;
}

void DiffuseLight::setLocation(Coordinate newlocation)
{
  location = newlocation;
}

void DiffuseLight::setLightNumber(int index)
{
  number = index;
}

void DiffuseLight::enable()
{
   int lightnumber = number + GL_LIGHT0;
   glEnable(lightnumber);
}

void DiffuseLight::disable()
{
   int lightnumber = number + GL_LIGHT0;
   glDisable(lightnumber);

}

Coordinate DiffuseLight::getLocation() const
{
  return location;
}

bool DiffuseLight::instanceof(char name [ ])
{
  if(strcmp(name, "DiffuseLight") == 0)
  {
	return true;
  }
  return Light::instanceof(name);
}

void DiffuseLight::setPriority(int newpriority)
{
  priority = newpriority;
}

int DiffuseLight::getPriority() const
{
  return priority;
}

Color DiffuseLight::getDiffuse()
{
  return diffusecolor;
}
