// DisplayMode.cpp: implementation of the DisplayMode class.
//
//////////////////////////////////////////////////////////////////////

#include "DisplayMode.h"
#ifdef WIN32
#include <windows.h>
#endif
#include <set>

#pragma warning( disable : 4786 ) 

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DisplayMode::DisplayMode()
{

}

DisplayMode::DisplayMode(int nwidth, int nheight, int ndepth)
{
  width = nwidth;
  height = nheight;
  depth = ndepth;
}

DisplayMode::~DisplayMode()
{

}



bool operator<(const DisplayMode & d, const DisplayMode & d2)
{
  if(d.width == d2.width)
  {
    return d.depth < d2.depth;
  }
  return d.width < d2.width;
}

bool operator==(const DisplayMode & d, const DisplayMode & d2)
{
  return d.width == d2.width && 
	     d.height == d2.height &&
		 d2.depth == d2.depth;
}


std::vector<DisplayMode> DisplayMode::getDisplayModes()
{
#ifdef WIN32
  int index;
  DEVMODE mode;
  BOOL result;
  std::set<DisplayMode> modes;
  std::set<DisplayMode>::iterator i;
  DisplayMode currentmode;
  std::vector<DisplayMode> modevector;

  index = 0;
  result = EnumDisplaySettings(NULL, index, &mode);
  while(result != FALSE)
  {
    index++;
	float ratio = (float)mode.dmPelsWidth / (float) mode.dmPelsHeight;
	float goodratio = 640.0f / 480.0f;
	if(mode.dmBitsPerPel >= 16 && mode.dmPelsWidth >= 640
		&& (ratio == goodratio) )
	{
        currentmode = DisplayMode(mode.dmPelsWidth, mode.dmPelsHeight, 
			                   mode.dmBitsPerPel);
		modes.insert(currentmode);

	}
    
    result = EnumDisplaySettings(NULL, index, &mode);

  }

  for (i=modes.begin();i!=modes.end();i++)
  {
	 currentmode = *i;
	 modevector.push_back(currentmode);
  }
#else
  std::vector<DisplayMode> modevector; 
  DisplayMode currentmode;

  currentmode = DisplayMode(640, 480, 32);
  modevector.push_back(currentmode);
  currentmode = DisplayMode(800, 600, 32);
  modevector.push_back(currentmode);
  currentmode = DisplayMode(1024, 768, 32);  
  modevector.push_back(currentmode);
#endif

  return modevector;

}


std::string DisplayMode::toString()
{
  char buffer[100];

  sprintf(buffer, "%dx%d:%d", width, height, depth);
  return buffer;

}

DisplayMode::DisplayMode(std::string text)
{
   sscanf((char *) text.c_str(), "%dx%d:%d", &width, &height, &depth);
}
