// EjectCutscene.cpp: implementation of the EjectCutscene class.
//
//////////////////////////////////////////////////////////////////////

#include "EjectCutscene.h"
#include "SoundEffect.h"
#include "SoundFactory.h"
#include "World.h"
#include "shipfactory.h"
#include "Camera.h"
#include "ChaseCamera.h"
#include "FixedCamera.h"
#include "EjectionPod.h"
#include "AutoPilotCamera.h"
#include "Constants.h"
#include "Mission.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EjectCutscene::EjectCutscene()
{
  duration = 35000;
  scene2starttime = 5000;
  scene2duration = 20000;
  started2 = false;
  started3 = false;
  sphere = gluNewQuadric();
  cylinder = gluNewQuadric();
}

EjectCutscene::~EjectCutscene()
{

}

bool EjectCutscene::isDone()
{
  TimeValue currenttime = Time::getCurrentTime();

  return currenttime - starttime > duration;
}

void EjectCutscene::start()
{
  starttime = Time::getCurrentTime();
}

void EjectCutscene::advance()
{
  FixedCamera * camera;

  if(started2) // move pod and camera up together
  {
	float t;

	TimeValue scene2start = starttime + scene2starttime;
	TimeValue now =  Time::getCurrentTime();
    t = (float)(now - scene2start) / (float)scene2duration;

	if(t > 1.0f) // autopilot the sar away
	{
	   if(! started3)
	   {
		 tractorbeamsound->stop();
  	     World::getWorld().remove(pod);
	     sar->setMaxVelocity();
         Camera::setCamera(AutoPilotCamera::getInstance() );
	     Camera::setFocusObject(sar);
		 started3 = true;
	   }
	}
	else
	{
	   Coordinate newpod = podstart.interpolate(podend, t);
       pod->setLocation(newpod);
	   cameralocation.y = newpod.y;
	   camera = (FixedCamera *) FixedCamera::getInstance();
	   camera->setLocation(cameralocation);
	}
  }
  else
  {
	TimeValue currenttime = Time::getCurrentTime();

	  if(currenttime > starttime + scene2starttime)
	  {
        tractorbeamsound = SoundFactory::getSoundFactory()->getTractorBeamSound();
        tractorbeamsound->setLoop(true);
        tractorbeamsound->play();

  	    World::getWorld().clear();
		string shipname;
		if(Mission::getCurrentMission()->canEjectToNextMission() )
		{
		   shipname = Constants::getGoodEject() + ".ship.xml";
		}
		else
		{
		   shipname = Constants::getBadEject() + ".ship.xml";
		}
		sar = shipfactory::getshipfactory().getship((char *) shipname.c_str() );
		sar->setCurrentVelocity(0.0f);
		sar->setAI(NULL);
        sar->setDesiredVelocity(0.0f);
		float radius = sar->getradius();
		pod = new EjectionPod("", 
			sar->getLocation() + Coordinate(0.0f, 0.0f, 0.0f),
			sar->getHeading(),
			sar->getUp(),
			sar->getleft()
			);
		float yoffset = sar->getExtents().y * 2.0f + pod->getradius();
		podstart = Coordinate(0.0f, -yoffset, 0.0f);
		podend = sar->getLocation() - Coordinate(0.0f, sar->getExtents().y * .5f, 0.0f) ;
		pod->setLocation(podstart);
		pod->setVelocity(0.0f);
        World::getWorld().add(pod);  

		camera = (FixedCamera *) FixedCamera::getInstance();
        cameralocation = sar->getLocation() + Coordinate(1.5f * radius, -yoffset, 3.0f * radius);
        camera->setLocation(cameralocation);
		camera->setLookAt( pod->getLocation() );
		camera->setUp(sar->getUp() );


		Camera::setCamera(camera );
		Camera::setFocusObject(sar);
		started2 = true;
	  }
  }
}

void EjectCutscene::render()
{
   if(started2  && ! started3)
   {
	float distance = pod->getLocation().distance( sar->getLocation() );
    if(distance > pod->getradius()  )
	{

		pod->render();
		glPushMatrix();
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		
		Coordinate podlocation = pod->getLocation();
		
		glTranslatef(podlocation.x, podlocation.y, podlocation.z);
		glColor4f(0.0f, 0.0f, 1.0f, 0.25f);
	//	glColor3f(1.0f, 1.0f, 1.0f);
		gluSphere(sphere, pod->getradius() * 2.0f, 30, 30);
	
	
	
		glRotatef(-90.0, 1.0, 0.0, 0.0);
		gluCylinder( cylinder,  
		pod->getradius() * .75f,   
		pod->getradius() * .75f, 
		distance, 
			5, 8); 
 
		glDisable(GL_BLEND);
		glEnable(GL_LIGHTING);
		glEnable(GL_TEXTURE_2D);
		glPopMatrix();
	}
   }
}

bool EjectCutscene::instanceof(char classname [ ])
{
  if(strcmp(classname, "EjectCutscene") == 0)
  {
	  return true;
  }
  return false;
}
