// Explosion.cpp: implementation of the Explosion class.
//
//////////////////////////////////////////////////////////////////////

#include "GL/glew.h"
#include "Explosion.h"
#include <glut.h>
#include <time.h>
#include "coord.h"
#include "Color.h"
#include <iostream.h>
#include "TextureFactory.h"
#include "glerr.h"
#include "Settings.h"
#include "LightManager.h"
#include "DiffuseLight.h"
#include <math.h>

#ifndef __max
#define __max(a, b) (a>b)?a:b
#define __min(a, b) (a<b)?a:b
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

static GLfloat constant[3] = { 1.0, 0.0, 0.0 };
static GLfloat linear[3] = { 0.0, 1/5.0, 0.0 };
static GLfloat quadratic[3] = { 0.25, 0.0, 1/60.0 };

Texture * Explosion::texture = NULL;

Explosion::Explosion(Coordinate newcenter)
  : center(newcenter)
{
  DisplayMode displaymode;
  const float defaultwidth = 640.0f;

  endtime = 6000;
  totalparticles = 800;
  numparticles = 0;  
  v0 = 5.5f;
  acceleration = -.5;


//  isdead = new bool[totalparticles];
//  v0s = new float[totalparticles];

  //headings = new Coordinate[totalparticles];

  starttime = Time::getCurrentTime();

  maxdistance = 45.0f;

  addParticles(50);

  displaymode = Settings::getSettings()->getDisplayMode();
  sizemultiplier = ((float)displaymode.width) / defaultwidth;

  LightManager::getInstance()->addLocalLight(&light);

  lastcolor = Color(1.0f, 1.0f, 1.0f);
}

Explosion::Explosion(Coordinate newcenter, float newmaxdistance)
  : center(newcenter)
{
  totalparticles = 800;
  numparticles = 0;  
  v0 = 2.0f;
  acceleration = -.4;
  endtime = 9000;
  
//  isdead = new bool[totalparticles];
//  v0s = new float[totalparticles];
//  headings = new Coordinate[totalparticles];

  starttime = Time::getCurrentTime();
  maxdistance = newmaxdistance;
  addParticles(50);
  LightManager::getInstance()->addLocalLight(&light);
  lastcolor = Color(1.0f, 1.0f, 1.0f);
}


Explosion::~Explosion()
{
//  delete [] isdead;


/*  delete [] v0s;
  delete [] headings;
*/
  LightManager::getInstance()->removeLocalLight(&light);
}

void Explosion::render()
{
  glDisable(GL_LIGHTING);
  int i;
  float timechange = (float)( Time::getCurrentTime() - starttime);
  float myv0;
  float acceleration = this->acceleration / 500.0f /500.0f;
  Color color;
  float alpharatio;
  float candidatesize;

  alpharatio = 1.0f - (float) (Time::getCurrentTime() - starttime) / (float) endtime;
  alpharatio = __max(0.0f, alpharatio);
  alpharatio = __min(1.0f, alpharatio);
 
  if(GLEW_EXT_point_parameters)
  {
    glPointParameterfvEXT(GL_DISTANCE_ATTENUATION_EXT, linear);
	GLERRORCHECK
  }
  GLERRORCHECK

  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);
  GLERRORCHECK
//  glBlendFunc( GL_SRC_ALPHA, GL_ZERO );

  glDisable(GL_TEXTURE_2D);
  glPushMatrix();

  glTranslatef(center.x, center.y, center.z);
  if(GLEW_EXT_point_parameters)
  {
    if(GLEW_NV_point_sprite)
	{
//    glPointParameterfARB( GL_POINT_FADE_THRESHOLD_SIZE_ARB, 80.0f );
	  candidatesize = 100.0f * alpharatio * sizemultiplier;
	  candidatesize = __max(1.0f, candidatesize);
      glPointSize(candidatesize);
	  GLERRORCHECK
      glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );
	  GLERRORCHECK
      glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
      glEnable(GL_TEXTURE_2D);
	  GLERRORCHECK
	  if(texture == NULL)
	  {
		  texture = TextureFactory::getTransparentTextureAveraged("particle2.bmp");
	  }
	  texture->render();
	  GLERRORCHECK
      glEnable( GL_POINT_SPRITE_ARB );	
	}  
	else
	{
      candidatesize = (float)alpharatio * 10.0f * sizemultiplier;
	  candidatesize = __max(1.0f, candidatesize);
      glPointSize(candidatesize);
	}
  }
  else
  {
    glPointSize(3.0f);
  }
  GLERRORCHECK


//  glEnable(GL_COLOR);
  GLERRORCHECK
  glBegin(GL_POINTS);
  

  for(i=0; i < numparticles; i++)
  {
     myv0 = v0s[i] /500.0f;

  //   if(! isdead[i] )
//     {
        float xf; // final position;
        Coordinate position;
 
        xf = .5f * acceleration * timechange * timechange + myv0 * timechange;
        position = headings[i] * xf /*+ p0s[i]*/;
        float distance = position.getlength();
        if(distance < maxdistance)
        {

        float ratio = distance / maxdistance;
        if(ratio < .15f)
        {
           float s = ratio/.15f;
           color.r =  s * (1.0f)  + (1.0f - s)* (1.0f);
           color.g =  s * (1.0f)  + (1.0f - s)* (1.0f);
           color.b =  s * (0.0f)  + (1.0f - s)* (1.0f);
        }
        else if(ratio < .25f)
        {
           float s = (ratio - .15f) /(.25f- .15f);
           color.r =  s * (1.0f)  + (1.0f - s)* (1.0f);
           color.g =  s * (0.1f)  + (1.0f - s)* (1.0f);
           color.b =  s * (0.1f)  + (1.0f - s)* (0.0f);
        }
        else 
        {
           float s = (ratio - .25f) /.75f;
           color.r =  s * (0.0f)  + (1.0f - s)* (1.0f);
           color.g =  s * (0.0f)  + (1.0f - s)* (0.0f);
           color.b =  s * (0.0f)  + (1.0f - s)* (0.0f);
        }           
		lastcolor = color;
        glColor4f(color.r, color.g, color.b, alpharatio);
		//glColor3f(color.r, color.g, color.b);
        glVertex3f(position.x, position.y, position.z);    
     }
  }
  glEnd();
  glPopMatrix();
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
  GLERRORCHECK
//  glDisable(GL_COLOR);
  GLERRORCHECK
  if(GLEW_EXT_point_parameters)
  {
    glPointParameterfvEXT(GL_DISTANCE_ATTENUATION_EXT, constant);
    if(GLEW_NV_point_sprite)
	{
      glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_FALSE );
      glDisable( GL_POINT_SPRITE_ARB );	
	}
  }
  GLERRORCHECK
  glEnable(GL_LIGHTING);
}

void Explosion::advance()
{
  if(numparticles < totalparticles)
    addParticles(25);
  if(Time::getCurrentTime() - starttime >= endtime)
    setDestroyed();

  light.setDiffuse(lastcolor);
  light.setLocation(center);
     
}

void Explosion::addParticles(int moreparticles)
{
  int i;
  float rnum;
  const int maxparticles = 860;

  const int numgroups = 1;
  Coordinate groups[numgroups];

  for(i=0; i < numgroups; i++)
  {
     groups[i].x = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1
     groups[i].y = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1
     groups[i].z = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1    
	 groups[i] = groups[i] * 2.0f;
  }

  for(i=numparticles; i < numparticles + moreparticles && i < maxparticles; i++)
  {

     rnum = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1
     Coordinate c;
	 Coordinate p;

     c.x = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1
     c.y = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1
     c.z = rand()* 2.0f / RAND_MAX - 1.0f; // between -1 and 1
     c.normalize();

     headings[i] = c;

     p0s[i] = groups[i % numgroups];
     v0s[i] = v0 + rnum;
//     isdead[i] = false;
  }
  numparticles += moreparticles;
  if(numparticles > maxparticles)
    numparticles = maxparticles;
}

bool Explosion::instanceof(char classname [ ])
{
  if(0 == strcmp(classname, "Explosion")) 
    return true;
  return Object3d::instanceof(classname);
}	


void Explosion::clear()
{
  texture = NULL;
}

bool Explosion::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_EXPLOSION);
}
