// ForceEffect.cpp: implementation of the ForceEffect class.
//
//////////////////////////////////////////////////////////////////////

#include "ForceEffect.h"
#include <iostream.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
ForceEffect::ForceEffect(IDirectInputEffect * newieffect)
{
  ieffect = newieffect;
  ZeroMemory(&eff, sizeof(eff) );
  eff.dwSize = sizeof(eff);
  eff.dwFlags = DIEFF_POLAR | DIEFF_OBJECTOFFSETS;
  eff.dwTriggerButton = DIEB_NOTRIGGER;
  eff.dwGain = DI_FFNOMINALMAX;         // no scaling
  dwFlags = DIEP_ALLPARAMS;
  dwAxes[0] =  DIJOFS_X;
  dwAxes[1] =  DIJOFS_Y;
  eff.cAxes = 2; 
  eff.rgdwAxes = &dwAxes[0];
}
#endif

ForceEffect::~ForceEffect()
{
#ifdef WIN32
  ieffect->Release();
#endif
}


void ForceEffect::setDelay(int start_delay)
{
#ifdef WIN32
  delay = start_delay;
#endif
}

void ForceEffect::setDuration(int duration)
{
#ifdef WIN32
  eff.dwDuration = duration;	
#endif
	
}
	
void ForceEffect::update()
{
#ifdef WIN32
  setParameters();
  ieffect->Download();
#endif
}

void ForceEffect::stop()
{
#ifdef WIN32
  ieffect->Stop();
#endif
}

void ForceEffect::play()
{
#ifdef WIN32
  setParameters();
  HRESULT result = ieffect->Start(1, 0);
  if(result != DI_OK)
  {
	 cout << "bad force effect play" << endl;
  }
#endif

}

#ifdef WIN32
void ForceEffect::setParameters()
{
  HRESULT result = ieffect->SetParameters(&eff, dwFlags);
  if(result != DI_OK)
  {
//	 cout << "bad force set parameters" << endl;
	 if(result == DIERR_NOTINITIALIZED)
	 {
       result = result;
	 }
	 else if(result == DIERR_NOTINITIALIZED)
	 {
       result = result;
	 }
	 else if(result == DIERR_INCOMPLETEEFFECT)
	 {
       result = result;
	 }
	 else if(result == DIERR_INPUTLOST)
	 {
       result = result;
	 }
	 else if(result == DIERR_INVALIDPARAM)
	 {
       result = result;
	 }
	 else if(result == DIERR_EFFECTPLAYING)
	 {
       result = result;
	 }
  }
}
#endif

bool ForceEffect::isPlaying()
{
#ifdef WIN32
  DWORD status=0;

  ieffect->GetEffectStatus(&status);
  return status & DIEGES_PLAYING;
#else
  return false;
#endif
}

int ForceEffect::getDelay()
{
#ifdef WIN32
  return delay;
#else
  return 0;
#endif
}
