// ForceXmlAdapter.cpp: implementation of the ForceXmlAdapter class.
//
//////////////////////////////////////////////////////////////////////

#include "ForceXmlAdapter.h"
#include "ErrorHandlerE.h"
#include "saxexpat.h"
#include "InputSource.h"
#include "ForceEffectSet.h"
#include "ForceEffect.h"
#include "AttributeList.h"
#include <ctype.h>
#include "ConstantForceEffect.h"
#include "DirectionalForceEffect.h"
#include "ForceFeedbackJoystick.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ForceXmlAdapter::ForceXmlAdapter(std::string filename, 
								 ForceFeedbackJoystick * newjoystick)
{
   expat myParser;

   forceset = new ForceEffectSet();
   joystick = newjoystick;
   myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(*(new SAX::InputSource(filename)));
}

ForceXmlAdapter::~ForceXmlAdapter()
{

}

void ForceXmlAdapter::startDocument()
{

}

void ForceXmlAdapter::endDocument()
{

}

void ForceXmlAdapter::startElement(const std::string& name, 
								   const SAX::AttributeList* atts)
{
   int i;
   if(name == "constantforce" || name == "rampforce" || name == "periodicforce")
   {

      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "start_delay")
		 {
            start_delay = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "duration")
		 {
            duration = atoi( currentvalue.c_str() );        
		 }
	  }
   }
   else if(name == "direction")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "value")
		 {
            direction = atoi( currentvalue.c_str() );        
		 }
	  }
   }
   else if(name == "envelope")
   {

      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "attack_level")
		 {
            attack_level = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "attack_time")
		 {
            attack_time = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "decay_level")
		 {
            decay_level = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "decay_time")
		 {
            decay_time = atoi( currentvalue.c_str() );        
		 }

	  }
   }
   else if(name == "constant")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "value")
		 {
            constant_value = atoi( currentvalue.c_str() );        
		 }
	  }
   }
   else if(name == "ramp")
   {

      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "start")
		 {
            ramp_start = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "end")
		 {
            ramp_end = atoi( currentvalue.c_str() );        
		 }
	  }
   }
   else if(name == "periodic")
   {

      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "type")
		 {
            periodic_type =  currentvalue.c_str() ;        
		 }
		 else if(currentname == "phase")
		 {
            phase = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "period")
		 {
            period = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "offset")
		 {
            offset = atoi( currentvalue.c_str() );        
		 }
		 else if(currentname == "amplitude")
		 {
            amplitude = atoi( currentvalue.c_str() );        
		 }

	  }
   }


}

void ForceXmlAdapter::endElement(const std::string& name)
{
	if(name == "constantforce")
	{
       ConstantForceEffect * constant_force = joystick->getConstantForce();
	   setBasics(constant_force);
	   setDirectional(constant_force);
	   constant_force->setConstant(constant_value);
	   forceset->addForceEffect(constant_force);
	}
	else if(name == "rampforce")
	{
		RampForceEffect * ramp = joystick->getRampForce();
  	    setBasics(ramp);
	    setDirectional(ramp);
		ramp->setStart(ramp_start);
		ramp->setEnd(ramp_end);
	    forceset->addForceEffect(ramp);		
	}
	else if(name == "periodicforce")
	{
		PeriodicForceEffect * periodic = NULL;

		if(periodic_type == "square")
		{
			periodic = joystick->getSquareForce();
		}
		else if(periodic_type == "triangle")
		{
			periodic = joystick->getTriangleForce();
		}
		else if(periodic_type == "sine")
		{
			periodic = joystick->getSineForce();
		}
		else if(periodic_type == "sawtooth up")
		{
			periodic = joystick->getSawToothUpForce();
		}
		else if(periodic_type == "sawtooth down")
		{
			periodic = joystick->getSawToothDownForce();
		}
		else
		{
		   // throw an exception
		}
  	    setBasics(periodic);
	    setDirectional(periodic);
		periodic->setMagnitude(amplitude);
		periodic->setOffset(offset);
		periodic->setPeriod(period);
		periodic->setPhase(phase);
	    forceset->addForceEffect(periodic);		
	}
}

void ForceXmlAdapter::characters(const std::string& ch)
{

}

void ForceXmlAdapter::setBasics(ForceEffect * fe)
{
  fe->setDelay(start_delay);
  fe->setDuration(duration);
}

void ForceXmlAdapter::setDirectional(DirectionalForceEffect * directional)
{
  directional->setDirection(direction);
  directional->setEnvelope(attack_level, attack_time, decay_level, decay_time);
}

ForceEffectSet * ForceXmlAdapter::getForce()
{
  return forceset;
}
