// GameEngine.cpp: implementation of the GameEngine class.
//
//////////////////////////////////////////////////////////////////////
#include "GL/glew.h"
#ifdef WIN32
#include "GL/wglew.h"
#endif
#include "GameEngine.h"
#include "StarField.h"
#include "ai.h"
#include "mesh.h"
#include "ship.h"
#include "World.h"
#include "shipfactory.h"
#include "Camera.h"
#include "Cockpit.h"
#include "PursueBehavior.h"
#include "PursueWhenStrongBehavior.h"
#include "SoundFactory.h"
#include "SoundEffect.h"
#include "WanderBehavior.h"
#include "IffCode.h"
#include "Mission.h"
#include "CutScene.h"
#include "ChaseCamera.h"
#include "AutoPilotCamera.h"
#include "LeftCamera.h"
#include "CockpitCamera.h"
#include "BackCamera.h"
#include "RightCamera.h"
#include "Time.h"
#include "GameFlow.h"
#include "Thruster.h"
#include "weaponfactory.h"
#include "mesh.h"
#include "TextureFactory.h"
#include "TakeOffCutScene.h"
#include "Settings.h"
#include "LandingCutScene.h"
#include "Sector.h"
#include "SoundFactory.h"
#include "MusicManager.h"
#include "ForceFeedbackJoystick.h"
#include "JoystickFactory.h"
#include "JoystickRange.h"
#include <stdio.h>
#include "SoundFactory.h"
#include "SoundObject3d.h"

#ifdef WIN32
#include "ConstantForceEffect.h"
#include "RampForceEffect.h"
#include "PeriodicForceEffect.h"
#endif
#include "ForceEffectFactory.h" 
#include "ForceEffectSet.h"
#include "Explosion.h"
#include "TextOverlay.h"
#include "CommFactory.h"
#include "FontFactory.h"
#include "EjectionPod.h"
#include <iostream.h>

#include "NavGrid.h"
#include "FileNotFoundException.h"
#include "glerr.h"
#include "ReticleFactory.h"
// for turret testing
#include "Turret.h"
#include "LightManager.h"
#include "MovieFactory.h"
#include "LensFlare.h"
#include "FixedCamera.h"
#include "CubeMap.h"
#include "Decoy.h"
#include <stdlib.h>
#include "MeshFileFactory.h"


void RestoreDisplayMode();

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GameEngine::GameEngine()
{
  cockpit = NULL;
  upkeypressed=false;
  downkeypressed=false;
  leftkeypressed=false;
  rightkeypressed=false;
  firekeypressed=false;
  bankrightkeypressed=false;
  bankleftkeypressed=false;
  stars = NULL;
  mission = NULL;
  gameflow = NULL;
  music = NULL;
  sector = NULL;
  joystick = NULL;
  xrange = NULL;
  yrange = NULL;
  rzrange = NULL;
  throttlerange = NULL;
  show_comms = false;
  show_damage = false;
  missilecamenabled = false;
  width = 0.0f;
  height = 0.0f;
  cubemapcreated = false;
  srand( 0 );
}

GameEngine::~GameEngine()
{
 // delete stars;
//  if(mission != NULL)
//     delete mission;
}

void GameEngine::onIdle()
{
   Missile * missile;

   Time::advance();


   CutScene * scene = CutScene::getCurrentCutScene();
   if(scene != NULL)
   {
     if( scene->isDone() )
     {
       if(scene->instanceof("AutoPilotCutScene"))
       {
         mission->onAutoPilotDone();
         clearInputState();
       }
       else if(scene->instanceof("TakeOffCutScene"))
       {
         onLaunchDone();
         clearInputState();
       }
	   else if(scene->instanceof("EjectCutscene") )
	   {
		   if(mission->canEjectToNextMission() )
		   {
              exitEngine(MISSION_LOST);
		   }
		   else
		   {
              exitEngine(MISSION_RESTART);
		   }
           CutScene::destroy(); 
           return;
	   }
       else // landing
       {
         if(mission->getFailed() )
         {
            exitEngine(MISSION_LOST);
         }
         else
         {
            exitEngine(MISSION_WON);
         }
         CutScene::destroy(); 
         return;
       }
       CutScene::destroy();
     }
     else
     {
        scene->advance();
     }
   }
   else
   {
     ship * player = mission->getPlayer();
     if(player != NULL)
     {
        if(upkeypressed)
        {
          player->rotateup();
          stars->rotateup();
        }
        else if(downkeypressed)
        {
          player->rotatedown();
          stars->rotatedown();
        }
        if(leftkeypressed)
        {
          stars->rotateleft();
          player->rotateleft();
        }
        else if(rightkeypressed)
        {
          stars->rotateright();
          player->rotateright();
        }
        if(firekeypressed)
        {
		  if(player->isCurrentGunChargeable() )
		  {

		  }
		  else
		  {
            player->fire();
		  }
        }
        if(bankrightkeypressed)
        {
          player->bankleft();
          stars->bankleft();
        }
        else if(bankleftkeypressed)
        {
          player->bankright();
          stars->bankright();
        }

		// joystick stuff
		if(joystick != NULL)
		{
           joystick->update();
           if( joystick->isButtonPressed(0) )
		   {
			  if(player->isCurrentGunChargeable() )
			  {
				 player->beginCharging();
			  }
			  else
			  {
                player->fire();
			  }
		   }
		   else
		   {
			  if(player->isCurrentGunChargeable() && !(firekeypressed) )
			  {
				 player->endCharging();
			  }
		   }
		   if(joystick->isButtonPressed(1) )
		   {
             missile = player->fireMissile();
		    if(missile != NULL && missilecamenabled)
			{
	          Camera::setCamera(ChaseCamera::getInstance() );
		      Camera::setFocusObject(missile);
			}
		   }

			bool currentdown2 = joystick->isButtonPressed(2);
			bool currentdown3 = joystick->isButtonPressed(3);
			bool currentdown4 = joystick->isButtonPressed(4);
			bool currentdown5 = joystick->isButtonPressed(5);

		   if(button2down && ! currentdown2  )
		   {
			   button2down = false;
			   nextTarget(player);
		   }

  		   if(button3down && ! currentdown3  )
		   {
			   button3down = false;
			   nextGun(player);
		   }
  		   if(button4down && ! currentdown4  )
		   {
			   button4down = false;
			   nextMissile(player);
		   }

		   if(currentdown5)
		   {
			  player->setAfterburner(true);
		   }
		   else
		   {
             if(button5down)
			 {
			    player->setAfterburner(false);
			 }
			 button5down = false;
		   } 

		   if(currentdown2 )
		   {
			   button2down = true;
		   }
		   if(currentdown3 )
		   {
			   button3down = true;
		   }
		   if(currentdown4)
		   {
			   button4down = true;
		   }
		   if(currentdown5)
		   {
			   button5down = true;
		   }



        float joyx = xrange->getPositionPercentageSigned(joystick->getXValue() );
		float joyy = yrange->getPositionPercentageSigned(joystick->getYValue() );
		const float epsilon = 1e-5;

        if(fabs(joyx) > epsilon)
        {
           if(joyx < 0.0f)
           {
              player->rotateLeftPercent(- joyx) ;
           }
           else
           {
              player->rotateRightPercent(joyx) ;
           }
        }
        if(fabs(joyy) > epsilon)
        {
           if(joyy < 0.0f)
           {
              player->rotateUpPercent( -joyy) ;
           }
           else
           {
              player->rotateDownPercent(joyy) ;
           }
        }
		if(joystick->hasRZAxis())
		{
           float joyrz = rzrange->getPositionPercentageSigned(joystick->getRZValue() );
		   if(fabs(joyrz) > epsilon)
		   {
			   if(joyrz < 0.0f)
			   {
				  player->bankRightPercent( -joyrz);
			   }
			   else
			   {
				  player->bankLeftPercent( joyrz);
			   }
		   }
		}
		if(joystick->hasThrottle() )
		{
			float throttle = throttlerange->getPositionPercentage(joystick->getThrottleValue() );
            player->setDesiredVelocity(throttle);
		}
		if(joystick->getNumHats() > 0)
		{
           int direction = joystick->getHatDirection(0);
           if(direction != -1)
		   {
             Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
		     if(direction == 0)
			 {
                Camera::setCamera(CockpitCamera::getInstance() );
			 }
		     else if(direction == 18000)
			 {
                Camera::setCamera(BackCamera::getInstance() );
			 }
		     else if(direction > 18000)
			 {
                Camera::setCamera(LeftCamera::getInstance() );
			 }
		     else if(direction < 18000)
			 {
                Camera::setCamera(RightCamera::getInstance() );
			 }
		   }
		}
		}
     }
   }
    MusicManager::getInstance()->advance();
    Comm::update();
    World::getWorld().advance();
	SoundFactory::getSoundFactory()->advance();
  stars->advance();
  SoundObject3d * listener = SoundFactory::getSoundFactory()->getListener();
  Camera * camera = Camera::getCamera();
  listener->setPosition(camera->getLocation() );
  listener->setHeading(camera->getHeading() );
  listener->setUp(camera->getUp() );
  listener->render();
  onDisplay();
}

//bool cubecapturemode = false;
//int cubeindex = -1;

void GameEngine::onSpecialKeyboard(int key, int x, int y)
{
   CutScene * scene = CutScene::getCurrentCutScene();
   if(scene != NULL)
   {
     if(! scene->isDone() )
       return;
   }
   ship * player =  mission->getPlayer();
   if(player == NULL)
     return;


  switch (key) {
  case GLUT_KEY_RIGHT:
    rightkeypressed = true;
    break;
  case GLUT_KEY_LEFT:
    leftkeypressed = true;
    break;
  case GLUT_KEY_UP:
    upkeypressed = true;
    break;
  case GLUT_KEY_DOWN:
    downkeypressed = true;
    break;
  case GLUT_KEY_INSERT:
    bankrightkeypressed = true;
    break;
  case GLUT_KEY_HOME:
    bankleftkeypressed = true;
    break;
  case GLUT_KEY_PAGE_UP:
    bankrightkeypressed = true;
    break;
  case GLUT_KEY_F1:
	Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
    Camera::setCamera(CockpitCamera::getInstance() );
	// force test
    break;
  case GLUT_KEY_F2:
	Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
    Camera::setCamera(LeftCamera::getInstance() );
    break;
  case GLUT_KEY_F3:
	Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
    Camera::setCamera(RightCamera::getInstance() );
    break;
  case GLUT_KEY_F4:
	Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
    Camera::setCamera(BackCamera::getInstance() );
    break;
  case GLUT_KEY_F5:
	Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
    Camera::setCamera(ChaseCamera::getInstance() );
    break;
  case GLUT_KEY_F6:
	Camera::setFocusObject(Mission::getCurrentMission()->getPlayer() );
    Camera::setCamera(AutoPilotCamera::getInstance() );
    break;
  case GLUT_KEY_F9:
	missilecamenabled = ! 	missilecamenabled;
    break;
/*  case GLUT_KEY_F11:
	cubecapturemode = true;
	cubeindex ++;
    break;
  case GLUT_KEY_F12:
	char buffer[100];
	sprintf(buffer, "screenshot%d.bmp", cubeindex);
	CaptureViewport(buffer);
    break;
*/


  }
}

void GameEngine::onSpecialUpKeyboard(int key, int x, int y)
{
   CutScene * scene = CutScene::getCurrentCutScene();
   if(scene != NULL)
   {
     if(! scene->isDone() )
       return;
   }
   ship * player =  mission->getPlayer();
   if(player == NULL)
     return;

  switch (key) {
  case GLUT_KEY_RIGHT:
    rightkeypressed = false;
    break;
  case GLUT_KEY_LEFT:
    leftkeypressed = false;
    break;
  case GLUT_KEY_UP:
    upkeypressed = false;
    break;
  case GLUT_KEY_DOWN:
    downkeypressed = false;
    break;
  case GLUT_KEY_INSERT:
    bankrightkeypressed = false;
    break;
  case GLUT_KEY_HOME:
    bankleftkeypressed = false;
    break;
  case GLUT_KEY_PAGE_UP:
    bankrightkeypressed = false;
    break;
  }
}

void GameEngine::onKeyboard(unsigned char key, int x, int y)
{
   CutScene * scene = CutScene::getCurrentCutScene();
   SoundEffect * effect;
   Missile * missile;
   Turret * turret;

   if(scene != NULL)
   {
     if(! scene->isDone() )
       return;
   }
   ship * player = mission->getPlayer();
   if(player == NULL)
   {
     if(key == 27)
     {
        exitEngine(MISSION_RESTART);
     }
     return;
   }

   switch (key)
   {
      case 27:
         exitEngine(MISSION_RESTART);
         break;
	  case 8:
		  player->setDesiredVelocity(0.0f);
		  break;
      case 32:
         firekeypressed = true;
		 if(player->isCurrentGunChargeable() )
		 {
			player->beginCharging();
		 }
         break;
     case 13:
         missile = player->fireMissile();
		 if(missile != NULL && missilecamenabled)
		 {
	       Camera::setCamera(ChaseCamera::getInstance() );
		   Camera::setFocusObject(missile);
		 }

         break;
     case 5: // control e
		player->eject();
		break;
     case 'a':
		 if(glutGetModifiers() & GLUT_ACTIVE_ALT)
		 {
           doCommSelection('3'); //attack my target
		 }
         else
		 {
            mission->onAutoPilot();
		 }
        break;
     case 'c':
        effect = SoundFactory::getSoundFactory()->getBeepSound();
	    effect->play();
        show_comms = ! show_comms;
        cockpit->setShowComm(show_comms);
        break;
	 case 'd':
		 // test turret rotation
//		 turret = (Turret *)player->getTargetedCapshipComponent();
//		 turret->rotateSide(1.0f);
 	    if(glutGetModifiers() & GLUT_ACTIVE_ALT)
		{
           doCommSelection('4'); //status
		}
		else
		{
           effect = SoundFactory::getSoundFactory()->getBeepSound();
	       effect->play();
           show_damage = ! show_damage;
           cockpit->setShowDamage(show_damage);
		}
        break;

		 break;
#if 0
       	 case 'D':
		 // test turret rotation
		 turret = (Turret *)player->getTargetedCapshipComponent();
		 turret->rotateSide(-1.0f);
		 break;
	 case 's':
		 // test turret rotation
		 turret = (Turret *)player->getTargetedCapshipComponent();
		 turret->rotateUp(1.0f);
		 break;
	 case 'S':
		 // test turret rotation
		 turret = (Turret *)player->getTargetedCapshipComponent();
		 turret->rotateUp(-1.0f);
		 break;
	 case 'x':
		 // test turret fire
		 turret = (Turret *)player->getTargetedCapshipComponent();
		 turret->fire();
		 break;
#endif
	 case 'p':
		 // test turret fire
		 LightManager::getInstance()->printLightState();
		 break;
	 case 'f':
		 fullGuns(player);
		 break;
	 case 'F':
		 cockpit->toggleFPSEnabled();
		 break;
	 case 'b':
		 if(glutGetModifiers() & GLUT_ACTIVE_ALT)
		 {
           doCommSelection('2');
		 }
		 break;
	 case 'h':
		 if(glutGetModifiers() & GLUT_ACTIVE_ALT)
		 {
           doCommSelection('5'); // help
		 }
		 break;

     case '1':// taunt or status
     case '2':// break and attack
     case '3'://attack my target
     case '4'://status of wing
     case '5'://help
//     case '6':
        if(show_comms)
        {
           doCommSelection(key);
           show_comms = false;
           cockpit->setShowComm(show_comms);
        }
        break;
     case '=':
     case '+':
       player->incrementVelocity();
       break;
     case '-':
       player->decrementVelocity();
       break;
     case 't':
	   nextTarget(player);
       break;
     case 'g':
	   nextGun(player);
       break;
     case 'm':
       nextMissile(player);	   
       break;
     case 'u':
       effect = SoundFactory::getSoundFactory()->getBeepSound();
	   effect->play();
       player->targetClosest();
       break;
     case 'r':
       effect = SoundFactory::getSoundFactory()->getBeepSound();
	   effect->play();
       player->targetNextComponent();
       break;
     case '\t':
       player->setAfterburner(true);
       break;
     case 'e':
       player->fireDecoy();
       break;
	 case 'I':
       player->setInvulnerable(true);
       break;
	 case 'q':
		 bankleftkeypressed = true;
		 break;
	 case 'w':
		 bankrightkeypressed = false;
		 break;
     case 'l':
       effect = SoundFactory::getSoundFactory()->getBeepSound();
	   effect->play();
       player->setTargetLock(!player->isTargetLockSelected() );
       break;
     case ';':
	   if(player->getTarget() != NULL)
	   {
		   Targetable * target = player->getTarget();
		   if(target->instanceof(INST_SHIP) )
		   {
			  ship * starget= (ship *) target;
//			  starget->destroy();
			  starget->damage(10000, Coordinate(0.0f, 5.0f, 0.0f));
		   }		   
	   }
       break;
     case ':':
	   if(player->getTarget() != NULL)
	   {
		   Targetable * target = player->getTarget();
		   if(target->instanceof(INST_SHIP) )
		   {
			  ship * starget= (ship *) target;
			  starget->eject();
		   }		   
	   }
       break;
     case '\'':
       Settings * settings = Settings::getSettings();
	   settings->setShowCollisionSpheres(! settings->getShowCollisionSpheres() );
       break;
   }
}

void GameEngine::onKeyboardUp(unsigned char key, int x, int y)
{
   if(isdoingbriefing)
   {
      isdoingbriefing = false;
	  doTakeoffCutscene();
	  return;

   }

   CutScene * scene = CutScene::getCurrentCutScene();

   if(scene != NULL)
   {
     if(! scene->isDone() )
       return;
   }
   ship * player = mission->getPlayer();
   switch (key)
   {
      case 32:
        firekeypressed = false;
		if(player != NULL)
		{
          if(player->isCurrentGunChargeable() )
		  {
			  player->endCharging();
		  }
		}
        break;
	 case 'q':
		 bankleftkeypressed = false;
		 break;
	 case 'w':
		 bankrightkeypressed = false;
		 break;
     case '\t':
       if(player != NULL)
         player->setAfterburner(false);
       break;

   }
}

void GameEngine::onReshape(int w, int h)
{
   double aspect;

   glViewport (0, 0, (GLsizei)width, (GLsizei) height);
   aspect = (double) w  /(double) h;
   width = w;
   height = h;
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
//   gluPerspective(45.0, aspect, .1, 5000.0);
   gluPerspective(45.0, aspect, .5, 5000.0);
   glMatrixMode (GL_MODELVIEW);
   GLERRORCHECK
}


void GameEngine::onDisplay()
{
	if(! cubemapcreated)
	{
		if(GLEW_EXT_texture_cube_map && Settings::getSettings()->getReflections() )
		{
		  createCubeMaps2();
		}
  	    cubemapcreated = true;
	}	

    ship * player = mission->getPlayer();
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

   glViewport (0, 0, (GLsizei)width, (GLsizei) height);
GLERRORCHECK
//    glColor3f (1.0f, 1.0f, 1.0f);
// ???? what is this for?
    glColor3f (.5f, .5f, .5f);
    glLoadIdentity ();
    stars->render();
    Camera::getCamera()->render();
    sector->renderPlanets();


   CutScene * scene = CutScene::getCurrentCutScene();
   if(scene != NULL)
   {
     if(! scene->isDone() )
	 {
		scene->render();
	 }
   }

    World::getWorld().render();

//    glEnd();
	GLERRORCHECK
    if(cockpit == NULL && player != NULL)
	{
      cockpit = new Cockpit(player);
	}
	lensflare->render();

    if(Camera::getCamera()->shouldRenderCockpit() && cockpit != NULL)
	{
      cockpit->render();
	}
	GLERRORCHECK
	TextOverlay * overlay = TextOverlay::getInstance();
	if(overlay->isEnabled() )
      overlay->render();

    glFlush ();
    glutSwapBuffers();
	GLERRORCHECK
   glViewport (0, 0, (GLsizei)width, (GLsizei) height);
}


void GameEngine::setMission(char missionname [ ])
{
#ifdef WIN32
  if(WGLEW_EXT_swap_control)
  {
  // disble veritcal retrace
	if(Settings::getSettings()->getVSync() )
	{
      wglSwapIntervalEXT(1);
	}
    else
	{
      wglSwapIntervalEXT(0);	
	}
  }
#endif

  SoundFactory::getSoundFactory()->refresh();
  initJoystickVariables();
  lensflare = new LensFlare();
  
   bool smooth = Settings::getSettings()->smooth();

   if(smooth)
   {
     glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
     glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
     glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
     glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
//     glEnable(GL_DITHER);
     glDisable(GL_DITHER);
   }
   else
   {
     glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
     glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
     glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
     glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
     glDisable(GL_DITHER);
   }
   glEnable(GL_POINT_SMOOTH);
//   glEnable(GL_LINE_SMOOTH);
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glEnable(GL_DEPTH_TEST);
   glDepthMask(GL_TRUE);
   glEnable(GL_CULL_FACE);


   /** Fixme: This initiates the lights twice. Without it the screen is black on some systems
              When I find the cause of the blackness, remove this
   */
   glEnable(GL_LIGHTING);
/*   glLightfv(GL_LIGHT0, GL_AMBIENT, ambientlight);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuselight);
   glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
   glEnable(GL_LIGHT0);
*/

   glEnable(GL_TEXTURE_2D);
   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);



   glEnable(GL_COLOR_MATERIAL);
   glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
   ///!!!
//   glColorMaterial(GL_FRONT, GL_SPECULAR);

   World::getWorld().clear();
   World::getWorld().setCollisionEnabled(false);
   Explosion::clear();
   Decoy::clear();
   MovieFactory::getInstance();
   MeshFileFactory::clear();
   


   if(mission != NULL)
     delete mission;
   // make sure time is initialized
   Time::init();
   Time::advance();
   mission = new Mission(missionname);
   MusicManager::getInstance()->setMission(mission);
   sector = new Sector((char *) mission->getSectorName().c_str() );
   sector->createLights();
   stars = new StarField(128, sector->getBackgroundTexture() , sector->showStars()  );
   // set
     /////////////
   string namecopy = missionname;
  string briefingname = strtok( (char *) namecopy.c_str(), ".");
  briefingname += ".briefing.xml";

  try
  {
    NavGrid * navgrid = new NavGrid((char *) briefingname.c_str());
    World::getWorld().addVisualOnly(navgrid );
    isdoingbriefing = true;
    TextOverlay * overlay = TextOverlay::getInstance();
    overlay->setEnabled(true);
    MusicManager::getInstance()->briefing();

  }
  catch(FileNotFoundException e)
  {
    isdoingbriefing = false;
    e.print(cout);
    cout.flush();
	doTakeoffCutscene();
	// skip the briefing, just launch
  }

  GLERRORCHECK

}

void GameEngine::setGameFlow(GameFlow * newgameflow)
{
  gameflow = newgameflow;
}

void GameEngine::exitEngine(int exittype)
{
   mesh::clear();
   EjectionPod::clear();
   CubeMap::setEnvironmentMap(NULL);
   cubemapcreated = false;
   TextureFactory::clear();
   shipfactory::getshipfactory().clear();
   weaponfactory::clear();
   Thruster::clear();
   MusicManager::clear();
   CommFactory::getInstance()->clear();
   ReticleFactory::clear();
   ForceEffectFactory::clear();
   SoundFactory::getSoundFactory()->clear();
   LightManager::getInstance()->clear();
   MovieFactory::getInstance()->clear();
   FontFactory::clear();
   delete cockpit;
   cockpit = NULL;
   delete sector;
   sector = NULL;
   delete stars;
   deleteJoystickVariables();

   joystick = NULL;
//   music->stop();
   Time::clear();
   delete lensflare;
   if(gameflow != NULL)
      gameflow->onEngineExit(exittype, mission->getDebriefingInfo() );
}

void GameEngine::onLaunchDone()
{
   mission->activateNavPoint();
   ship * player = mission->getPlayer();
   Camera::setFocusObject(player);
   Camera::setCamera(CockpitCamera::getInstance() );
   player->setMaxVelocity();
}

void GameEngine::clearInputState()
{
  upkeypressed=false;
  downkeypressed=false;
  leftkeypressed=false;
  rightkeypressed=false;
  firekeypressed=false;
  bankrightkeypressed=false;
  bankleftkeypressed=false;
  button2down=false;
  button3down=false;
  button4down=false;
  button5down=false;

}

void GameEngine::doCommSelection(char key)
{
  commhandler.update();
  bool enemy;

  switch(key)
  {
     case '1':
		if(mission->getPlayer()->getTarget() != NULL)
		{
			if(mission->getPlayer()->getTarget()->instanceof(INST_SHIP) )
			{
				ship * target = (ship *) mission->getPlayer()->getTarget();
				if(target->getIffCode() == ENEMY)
				{
					commhandler.taunt();
				}
				else if(target == mission->getLandingShip() )
				{
					commhandler.requestLanding();
				}
				else
				{
					commhandler.targetStatus();
				}
			}
		}
        break;
     case '2':
		commhandler.breakAndAttack();
        break;
     case '3':
		commhandler.attackMyTarget();
        break;
     case '4':
  	    commhandler.wingStatus();
        break;
     case '5':
		commhandler.help();
        break;
  };
}

void GameEngine::initJoystickVariables()
{
  joystick = JoystickFactory::getInstance()->getJoystick();
  forcejoystick = JoystickFactory::getInstance()->getForceFeedbackJoystick();
  if(joystick != NULL)
  {
    xrange = joystick->getXAxisRange();
	yrange = joystick->getYAxisRange();
	if(joystick->hasRZAxis() )
	{
	  rzrange = joystick->getRZAxisRange();
	}
	if(joystick->hasThrottle() )
	{
      throttlerange = joystick->getThrottleRange();
	}
  }
}

void GameEngine::deleteJoystickVariables()
{
	JoystickFactory::getInstance()->clear();

}

void GameEngine::doTakeoffCutscene()
{
   World::getWorld().clear();
   World::getWorld().setCollisionEnabled(true);
   TextOverlay::getInstance()->setEnabled(false);


   Object3d * tempplayer = mission->preloadPlayer();
   Camera::setFocusObject(tempplayer);
   mission->preloadShips();

   TakeOffCutScene * cutscene;
   cutscene = new TakeOffCutScene();
//   LandingCutScene * cutscene;
//   cutscene = new LandingCutScene();

   cutscene->setPlayer(tempplayer);
   cutscene->start();

}

void GameEngine::nextGun(ship * player)
{
   SoundEffect * effect;

   player->nextGun();
   effect = SoundFactory::getSoundFactory()->getGunChangeSound();
   effect->play();

}

void GameEngine::nextMissile(ship * player)
{
   SoundEffect * effect;

   player->nextMissile();
   effect = SoundFactory::getSoundFactory()->getMissileChangeSound();
   effect->play();
}

void GameEngine::nextTarget(ship * player)
{
   SoundEffect * effect;

   player->nextTarget();
   effect = SoundFactory::getSoundFactory()->getBeepSound();
   effect->play();
}

void GameEngine::createCubeMaps(int index)
{
	  FixedCamera * camera = (FixedCamera *) FixedCamera::getInstance();

  	  Camera::setCamera(camera);
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);



    glViewport (0, 0, (GLsizei)256, (GLsizei) 256);

   glMatrixMode (GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity ();
   gluPerspective(90.0f, 1.0f, .5, 500000.0);

   glMatrixMode (GL_MODELVIEW);


if(index == 0)
{

	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();
	  // +x

	  camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(6.0f, 0.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	  camera->render();
	  GLERRORCHECK
      sector->renderPlanets();
	  lensflare->render();
	  GLERRORCHECK
	  GLERRORCHECK
    glPopMatrix();
    glFlush ();
    glutSwapBuffers();
   // glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
}
else if(index == 1)
{

	GLERRORCHECK
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	// -x
	  camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(-6.0f, 0.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	  camera->render();
      sector->renderPlanets();
	  lensflare->render();
    glPopMatrix();
    glFlush ();
    glutSwapBuffers();
//    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK
}
else if(index == 2)
{
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	// +y
      camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(0.0f, 6.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 0.0f, -1.0f) );
	  camera->render();
      sector->renderPlanets();
	  lensflare->render();
    glPopMatrix();
    glFlush ();
    glutSwapBuffers();
//    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK
}
else if(index == 3)
{
	// -y
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	  camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(0.0f, -6.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 0.0f, 1.0f) );
	  camera->render();
      sector->renderPlanets();
	  lensflare->render();
    glPopMatrix();
    glFlush ();
    glutSwapBuffers();
//    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    GLERRORCHECK
}
else if(index == 4)
{

	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	// +z
	camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	camera->setLookAt(Coordinate(0.0f, 0.0f, 6.0f));
	camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	camera->render();
    sector->renderPlanets();
    lensflare->render();
    glPopMatrix();
    glFlush ();
    glutSwapBuffers();
//    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK
}
else if(index == 5)
{
	// -z
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	camera->setLookAt(Coordinate(0.0f, 0.0f, -6.0f));
	camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	camera->render();
    sector->renderPlanets();
    lensflare->render();
    glPopMatrix();
    glFlush ();
    glutSwapBuffers();
//    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK
}


   glMatrixMode (GL_PROJECTION);
   glPopMatrix();
   glMatrixMode (GL_MODELVIEW);

}

void GameEngine::createCubeMaps2()
{

  Camera * oldcamera;
  CubeMap * cube2 = NULL;

  oldcamera = Camera::getCamera();

  if(cube2 != NULL)
  {
	return;
  }
  cube2 = new CubeMap();

  FixedCamera * camera = (FixedCamera *) FixedCamera::getInstance();

  Camera::setCamera(camera);
  glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);



    glViewport (0, 0, (GLsizei)256, (GLsizei) 256);

   glMatrixMode (GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity ();
   gluPerspective(90.0f, 1.0f, .5, 500000.0);

   glMatrixMode (GL_MODELVIEW);

// 0

	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();
	  // +x

	  camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(6.0f, 0.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	  camera->render();
	  GLERRORCHECK
//      sector->renderPlanets();
//	  lensflare->render();
	  GLERRORCHECK
	  GLERRORCHECK
    glPopMatrix();

	cube2->copyFromScreen(0, 256, 256);
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	GLERRORCHECK
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	// -x
	  camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(-6.0f, 0.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	  camera->render();
//      sector->renderPlanets();
//	  lensflare->render();
    glPopMatrix();
	cube2->copyFromScreen(1, 256, 256);
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK

// 2
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	// +y
      camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(0.0f, 6.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 0.0f, -1.0f) );
	  camera->render();
//      sector->renderPlanets();
//	  lensflare->render();
    glPopMatrix();
	cube2->copyFromScreen(2, 256, 256);
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK
// 3
	// -y
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	  camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	  camera->setLookAt(Coordinate(0.0f, -6.0f, 0.0f));
	  camera->setUp(Coordinate(0.0f, 0.0f, 1.0f) );
	  camera->render();
//      sector->renderPlanets();
//	  lensflare->render();
    glPopMatrix();
	cube2->copyFromScreen(3, 256, 256);
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    GLERRORCHECK

///4
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	// +z
	camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	camera->setLookAt(Coordinate(0.0f, 0.0f, 6.0f));
	camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	camera->render();
//    sector->renderPlanets();
//    lensflare->render();
	cube2->copyFromScreen(4, 256, 256);
    glPopMatrix();
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK
//5
	// -z
	glPushMatrix();
      GLERRORCHECK
      glColor3f (.5f, .5f, .5f);
      glLoadIdentity ();
      stars->render();

	camera->setLocation(Coordinate(0.0f, 0.0f, 0.0f));
	camera->setLookAt(Coordinate(0.0f, 0.0f, -6.0f));
	camera->setUp(Coordinate(0.0f, 1.0f, 0.0f) );
	camera->render();
//    sector->renderPlanets();
//    lensflare->render();
    glPopMatrix();
    cube2->copyFromScreen(5, 256, 256);
    glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	GLERRORCHECK

   glMatrixMode (GL_PROJECTION);
   glPopMatrix();
   glMatrixMode (GL_MODELVIEW);

   Camera::setCamera(oldcamera);

   CubeMap::setEnvironmentMap(cube2);
}

void GameEngine::fullGuns(ship * player)
{
   SoundEffect * effect;

   player->setFullGuns(true);
   effect = SoundFactory::getSoundFactory()->getGunChangeSound();
   effect->play();
}
