// GameFlow.cpp: implementation of the GameFlow class.
//
//////////////////////////////////////////////////////////////////////
#include "GameFlowXmlAdapter.h"
#include "GameFlow.h"
#include "Font.h"
#include "SoundFactory.h"
#include "Settings.h"
#include "GameEngine.h"
#include "MusicFactory.h"
#include "Panel.h"
#include "SettingsPanel.h"
#include "CampaignPanel.h"
#include "LoadPanel.h"
#include "SavePanel.h"
#include <stdio.h>
#include "ImageFactory.h"
#include "SettingsPanel.h"
#include "Campaign.h"
#include "BriefingPanel.h"
#include "DebriefingPanel.h"
#include "Color.h"
#include "Image.h"
#include "FontFactory.h"
#include "Font.h"
#include "TextureFactory.h"
#include "TextPanel.h"
#include "glerr.h"


#define MOUSE_OVER_EXIT 1
#define MOUSE_OVER_SETTINGS 2
#define MOUSE_OVER_MISSION 3
#define MOUSE_OVER_CAMPAIGN 4
#define MOUSE_OVER_LOAD 5
#define MOUSE_OVER_SAVE 6
#define MOUSE_OVER_EMAIL 7


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GameFlow::GameFlow()
{
  GameFlowXmlAdapter adapter("gameflow.xml");

  glDisable(GL_TEXTURE_2D);
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_CULL_FACE);
  glDisable(GL_LIGHTING);
  glDepthMask(GL_FALSE);


  background = adapter.getBackgroundImageName();
  shell = ImageFactory::getImage((char *) background.c_str() , 640, 480);
  cursor = ImageFactory::getTransparentImage("cursor.bmp", Color(0.0f, 0.0f, 0.0f) );  
  cursor_offset_x = cursor->getWidth() /2; // divide by 2
  cursor_offset_y = cursor->getHeight() /2; // divide by 2

  exitrect = adapter.getExitArea();
  missionrect = adapter.getStartMissionArea();
  settingsrect = adapter.getSettingsArea();
  campaignrect = adapter.getCampaignArea();
  loadrect = adapter.getLoadArea();
  saverect = adapter.getSaveArea();
  emailrect = adapter.getEmailArea();
  engine = new GameEngine();
  engine->setGameFlow(this);
  mouseoverstate = 0;
#ifdef WIN32
  ShowCursor(FALSE);
#endif
  flyx = missionrect.x + missionrect.width / 2;
  flyy = 480 - (missionrect.y + missionrect.height / 2);

  exitx = exitrect.x + exitrect.width / 2;
  exity = 480 - (exitrect.y + exitrect.height / 2);

  settingsx = settingsrect.x + settingsrect.width / 2;
  settingsy = 480 - (settingsrect.y + settingsrect.height / 2);

  campaignx = campaignrect.x + campaignrect.width / 2;
  campaigny = 480 - (campaignrect.y + campaignrect.height / 2);

  loadx = loadrect.x + loadrect.width / 2;
  loady = 480 - (loadrect.y + loadrect.height / 2);

  savex = saverect.x +saverect.width / 2;
  savey = 480 - (saverect.y + saverect.height / 2);

  music = MusicFactory::getMusicFactory()->getMusic("fcgameflow.mp3");
//  music->play();
  startedmusic = false;
  currentpanel = NULL;
  settingspanel = new SettingsPanel();
  campaignpanel = new CampaignPanel();
  loadpanel = new LoadPanel();
  savepanel = new SavePanel();
  briefingpanel = new BriefingPanel();
  debriefingpanel = new DebriefingPanel();
  emailpanel = new TextPanel();
  panelactive = false;
  currentcampaign = NULL;
  displayed = false;
  under_mouse_bits = NULL;
  text_background_bits = NULL;
  prevx=-1.0f;
  prevy=-1.0f;
  prev_mousover_state= -1;
  font = FontFactory::getInstance().getFont();

}

GameFlow::~GameFlow()
{
  if(shell != NULL)
    delete shell;
  delete engine;
  if(music != NULL)
    delete music;
#ifdef WIN32
  ShowCursor(TRUE);
#endif
  delete settingspanel;
  delete campaignpanel;
  delete briefingpanel;
  delete debriefingpanel;
  delete font;
}



void GameFlow::onDisplay()
{
   float 	winWidth ,winHeight;

   if(! startedmusic)
   {
     music->play();
	 startedmusic = true;
   }

//	float radarWidth;
//	float radarHeight;

//   glClear (GL_DEPTH_BUFFER_BIT);
	winWidth = 640;
	winHeight = 480;


    glMatrixMode (GL_PROJECTION);
    glPushMatrix();//1
    glLoadIdentity ();
	  
    gluOrtho2D(-0.01f, winWidth + 0.01f, 0.0f, winHeight);

    glMatrixMode (GL_MODELVIEW);
	 glPushMatrix();//2
    glLoadIdentity ();

    if(panelactive)
    {
       currentpanel->onDisplay();
    }
    else
    {
       shell->render(0.0f, 0.0f);
       renderSelf();
    }
    /* draw the mouse pointer */
////////////////////////
/*    if(prevx <0.0f)
    {
      glReadPixels((int)(x- cursor_offset_x), (int) (y- cursor_offset_y), cursor->getWidth(), cursor->getHeight(), GL_RGB, GL_UNSIGNED_BYTE, under_mouse_bits);
      prevx = x;
      prevy = y;
    }
*/
//////////////////////////
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
    cursor->render(x - cursor_offset_x, y- cursor_offset_y);   
     glDisable(GL_BLEND);



	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
/*   glEnable(GL_TEXTURE_2D);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_CULL_FACE);
   glEnable(GL_LIGHTING);
*/
    glFlush ();
    glutSwapBuffers();
   displayed = true;
   GLERRORCHECK
}

void GameFlow::onReshape(int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   glFrustum (-1.0, 1.0, -1.0, 1.0, 1.0, 500.0);
   glMatrixMode (GL_MODELVIEW);
//   glDisable(GL_TEXTURE_2D);
   glEnable(GL_TEXTURE_2D);
   glDisable(GL_DEPTH_TEST);
   glDepthMask(GL_FALSE);   
   glDisable(GL_CULL_FACE);
   glDisable(GL_LIGHTING);
   GLERRORCHECK
}

void registerlisteners();

void GameFlow::onKeyboard(unsigned char key, int x, int y)
{


  if(panelactive)
  {
     bool result = currentpanel->onKeyboard(key, x, y);
     panelactive = ! result;
     if(result)
     {
        if(currentpanel == briefingpanel)
        {
           if(briefingpanel->getStart() )
           {
             startMission();
             return;
           }
        }
     }
  }
  
//  currentpanel = loadpanel;
  //loadpanel->update();
    onDisplay();
}

void GameFlow::onMouseDown(int button, int state, int x, int y)
{
  bool dodisplay=true;

  if(panelactive)
  {
    bool result =
      currentpanel->onMouseDown(button, state, x, y);
    panelactive = ! result;
    if(! panelactive) 
    {
      if(currentpanel == campaignpanel)
      {      
         CampaignPanel * cp = (CampaignPanel *) currentpanel;

         if(cp->campaignChanged() )
         {
            setCampaign(cp->getCampaignName() );
         }
      }
      else if(currentpanel == loadpanel)
      {
         LoadPanel * cp = (LoadPanel *) currentpanel;

         if(cp->hasSelection() )
         {
            loadGame(cp->getMission(), cp->getCampaign() );
         }

         
      }
      else if(currentpanel == briefingpanel)
      {
         if(briefingpanel->getStart() )
         {
           dodisplay = false;
           startMission();
		   return;
         }
      }
      else if(currentpanel == debriefingpanel)
      {
         debriefingmusic->stop();
         delete debriefingmusic;
         debriefingmusic = NULL;       
         music->play();
      }
	  else if(currentpanel == settingspanel)
	  {
		if(! Settings::getSettings()->getMusic() )
		{
			music->stop();
			
		}
	  }
    }
  }
  else
  {
     if( exitrect.inside(x, y) )
     {
       this->exit();
     }
     else if(missionrect.inside(x, y) )
     {
       if(currentcampaign != NULL)
       {
          startMission();
		  return;
/*          panelactive = true;
          currentpanel = briefingpanel;
		  briefingpanel->update();
*/
       }
     }
     else if(settingsrect.inside(x, y) )
     {
       panelactive = true;
       currentpanel = settingspanel;
     }
     else if(campaignrect.inside(x, y))
     {
       currentpanel = campaignpanel;
       panelactive = true;
     }
     else if(loadrect.inside(x, y) )
     {
        panelactive = true;
        currentpanel = loadpanel;
        loadpanel->update();
     }
     else if(saverect.inside(x, y) )
     {
        if(currentcampaign != NULL)
        {
          panelactive = true;
          currentpanel = savepanel;
          savepanel->setMission(currentmission);
          savepanel->setCampaign(currentcampaign->getFileName() );
          savepanel->update();
        }
     }
	 else if(emailrect.inside(x, y) )
	 {
		if(currentcampaign != NULL)
		{
          char backup[200];
		  panelactive = true;
		  string newnamecopy = currentmission;
		  strcpy(backup, (char *) currentmission.c_str() );
		  string messagesname = strtok( (char *) newnamecopy.c_str(), ".");
		  currentmission = backup;
		  messagesname += ".messages.xml";
		  emailpanel->setMessageFile( (char *)  messagesname.c_str() );
		  currentpanel = emailpanel;
		}
	 }
  }
  if(dodisplay)
  {
    clearMouseUnderState();
    onDisplay();
  }
}
void RestoreDisplayMode();

void GameFlow::exit()
{
         RestoreDisplayMode();
         delete SoundFactory::getSoundFactory();
         Panel::clean();
         ::exit(0);
}

void GameFlow::onEngineExit(int mission_status, DebriefingInfo debrifinginfo)
{
  music = MusicFactory::getMusicFactory()->getMusic("fcgameflow.mp3");
 
  clearMouseUnderState();
  displayed = false;
#ifdef GAMEMODE
  glutLeaveGameMode();
  glutGameModeString("640x480:16");
  glutEnterGameMode();
#else
    glutReshapeWindow(640, 480);  
#endif

  font = FontFactory::getInstance().getFont();
  shell = ImageFactory::getImage((char *) background.c_str() , 640, 480);
  cursor = ImageFactory::getTransparentImage("cursor.bmp", Color(0.0f, 0.0f, 0.0f) );
  emailpanel = new TextPanel();


  registerlisteners();
  setEventListener(this);

   //glDisable(GL_TEXTURE_2D);
   glEnable(GL_TEXTURE_2D);
   glDisable(GL_DEPTH_TEST);
   glDepthMask(GL_FALSE);
   glDisable(GL_CULL_FACE);
   glDisable(GL_LIGHTING);

  if(mission_status == MISSION_LOST)
  {
    string nextmission;

    nextmission = currentcampaign->getLosingMission(currentmission);
    if(nextmission != "")
    {
      currentmission = nextmission;
    }
    nextmission = currentcampaign->getWinningMission(currentmission);
    if(nextmission != "")
    {
      currentmission = nextmission;
      briefingpanel->setMission(currentmission);
    }
    debriefingmusic = MusicFactory::getMusicFactory()->getMusic("wc1-debriefbad.mp3");
    debriefingmusic->play();
    debriefingpanel->setDebriefingInfo(mission_status, &debrifinginfo);
    panelactive = true;
    currentpanel = debriefingpanel;
  }
  else if(mission_status == MISSION_WON)
  {
    string nextmission;

    nextmission = currentcampaign->getWinningMission(currentmission);
    if(nextmission != "")
    {
      currentmission = nextmission;
      briefingpanel->setMission(currentmission);
    }
    debriefingmusic = MusicFactory::getMusicFactory()->getMusic("wc1-debriefgood.mp3");
    debriefingmusic->play();
    debriefingpanel->setDebriefingInfo(mission_status, &debrifinginfo);
    panelactive = true;
    currentpanel = debriefingpanel;

  }
  else // restart 
  {
    music->play();
  }
  GLERRORCHECK  
}




void GameFlow::onMouseMove(int newx, int newy)
{
  static bool called=false;
  bool render_self_call = false;
  const int text_coverx=320, text_covery=20-5, text_cover_width=200, text_cover_height=25;


//  if(!displayed)
//    return;

  prevx = x;
  prevy = y;
  x = newx;
  y = newy;
// save the area where we would draw text
/*  if(text_background_bits == NULL)
  {
      text_background_bits = new unsigned char[3 * text_cover_width * text_cover_height ];
      glReadPixels(text_coverx, text_covery,text_cover_width, text_cover_height, GL_RGB, GL_UNSIGNED_BYTE, text_background_bits);
  }
*/

  
  if(! panelactive)
  {
     if( exitrect.inside(x, y) )
     {
         mouseoverstate =  MOUSE_OVER_EXIT;
     }
     else if(missionrect.inside(x, y) )
     {
         mouseoverstate =  MOUSE_OVER_MISSION;
     }
     else if(settingsrect.inside(x, y))
     {
         mouseoverstate =  MOUSE_OVER_SETTINGS;
     }
     else if(campaignrect.inside(x, y))
     {
        mouseoverstate = MOUSE_OVER_CAMPAIGN;
     }
     else if(loadrect.inside(x, y) )
     {
        mouseoverstate = MOUSE_OVER_LOAD;
     }
     else if(saverect.inside(x,y) )
     {
        mouseoverstate = MOUSE_OVER_SAVE;
     }
     else if(emailrect.inside(x,y) )
     {
        mouseoverstate = MOUSE_OVER_EMAIL;
     }
     else
     {
        mouseoverstate = 0;
     }
     if(mouseoverstate != prev_mousover_state)
     {
       prev_mousover_state = mouseoverstate;
       render_self_call = true;
     }
  }
  y = 480 - y;
  onDisplay();
  return;


///////////////////private drawing

   float 	winWidth ,winHeight;



	winWidth = 640;
	winHeight = 480;


    glMatrixMode (GL_PROJECTION);
    glPushMatrix();//1
    glLoadIdentity ();
	  
    gluOrtho2D(-0.01f, winWidth + 0.01f, 0.0f, winHeight);

    glMatrixMode (GL_MODELVIEW);
	 glPushMatrix();//2
    glLoadIdentity ();

    if(under_mouse_bits == NULL)
    {
      under_mouse_bits = new unsigned char[3 * cursor->getWidth() * cursor->getHeight() ];
    }
    if(render_self_call)
    {
       glRasterPos2i(text_coverx, text_covery);
       glDrawPixels(text_cover_width, text_cover_height, GL_RGB, GL_UNSIGNED_BYTE, text_background_bits);
       renderSelf();
    }


 
    if(prevx >= 0.0f)
    {
      glRasterPos2f(prevx- cursor_offset_x, prevy - cursor_offset_y);
      glDrawPixels(cursor->getWidth(), cursor->getHeight(), GL_RGB, GL_UNSIGNED_BYTE, under_mouse_bits);
      glReadPixels((int)(x- cursor_offset_x), (int) (y- cursor_offset_y), cursor->getWidth(), cursor->getHeight(), GL_RGB, GL_UNSIGNED_BYTE, under_mouse_bits);
    }
    else  /* save the previous hole before drawing */
    {
      glReadPixels((int)(x- cursor_offset_x), (int) (y- cursor_offset_y), cursor->getWidth(), cursor->getHeight(), GL_RGB, GL_UNSIGNED_BYTE, under_mouse_bits);
      prevx = x;
      prevy = y;
    }
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
    cursor->render(x- cursor_offset_x, y- cursor_offset_y);   
     glDisable(GL_BLEND);



	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
    glFlush ();
    glutSwapBuffers();
    GLERRORCHECK
}



void GameFlow::renderSelf()
{
    string text;
    if(mouseoverstate == MOUSE_OVER_EXIT)
    {
      text = "Exit";
    }
    else if(mouseoverstate == MOUSE_OVER_MISSION)
    {

      if(currentcampaign == NULL)
      {
        text = "Load a game or choose a campaign";
      }
      else
      {
        text = "Fly mission";
      }
    }
    else if (mouseoverstate == MOUSE_OVER_SETTINGS)
    {
        text = "Settings";
    }
    else if(mouseoverstate == MOUSE_OVER_CAMPAIGN)
    {
        text = "Start campaign";        
    }
    else if(mouseoverstate == MOUSE_OVER_LOAD)
    {
        text = "Load a game";
    }
    else if(mouseoverstate == MOUSE_OVER_SAVE)
    {
      if(currentcampaign != NULL)
      {
        text = "Save a game";
      }
    }
    else if(mouseoverstate == MOUSE_OVER_EMAIL)
    {
      if(currentcampaign != NULL)
      {
        text = "Communications";
      }
    }
    font->print(320.0f, 20.0f, (char  *) text.c_str() );

}

void GameFlow::setCampaign(string name)
{
   if(currentcampaign != NULL)
      delete currentcampaign;

   currentcampaign = new Campaign((char *) name.c_str() );
   currentmission = currentcampaign->getStartMission();
   briefingpanel->setMission(currentmission);
}

void GameFlow::loadGame(string mission, string campaign)
{
  setCampaign(campaign);
  currentmission = mission;
  briefingpanel->setMission(mission);
}

void GameFlow::startMission()
{
         FontFactory::clear();
	     TextureFactory::clear();
         Panel::clean();
		 delete emailpanel;
		 emailpanel = NULL;
#ifdef GAMEMODE
         glutLeaveGameMode();
         glutGameModeString((char *) Settings::getSettings()->getResolutionString().c_str() );
         glutEnterGameMode();
#else    
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();	  
    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();
    glutReshapeWindow(1024, 768);
#endif
         registerlisteners();
         setEventListener(engine);
         delete shell;
         shell = NULL;
         music->stop();
         delete music;
         music = NULL;
         engine->setMission((char *) currentmission.c_str());

}

void GameFlow::clearMouseUnderState()
{
//  delete []  under_mouse_bits;
//  under_mouse_bits = NULL;
  prevx = -1.0f;
  prevy = -1.0f;
}
