// GameFlowXmlAdapter.cpp: implementation of the GameFlowXmlAdapter class.
//
//////////////////////////////////////////////////////////////////////

#include "GameFlowXmlAdapter.h"
#include "AttributeList.h"
#include <ctype.h>
#include "saxexpat.h"
#include "InputSource.h"
#include "ErrorHandlerE.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GameFlowXmlAdapter::GameFlowXmlAdapter(char filename[])
{
	expat myParser;
   ErrorHandlerE::checkFile(filename);
	myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(*(new SAX::InputSource(filename)));
}

GameFlowXmlAdapter::~GameFlowXmlAdapter()
{

}


void GameFlowXmlAdapter::startDocument()
{

}

void GameFlowXmlAdapter::endDocument()
{

}

void GameFlowXmlAdapter::startElement(const std::string& name, const SAX::AttributeList* atts)
{
   int i;

   if(name == "gameflow")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

         if(currentname == "background")
         {
            backgroundimagename =  currentvalue;        
         }
      }
   }
   else if(name == "rect")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

         if(currentname == "x")
         {
            currentrectangle.x  =  atoi(currentvalue.c_str() );        
         }
         else if(currentname == "y")
         {
            currentrectangle.y  =  atoi(currentvalue.c_str() );        
         }
         else if(currentname == "width")
         {
            currentrectangle.width  =  atoi(currentvalue.c_str() );        
         }
         else if(currentname == "height")
         {
            currentrectangle.height  =  atoi(currentvalue.c_str() );        
         }
      }
   }
}

void GameFlowXmlAdapter::endElement(const std::string& name)
{
  if(name == "startmission")
  {
     startmissionarea = currentrectangle;
  }
  else if(name == "exit")
  {
     exitarea = currentrectangle;
  }
  else if(name == "settings")
  {
     settingsrectangle = currentrectangle;
  }
  else if(name == "campaign")
  {
     campaignrectangle = currentrectangle;
  }
  else if(name == "load")
  {
     loadrectangle = currentrectangle;
  }
  else if(name == "save")
  {
     saverectangle = currentrectangle;
  }
  else if(name == "email")
  {
     emailrectangle = currentrectangle;
  }

}


void GameFlowXmlAdapter::characters(const std::string& ch)
{


}

string GameFlowXmlAdapter::getBackgroundImageName()
{
  return backgroundimagename;
}

Rect GameFlowXmlAdapter::getExitArea()
{
  return exitarea;
}

Rect GameFlowXmlAdapter::getStartMissionArea()
{
  return startmissionarea;
}

Rect GameFlowXmlAdapter::getSettingsArea()
{
  return settingsrectangle;
}

Rect GameFlowXmlAdapter::getCampaignArea()
{
  return campaignrectangle;
}

Rect GameFlowXmlAdapter::getLoadArea()
{
  return loadrectangle;
}

Rect GameFlowXmlAdapter::getSaveArea()
{
  return saverectangle;
}

Rect GameFlowXmlAdapter::getEmailArea()
{
  return emailrectangle;
}
