// ImageFile.cpp: implementation of the ImageFile class.
//
//////////////////////////////////////////////////////////////////////

#include "ImageFile.h"
#include "il.h"
#include "FileNotFoundException.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ImageFile::ImageFile(char filename[], bool wantalpha)
{
  ilInit();

  ilEnable(IL_ORIGIN_SET);
  ilOriginFunc(IL_ORIGIN_LOWER_LEFT);

  ilGenImages(1, &image);
  ilBindImage(image);
  ilLoadImage(filename);
  bytesperpixel   = ilGetInteger(IL_IMAGE_BYTES_PER_PIXEL);
  hasalpha = (bytesperpixel == 4);
  
  if(wantalpha)
  {
    ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE);
  }
  else
  {
    ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
  }
  if(ilGetError() != IL_NO_ERROR)
  {
	throw FileNotFoundException(filename);
  }
}

ImageFile::~ImageFile()
{
  ilDeleteImages(1, &image);
  if(ilGetError() != IL_NO_ERROR)
  {
	throw FileNotFoundException("imagefile");
  }

}

int ImageFile::getWidth()
{
   ilBindImage(image);
   return ilGetInteger(IL_IMAGE_WIDTH);
}

int ImageFile::getHeight()
{
   ilBindImage(image);
   return ilGetInteger(IL_IMAGE_HEIGHT);
}

unsigned char * ImageFile::getData()
{
  ilBindImage(image);
  if(ilGetError() != IL_NO_ERROR)
  {
	throw FileNotFoundException("imagefile");
  }

  return  ilGetData();
}

bool ImageFile::hasAlpha()
{
  return hasalpha;
}

int ImageFile::getBytesPerPixel()
{
  return bytesperpixel;
}
