// JoystickDirectX.cpp: implementation of the JoystickDirectX class.
//
//////////////////////////////////////////////////////////////////////

#include "JoystickDirectX.h"
#include <iostream.h>
#include "JoystickRange.h"
#include "JoystickException.h"


struct callbackstruct
{
  bool hasrz;
  bool hasthrottle;
  DIDEVICEOBJECTINSTANCE x;
  DIDEVICEOBJECTINSTANCE y;
  DIDEVICEOBJECTINSTANCE rz;
  DIDEVICEOBJECTINSTANCE throttle;

};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BOOL CALLBACK DIEnumJoyProc(LPCDIDEVICEINSTANCE lpddi, 
                            LPVOID pvRef) 
{ 
  *(GUID*) pvRef = lpddi->guidInstance; 
  return DIENUM_STOP; 
} // end of callback 


BOOL CALLBACK DIEnumDeviceObjectsProc(
  LPCDIDEVICEOBJECTINSTANCE deviceobject,  
  LPVOID pvRef                       
)
{
  struct callbackstruct * data = (struct callbackstruct *) pvRef;

  
  if(deviceobject->guidType == GUID_XAxis  )
  {
	data->x = *deviceobject;
  }
  else if(deviceobject->guidType == GUID_YAxis  )
  {
	data->y = *deviceobject;
  }
  else if(deviceobject->guidType == GUID_RzAxis)
  {
    data->hasrz = true;
	data->rz = *deviceobject;
  }
  else if((deviceobject->guidType == GUID_Slider) || deviceobject->guidType == GUID_ZAxis)
  {
    data->hasthrottle = true;
	data->throttle = *deviceobject;
  }
  return DIENUM_CONTINUE;
}


JoystickDirectX::JoystickDirectX()
{
  HRESULT        hr; 
  HINSTANCE instance;
  directinput = NULL;
  GUID joystickguid;


  hasthrottle = false;
  hasrzaxis = false;
  // this is a hack to get the window handle and instance from glut
  HWND wnd = FindWindow(NULL, "GLUT");
  instance = (HINSTANCE) GetWindowLong(wnd, GWL_HINSTANCE);
  if(wnd == NULL)
  {
    cout << "Window is null in joystick create" << endl;
	throw JoystickException();
  }

  hr = DirectInputCreate(instance, DIRECTINPUT_VERSION, &directinput, NULL); 
  if FAILED(hr) 
  { 
    cout << "Could not create directinput" << endl;
	throw JoystickException();
  } 
  directinput->EnumDevices(DIDEVTYPE_JOYSTICK,DIEnumJoyProc, (void *) &joystickguid ,
                         DIEDFL_ATTACHEDONLY | DIEDFL_FORCEFEEDBACK); 


  LPDIRECTINPUTDEVICE tempdirectinputdevice;
  if (directinput->CreateDevice(joystickguid, 
                                 &tempdirectinputdevice, NULL) != DI_OK) 
  { 
    // try again, but without specifying the force feedback flag
    directinput->EnumDevices(DIDEVTYPE_JOYSTICK,DIEnumJoyProc, (void *) &joystickguid ,
                         DIEDFL_ATTACHEDONLY); 

    if (directinput->CreateDevice(joystickguid, 
                                 &tempdirectinputdevice, NULL) != DI_OK) 
	{
      cout << "Could not create directinput joystick device" << endl;
	  throw JoystickException();
	}
  }

   // convert to direct input 2

  if(tempdirectinputdevice->QueryInterface(IID_IDirectInputDevice2,(void **) &directinputdevice) != DI_OK)
  {
    cout << "Could not create directinput device2" << endl;
	throw JoystickException();
  }

  capabilities.dwSize = sizeof(capabilities);
  directinputdevice->GetCapabilities(&capabilities);
  hasforcefeedback = (capabilities.dwFlags & DIDC_FORCEFEEDBACK) ;

  if (directinputdevice->SetDataFormat(&c_dfDIJoystick) != DI_OK) 
  { 
    cout << "Could not set data format" << endl;
	throw JoystickException();
  } 
//  if (directinputdevice->SetCooperativeLevel(wnd, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE) != DI_OK) 
  if (directinputdevice->SetCooperativeLevel(wnd, DISCL_FOREGROUND | DISCL_EXCLUSIVE) != DI_OK) 
  { 
    cout << "Could not set cooperative level" << endl;
	throw JoystickException();
  } 

/*  hr=directinputdevice->Acquire();
  if (hr != DI_OK && hr != S_FALSE) 
  { 
    cout << "Could not acquire 1" << endl;
  } 
*/  


  struct callbackstruct data;
  data.hasrz = false;
  data.hasthrottle = false;
  directinputdevice->EnumObjects(DIEnumDeviceObjectsProc, (void *)&data, DIDFT_AXIS);
  hasrzaxis = data.hasrz;
  hasthrottle = data.hasthrottle;
  xaxisobject = data.x;
  yaxisobject = data.y;
  rzaxisobject = data.rz;
  throttleobject =data.throttle;
}

JoystickDirectX::~JoystickDirectX()
{
  if( directinputdevice != NULL)
  {
    directinputdevice->Unacquire();
    directinputdevice->Release();
  }
  if(directinput != NULL)
  {
	directinput->Release();
  }
}



int JoystickDirectX::getNumButtons()
{
  return capabilities.dwButtons;
}

int JoystickDirectX::getNumAxes()
{
  return capabilities.dwAxes;
}

bool JoystickDirectX::isButtonPressed(int button_index)
{
  if(button_index >= getNumButtons() )
    return false;
  return joystickstate.rgbButtons[button_index] != 0;
}

JoystickRange * JoystickDirectX::getXAxisRange()
{
  return getAxisRange(&xaxisobject);
}

JoystickRange * JoystickDirectX::getYAxisRange()
{
  return getAxisRange(&yaxisobject);
}

JoystickRange * JoystickDirectX::getRZAxisRange()
{
  return getAxisRange(&rzaxisobject);
}

JoystickRange * JoystickDirectX::getThrottleRange()
{
  return getAxisRange(&throttleobject);
}

int JoystickDirectX::getXValue()
{
  return joystickstate.lX;
}

int JoystickDirectX::getYValue()
{
  return joystickstate.lY;
}


int JoystickDirectX::getRZValue()
{
  return joystickstate.lRz;
}

int JoystickDirectX::getThrottleValue()
{
  return joystickstate.lZ;
}

void JoystickDirectX::update()
{
  HRESULT  res= directinputdevice->Acquire();
  if (res != DI_OK && res != S_FALSE) 
  { 
    cout << "Could not acquire 2" << endl;
  } 

  if( directinputdevice->Poll() != DI_OK)
  {
    cout << "Could not poll" << endl;
  }
  if (directinputdevice->GetDeviceState(sizeof(DIJOYSTATE), & joystickstate) != DI_OK) 
  {
    cout << "Could not get device state" << endl;
  }
}

int JoystickDirectX::getNumHats()
{
	return capabilities.dwPOVs;
}

int JoystickDirectX::getHatDirection(int hat_index)
{
	return joystickstate.rgdwPOV[hat_index];
}

bool JoystickDirectX::hasThrottle()
{
  return hasthrottle;
}

bool JoystickDirectX::hasRZAxis()
{
  return hasrzaxis;
}

JoystickRange * JoystickDirectX::getAxisRange(DIDEVICEOBJECTINSTANCE * deviceobject)
{
  int min, max;
  DIPROPRANGE range;

  range.diph.dwSize = sizeof(DIPROPRANGE);
  range.diph.dwHeaderSize = sizeof(DIPROPHEADER);
  range.diph.dwHow = DIPH_BYID;
  range.diph.dwObj = deviceobject->dwType;
  
  directinputdevice->GetProperty( DIPROP_RANGE,(DIPROPHEADER *) &range);
  min = range.lMin;
  max = range.lMax;
  return new JoystickRange(min, max);
}


IDirectInputEffect * JoystickDirectX::getEffect(REFGUID rguid)
{
  IDirectInputEffect * effect=NULL;


  HRESULT result = directinputdevice->CreateEffect(rguid
	  , NULL, &effect, NULL);
  if(result != DI_OK)
  {
     throw JoystickException();
	 return NULL;
  }
  return effect;
}

RampForceEffect * JoystickDirectX::getRampForce()
{
  try
  {
    return new RampForceEffect(getEffect(GUID_RampForce));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }
}

PeriodicForceEffect * JoystickDirectX::getSquareForce()
{
  try
  {
    return new PeriodicForceEffect(getEffect(GUID_Square));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }

}

PeriodicForceEffect * JoystickDirectX::getSineForce()
{
  try
  {
    return new PeriodicForceEffect(getEffect(GUID_Sine));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }

}

PeriodicForceEffect * JoystickDirectX::getTriangleForce()
{
  try
  {
    return new PeriodicForceEffect(getEffect(GUID_Triangle));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }

}

PeriodicForceEffect * JoystickDirectX::getSawToothUpForce()
{
  try
  {
    return new PeriodicForceEffect(getEffect(GUID_SawtoothUp));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }

}

PeriodicForceEffect * JoystickDirectX::getSawToothDownForce()
{
  try
  {
    return new PeriodicForceEffect(getEffect(GUID_SawtoothDown));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }

}

ConstantForceEffect * JoystickDirectX::getConstantForce()
{
  try
  {
    return new ConstantForceEffect(getEffect(GUID_ConstantForce));
  }
  catch(JoystickException joyexception)
  {
    return NULL;
  }

}

bool JoystickDirectX::hasForceFeedback()
{
  return hasforcefeedback;
}

std::string JoystickDirectX::getName()
{
  DIDEVICEINSTANCE info;

  info.dwSize = sizeof(DIDEVICEINSTANCE);
  directinputdevice->GetDeviceInfo(& info);
  return std::string(info.tszProductName);

}

void JoystickDirectX::printInfo()
{
   cout << "Joystick:" << getName().c_str()  << endl;
   if(hasForceFeedback() )
   {
	  cout << "Force feedback enabled" << endl;
   }
   else
   {
	  cout << "No Force feedback" << endl;
   }
   if(hasThrottle() )
   {
	  cout << "Has throttle" << endl;
   }
   else
   {
	  cout << "No throttle" << endl;
   }
   if(hasRZAxis() )
   {
	  cout << "Has RZ axis" << endl;
   }
   else
   {
	  cout << "No RZ axis" << endl;
   }
   cout << getNumButtons() << " buttons" << endl;
   cout << getNumHats() << " hats" << endl;

}
