// JoystickFactory.cpp: implementation of the JoystickFactory class.
//
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#include "JoystickDirectX.h"
#else
#include "JoystickLinux.h"
#endif
#include "JoystickFactory.h"
#include "JoystickException.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

JoystickFactory  JoystickFactory::instance;

JoystickFactory::JoystickFactory()
{
  joystick = NULL;
  nojoystick = false;
}

JoystickFactory::~JoystickFactory()
{

}

JoystickFactory * JoystickFactory::getInstance()
{
  return &instance;
}

Joystick * JoystickFactory::getJoystick()
{
  initJoystick();
  return joystick;
}

ForceFeedbackJoystick * JoystickFactory::getForceFeedbackJoystick()
{
  initJoystick();
  if(joystick != NULL)
  {
     if(!joystick->hasForceFeedback() )
	 {
		 return NULL;
	 }
  }
  return joystick;
}

void JoystickFactory::initJoystick()
{
  if((joystick == NULL) && (!nojoystick) )
  {
    try
	{
#ifdef WIN32
      joystick =  new JoystickDirectX();
#else
      joystick = NULL;
            joystick = new JoystickLinux();
#endif
	}
    catch(JoystickException joy)
	{
     nojoystick = true;
	}
  }
}

void JoystickFactory::clear()
{
  if(joystick != NULL)
  {
     delete joystick;
	 joystick = NULL;
  }
  joystick = NULL;
  nojoystick = false;
}
