// LandingCutScene.cpp: implementation of the LandingCutScene class.
//
//////////////////////////////////////////////////////////////////////

#include "LandingCutScene.h"
#include "World.h"
#include "Camera.h"
#include "FixedCamera.h"
#include "ChaseCamera.h"
#include "ship.h"
#include "SoundFactory.h"
#include "SoundEffect.h"
#include "MusicManager.h"
#include "LightManager.h"
#include "Settings.h"
#include "Math.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LandingCutScene::LandingCutScene()
: ambient(Color(.8f, .8f, .8f) )
{
  hangar = new Object3d( new mesh("hangerc.mesh.bin"), 0.0f, 0.0f, 0.0f , 0.0f, 0.0f, 0.0f);
  hangar->setCurrentVelocity(0.0f);
  player = NULL;
  duration = 9000;
  starttime = 0;
  if(Settings::getSettings()->getDynamicLights() )
  {
    light.setAmbient(Color(.8f, .8f, .8f) );
    light.setLocation(Coordinate(0.0f, 0.0f, 0.0f));
    LightManager::getInstance()->addLocalLight(&light);
  }
  else
  {
	LightManager::getInstance()->setAmbientLight(&ambient);
  }
}

LandingCutScene::~LandingCutScene()
{
  delete hangar;//???
}

bool LandingCutScene::isDone()
{
  TimeValue currenttime = Time::getCurrentTime();
  bool done;

  if(starttime == 0)
    return true;
   done = currenttime - starttime > duration;
   bool playing = MusicManager::getInstance()->isPlaying();
   done = done && !(playing);
   if(done)
   {
       World::getWorld().remove(hangar);
       World::getWorld().remove(player);
       hangar->setDestroyed();
	   if(Settings::getSettings()->getDynamicLights() )
	   {
         LightManager::getInstance()->removeLocalLight(&light);
	   }
   }
   return done ;
}

void LandingCutScene::start()
{
  MusicManager::getInstance()->landing();
  starttime = Time::getCurrentTime();

//  World::getWorld().add(hangar);

/*  Camera::setCamera(ChaseCamera::getInstance() );
  player->setOrientation(Coordinate(0.0f, 0.0f, 1.0f), 
                         Coordinate(0.0f, 1.0f, 0.0f),
                         Coordinate(1.0f, 0.0f, 0.0f) );

  float y = player->getExtents().y /2.0f - hangar->getExtents().y / 2.0f;

  player->setLocation(Coordinate(0.0f, y, -11.0f) );
*/
  FixedCamera * camera = (FixedCamera *) FixedCamera::getInstance();
  camera->setLocation(Coordinate(7.0f, 0.0f, 30.0f));
  camera->setLookAt(Coordinate(0.0f, 0.0f, - hangar->getExtents().z / 2.0f  ));
  camera->setUp(Coordinate(0.0f, 1.0f, 0.0f));
  Camera::setCamera(camera);
  player->setOrientation(Coordinate(0.0f, 0.0f, 1.0f), 
                         Coordinate(0.0f, 1.0f, 0.0f),
                         Coordinate(1.0f, 0.0f, 0.0f) );
  player->setShadowLocation(hangar->getExtents().y * -.5f);
  player->setVelocity(.1f);
  player->setDesiredVelocity(.1f);

  ((ship *)player)->clearVelocityVector();
//  float y =  MIN(player->getExtents().y * .4f, hangar->getExtents().y * .4f);
  float y =  MIN(player->getExtents().y * .25f, hangar->getExtents().y * .25f);
  starty = y;
  endy = -y;
  player->setLocation(Coordinate(0.0f, y, - 22.0f) );
//  player->setMaxVelocity();

  World & world= World::getWorld();
  world.setCollisionEnabled(false);
//  world.clear();
//  world.add(player);
  world.add(hangar);
  
  SoundEffect * effect = SoundFactory::getSoundFactory()->getLandingSound();
  effect->play();
}


void LandingCutScene::setPlayer(Object3d * newplayer)
{
  player = (ship *) newplayer;
  player->setAfterburner(false);
}

void LandingCutScene::render()
{
  // setup stencil buffer 

  glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
  glDepthMask(GL_FALSE);


  // enable stencil buffer
  glEnable(GL_STENCIL_TEST);
  glClear(GL_STENCIL_BUFFER_BIT);
  // write a one to the stencil buffer everywhere we are about to draw
  glStencilFunc(GL_ALWAYS, 1, 0xFFFFFFFF);
  // this is to always pass a one to the stencil buffer where we draw
  glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
				
  // render the plane which the shadow will be on
  // color and depth buffer are disabled, only the stencil buffer
  // will be modified
  hangar->render();

  // turn the color and depth buffers back on
  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
  glDepthMask(GL_TRUE);
  glDisable(GL_STENCIL_TEST);

}

void LandingCutScene::advance()
{
  TimeValue  currenttime = Time::getCurrentTime();
  float percentage = (float) (currenttime - starttime) / (float)duration;

  if(percentage > 1.0f)
  {
	percentage = 1.0f;
  }
  Coordinate location = player->getLocation();
  location.y = endy * percentage + starty * (1.0f - percentage);
  player->setLocation(location);

// camera
  Coordinate camerastart(7.0f, 0.0f, 30.0f);
  Coordinate cameraend(-7.0f, 0.0f, 30.0f);

  Coordinate camerapos = camerastart.interpolate(cameraend, percentage);
  FixedCamera * camera = (FixedCamera *) FixedCamera::getInstance();
  camera->setLocation(camerapos);
}
