// LensFlare.cpp: implementation of the LensFlare class.
//
//////////////////////////////////////////////////////////////////////

#include "ImageFactory.h"
#include "LensFlare.h"
#include "Color.h"
#include "Image.h"
#include "glut.h"
#include "LightManager.h"
#include "Camera.h"
#include "Settings.h"
#include "Math.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LensFlare::LensFlare()
{
	halo = ImageFactory::getTransparentImage("Halo.bmp", Color(0.0f, 0.0f, 0.0f) );
	hardglow = ImageFactory::getTransparentImage("HardGlow.bmp", Color(0.0f, 0.0f, 0.0f) );
	bigglow = ImageFactory::getTransparentImage("BigGlow.bmp", Color(0.0f, 0.0f, 0.0f) );
	streak =  ImageFactory::getTransparentImage("Streaks.bmp", Color(0.0f, 0.0f, 0.0f) );
}

LensFlare::~LensFlare()
{

}

void LensFlare::render(Coordinate screencenter, Coordinate lightlocation, Color starcolor)
{
	center = screencenter;
	light = lightlocation;

    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

    glDisable(GL_DEPTH_TEST);

    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glEnable(GL_BLEND);
//	   render(bigglow, 0.0f, Color(0.60f, 0.60f, 0.8f, 1.0f), 8.0f);
//	   render(streak, 0.0f, Color(0.60f, 0.60f, 0.8f, 1.0f), 8.0f);
	   render(bigglow, 0.0f, starcolor * 1.05f, 8.0f);
	   render(streak, 0.0f, starcolor * 1.05f, 8.0f);
	   render(hardglow, 0.0f, Color(0.8f, 0.8f, 1.0f, .5f), 1.25);

	   if(Settings::getSettings()->getCockpitDetails() )
	   {

   	   render(bigglow, .20f, Color(0.9f, 0.6f, 0.4f, 0.5f), .6f);

	   render(halo, .30f, Color(0.8f, 0.5f, 0.6f, 0.5f), 1.7f);

	   render(halo, .35f, Color(0.9f, 0.2f, 0.1f, 0.5f), .83f);

	   render(halo, .57f, Color(0.7f, 0.7f, 0.4f, 0.5f), .83f);

       render(bigglow, .551f, Color(0.9f, 0.9f, 0.2f, 0.5f), .8f);
 
       render(bigglow,  .95f, Color(0.93f, 0.82f, 0.73f, 0.5f), 1.0f);

	   render(halo, .98f, Color(0.7f, 0.6f, 0.5f, 0.5f), 1.4f);


	   render(bigglow,  1.3f, Color(0.7f, 0.8f, 0.3f, 0.5f), 1.8f);

	   render(bigglow,  1.26f, Color(0.4f, 0.3f, 0.2f, 0.5f), 1.4f);
	   render(halo, 1.6f, Color(0.7f, 0.5f, 0.5f, 0.5f), 1.4f);
       render(bigglow,  1.56f, Color(0.8f, 0.5f, 0.1f, 0.5f), .6f);
	   render(halo, 2.0f, Color(0.5f, 0.5f, 0.7f, 0.5f), 1.7f);
	   }
	
	glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
}

void LensFlare::render(Image * image, float percent, Color newcolor, float scale)
{
	Coordinate diff;
	Coordinate half;
	Coordinate final;
	float length;


	diff = center - light;
	length = fabs_f(diff.getlength() * percent);
	Coordinate normal = diff;
	normal.normalize();

	if(percent > 1.0f)
	{
		percent = percent;
	}
	final = light + normal * length;

//	half = diff * percent;
//	final = half + center;
//	final = center * scale +  light * (1 - scale);
	image->setScale(scale);
	image->setColor(newcolor);
	image->render(final.x - .5f * image->getWidth(),
		final.y - .5f * image->getHeight() );
}

void LensFlare::render()
{
  	float winWidth = glutGet(GLUT_WINDOW_WIDTH);
	float winHeight = glutGet(GLUT_WINDOW_HEIGHT);

// project light source to screen coordinates
   Coordinate location = LightManager::getInstance()->getLensFlareSourceLocation();
  double modelMatrix[16];
  double projMatrix[16];
  int viewport[4];
  double centerxd, centeryd, centerzd;

  glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
  glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
  glGetIntegerv(GL_VIEWPORT, viewport);

  location = location + Camera::getCamera()->getLocation();
  int result = gluProject(location.x, location.y, location.z,
	  modelMatrix, projMatrix, viewport, & centerxd, & centeryd, & centerzd);

  float zbuffervalue;
  bool occluded;
  glReadPixels(centerxd, centeryd,1,1,GL_DEPTH_COMPONENT, GL_FLOAT, &zbuffervalue);
  occluded = zbuffervalue < centerzd;

// switch to 2d mode
  switchTo2dMode();

// lens flare
   if(result != GL_FALSE && centerzd < 1.0 && (! occluded) )
   {
	   Coordinate screencenter(winWidth * .5f, winHeight * .5f, 0.0f);
	   Coordinate lightloc(centerxd, centeryd, 0.0f);

	   render(screencenter, lightloc, LightManager::getInstance()->getLensFlareColor() );
   }

// end lens flare

// end lens flare projection

   // back to 3d mode
   switchTo3dMode();

}

void LensFlare::switchTo2dMode()
{

  	float winWidth = glutGet(GLUT_WINDOW_WIDTH);
	float winHeight = glutGet(GLUT_WINDOW_HEIGHT);

    glMatrixMode (GL_PROJECTION);
    glPushMatrix();//1
    glLoadIdentity ();

    gluOrtho2D(-0.01f, winWidth + 0.01f, 0.0f, winHeight);

    glMatrixMode (GL_MODELVIEW);
	 glPushMatrix();//2
    glLoadIdentity ();

   glDisable(GL_TEXTURE_2D);
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_CULL_FACE);
   glDisable(GL_LIGHTING);


}

void LensFlare::switchTo3dMode()
{

   glEnable(GL_TEXTURE_2D);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_CULL_FACE);
   glEnable(GL_LIGHTING);


   glPopMatrix();
   glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
}
