// LightManager.cpp: implementation of the LightManager class.
//
//////////////////////////////////////////////////////////////////////

#include "LightManager.h"
#include "glut.h"
#include <algorithm>
#include "Camera.h"
#include <iostream.h>
#include "Settings.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LightManager LightManager::instance;

LightManager::LightManager()
{
  ambient = NULL;
}

LightManager::~LightManager()
{

}

void LightManager::renderGlobalLights()
{
       glPushMatrix();
//       glLoadIdentity();

  glGetIntegerv( GL_MAX_LIGHTS , &num_gl_lights );   

	std::list<Light *>::iterator it;

    for (it = globallights.begin(); it != globallights.end(); ++it)
	{
		Light * light = *it;
		light->enable();
		light->render();
	}
	if(ambient != NULL)
	  ambient->render();
	glPopMatrix();
}


struct compareByRS : public std::greater<DiffuseLight *>
{
  bool operator()(const DiffuseLight * obj1, const DiffuseLight* obj2)const
  { 
	  if(obj1->getPriority() != obj2->getPriority() )
	  {
        return obj1->getPriority() > obj2->getPriority();
	  }
	  else
	  {
	    float d1 = obj1->getLocation().distance(cameralocation);
	    float d2 = obj2->getLocation().distance(cameralocation);
	    return d2 > d1;
	  }
  }
  Coordinate cameralocation;
};


void LightManager::renderLocalLights(Coordinate position)
{
  int size;
  std::list<DiffuseLight *>::iterator it; 
  compareByRS s;
  int i;
  int max;
  int localoffset;
  

  if(! Settings::getSettings()->getDynamicLights() )
  {

    return;
  }

  localoffset = globallights.size();
  max = num_gl_lights - localoffset;
  

  s.cameralocation = position/* Camera::getCamera()->getLocation()*/;
  locallights.sort(s);

  it = locallights.begin();
  for(i=0; (i < max) && (it != locallights.end() ); i++)
  {
		DiffuseLight * light = *it;
		light->setLightNumber(i+localoffset);
		light->render();
		light->enable();
		it++;
  }

  for(; i < num_gl_lights - localoffset; i++)
  {
    glDisable(GL_LIGHT0 + i + localoffset);
	// zero out for shader
	float zero[] = {0.0f, 0.0f, 0.0f, 0.0f};
    glLightfv(GL_LIGHT0 + i + localoffset, GL_DIFFUSE, zero);
  }
}

void LightManager::clear()
{
  int i;

  locallights.clear();
  globallights.clear();
  for(i=0; i < num_gl_lights; i++)
  {
    glDisable(GL_LIGHT0 + i);
  }
}

void LightManager::addLocalLight(DiffuseLight * light)
{
  int size;

  size = locallights.size();
  locallights.push_back(light);
  size = locallights.size();
}

void LightManager::addGlobalLight(DiffuseLight * light)
{
  globallights.push_back(light);
  light->setLightNumber(globallights.size() - 1);
  lensflarelight = light;
}

void LightManager::removeLocalLight(DiffuseLight * light)
{
  int size;
	size = locallights.size();
  locallights.remove(light);
//  light->disable();

  size = locallights.size();
}

LightManager * LightManager::getInstance()
{
  return & instance;
}

void LightManager::printLightState()
{
  int i;
  int j;
  float variables[4];
  cout << " -------------------------" << endl;

  cout << "Ambient light" << endl;

  glGetFloatv(GL_LIGHT_MODEL_AMBIENT, variables);
  for(i=0; i < 4; i++)
  {
     cout << variables[i] << " ";
  }
  cout << endl;

  for(i=0; i < num_gl_lights; i++)
  {
	if( glIsEnabled(GL_LIGHT0 + i) )
	{
		cout << "Light " << i << endl;
		cout << "Ambient color" << i << endl;
		glGetLightfv(GL_LIGHT0 + i, GL_AMBIENT, variables);
		for(j=0; j < 4; j++)
		{
			cout << variables[j] << " ";
		}
		cout << endl;
		cout << "Diffuse color" << i << endl;
		glGetLightfv(GL_LIGHT0 + i, GL_DIFFUSE, variables);
		for(j=0; j < 4; j++)
		{
			cout << variables[j] << " ";
		}
		cout << endl;
		cout << "Position" << i << endl;
		glGetLightfv(GL_LIGHT0 + i, GL_POSITION , variables);
		for(j=0; j < 4; j++)
		{
			cout << variables[j] << " ";
		}
        cout << endl;
	}
	else
	{
       cout << "Light " << i << "is disabled" << endl;

	}
  }
}

void LightManager::setAmbientLight(AmbientLight * light)
{
  ambient = light;
}

Coordinate LightManager::getLensFlareSourceLocation()
{
  return lensflarelight->getLocation();
}

Color LightManager::getLensFlareColor()
{
  return lensflarelight->getDiffuse();
}

void LightManager::removeGlobalLight(DiffuseLight * light)
{
  light->disable();
  globallights.remove(light);
}

AmbientLight * LightManager::getAmbientLight()
{
  return ambient;
}
