// LoadPanel.cpp: implementation of the LoadPanel class.
//
//////////////////////////////////////////////////////////////////////

#include "LoadPanel.h"
#include "TextToSpeech.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LoadPanel::LoadPanel()
{
  adapter = NULL;
  setTitle("Load a game");
  setConfirmText("Load");
  update();
}

LoadPanel::~LoadPanel()
{

}

void LoadPanel::update()
{
  int i;

  selected = false;

  if(adapter != NULL)
  {
    delete adapter;
  }
  adapter = new SaveGameXmlAdapter("savegames.xml");

  int max;

  if(adapter->getNumSaves() > 17)
  {
    max = 17;
  }
  else 
  {
    max = adapter->getNumSaves();
  }

  setNumLines(max);
  for(i=0; i < max; i++)
  {
    setLine(adapter->getNames()[i], i);
  }
}

void LoadPanel::onConfirm()
{
  if(getStatusText() != "")
  {
    selected = true;
    TextToSpeech::getInstance()->synthesizeAndPlay((char *) getStatusText().c_str() );
  }
  setStatusText("");

}

void LoadPanel::onLineClick(int linenumber)
{
  if(linenumber < adapter->getNumSaves() )
  {
    setStatusText(adapter->getNames()[linenumber] );
    selection = linenumber;
  }
}

void LoadPanel::onCancel()
{
  setStatusText("");
  selected = false;
}

bool LoadPanel::hasSelection()
{
  return selected;
}

string LoadPanel::getMission()
{
  return adapter->getMissions()[selection];
}

string LoadPanel::getCampaign()
{
  return adapter->getCampaigns()[selection];
}
