// MP3Music.cpp: implementation of the MP3Music class.
//
//////////////////////////////////////////////////////////////////////

#include "MP3Music.h"
#include "MusicManager.h"
#include "Settings.h"
#include <iostream.h>
#ifdef HAS_FMOD
#include "fmod.h"
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#ifdef HAS_BASS
void __stdcall sync_callback(HSYNC handle, DWORD channel, DWORD data, DWORD user);
#endif

#ifdef HAS_FMOD
signed char  myendcallback(FSOUND_STREAM *stream,
			   void *buff,
			   int len,
			   void *userdata)
{
  MusicManager::getInstance()->onMusicFinish();
}
#endif


MP3Music::MP3Music(char filename[], bool repeat)
{

   if(! Settings::getSettings()->getMusic() )
   {
#ifdef HAS_BASS
      stream = NULL;
#endif
#ifdef HAS_FMOD
	  stream = NULL;
#endif
      return;
   }
#ifdef HAS_BASS
	stream = BASS_StreamCreateFile(false, filename, 0, 0, 0 );
    if(! stream)
    {
         cout << "Could not load music " << filename << endl;
    }
#endif
#ifdef HAS_FMOD
    if(repeat)
    {
       stream = FSOUND_Stream_Open(filename, FSOUND_LOOP_NORMAL, 0, 0);
    }
    else
    {
       stream = FSOUND_Stream_Open(filename, 0, 0, 0);
       FSOUND_Stream_SetEndCallback(stream, myendcallback, NULL);
    }
    if(stream == NULL)
    {
       cout << "Could not load music " << filename << endl;
    }
#endif
    loop = repeat;
}

MP3Music::~MP3Music()
{
   if(! Settings::getSettings()->getMusic() )
   {
      return;
   }
#ifdef HAS_BASS
    if(! stream)
		return;

	BASS_StreamFree(stream);
#endif
#ifdef HAS_FMOD
	if(stream == NULL)
	{
	  return;
	}
	FSOUND_Stream_Close(stream);
#endif
}

void MP3Music::play()
{
   if(! Settings::getSettings()->getMusic() )
   {
      return;
   }
#ifdef HAS_BASS
   if(! stream)
		return;


	DWORD flags =  (loop) ?  BASS_SAMPLE_LOOP : 0;

	BASS_StreamPlay(stream, true, flags);	
    BASS_ChannelSetSync(stream, BASS_SYNC_END, 0, sync_callback, (DWORD)this);
#endif

#ifdef HAS_FMOD

   FSOUND_Stream_SetPosition(stream, 0);
   FSOUND_Stream_Play(FSOUND_FREE, stream);
#endif

}

void MP3Music::stop()
{
#ifdef HAS_BASS
   if(stream == NULL) 
   {
      return;
   }
    if(! stream)
		return;


	BASS_ChannelStop(stream);
#endif
#ifdef HAS_FMOD
  if(stream == NULL)
  {
	return;
  }
  FSOUND_Stream_Stop(stream);
#endif
}

bool MP3Music::isPlaying()
{
   if(! Settings::getSettings()->getMusic() )
   {
      return false;
   }
#ifdef HAS_BASS
    if(! stream)
		return false;

  return BASS_ChannelIsActive(stream) == BASS_ACTIVE_PLAYING;
#elif HAS_FMOD
  if(stream == NULL)
  {
	return false;
  }
  return FSOUND_Stream_GetPosition(stream) < FSOUND_Stream_GetLength(stream);
#else
  return false;
#endif
}
