// MissileDescription.cpp: implementation of the MissileDescription class.
//
//////////////////////////////////////////////////////////////////////

#include "MissileDescription.h"
#include "ImageFactory.h"
#include "ReticleFactory.h"
#include "TextureFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MissileDescription::MissileDescription(int newrange, int newdamage, std::string newmesh,
     float newvelocity, float newroll, float newpitch, float newyaw,
	 float newaccel, string name,
	 std::string newimagename, shipareas newarmor, int newlocktime, bool newtracking,
	 bool newdamage_capships, bool newdamage_turrets, int refire_delay, float newspoof,
 	 int newsoundindex,
	 std::string newreticlename, 
	 std::string newreticletype,
	 std::string icon,
	 std::string exhaustname
	 )
   :weapondescription(newrange, newdamage, newmesh, newvelocity, newroll, newpitch, 
     newyaw, name, refire_delay, newaccel, icon, false, 
	 newdamage_capships, newdamage_turrets, newsoundindex),
   image(NULL), armor(newarmor), locktime(newlocktime), tracking(newtracking), 
     reticle(NULL), imagename(newimagename),
	 reticlename(newreticlename),
	 reticletype(newreticletype),
	 exhaust(exhaustname),
	 spoof(newspoof)
{
  loaded = false;
  exhausttexture = NULL;
}

MissileDescription::~MissileDescription()
{

}

Image * MissileDescription::getImage()
{
  return image;
}

shipareas MissileDescription::getArmor()
{
  return armor;
}

bool MissileDescription::isMissile()
{
  return true;
}

int MissileDescription::getLockTime()
{
  return locktime;
}

bool MissileDescription::isTracking()
{
  return tracking;
}

Reticle * MissileDescription::getReticle()
{
  return reticle;
}

void MissileDescription::load()
{
  if(loaded)
  {
    return;
  }
  weapondescription::load();
  if(imagename != "")
  {
    image = ImageFactory::getImage((char *) imagename.c_str() );
  }

  if(reticlename != "")
  {
	  reticle = ReticleFactory::getInstance()->getReticle(reticletype, reticlename);

  }

  if(exhaust != "")
  {
    exhausttexture  = TextureFactory::getTransparentTexture((char *) exhaust.c_str() );
  }
  loaded = true;
}

Texture * MissileDescription::getExhaustTexture()
{
  return exhausttexture;
}

float MissileDescription::getSpoof()
{
  return spoof;
}
