// Mission.cpp: implementation of the Mission class.
//
//////////////////////////////////////////////////////////////////////

#include "Mission.h"
#include "MissionXmlAdapter.h"
#include "shipfactory.h"
#include "World.h"
#include "PursueWhenStrongBehavior.h"
#include "CockpitCamera.h"
#include "FixedCamera.h"
#include "LandingCutScene.h"
#include "ChaseCamera.h"
#include "MusicManager.h"
#include <iostream.h>
#include "NavGrid.h"
#include "CapShipAI.h"
#include "AsteroidField.h"
#include "EjectCutscene.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
Mission * Mission::currentmission = NULL;


Mission::Mission(char filename[])
{
  MissionXmlAdapter adapter(filename);

  name = adapter.getName();
  can_eject = adapter.getCanEject();
  navpoints = adapter.getNavPoints(numnavpoints);
  sectorname = adapter.getSectorName() + ".sector.xml";
  currentnavpoint = 0;  
  player = NULL;
  currentmission = this;
  numshipsdestroyed = 0;
  numenemyships = 0;
  friendly_ships_lost = 0;
  landingship = NULL;
  numenemyshipsdestroyed = 0;
  failed = false;
  num_player_kills=0;
  num_player_gun_hits=0;
  num_player_gun_fired=0;
  num_player_missile_hits=0;
  num_player_missile_fired=0;

}

Mission::~Mission()
{

}

ship * Mission::getPlayer()
{
  return player;
}

Mission * Mission::getCurrentMission()
{
  return currentmission;
}

bool Mission::canAutoPilot()
{
  return (numenemyshipsdestroyed == numenemyships) && (currentnavpoint != numnavpoints -1);
}

void Mission::activateNavPoint()
{
  int i;
  int numnewships;
  NavPoint * navpoint = navpoints[currentnavpoint];
  ship * currentship;
  std::vector<ship *>::iterator it;
  Coordinate heading, up, left;

  numenemyshipsdestroyed = 0;
  numenemyships = 0;

  
  World::getWorld().clear();

/*
  copy over remaining ships from before

*/
 
  for(it = shipsv.begin(); it != shipsv.end() ;)
  {
	  currentship = *it;	
      if((!currentship->doesAutoPilot(currentnavpoint)) && (currentship != player) )
      {
		 // remove it from my list
		 it = shipsv.erase(it);
      }
      else if(! currentship->isdestroyed() )
      {
        World::getWorld().add(currentship); 
        currentship->setLocation(currentship->getInitialPosition() );

        heading.x = 0.0;
        heading.y = 0.0;
        heading.z = 1.0;
        up.x = 0.0;
        up.y = 1.0;
        up.z = 0.0;
        left.x = 1.0;
        left.y = 0.0;
        left.z = 0.0;
        currentship->setOrientation(heading, up, left);
		it++;
      }
  }

  // add in new ships
  numnewships = navpoint->getNumShips();
  shipfactory & factory = shipfactory::getshipfactory();
  for(i=0; i < numnewships; i++)
  {
    ShipInfo currentinfo = navpoint->getShipInfo(i);
    currentship = factory.getShip(currentinfo);
	shipsv.push_back(currentship);
	// don't add in new ships, the ship factory already added them to the world
//    World::getWorld().add(currentship); 
    if(currentinfo.getPilot() == "player" )
    {
       player = currentship;
       player->setPlayer();
    }
    else if(currentinfo.getLand() )
    {
      landingship = currentship;
    }
    else
    {
       if(currentship->getIffCode() == ENEMY)
       {
          numenemyships++;
       }
       if(currentship->instanceof(INST_CAPITALSHIP) )
	   {
         currentship->setAI(new CapShipAI() );
	   }
	   else
	   {
         currentship->setAI(new PursueWhenStrongBehavior() );
	   }
    }
  }

  for(i=0; i < shipsv.size(); i++)
  {
    if(shipsv[i] != player)
    {
      if(! shipsv[i]->isdestroyed() )
        shipsv[i]->targetClosest();
    }
    else 
    {
       player->setTarget(NULL);
    }
  }
  // added, reflects num ships destroyed at the next nav
  numshipsdestroyed = 0;


// ast field testing
  if(navpoint->hasAsteroidBelt() )
  {
    AsteroidField field;
    field.generate(player->getLocation() , 55, 50.0f); 
  }

  
}

void Mission::nextNavPoint()
{
  currentnavpoint++;
}

void Mission::onDestroyed(ship * destroyedship, bool ejected)
{
  std::vector<ship *>::iterator it;
  ship * currentship;

  if(destroyedship->getIffCode() == ENEMY)
  {
    numenemyshipsdestroyed++;
    MusicManager::getInstance()->onKillEnemy();
  }
  else
  {
     friendly_ships_lost++;
     MusicManager::getInstance()->onKillFriend();
  }
  numshipsdestroyed++;
  for(it=shipsv.begin(); it != shipsv.end(); )
  {
	 currentship = *it;
     if(currentship == destroyedship)
	 {
		it = shipsv.erase(it);
	 }
	 else
	 {
		it++;
	 }
  }
  if(destroyedship->getMustSurvive() )
  {
    failed = true;
  }

  if(destroyedship == player)
  {
    Coordinate up = player->getUp();
    Coordinate lookat = player->getLocation();
    Coordinate location = lookat;
    location.x -= 15.0f;
    FixedCamera * camera = (FixedCamera *) FixedCamera::getInstance();
    camera->setLocation(location);
    camera->setLookAt(lookat);
    camera->setUp(up);
    Camera::setCamera(camera);
    player = NULL;
	if(ejected)
	{
	  failed = true;
      MusicManager::getInstance()->onPlayerEject();
      EjectCutscene * ejcutscene = new EjectCutscene();
	  ejcutscene->start();
	}
	else
	{
      MusicManager::getInstance()->onPlayerDeath();
	}
  }
}

void Mission::onAutoPilot()
{
  int numwingmen;
  Object3d * * wingmen;
  ship * myship;

  if(!canAutoPilot() )
    return;

  cutscene = new AutoPilotCutScene(Camera::getCamera() );
  
  numwingmen = countWingmen();
  wingmen = new Object3d *[numwingmen];
  int i;
  int currentship = 0;

  for(i=0; i < shipsv.size() ; i++)
  {
    myship = shipsv[i];
    if(myship != NULL)
    {
       if(myship->getIffCode() == FRIEND && myship != player && myship->doesAutoPilot(currentnavpoint + 1) ) 
       {
          wingmen[currentship] = myship;
          currentship++;
       }
    }
  }
  cutscene->setPlayer(player);
  cutscene->setWing(wingmen, numwingmen);

/* 
  LandingCutScene * cutscene;
  cutscene = new LandingCutScene();
  cutscene->setPlayer(player);
*/
  cutscene->start();

}

int Mission::countWingmen()
{
  int count = 0;
  int i;

  for(i = 0; i < shipsv.size() ; i++)
  {
    if(shipsv[i] != NULL)
    {
       if(shipsv[i]->getIffCode() == FRIEND && shipsv[i] != player && shipsv[i]->doesAutoPilot(currentnavpoint + 1) ) 
       {
          count++;
       }
    }
  }
  return count;
}

void Mission::onAutoPilotDone()
{
//  camera.setCameraType(Camera::cameratype::cockpit);

  Camera::setCamera(CockpitCamera::getInstance() );
  nextNavPoint();
  activateNavPoint();
  MusicManager::getInstance()->onNewNavPoint();
}



Object3d * Mission::preloadPlayer()
{
  shipfactory & factory = shipfactory::getshipfactory();
  NavPoint * navpoint = navpoints[0];
  int numships = navpoint->getNumShips();
  int i;


  for(i=0; i < numships; i++)
  {
    
    if(navpoint->getShipInfo(i).getPilot() == "player" )
    {
      ShipInfo tempinfo = navpoint->getShipInfo(i);
       return factory.getShip(tempinfo);
    }
  }
  return NULL;
}

bool Mission::isComplete()
{
  return (numenemyshipsdestroyed == numenemyships) && (currentnavpoint == numnavpoints -1);
}

void Mission::onLanding()
{
  Camera::setCamera(ChaseCamera::getInstance() );
  LandingCutScene * cutscene = new LandingCutScene();

  player->setrenderingenabled(true);
  landingship->setrenderingenabled(false);
  cutscene->setPlayer(player);
  cutscene->start();
}

bool Mission::getFailed()
{
  return failed;
}

void Mission::preloadShips()
{
   int n,s;
   int maxships;

   shipfactory & factory = shipfactory::getshipfactory();

   NavPoint * nav;

   for(n=0; n < numnavpoints; n++)
   {
     nav = navpoints[n];
     maxships = nav->getNumShips();
     for(s=0; s < maxships; s++)
     {
        factory.preloadShip((char *) nav->getShipInfo(s).getFileName().c_str() );
     }
   }  
}

string Mission::getSectorName()
{
  return sectorname;
}

bool Mission::enemiesAtCurrentNav()
{
  return numenemyshipsdestroyed != numenemyships;
}

ship * * Mission::getFriendlyFighters(int & num)
{
  num = 0;

  int count = 0;
  int i;

  for(i = 0; i < shipsv.size() ; i++)
  {
       if(shipsv[i]->getIffCode() == FRIEND && shipsv[i] != player && (! shipsv[i]->instanceof(INST_CAPITALSHIP) ) )
       {
          count++;
       }
  }
  num = count;
  ship * * returnships = new ship *[count];
  count = 0;
  for(i = 0; i < shipsv.size(); i++)
  {
       if(
          (shipsv[i]->getIffCode() == FRIEND) && 
          (shipsv[i] != player) && 
          (! shipsv[i]->instanceof("CapitalShip") ) 
         )
       {
          returnships[count] = shipsv[i];
          count++;
       }
  }
  return returnships;
}

ship * Mission::getLandingShip()
{
  return landingship;
}

void Mission::incrementPlayerKills()
{
  num_player_kills++;
}

void Mission::incrementPlayerGunHits()
{
  num_player_gun_hits++;
}

void Mission::incrementPlayerMissileHits()
{
  num_player_missile_hits++;
}

void Mission::incrementPlayerMissileFired()
{
  num_player_missile_fired++;
}

void Mission::incrementPlayerGunFired()
{
  num_player_gun_fired++;
}

DebriefingInfo Mission::getDebriefingInfo()
{
  DebriefingInfo info(num_player_gun_fired,  num_player_missile_fired, 
    num_player_gun_hits, num_player_missile_hits, num_player_kills, 
    friendly_ships_lost);

  return info;
}

bool Mission::hasFriendsAtCurrentNav()
{
  int count = 0;
  int i;

  for(i = 0; i < shipsv.size(); i++)
  {
       if(shipsv[i]->getIffCode() == FRIEND && shipsv[i] != player ) 
       {
          return true;
       }
  }
  return false;
}

bool Mission::canEjectToNextMission()
{
  return can_eject;
}
