// MissionXmlAdapter.cpp: implementation of the MissionXmlAdapter class.
//
//////////////////////////////////////////////////////////////////////

#include "MissionXmlAdapter.h"
#include "AttributeList.h"
#include "saxexpat.h"
#include "InputSource.h"
#include <ctype.h>
#include "IffCode.h"
#include "ErrorHandlerE.h"
#include "OutOfBoundsException.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MissionXmlAdapter::MissionXmlAdapter(char filename[])
{
   currentnavpoint = 0;
   currentship =0;
   briefing = false;
   briefingtextcount=0;
   landshipassigned = false;
   asteroidsvalid = false;
   can_eject = false;

	expat myParser;
   ErrorHandlerE::checkFile(filename);
 	myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(*(new SAX::InputSource(filename)));
}

MissionXmlAdapter::~MissionXmlAdapter()
{

}

void MissionXmlAdapter::startDocument()
{

}

void MissionXmlAdapter::endDocument()
{

}

void MissionXmlAdapter::startElement(const std::string& name, const SAX::AttributeList* atts)
{
   int i;

   if(name == "point")
   {

      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "x")
        {
            currentpoint.x =  atof( currentvalue.c_str() );        
        }
        else if(currentname == "y")
        {
            currentpoint.y =  atof( currentvalue.c_str() );        
        }
        else if(currentname == "z")
        {
            currentpoint.z =  atof( currentvalue.c_str() );        
        }
      }
   }
   else if(name == "mission")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "name")
        {
          missionname = currentvalue;   
        }
        else if(currentname == "sector")
        {
         sectorname = currentvalue;
        }
		else if(currentname == "can_eject") 
		{
		  can_eject = (currentvalue == "true");
		}
      }
   }
   else if(name == "navpoint")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "numships")
        {
           currentship = 0;
           numships = atoi( currentvalue.c_str() );        
           shipinfos = new ShipInfo[numships];
        }
      }
   }
   else if(name == "ship")
   {
      lastautopilotnav = -1;
      if(currentship >= numships)
	  {
        throw OutOfBoundsException("*.mission.xml (ship elements)");

	  }

      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "pilot")
        {
           shipinfos[currentship].setPilot(currentvalue.c_str() );
        }
        else if(currentname == "comm")
        {
           shipinfos[currentship].setComm(currentvalue.c_str() );
        }
        else if(currentname == "video")
        {
           shipinfos[currentship].setVideo(currentvalue.c_str() );
        }
        else if(currentname == "autopilot")
        {
          if(currentvalue == "true")
          {
            shipinfos[currentship].setAutoPilot(true);
          }
          else
          {
            shipinfos[currentship].setAutoPilot(false);
          }
        }
		else if(currentname == "last_autopilot_nav")
		{
          lastautopilotnav = atoi( currentvalue.c_str() );  
		  shipinfos[currentship].setLastAutopilotNav(lastautopilotnav);
		}
        else if(currentname == "name")
        {
          shipinfos[currentship].setFileName(currentvalue.c_str() );
        }
        else if(currentname == "alignment")
        {
          if(currentvalue == "friend")
          {
             shipinfos[currentship].setIffCode(FRIEND);
          }
          else if(currentvalue == "neutral")
          {
             shipinfos[currentship].setIffCode(NEUTRAL);
          }
          else
          {
             shipinfos[currentship].setIffCode(ENEMY);
          }
        }
        else if(currentname == "land")
        {
          if(currentvalue == "true")
          {
            shipinfos[currentship].setLand(true);
			if(landshipassigned)
			{
				// can't assign twice
              throw OutOfBoundsException("too many landing ships assigned in mission");
			}
			landshipassigned = true;
          }
          else
          {
            shipinfos[currentship].setLand(false);
          }
        }
        else if(currentname == "must_survive")
        {
          if(currentvalue == "true")
          {
            shipinfos[currentship].setMustSurvive(true);
          }// defaults to false;
        }
      }
   }
   else if(name == "navpoints")
   {
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);
        if(currentname == "number")
        {
          numnavpoints = atoi( currentvalue.c_str() );  
          navpoints = new NavPoint *[numnavpoints];
          currentnavpoint = 0;      
        }
      }
   }
   else if(name == "asteroids")
   {
	  asteroidsvalid = true;
      for(i=0; i < atts->getLength(); i++)
      {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);
        if(currentname == "number")
        {
           asteroidinfo.number = atoi( currentvalue.c_str() );  
        }
        else if(currentname == "radius")
        {
           asteroidinfo.radius = atof( currentvalue.c_str() );  
        }

      }
   }
   else if(name == "briefing")
   {
     briefing = true;

   }
}

void MissionXmlAdapter::endElement(const std::string& name)
{
   if(name == "location")
   {
     currentlocation = currentpoint;
   }
   else if(name == "ship")
   {
     shipinfos[currentship].setStartingLocation(currentlocation);
     currentship++;
   }
   else if(name == "navpoint")
   {
	  if(asteroidsvalid)
	  {
        navpoints[currentnavpoint] = new NavPoint(numships, shipinfos, currentlocation, asteroidinfo);
	  }
	  else
	  {
        navpoints[currentnavpoint] = new NavPoint(numships, shipinfos, currentlocation);
	  }
      currentnavpoint++;
	  asteroidsvalid = false;
   }
   else if(name == "asteroids")
   {
	   asteroidinfo.center = currentpoint;
   }
   else if(name == "briefing")
   {
     briefing = false;

   }
   else if(name == "navpoints")
   {
      if(currentnavpoint != numnavpoints)
	  {
        throw OutOfBoundsException("*.mission.xml (nav elements)");
	  }
   }
}

void MissionXmlAdapter::characters(const std::string& ch)
{
  std::string newstring;
// 18 is the number of briefing text lines
  if(briefing && briefingtextcount < 18)
  {
    briefingtext[briefingtextcount] = ch;
    briefingtextcount++;
  }
}

string MissionXmlAdapter::getName()
{
  return missionname;
}

NavPoint * * MissionXmlAdapter::getNavPoints(int & newnumnavpoints)
{
  newnumnavpoints = numnavpoints;
  return navpoints;
}

string MissionXmlAdapter::getSectorName()
{
  return sectorname;
}

int MissionXmlAdapter::getNumBriefingLines()
{
  return briefingtextcount;
}

string * MissionXmlAdapter::getBriefingText()
{
   return briefingtext;
}

bool MissionXmlAdapter::getCanEject()
{
  return can_eject;
}
