// MusicManager.cpp: implementation of the MusicManager class.
//
//////////////////////////////////////////////////////////////////////

#include "MusicManager.h"
#include "MusicFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MusicManager * MusicManager::manager = NULL;

MusicManager::MusicManager()
{
  currentmusic = NULL;
  musicfinished = false;
  no_enemies = NULL;
  mission_complete = NULL;
  landing_music = NULL;
  kill_enemy = NULL;
  combat1 = NULL;
  combat2 = NULL;
  kill_friend = NULL;
  briefingmusic = NULL;
}

MusicManager::~MusicManager()
{
  if(currentmusic != NULL)
    currentmusic->stop();
}

void MusicManager::onPlayerDeath()
{
  stopCurrent();
  currentmusic = MusicFactory::getMusicFactory()->getMusic("wc2-death-funeral.mp3", false);
  dead = currentmusic;
  currentmusic->play();

}

void MusicManager::onPlayerEject()
{
  stopCurrent();
  currentmusic = MusicFactory::getMusicFactory()->getMusic("wc2-eject.mp3", false);
  eject = currentmusic;
  currentmusic->play();

}


void MusicManager::onKillEnemy()
{
  if(currentmusic != NULL && (currentmusic == dead || currentmusic == eject ) )
     return; 

  stopCurrent();
  if(kill_enemy == NULL)
    kill_enemy = MusicFactory::getMusicFactory()->getMusic("wc2-inflight-success.mp3", false);
  kill_enemy->play();
  currentmusic = kill_enemy;

}

void MusicManager::takeoff()
{
  stopCurrent();
  currentmusic = takeoffmusic = MusicFactory::getMusicFactory()->getMusic("wc2-takeoff.mp3", false);
  currentmusic->play();
  musicfinished = false;
}

void MusicManager::landing()
{
  stopCurrent();
  if(landing_music == NULL)
    landing_music = MusicFactory::getMusicFactory()->getMusic("wc2land.mp3", false);
  currentmusic = landing_music;
  currentmusic->play();
}

void MusicManager::setMission(Mission * newmission)
{
  mission = newmission;
}

void MusicManager::stopCurrent()
{
  if((currentmusic != NULL) && currentmusic->isPlaying() )
    currentmusic->stop();
}

void MusicManager::onMusicFinish()
{
  musicfinished = true;
}

MusicManager * MusicManager::getInstance()
{
  if(manager == NULL)
    manager = new MusicManager();
  return  manager;
}

void MusicManager::clear()
{ 
  MusicManager * that = MusicManager::getInstance(); 
  if(that->currentmusic != NULL)
    that->currentmusic->stop();
  that->musicfinished = false;
}

void MusicManager::advance()
{
  if(musicfinished)
  {
     if(currentmusic == takeoffmusic)
       delete takeoffmusic;
     if(currentmusic != NULL && (currentmusic == landing_music || currentmusic == dead || currentmusic == briefingmusic || currentmusic == eject) )
        return; 
     if(mission->isComplete() )
     {
#ifndef HAS_FMOD
        if(mission_complete == NULL)
#endif
          mission_complete = MusicFactory::getMusicFactory()->getMusic("goodlanding.mp3", false);
        currentmusic = mission_complete;
        currentmusic->play();  
     }
     else if(mission->enemiesAtCurrentNav() )
     {
        if(currentmusic == combat1)
        {
          if(combat2 == NULL)
            combat2 = MusicFactory::getMusicFactory()->getMusic("combat2.mp3", false);
          currentmusic = combat2;
          currentmusic->play();  
        }
        else 
        {       
          if(combat1 == NULL)
            combat1 = MusicFactory::getMusicFactory()->getMusic("combat1.mp3", false);
          currentmusic = combat1;
          currentmusic->play();  
        }
     }
     else
     {
#ifndef HAS_FMOD
       /* a bad hack, fmod was crashing if not realloced each time */
       if(no_enemies ==NULL)
#endif
       {
         no_enemies = MusicFactory::getMusicFactory()->getMusic("noenemies.mp3", false);
       }
       currentmusic = no_enemies;
       no_enemies->play();
     }
     musicfinished = false;
  }
}

void MusicManager::onNewNavPoint()
{
     if(mission->isComplete() )
     {
        currentmusic->stop();
        if(mission_complete == NULL)
          mission_complete = MusicFactory::getMusicFactory()->getMusic("goodlanding.mp3", true);
        currentmusic = mission_complete;
        currentmusic->play();  
     }
     else if(mission->enemiesAtCurrentNav() )
     {
        currentmusic->stop();
        if(combat1 == NULL)
          combat1 = MusicFactory::getMusicFactory()->getMusic("combat1.mp3", false);
        currentmusic = combat1;
        currentmusic->play();  
     }
}

void MusicManager::onKillFriend()
{
  if(currentmusic != NULL && (currentmusic == dead || currentmusic == eject) )
     return; 

  stopCurrent();
  if(kill_friend == NULL)
    kill_friend = MusicFactory::getMusicFactory()->getMusic("wc2-wingmandied.mp3", false);
  kill_friend->play();
  currentmusic = kill_friend;

}

bool MusicManager::isPlaying()
{
  if(currentmusic == NULL)
	 return false;
  return currentmusic->isPlaying();
}

void MusicManager::briefing()
{
  stopCurrent();
  if(briefingmusic == NULL)
    briefingmusic = MusicFactory::getMusicFactory()->getMusic("briefing2.mp3", true);
  currentmusic = briefingmusic;
  currentmusic->play();
}
