// NavGrid.cpp: implementation of the NavGrid class.
//
//////////////////////////////////////////////////////////////////////
#include "GL/glew.h"
#include "glut.h"
#include "NavGrid.h"
#include "Circle.h"
#include <string.h>
#include <math.h>
#include "mesh.h"
#include "Camera.h"
#include "ChaseCamera.h"
#include "AutoPilotCamera.h"
#include <iostream.h>
#include "FixedCamera.h"
#include <string>
#include "BriefingShip.h"
#include "BriefingXmlAdapter.h"
#include "TextOverlay.h"
#include "glerr.h"

using std::string;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

NavGrid::NavGrid(char filename[])
{
  BriefingXmlAdapter adapter(filename);
  player = adapter.getPlayer();
  navs = adapter.getNavPoints();
  wingmen = adapter.getAutopilotShips();
  

  player->setLocation(navs->at(0)->getLocation() );

  currentfrom = navs->size() - 1;
  if(currentfrom == -1)
  {
    currentfrom = 0;
  }
  start = 0;
  s = 0.0f;
  rotatemode = true;

	FixedCamera * fixedcamera = (FixedCamera *) FixedCamera::getInstance();

    fixedcamera->setLocation(player->getcenter() );
	fixedcamera->setUp(Coordinate(0.0f, 1.0f, 0.0f));
	fixedcamera->setLookAt(player->getLocation() );
	fixedcamera->setFocusObject(player);
	Camera::setCamera(fixedcamera);
   
	glNewList(mydisplaylist = glGenLists(1), GL_COMPILE);
        renderGrid();
	glEndList();
	GLERRORCHECK

}

NavGrid::~NavGrid()
{
   glDeleteLists(mydisplaylist, 1); 
   GLERRORCHECK 
}

void NavGrid::render()
{
  
	Coordinate yoffset;
	int i;
	int mynavcount;


  glDisable(GL_TEXTURE_2D);
  glEnable(GL_COLOR_MATERIAL);
  glDisable(GL_LIGHTING);
GLERRORCHECK
//  renderGrid();
  glCallList(mydisplaylist);
GLERRORCHECK
  float newy, nextnewy;
  // current heading
  Coordinate currenta = navs->at(currentfrom)->getLocation();
  Coordinate currentb =navs->at(
	   (currentfrom + 1) % navs->size() )->getLocation() ;
  Coordinate newheading = currentb - currenta;
  newy = newheading.y;
  newheading.y = 0.0f;
  newheading.normalize();
  Coordinate desiredup(0.0f, 1.0f, 0.0f);
  Coordinate side = newheading.cross(desiredup);
  Coordinate trueup = newheading.cross(side);

  // next heading
  Coordinate nexta = navs->at(((currentfrom + 1) % navs->size()))->getLocation();
  Coordinate nextb =navs->at(((currentfrom + 2) % navs->size()))->getLocation() ;
  Coordinate nextnewheading = nextb - nexta;
  nextnewy = nextnewheading.y;
  nextnewheading.y = 0.0f;
  nextnewheading.normalize();
  Coordinate nextside = nextnewheading.cross(desiredup);
  Coordinate nexttrueup = nextnewheading.cross(side);


  if(newy > 0.0f)
  {
     yoffset = Coordinate(0.0f, -20.0f, 0.0f);
  }
  else
  {
     yoffset = Coordinate(0.0f, 20.0f, 0.0f);
  }

  //
  if(rotatemode)
  {

      // interpolate y

 	if(nextnewy > 0 && newy < 0)
	{
      yoffset = Coordinate(0.0f, 20.0f, 0.0f).interpolate(
		  Coordinate(0.0f, -20.0f, 0.0f), s );
	}
	else if(nextnewy < 0 && newy > 0   )
	{
      yoffset = Coordinate(0.0f, -20.0f, 0.0f).interpolate(
		  Coordinate(0.0f, 20.0f, 0.0f), s );

	}
	else
	{


	}

	Coordinate iheading;
	Coordinate iup(0.0f, 1.0f, 0.0f);
	Coordinate iside;
      if(((newheading + nextnewheading) == Coordinate(0.0f, 0.0f, 0.0f))  )
	  {
		  // singularity
		  float theta = s * 3.14159;
		  iheading = newheading.rotatearoundaxis(
			  Coordinate(0.0f, 1.0f, 0.0f), theta);
		  iside = side.rotatearoundaxis(
			  Coordinate(0.0f, 1.0f, 0.0f), theta);
	  }
	  else
	  {

	    iheading = newheading.interpolate(nextnewheading, s);
	    
	    iside = side.interpolate(nextside, s);
	  }

	  iheading.normalize();
	  iside.normalize();

	  player->setOrientation(iheading, iup, iside);
	  for(i=0; i < wingmen->size(); i++)
	  {
     	int mylastauto = wingmen->at(i)->getLastAutopilotNav() ;
        if((currentfrom + 1 < mylastauto) || (mylastauto == -1) )  
		{
	      wingmen->at(i)->setOrientation(iheading, iup, iside);
		}
	  }

  }
  else
  {
	  Coordinate location3 =  currenta.interpolate(currentb, s);
	  player->setLocation(location3);
      player->setOrientation(newheading, desiredup, side);	
	  for(i=0; i < wingmen->size(); i++)
	  {
     	int mylastauto = wingmen->at(i)->getLastAutopilotNav() ;
        if((currentfrom < mylastauto) || (mylastauto == -1) )  
        {
	      wingmen->at(i)->setOrientation(newheading, desiredup, side);
		}
	  }

  }

  for(i=0; i < wingmen->size(); i++)
  {
	int mylastauto = wingmen->at(i)->getLastAutopilotNav() ;
    if((currentfrom < mylastauto) || (mylastauto == -1) )  
	{
      wingmen->at(i)->setPositionRelative(player);
	}
  }// stop autopilot

	FixedCamera * fixedcamera = (FixedCamera *) FixedCamera::getInstance();

    fixedcamera->setLocation(player->getLocation() + 
		player->getHeading() * -50.0f +
		yoffset + 
		player->getleft() * -30.0f );
    
	fixedcamera->setUp(Coordinate(0.0f, 1.0f, 0.0f));
	fixedcamera->setLookAt(player->getLocation() );
	Camera::setCamera(fixedcamera);


    glLineWidth(1.0f);
	GLERRORCHECK
	player->render();
	for(i=0; i < wingmen->size(); i++)
	{
      wingmen->at(i)->render();
	}
	for(i=0; i < navs->size(); i++)
	{
      navs->at(i)->render();
	}
	GLERRORCHECK
}


bool NavGrid::instanceof(char classname [ ])
{
  if(0 == strcmp(classname, "Explosion")) 
    return true;
  return Object3d::instanceof(classname);
}	

void NavGrid::renderGrid()
{
  Color lightgreen(0.0f, .5f, 0.0f);
  lightgreen.render();
  int numlines = 10;
  float halflinelength = 2000.0f ;
  float lineseparation = 200.0f;


  float y = 0.0f;
  int i;

  glLineWidth(2.0f);
  for(i=-numlines; i <= numlines; i++)
  {
	 if(i != 0 )
	 {
        glBegin(GL_LINES);
          glVertex3f(-halflinelength, y, (float) (i * (lineseparation) ));
          glVertex3f(halflinelength, y, (float)(i * (lineseparation)  ));
        glEnd();
        glBegin(GL_LINES);
          glVertex3f( (float) (i * (lineseparation)), y, -halflinelength );
          glVertex3f( (float) (i * (lineseparation)), y,  halflinelength);
		glEnd();
	 }
   }

  
  /* axes */
  Color red(1.0f, 0.0f, 0.0f);
  red.render();

  glBegin(GL_LINES);
    glVertex3f(-halflinelength, 0.0f, 0.0f );
    glVertex3f(halflinelength, 0.0f, 0.0f );
    glVertex3f(0.0f, 0.0f, -halflinelength);
    glVertex3f(0.0f, 0.0f, halflinelength );
  glEnd();

  Color color2(1.0f, 1.0f, 0.0f);
  color2.render();
  glBegin(GL_LINES);
     glVertex3f(0.0f, -halflinelength, 0.0f );
     glVertex3f(0.0f, halflinelength, 0.0f );
  glEnd();
GLERRORCHECK

}

void NavGrid::advance()
{
  BriefingNavPoint * briefnav;

  if(start == 0)
  {
	  start = Time::getCurrentTimeImmediately();
  }
  unsigned int now = Time::getCurrentTimeImmediately();
  s = (double)(now - start)/ 10000.0;

  if(s == 0.0f)
  {
	if(rotatemode)
	{

/*		 char buffer[100];
		 sprintf(buffer, "You are currently at nav point %d.", ((currentfrom + 1) % navs.size()) + 1 );
*/
//         TextToSpeech::getInstance()->synthesizeAndPlay((char *) navs[(currentfrom + 1) % navs.size()]->getAtText().c_str() );
		briefnav = navs->at((currentfrom + 1) % navs->size());
		briefnav->getAtSpeech()->play();
        TextOverlay::getInstance()->setText(briefnav->getAtText() );
	}
	else
	{
/*		 char buffer2[100];
		 sprintf(buffer2, "You are traveling from  nav point %d to nav point %d.", 
			 ((currentfrom) % navs.size()) + 1, ((currentfrom + 1) % navs.size()) + 1);
*/
//         TextToSpeech::getInstance()->synthesizeAndPlay(buffer2 );       
//         TextToSpeech::getInstance()->synthesizeAndPlay((char *) navs[(currentfrom ) % navs.size()]->getLeavingText().c_str()  );
		briefnav = navs->at((currentfrom) % navs->size());
		briefnav->getLeavingSpeech()->play();
		TextOverlay::getInstance()->setText(briefnav->getLeavingText() );

	}

  }

  if(s > 1.0f)
  {
     if(rotatemode)
	 {
	   rotatemode = false;
	   int oldfrom = currentfrom;
	   currentfrom = ((currentfrom + 1) % navs->size() );
	 }
	 else
	 {
		 rotatemode = true;
	 }

     start = 0;
	 s = 0.0f;	 
  }
}

bool NavGrid::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_EXPLOSION);
}
