// Object3d.cpp: implementation of the Object3d class.
//
//////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include "Object3d.h"
#include <glut.h>
#include <math.h>
#include <stdio.h>
#include <iostream.h>
#include "coord.h"
#include <string.h>
#include "Time.h"
#include "glerr.h"
#include "Math.h"
#include "LightManager.h"
#include "instanceofobj.h"
#include "Math.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

void Object3d::resetheadings()
{
  heading.x = 0.0;
  heading.y = 0.0;
  heading.z = 1.0;
  up.x = 0.0;
  up.y = 1.0;
  up.z = 0.0;
  left.x = 1.0;
  left.y = 0.0;
  left.z = 0.0;

}


// allows several objects to share the same mesh

Object3d::Object3d(mesh * newmesh, float velocity, float newroll, float newpitch, float newyaw, float new_acceleration, float new_angular_acceleration)

{
  collision_model = newmesh->getCollisionModel();
  trackdistance =false;
  destroyed = false;
  mymesh = newmesh;
  location.x = 0.0f;
  location.y = 0.0f;
  location.z = 0.0f;
  resetheadings();

  init();
  currentvelocity = maxvelocity = velocity;
  velocityincrement = maxvelocity / 6.0f;
  roll = newroll;
  pitch = newpitch;
  yaw = newyaw;
  acceleration = new_acceleration;
  angular_acceleration = new_angular_acceleration;
  desired_velocity = maxvelocity;

}

void Object3d::init()
{
  renderme = true;
  location.x = 0.0f;
  location.y = 0.0f;
  location.z = 0.0f;

  resetheadings();
  scalenabled = false;
  if(mymesh != NULL)
  {
	radius = mymesh->getradius();
	radiussquared = radius * radius;
  }
  else
  {
	radius = 999999.9;
  }
}

Object3d::~Object3d()
{

}



void Object3d::render()
{

// render local lights

	   LightManager * lightmanager = LightManager::getInstance();

       glPushMatrix();
//       glLoadIdentity();
       lightmanager->renderGlobalLights();
	   lightmanager->renderLocalLights(getLocation() );
	   glPopMatrix();
// end lighting stuff 
	   

       glPushMatrix();
       glTranslatef(location.x, location.y, location.z);

       matchDOF(heading, up);    
	   if(scalenabled)
	   {
         glScalef(scalefactor.x, scalefactor.y, scalefactor.z);
	   }

       mymesh->render();
       glPopMatrix();

    glColor3f(1.0f, 1.0f, 1.0f);
	GLERRORCHECK
}


Coordinate Object3d::getLocation()
{
  return location;
}

void Object3d::setLocation(Coordinate newlocation)
{
  location = newlocation;
}

void Object3d::advance()
{
  float difference = fabs(currentvelocity - desired_velocity);
  float myaccel = acceleration * Time::getTimeScale();
  
  if(difference == 0.0f)
  {

  }
  else if(currentvelocity > desired_velocity)
  {
    currentvelocity -= MIN(myaccel, difference);
	currentvelocity = MAX(currentvelocity, 0.0f);
  }
  else
  {
    currentvelocity += MIN(myaccel, difference);
	currentvelocity = MIN(currentvelocity, maxvelocity);
  }


  float scaledvelocity = currentvelocity * Time::getTimeScale();


  location.x += heading.x * scaledvelocity;
  location.y += heading.y * scaledvelocity;
  location.z += heading.z * scaledvelocity;
  if(trackdistance)
  {
    distancetraveled += scaledvelocity;
    if(distancetraveled > maxdistance)
      setDestroyed();     
  }
}

/*void Object3d::setRoll(float newroll)
{
  roll = newroll;
}

void Object3d::setPitch(float newpitch)
{
  pitch = newpitch;
}

void Object3d::setYaw(float newyaw)
{
  yaw = newyaw;
}

*/
void Object3d::rotateleft()
{
  float scaledyaw = yaw * Time::getTimeScale();


  heading = heading.rotatearoundaxis(up, Math::degtorad(scaledyaw) );
  heading.normalize();
  left = left.rotatearoundaxis(up, Math::degtorad(scaledyaw) );
  left.normalize();

}

void Object3d::rotateright()
{
  float scaledyaw = yaw * Time::getTimeScale();


  heading = heading.rotatearoundaxis(up, Math::degtorad(-scaledyaw) );
  heading.normalize();
  left = left.rotatearoundaxis(up, Math::degtorad(-scaledyaw) );
  left.normalize();
}

void Object3d::rotateup()
{
  float scaledpitch = pitch * Time::getTimeScale();


  heading = heading.rotatearoundaxis( left, Math::degtorad(scaledpitch) );
  heading.normalize();
  up = up.rotatearoundaxis(left, Math::degtorad(scaledpitch) );
  up.normalize();
}

void Object3d::rotatedown()
{
  Coordinate oldup;
 // float angle;

  oldup = up;

  float scaledpitch = pitch * Time::getTimeScale();


  heading = heading.rotatearoundaxis( left, Math::degtorad(-scaledpitch) );
  heading.normalize();
  up = up.rotatearoundaxis(left, Math::degtorad(-scaledpitch) );
  up.normalize();
}

void Object3d::bankleft()
{
  float scaledroll = roll * Time::getTimeScale();


  left = left.rotatearoundaxis( heading, Math::degtorad(scaledroll) );
  left.normalize();
  up = up.rotatearoundaxis(heading, Math::degtorad(scaledroll) );
  up.normalize();
}

void Object3d::bankright()
{
  float scaledroll = roll * Time::getTimeScale();


  left = left.rotatearoundaxis( heading, Math::degtorad(-scaledroll) );
  left.normalize();
  up = up.rotatearoundaxis(heading, Math::degtorad(-scaledroll) );
  up.normalize();
}

Object3d::Object3d()
{
  destroyed = false;
  trackdistance =false;
  mymesh = NULL;
  init();
}

Coordinate Object3d::getHeading()
{
  return heading;
}


void Object3d::matchDOF(Coordinate heading, Coordinate up)
{
  GLfloat   m[16];

  matchDOF(heading, up, m);
  glMultMatrixf (m);
}

float Object3d::getradius()
{
  return radius;
}

Coordinate Object3d::getcenter()
{
 // return mymesh->getcenter() + location;
   GLfloat matrix[16];

   Coordinate centerlocation = mymesh->getcenter();
   Coordinate worldpoint;

   matchDOF(getHeading(), getUp(), matrix);
   worldpoint = centerlocation.multiplybyglmatrix(matrix);
   worldpoint = worldpoint  + getLocation();
   return worldpoint;
}

Coordinate Object3d::getUp()
{
  return up;
}

Coordinate Object3d::getleft()
{
  return left;
}

Object3d::Object3d(mesh * newmesh, Coordinate position, Coordinate newheading, Coordinate newup, Coordinate newleft, float newmaxvelocity, float newroll, float newpitch, float newyaw, float new_acceleration, float new_angular_acceleration)
{
   trackdistance =false;
   destroyed = false;
   mymesh = newmesh;
   init();
   roll = newroll;
   pitch = newpitch;
   yaw = newyaw;
   velocityincrement = newmaxvelocity / 6.0f;
   currentvelocity = maxvelocity = newmaxvelocity;
   location = position;
   heading = newheading;
   up = newup;
   left = newleft;
   collision_model = newmesh->getCollisionModel();
   acceleration = new_acceleration;
   angular_acceleration = new_angular_acceleration;

}


void Object3d::matchDOF(Coordinate heading, Coordinate up, GLfloat m [ 16 ])
{
  Coordinate    x;
  Coordinate    y;
  Coordinate    z;

  z = heading;
  z.normalize();
  y = up;
  x = y.cross(z);  
  y = z.cross(x);
  x.normalize();
  y.normalize();
#define M(col,row)  m[col*4+row]
  M(0,0) = x.x;  M(0,1) = x.y;  M(0,2) = x.z;  M(0,3) = 0.0;
  M(1,0) = y.x;  M(1,1) = y.y;  M(1,2) = y.z;  M(1,3) = 0.0;
  M(2,0) = z.x;  M(2,1) = z.y;  M(2,2) = z.z;  M(2,3) = 0.0;
  M(3,0) = 0.0;   M(3,1) = 0.0;   M(3,2) = 0.0;   M(3,3) = 1.0;
#undef M
}

bool Object3d::isdestroyed()
{
  return destroyed;
}

void Object3d::setrenderingenabled(bool enable)
{
  renderme = enable;
}

void Object3d::setCurrentVelocity(float velocity)
{
  if(velocity > maxvelocity)
     currentvelocity = maxvelocity;
  else if(velocity < 0.0f)
     currentvelocity = 0.0f;
  else
     currentvelocity = velocity;
}


void Object3d::incrementVelocity()
{
   if(desired_velocity + velocityincrement > maxvelocity)
     desired_velocity = maxvelocity;
   else
     desired_velocity += velocityincrement;
}

void Object3d::decrementVelocity()
{
   if(desired_velocity - velocityincrement < 0.0)
     desired_velocity = 0.0f;
   else
     desired_velocity -= velocityincrement;

}

float Object3d::getVelocity()
{
  return currentvelocity;
}

Object3d::Object3d(Object3d & clone)
{
  collision_model = clone.collision_model;
  trackdistance =false;
  destroyed = false;
  mymesh = clone.mymesh;
  resetheadings();
  init();
  velocityincrement = clone.velocityincrement;
  currentvelocity = maxvelocity = clone.maxvelocity;
  roll = clone.roll;
  pitch = clone.pitch;
  yaw = clone.yaw;
  location = clone.location;
  heading = clone.heading;
  up = clone.up;
  left = clone.left;
  acceleration = clone.acceleration;
  angular_acceleration = clone.angular_acceleration;


}

void Object3d::onCollisionWith(Object3d * otherobject)
{

}


void Object3d::damage(int amount, Coordinate fromlocation)
{

}

bool Object3d::cantHit(Object3d * other)
{
  return false;
}

Coordinate Object3d::getExtents()
{
   return mymesh->getExtents();
}

/*bool Object3d::isShip()
{
  return false;
}
*/

void Object3d::setDestroyed()
{
  destroyed = true;
}

void Object3d::setMaxDistance(float newmaxdistance)
{
  maxdistance = newmaxdistance;
  trackdistance = true;
  distancetraveled = 0.0f;
}

Object3d::Object3d(float newroll, float newpitch, float newyaw)
{
  trackdistance =false;
  destroyed = false;
  mymesh = NULL;
  location.x = 0.0f;
  location.y = 0.0f;
  location.z = 0.0f;
  resetheadings();

//  ntriangles = numtriangle;
  init();
  currentvelocity = 0.0f;
  maxvelocity = 5.0f;
  roll = newroll;
  pitch = newpitch;
  yaw = newyaw;
  acceleration = 0.0f;
  angular_acceleration = 0.0f;

   
//  collisionObj =  bounds;
}

bool Object3d::instanceof(char classname [ ])
{
  return  strcmp(classname, "Object3d") == 0;
}

float Object3d::getMaxVelocity()
{
  return maxvelocity;
}

void Object3d::rotateUpPercent(float percentage)
{
  float scaledpitch = pitch * Time::getTimeScale();


  float mypitch = scaledpitch * percentage;

  heading = heading.rotatearoundaxis( left, Math::degtorad(mypitch) );
  heading.normalize();
  up = up.rotatearoundaxis(left, Math::degtorad(mypitch) );
  up.normalize();

}

void Object3d::rotateDownPercent(float percent)
{
  float scaledpitch = pitch * Time::getTimeScale();


  float mypitch = scaledpitch * percent;
  Coordinate oldup;
 // float angle;

  oldup = up;

  heading = heading.rotatearoundaxis( left, Math::degtorad(-mypitch) );
  heading.normalize();
  up = up.rotatearoundaxis(left, Math::degtorad(-mypitch) );
  up.normalize();
}

void Object3d::rotateLeftPercent(float percent)
{
  float scaledyaw = yaw * Time::getTimeScale();


  float myyaw = scaledyaw * percent;

  heading = heading.rotatearoundaxis(up, Math::degtorad(myyaw) );
  heading.normalize();
  left = left.rotatearoundaxis(up, Math::degtorad(myyaw) );
  left.normalize();

}

void Object3d::rotateRightPercent(float percent)
{
  float scaledyaw = yaw * Time::getTimeScale();


  float myyaw = scaledyaw * percent;

  heading = heading.rotatearoundaxis(up, Math::degtorad(-myyaw) );
  heading.normalize();
  left = left.rotatearoundaxis(up, Math::degtorad(-myyaw) );
  left.normalize();

}

bool Object3d::shouldRender()
{
  return renderme;
}

void Object3d::setOrientation(Coordinate newheading, Coordinate newup, Coordinate newleft)
{
  heading = newheading;
  up = newup;
  left = newleft;
}


Coordinate Object3d::getMeshCenter()
{
  return mymesh->getcenter();
}

void Object3d::setMaxVelocity()
{
  desired_velocity = maxvelocity;
}

CollisionModel3D * Object3d::getCollisionModel()
{
  return collision_model;
}

void Object3d::getMatrix(float matrix [ 16 ])
{
  matchDOF(heading, up, matrix);
  matrix[12] = location.x; 
  matrix[13] = location.y;
  matrix[14] = location.z;
}

void Object3d::setVelocity(float newvelocity)
{
   currentvelocity = newvelocity;
}

void Object3d::bankLeftPercent(float percent)
{
  float scaledroll = roll * Time::getTimeScale() * percent;


  left = left.rotatearoundaxis( heading, Math::degtorad(scaledroll) );
  left.normalize();
  up = up.rotatearoundaxis(heading, Math::degtorad(scaledroll) );
  up.normalize();
}

void Object3d::bankRightPercent(float percent)
{
  float scaledroll = roll * Time::getTimeScale() * percent;


  left = left.rotatearoundaxis( heading, Math::degtorad(-scaledroll) );
  left.normalize();
  up = up.rotatearoundaxis(heading, Math::degtorad(-scaledroll) );
  up.normalize();
}

void Object3d::setDesiredVelocity(float percentage)
{
  desired_velocity = maxvelocity * percentage;
}

bool Object3d::isPointBehind(Coordinate & point)
{
   Coordinate vector = location - point;
   return heading.dot(vector) < 0.0f;
}

void Object3d::setAcceleration(float newacceleration)
{
  acceleration = newacceleration;
}

float Object3d::getDesiredVelocity()
{
  return desired_velocity;
}

void Object3d::setMaxVelocity(float value)
{
  maxvelocity = value;
}

Coordinate Object3d::getMeshMaxs()
{
  return mymesh->getMaxs(); 
}

Coordinate Object3d::getMeshMins()
{
  return mymesh->getMins();
}

Object3d::Object3d(mesh * newmesh)
{
  collision_model = newmesh->getCollisionModel();
  trackdistance =false;
  destroyed = false;
  mymesh = newmesh;
  location.x = 0.0f;
  location.y = 0.0f;
  location.z = 0.0f;
  resetheadings();

  init();
  currentvelocity = maxvelocity = 0.0f;
  velocityincrement = maxvelocity / 6.0f;
  roll = 0.0f;
  pitch = 0.0f;
  yaw = 0.0f;
  acceleration = 0.0f;
  angular_acceleration = 0.0f;
  desired_velocity = 0.0f;

}

void Object3d::setScale(Coordinate newscalefactor)
{
   scalenabled = true;
   scalefactor = newscalefactor;
}


void Object3d::setMeshDefaultColor(Color newcolor)
{
  mymesh->setDefaultColor(newcolor);
}

bool Object3d::instanceof(InstanceT it)
{
  return it & INST_OBJECT;
}

float Object3d::getRadiusSquared()
{
  return radiussquared;
}

float Object3d::getRange()
{
  return maxdistance;
}
