// Object3d.h: interface for the Object3d class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OBJECT3D_H__1115E321_1D07_11D3_922B_58B502C10627__INCLUDED_)
#define AFX_OBJECT3D_H__1115E321_1D07_11D3_922B_58B502C10627__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <GL/glew.h>
#include "coord.h"
#include "mesh.h"	// Added by ClassView
#include "mount.h"
#include <stdlib.h>
#include "quaternion.h"	// Added by ClassView
#include "instanceofobj.h"


class Object3d  
{
public:
	float getRadiusSquared();
	virtual bool instanceof(InstanceT it);
	void setScale(Coordinate newscalefactor);
    Object3d(mesh * newmesh);
	virtual Coordinate getMeshMins();
	virtual Coordinate getMeshMaxs();
	float getDesiredVelocity();
	bool isPointBehind(Coordinate & point);
	void setDesiredVelocity(float percentage);
	void bankRightPercent(float percent);
	void bankLeftPercent(float percent);
   /* for use by cutscenes and initialization only */
	void setVelocity(float newvelocity);
	virtual void getMatrix(float matrix[16]);
	virtual CollisionModel3D * getCollisionModel();
   /* for use by cutscenes and initialization only */	
	void setMaxVelocity();
	void setOrientation(Coordinate heading, Coordinate up, Coordinate left);
	bool shouldRender();
	void rotateRightPercent(float percent);
	void rotateLeftPercent(float percent);
	void rotateDownPercent(float percent);
	void rotateUpPercent(float percentage);
	float getMaxVelocity();
	virtual bool instanceof(char classname[]);
	virtual Coordinate getExtents();
	virtual bool cantHit(Object3d * other);
	virtual void damage(int amount, Coordinate fromlocation);
	virtual void onCollisionWith(Object3d * otherobject);
	 Object3d(Object3d & clone);
   Object3d(mesh * newmesh, Coordinate position, Coordinate heading, Coordinate up, 
	   Coordinate left, float maxvelocity, float roll, float pitch, float yaw, 
	   float acceleration, float angular_acceleration);
   Object3d(mesh * newmesh, float maxvelocity, float roll, float pitch, float yaw,
	   float acceleration, float angular_acceleration);
	float getVelocity();
	void decrementVelocity();
	void incrementVelocity();
	void setCurrentVelocity(float velocity);
	void setrenderingenabled(bool enable);
	virtual bool  isdestroyed();
	Coordinate getleft();
	Coordinate getUp();
	Coordinate getcenter();
	virtual float getradius();
	Coordinate getHeading();
	virtual void bankleft();
	virtual void bankright();
	virtual void rotatedown();
	virtual void rotateup();
	virtual void rotateright();
	virtual void rotateleft();
   // moves the object forward according to its velocity in its current heading direction.
	virtual void advance();
   // forward rate
	virtual void setLocation(Coordinate newlocation);
	virtual ~Object3d();
   virtual void render();
   virtual Coordinate getLocation();
	virtual void setDestroyed();
protected:
	float getRange();
	void setMeshDefaultColor(Color newcolor);
	virtual Coordinate getMeshCenter();
   Object3d(float roll, float pitch, float yaw);
	void setMaxDistance(float maxdistance);
	void matchDOF(Coordinate heading, Coordinate up, GLfloat glmatrix[16]);
 	void matchDOF(Coordinate heading, Coordinate up);
	Object3d();
	void setMaxVelocity(float value);
	void setAcceleration(float newacceleration);
private:
   float angular_acceleration;
   float acceleration;
   float maxdistance;
   float distancetraveled;
   bool trackdistance;
   CollisionModel3D * collision_model;

	Coordinate heading;
   Coordinate up;
   Coordinate left;
   Coordinate location;
	void rotatematrix(float, float, float);
   void resetheadings();
   void init();
   float currentvelocity;
   float maxvelocity;
   float desired_velocity;
   float velocityincrement;
   float roll;
   float pitch;
   float yaw;
	mesh * mymesh;
   bool renderme;
   bool destroyed;
   // scaling
   bool scalenabled;
   Coordinate scalefactor;
   float radiussquared;
   float radius;
};

#endif // !defined(AFX_OBJECT3D_H__1115E321_1D07_11D3_922B_58B502C10627__INCLUDED_)
