// Panel.cpp: implementation of the Panel class.
//
//////////////////////////////////////////////////////////////////////

#include "Panel.h"
#include "Rect.h"
#include <stdlib.h>
#include "Font.h"
#include "ImageFactory.h"
#include "Image.h"
#include "glut.h"
#include "FontFactory.h"
#include "glerr.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Image * Panel::panelimage = NULL;
Font * Panel::font = NULL;
Rect Panel::exitrect(502, 399, 66, 21);
Rect Panel::confirmrect(502, 70, 66, 21);

Panel::Panel()
{
  if(panelimage == NULL)
    panelimage = ImageFactory::getImage("console.jpg", 640, 480);
  numitems = 0;
  font = FontFactory::getInstance().getFont();
  lineincrement = font->getHeight("W") + 4.0f;
  halfincrement = .5f * font->getHeight("W");

}

Panel::~Panel()
{

}


// all setup is performed elsewhere, just draw 2d
void Panel::onDisplay()
{
  int i;

  if(panelimage == NULL)
    panelimage = ImageFactory::getImage("console.jpg", 640, 480);
  if(font == NULL)
  {
      font = FontFactory::getInstance().getFont();
  }

   panelimage->render(0.0f, 0.0f);
   glColor3f(.18f, .71f, .69f);
   font->print((float) 234, (float) 480 - 44, (char *) title.c_str() );

   float rasterx = 75 , rastery=80;
   for(i=0; i < numitems; i++)
   {
      font->print((float)(rasterx), (float) (480 - rastery), (char *) items[i].c_str() );
      rastery += lineincrement;
   }

   float fontwidth = font->getWidth((char *) confirmtext.c_str() );
   float confirmx = (confirmrect.width - 2)* .5f - .5f * fontwidth+ confirmrect.x;
   //font->print( (float) 520, (float) 480 - 87, (char *) confirmtext.c_str() );
   font->print( confirmx, (float) 480 - 87, (char *) confirmtext.c_str() );

   font->print( (float)(rasterx), (float) (480 - 420), (char *) statustext.c_str() );
   GLERRORCHECK
}


/* returns true if the panel is exiting */
bool Panel::onMouseDown(int button, int state, int x, int y)
{
   int i;

   if(state != GLUT_DOWN)
      return false;

   float rasterx = 75 , rastery=80;
   for(i=0; i < numitems; i++)
   {
      if(x > rasterx && x < 380 && y <= rastery + halfincrement && y > rastery - lineincrement- halfincrement) 
      {
         onLineClick(i);
         return false;
      }
      rastery += lineincrement;
   }


  if(exitrect.inside(x, y) )
  {
    onCancel();
    return true;
  }
  else if(confirmrect.inside(x, y) )
  {
     onConfirm();
     return true;
  }
  return false;
}

void Panel::clean()
{
   if(panelimage != NULL)
   {
      delete panelimage;
      panelimage = NULL;
	  font = NULL;
   }
}

void Panel::setTitle(char newtitle [ ])
{
  title = newtitle;
}

void Panel::setNumLines(int numlines)
{
   numitems = numlines;
   items = new string[numitems];
}

void Panel::setLine(string text, int linenumber)
{
  if(linenumber < numitems)
  {
     items[linenumber] = text;
  } 
}

void Panel::setConfirmText(string newconfirmtext)
{
  confirmtext = newconfirmtext;
}

void Panel::onConfirm()
{

}

void Panel::onCancel()
{

}

void Panel::setStatusText(string newtext)
{
  statustext = newtext;
}


void Panel::onLineClick(int linenumber)
{

}

string Panel::getStatusText()
{
  return statustext;
}

bool Panel::onKeyboard(unsigned char key, int x, int y)
{
  return false;

}

string Panel::getLineText(int linenumber)
{
  if(linenumber >= numitems || linenumber < 0)
    return string("");
  return items[linenumber];  
}
