// SavePanel.cpp: implementation of the SavePanel class.
//
//////////////////////////////////////////////////////////////////////

#include "SavePanel.h"
#include "XmlOutput.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SavePanel::SavePanel()
{
  adapter = NULL;
  setTitle("Choose a save game slot");
  setConfirmText("Save");
  update();
}

SavePanel::~SavePanel()
{

}

void SavePanel::update()
{
  int i;

  selected = false;

  if(adapter != NULL)
  {
    delete adapter;
  }
  adapter = new SaveGameXmlAdapter("savegames.xml");

  int max;

  if(adapter->getNumSaves() + 1> 17)
  {
    max = 17;
  }
  else 
  {
    max = adapter->getNumSaves() + 1;
  }

  setNumLines(max);
  for(i=0; i < max - 1; i++)
  {
    setLine(adapter->getNames()[i], i);
  }
  if(max != 17)
    setLine("---", i);
  setTitle("Choose a save game slot");
}

void SavePanel::onConfirm()
{
  if(selected)
     write();
  setStatusText("");
  selected = false;
}

void SavePanel::onLineClick(int linenumber)
{
  int max;

  max = adapter->getNumSaves() + 1;
  if(max > 17)
    max = 17;



  if(linenumber < max )
  {
    setStatusText(getLineText(linenumber) );
  }
  selection = linenumber;
  selected = true;
  setTitle("Choose a save game name or choose another slot");
}

void SavePanel::onCancel()
{
  setStatusText("");
  selected = false;
}


void SavePanel::setCampaign(string name)
{
   campaign = name;
}

void SavePanel::setMission(string name)
{
   mission = name;
}

bool SavePanel::onKeyboard(unsigned char key, int x, int y)
{
  if(selected)
  {
     if(key == 8) // backspace
     {
        string statustext = getStatusText();
        int length = statustext.length();
        if(length == 0)
          return false;
        setStatusText(statustext.erase(length-1, 1 ) );
     }
     else if(key != '\n' && key != '\r' && key != '\t')
     {
        setStatusText(getStatusText() + (char) key );
     }
  }
  return false;
}

void SavePanel::write()
{
  int numtowrite;
  int i;

  numtowrite = adapter->getNumSaves();
  if(selection >= numtowrite)
    numtowrite++;


  ofstream out;
  
  out.open("savegames.xml", ios::out);  

  XmlOutput xmloutput(out);
  xmloutput.header("savegames");
  xmloutput.beginElement("savegames", true);
    xmloutput.attribute("numsaves", numtowrite);
    xmloutput.endAttributes();
    for(i=0; i < numtowrite; i++)
    {
      xmloutput.beginElement("save", true);
      if(i == selection)
      {
         xmloutput.attribute("name", (char *) getStatusText().c_str() );
         xmloutput.attribute("mission", (char *) mission.c_str() );
         xmloutput.attribute("campaign", (char *) campaign.c_str() );
      }
      else
      {
         xmloutput.attribute("name", (char *) adapter->getNames()[i].c_str() );
         xmloutput.attribute("mission", (char *) adapter->getMissions()[i].c_str() );
         xmloutput.attribute("campaign", (char *) adapter->getCampaigns()[i].c_str() );
      }
      xmloutput.endElement();
    }
  xmloutput.endElement("savegames");
  out.close();
}
