// Sector.cpp: implementation of the Sector class.
//
//////////////////////////////////////////////////////////////////////

#include "Sector.h"
#include "SectorXmlAdapter.h"
#include <stdio.h>
#include "WorldOrientedBillboard.h"
#include "glerr.h"
#include "LightManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Sector::Sector(char filename[])
{
  int i;
  LightManager * manager = LightManager::getInstance();

  SectorXmlAdapter adapter(filename);

  planets = adapter.getPlanets();
  numplanets = adapter.getNumPlanets();
  background = adapter.getBackgroundTexture();
  show_stars = adapter.showStars();
  numlights = adapter.getNumLights();
  lights = adapter.getLights();
  for(i=0; i < numlights; i++)
  {
	 Light * light = lights[i];
	 if(light->instanceof("AmbientLight") )
	 {
	   manager->setAmbientLight((AmbientLight *) light);
	 }
	 else
	 {
	   manager->addGlobalLight((DiffuseLight *) lights[i]);
	 }
  }
}

Sector::~Sector()
{
  int i;

  for(i=0; i < numplanets; i++)
  {
    delete planets[i];
  }
  if(planets != NULL)
    delete [] planets;
  for(i=0; i < numlights; i++)
  {
     delete lights[i];
  }
  if(lights != NULL)
    delete [] lights;
}

Texture * Sector::getBackgroundTexture()
{
  return background;
}

void Sector::renderPlanets()
{
   int i;
   for(i=0; i < numplanets; i++)
   {
     planets[i]->render();
   }

   glClear(GL_DEPTH_BUFFER_BIT);
   GLERRORCHECK
}

bool Sector::showStars()
{
   return show_stars;
}

void Sector::createLights()
{
/*  int i;

  for(i=0; i < numlights; i++)
  {
    lights[i]->render();
    lights[i]->enable();
  }
*/
  LightManager * manager = LightManager::getInstance();
  manager->renderGlobalLights();
}
