// SectorXmlAdapter.cpp: implementation of the SectorXmlAdapter class.
//
//////////////////////////////////////////////////////////////////////


#include "SectorXmlAdapter.h"
#include "saxexpat.h"
#include "InputSource.h"
#include "AttributeList.h"
#include "ErrorHandlerE.h"
#include "TextureFactory.h"
#include "AmbientLight.h"
#include "DiffuseLight.h"
#include "OutOfBoundsException.h"
#include "Settings.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SectorXmlAdapter::SectorXmlAdapter(char xmlfilename[] )
{
	expat myParser;

   currentplanet = 0;
   currentlight = 0;
   numlights = 0;
   numplanets = 0;
   planets = NULL;
   background = NULL;
   lights = NULL;
   show_stars = true;
   currentwidth = 3.0f;
   currentheight = 3.0f;

   ErrorHandlerE::checkFile(xmlfilename);
	myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(*(new SAX::InputSource(xmlfilename)));

}

SectorXmlAdapter::~SectorXmlAdapter()
{

}

void SectorXmlAdapter::startDocument()
{

}

void SectorXmlAdapter::endDocument()
{

}

void SectorXmlAdapter::startElement(const std::string& name, const SAX::AttributeList* atts)
{
  int i;

  if(name == "sector")
  {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "background_texture" )
         {
            if(Settings::getSettings()->getNebula() )
            {
              background = TextureFactory::getTexture((char *) currentvalue.c_str() );
            }
            else
            {
              background = NULL;
            }
         }
         else if(currentname == "stars")
         {
            if(currentvalue == "false")
               show_stars = false;
         }
      }
  }
  else if(name == "point")
  {

     for(i=0; i < atts->getLength(); i++)
     {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "x")
        {
            currentpoint.x =  atof( currentvalue.c_str() );
        }
        else if(currentname == "y")
        {
            currentpoint.y =  atof( currentvalue.c_str() );
        }
        else if(currentname == "z")
        {
            currentpoint.z =  atof( currentvalue.c_str() );
        }
     }
  }
  else if(name == "color")
  {

     for(i=0; i < atts->getLength(); i++)
     {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "r")
        {
            currentcolor.r =  atof( currentvalue.c_str() );
        }
        else if(currentname == "g")
        {
            currentcolor.g =  atof( currentvalue.c_str() );
        }
        else if(currentname == "b")
        {
            currentcolor.b =  atof( currentvalue.c_str() );
        }
     }
  }

  else if(name == "planet")
  {

     for(i=0; i < atts->getLength(); i++)
     {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "texture")
        {
           texturename = currentvalue;
        }
		else if(currentname == "width")
		{
            currentwidth = atof( currentvalue.c_str() );
		}
		else if(currentname == "height")
		{
            currentheight = atof( currentvalue.c_str() );
		}
     }
  }
  else if(name == "planets")
  {
     for(i=0; i < atts->getLength(); i++)
     {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "num_planets")
        {
           numplanets = atoi(currentvalue.c_str() );
           planets = new WorldOrientedBillboard *[numplanets];
        }
     }
  }
  else if(name == "lights")
  {
     for(i=0; i < atts->getLength(); i++)
     {
        std::string  currentname = atts->getName(i);
        std::string  currentvalue = atts->getValue(i);

        if(currentname == "num_lights")
        {
           numlights = atoi(currentvalue.c_str() );
           lights = new Light *[numlights];
        }
     }
  }

}


void SectorXmlAdapter::endElement(const std::string& name)
{
  if(name == "planet")
  {
    if(currentplanet >= numplanets)
       throw OutOfBoundsException("too many lights in *.sector.xml");	
    planets[currentplanet] = new WorldOrientedBillboard((char *) texturename.c_str(), currentpoint, 
		Coordinate(1.0f, 0.0f, 0.0f), currentwidth, currentheight);
    currentplanet++;
	currentwidth = 3.0f;
	currentheight = 3.0f;
  }
  else if(name == "ambientlight")
  {
    int maxlights;
    glGetIntegerv(GL_MAX_LIGHTS, & maxlights);
    if(currentlight >= numlights || currentlight >=  maxlights)
       throw OutOfBoundsException("too many lights in *.sector.xml");
    lights[currentlight] = new AmbientLight(currentcolor);
    currentlight++;
  }
  else if(name == "diffuselight")
  {
    int maxlights;
    glGetIntegerv(GL_MAX_LIGHTS, & maxlights);
    if(currentlight >= numlights || currentlight >=  maxlights)
       throw OutOfBoundsException("too many lights in *.sector.xml");
    lights[currentlight] = new DiffuseLight(currentlight, currentcolor, currentpoint);
    currentlight++;
  }
}

void SectorXmlAdapter::characters(const std::string& ch)
{

}

WorldOrientedBillboard * * SectorXmlAdapter::getPlanets()
{
  return planets;
}

Texture * SectorXmlAdapter::getBackgroundTexture()
{
  return background;
}

int SectorXmlAdapter::getNumPlanets()
{
  return numplanets;
}

bool SectorXmlAdapter::showStars()
{
  return show_stars;
}

int SectorXmlAdapter::getNumLights()
{
  return numlights;
}

Light * * SectorXmlAdapter::getLights()
{
  if(lights == NULL)
    throw OutOfBoundsException("Not enough lights were specified in *.sector.xml");
  return lights;
}
