// Settings->cpp: implementation of the Settings class->
//
//////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include "Settings.h"
#include "SettingsXmlAdapter.h"
#include "XmlOutput.h"
#include <iostream.h>
#include <fstream.h>
#include "TextureFactory.h"
#include "Math.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Settings * Settings::settings = NULL;

Settings::Settings()

{
  SettingsXmlAdapter adapter("settings.xml");

  resolution = adapter.getResolution();
  shouldsmooth = adapter.shouldSmooth();
  music = adapter.getMusic();
  sound = adapter.getSound();
  mipmap = adapter.getMipMap();
  nebula = adapter.getNebula();
  cockpitdetails = adapter.getCockpitDetails();
  dynamiclights = adapter.getDynamicLights();
  collision_spheres = false;
  forcenosound = false;
  texturecompression = adapter.getTextureCompression();
  texturecompression = MIN(texturecompression, TextureFactory::getMaxTextureCompression() );
  anisotropy = adapter.getAnisotropy();
  anisotropy = MIN(anisotropy, TextureFactory::getMaxAnisotropy() );
  anisotropy = MAX(anisotropy, 1 );
  vsync = adapter.getVSync();
  if(GLEW_EXT_texture_cube_map)
  {
    reflections = adapter.getReflections();
  }
  else
  {
	reflections = false;
  }

  computeDisplayMode();
}

Settings::~Settings()
{

}


string Settings::getResolutionString()
{
   return resolution;
}
 
Settings * Settings::getSettings()
{
  if(settings == NULL)
  {
    settings = new Settings();
  }
  return settings;
}

bool Settings::smooth()
{
   return shouldsmooth;
}


void Settings::setSmooth(bool newvalue)
{
  shouldsmooth = newvalue;
}

void Settings::setResolutionString(string newresolution)
{
  resolution = newresolution;
  computeDisplayMode();
}

void Settings::write()
{
  ofstream out;
  
  out.open("settings.xml", ios::out);  

  XmlOutput xmloutput(out);
  xmloutput.header("settings");
  xmloutput.beginElement("settings", true);
    xmloutput.attribute("resolution", (char *) resolution.c_str());
    xmloutput.attribute("smooth", shouldsmooth);
    xmloutput.attribute("music", music);
    xmloutput.attribute("sound", sound);
    xmloutput.attribute("mipmap", mipmap);
    xmloutput.attribute("nebula", nebula);
	xmloutput.attribute("cockpitdetails", cockpitdetails);
	xmloutput.attribute("dynamiclights", dynamiclights);
	xmloutput.attribute("texturecompression", texturecompression);
	xmloutput.attribute("anisotropy", anisotropy);	
	xmloutput.attribute("vsync", vsync);	
	xmloutput.attribute("reflections", reflections);	
  xmloutput.endElement();
  out.close();

}

bool Settings::getMusic()
{
  if(forcenosound)
	  return false;
  return music;
}

void Settings::setMusic(bool newmusic)
{
  music = newmusic;
}

bool Settings::getSound()
{
  if(forcenosound)
	  return false;
  return sound;
}

void Settings::setSound(bool newsound)
{
  sound = newsound;
}

bool Settings::getMipMap()
{
  return mipmap;
}

void Settings::setMipMap(bool newmipmap)
{
  mipmap = newmipmap;
}

bool Settings::getNebula()
{
  return nebula;
}

void Settings::setNebula(bool newnebula)
{
  nebula = newnebula;
}

bool Settings::getShowCollisionSpheres()
{
  return collision_spheres;
}

void Settings::setShowCollisionSpheres(bool collision)
{
  collision_spheres = collision;
}

bool Settings::getCockpitDetails()
{
  return cockpitdetails;
}

void Settings::setCockpitDetails(bool value)
{
  cockpitdetails = value;
}

void Settings::forceSoundDisable()
{
  forcenosound =true;
}

void Settings::computeDisplayMode()
{
  sscanf((char *) resolution.c_str(), "%dx%d:%d", 
	  &displaymode.width, &displaymode.height, &displaymode.depth);
}

DisplayMode Settings::getDisplayMode()
{
  return displaymode;
}

void Settings::setDynamicLights(bool on)
{
  dynamiclights = on;
}

bool Settings::getDynamicLights()
{
  return dynamiclights;
}

int Settings::getTextureCompression()
{
  return texturecompression;
}

void Settings::setTextureCompression(int newcompression)
{
  texturecompression = newcompression;
}

int Settings::getAnisotropy()
{
  return anisotropy;
}

void Settings::setAnisotropy(int newanisotropy)
{
  anisotropy = newanisotropy;
}

bool Settings::getVSync()
{
  return vsync;
}

void Settings::setVSync(bool newsync)
{
  vsync = newsync;
}

bool Settings::getReflections()
{
  return reflections;
}

void Settings::setReflections(bool newreflect)
{
  reflections = newreflect;
}
