// Shader.cpp: implementation of the Shader class.
//
//////////////////////////////////////////////////////////////////////

#include "gl/glew.h"
#include "Shader.h"
#include <iostream.h>
#include <fstream.h>
#include "FileNotFoundException.h"
#include "glerr.h"

using std::string;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Shader::Shader(std::string vertexfile, std::string fragmentfile)
{
  string vertstring;
  string fragstring;

  fileToString(vertexfile, vertstring);
  fileToString(fragmentfile, fragstring);

  vertexprogram = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
  GLERRORCHECK
  fragmentprogram = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
  GLERRORCHECK
  compile(vertexprogram, vertstring);
  compile(fragmentprogram, fragstring);

  my_program = glCreateProgramObjectARB();
  GLERRORCHECK
  glAttachObjectARB(my_program, vertexprogram);
  glAttachObjectARB(my_program, fragmentprogram);
  glLinkProgramARB(my_program);

}

Shader::~Shader()
{
   glDeleteObjectARB(my_program);
}

void Shader::begin()
{
  glUseProgramObjectARB(my_program);
}

void Shader::end()
{
  glUseProgramObjectARB(0);
}

void Shader::fileToString(std::string filename, std::string & newstring)
{
  ifstream in(filename.c_str(), ios::in | ios::nocreate);
  char byte;

  if(! in )
  {
     throw FileNotFoundException((char *) filename.c_str() );
  }

  while(! in.eof() )
  {
    in.read(&byte, 1) ;
	newstring += byte;
  }  
  newstring += '\0';
}

string Shader::getCompileLog(GLhandleARB handle)
{
  GLcharARB buffer[500];
  GLsizei len;

  glGetInfoLogARB(handle, (GLsizei) 500, &len, buffer);
  cout << buffer << endl;
  return buffer;
}

void Shader::compile(GLhandleARB handle, std::string textstring)
{
  int strlength;
  char * text;
  int compiled;

  strlength = (int) textstring.length();
  text = (char *) textstring.c_str();
//  cout << text << endl;
  glShaderSourceARB(handle, 1, (const char * *)& text, NULL);
  GLERRORCHECK
  glCompileShaderARB(handle);  
  GLERRORCHECK
  // see if it compiled
  glGetObjectParameterivARB(handle, GL_OBJECT_COMPILE_STATUS_ARB, &compiled);
  if(compiled)
  {
    cout << "compiled" << endl;
  }
  else
  {
    cout << "not compiled" << endl;
  }
  GLERRORCHECK
  getCompileLog(handle);

}

void Shader::sendUniform4f(char * varname, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
{
    GLint loc;

	loc = glGetUniformLocationARB(my_program, varname);

    if (loc==-1) 
	{
		cout << "bad send Uniform" << endl;
	}
    
    glUniform4fARB(loc, v0, v1, v2, v3);
    GLERRORCHECK
}
