// SoundEffect.cpp: implementation of the SoundEffect class.
//
//////////////////////////////////////////////////////////////////////

#include "SoundEffect.h"
#include <iostream.h>
#include "Settings.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SoundEffect::SoundEffect(char filename[])
{
   loop = false;
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }

#ifdef HAS_BASS
   const int num_simultaneous = 10;

   sam= (HSAMPLE) BASS_SampleLoad(FALSE,filename,0,0,num_simultaneous,
	   BASS_SAMPLE_OVER_POS);
   if(! sam)
   {
        cout << "Could not load sample " << filename << endl;
   }
   channel = NULL;
#endif
#ifdef HAS_FMOD
   sam = FSOUND_Sample_Load(FSOUND_FREE , filename, 0, 0, 0);	
   if(sam == NULL)
   {
        cout << "Could not load sample " << filename << endl;
   }
   channel = 0;
#endif
}

SoundEffect::~SoundEffect()
{
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }
#ifdef HAS_BASS

   if(! sam)
	   return;

  BASS_SampleFree(sam);
#endif

#ifdef HAS_FMOD
  FSOUND_Sample_Free(sam);
#endif
}

void SoundEffect::play()
{
#ifdef HAS_BASS
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }
   if(! sam)
   {
	 return;
   }
   if(loop)
   {
      channel = BASS_SamplePlayEx(
        sam,
        0,
        -1,
         100,
       -101, 
          1
	  );
   }
   else
   {
     channel= BASS_SamplePlay(sam);
   }
   if (!  channel)
   {
        cout << "Could not play sample" << endl;
   }
#endif
#ifdef HAS_FMOD

  channel = FSOUND_PlaySound(FSOUND_FREE, sam);
  if(channel == -1)
  {
        cout << "Could not play sample" << endl;
  }


#endif

}


bool SoundEffect::isPlaying()
{
   if(! Settings::getSettings()->getSound() )
   {
      return false;
   }

#ifdef HAS_BASS
   return BASS_ChannelIsActive(channel) == BASS_ACTIVE_PLAYING;
#elif defined HAS_FMOD
	return FSOUND_IsPlaying(channel);
#else
   return false;
#endif
}

void SoundEffect::stop()
{
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }

#ifdef HAS_BASS
   BASS_ChannelStop(channel);
#endif
#ifdef HAS_FMOD
   FSOUND_StopSound(channel);
#endif
}

void SoundEffect::setLoop(bool newloop)
{
  loop = newloop;
#ifdef HAS_FMOD
  if(loop)
  {
     FSOUND_Sample_SetMode(sam, FSOUND_LOOP_NORMAL);
  }
  else
  {
     FSOUND_Sample_SetMode(sam, FSOUND_LOOP_OFF);
  }
#endif
}
