// SoundFactory.cpp: implementation of the SoundFactory class.
//
//////////////////////////////////////////////////////////////////////

#include "SoundFactory.h"
#ifdef HAS_BASS
#include "bass.h"
#endif
#include <iostream.h>
#ifdef WIN32
#include <windows.h>
#endif
#include "SoundsXmlAdapter.h"
#include <string>
#include "Settings.h"
#include "Listener.h"
#include "SoundEffect3d.h"
#include "SoundSample3d.h"
#include "NullListener.h"
#include <list>

#ifdef HAS_BASS
#include "SurroundSoundEffect3d.h"
#include "StereoSoundEffect3d.h"
#endif

#ifdef HAS_FMOD
#include "StereoSoundEffect3d.h"
#endif

#ifdef HAS_FMOD
#include "fmod.h"
#endif


#include "NullSoundEffect3d.h"
#include <assert.h>

using std::string;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SoundFactory * SoundFactory::factory = NULL;

SoundFactory::SoundFactory()
{
   inited = false;
   refresh();
   garbage = new std::list<SoundEffect3d *>();
}

SoundFactory::~SoundFactory()
{
  std::list<SoundEffect3d *>::iterator iter;

  iter = garbage->begin();
  while(iter != garbage->end() )
  {
	  SoundEffect3d * effect = *iter;
	  if(effect != NULL)
	  {
        delete effect;
	  }
  }

  delete garbage;
  delete listener;
#ifdef HAS_BASS
  BASS_Free();
#endif

  


}

SoundFactory * SoundFactory::getSoundFactory()
{
  if(factory == NULL)
    factory = new SoundFactory();
  return factory;
}

SoundEffect3d * SoundFactory::getWeaponSound(int index)
{
  // these aren't registered, since the weapon itself will delete the effect
  return createSoundEffect(weaponsounds[index]);
}

SoundEffect3d * SoundFactory::getShieldSound()
{
  return regsiterEffect(createSoundEffect(shieldsound));
}

SoundEffect3d * SoundFactory::getArmorSound()
{
  return regsiterEffect(createSoundEffect(armorsound));
}

SoundEffect3d * SoundFactory::getExplosionSound()
{
  return regsiterEffect(createSoundEffect(explosionsound));
}

SoundEffect3d * SoundFactory::getMissileSound(int index)
{
// these aren't registered, since the weapon itself will delete the effect
  return createSoundEffect(missilesounds[index]);
}

SoundEffect * SoundFactory::getAutoPilotSound()
{
  return autopilotsound;
}

SoundEffect * SoundFactory::getLockSound()
{
  return locksound;
}

SoundEffect * SoundFactory::getLandingSound()
{
  return landingsound;
}

void SoundFactory::refresh()
{
   int i;
   bool disableall= false;

   if((Settings::getSettings()->getSound() || Settings::getSettings()->getMusic()  )&& ! inited)
   {
#ifdef HAS_BASS
     inited = true;

     HWND wnd = FindWindow(NULL, "GLUT");

     if(wnd == NULL)
     cout << "Window is null " << endl;

     if (!BASS_Init(1,44100,BASS_DEVICE_3D,wnd, NULL) )
	 {

       // try 2d
       if (!BASS_Init(1,44100,0,wnd, NULL))
	   {
           cout << "Could not init sound, disabling sound and music" << endl;
		   Settings::getSettings()->forceSoundDisable();

		   disableall = true;
	   }
	   else
	   {
           cout << "Initialized sound, no 3d sound support found" << endl;
	   }
       sound3d = false;
	 }
	 else
	 {
		// check to see if emulated, if emulated, disable 3d sound
        BASS_INFO info;

		info.size = sizeof(BASS_INFO);
		BASS_GetInfo(&info);
		if(info.flags & DSCAPS_EMULDRIVER)
		{
           cout << "Emulated driver detected, disabling 3d sound" << endl;
		   sound3d = false;
		}
		else
		{
          cout << "Initialized 3d sound" << endl;
          sound3d = true;
		}
		cout << "DirectSound version: " << info.dsver << endl;
	 }
	 if(! disableall)
	 {
       cout << "Sound card: " << BASS_GetDeviceDescription(1)  << endl;
       BASS_Start();	/* Start digital output */
	   if(sound3d)
	   {
     	   if(! BASS_Set3DFactors(1000000.0f, .001f, .001f) )
		   {
             cout << "Could not set factors" << endl;
		   }
	       BASS_Apply3D();
	   }
	 }

//     BASS_SetBufferLength(1.0f);
#endif
#ifdef HAS_FMOD

#ifdef WIN32
     HWND wnd = FindWindow(NULL, "GLUT");

     if(wnd == NULL)
	     cout << "Window is null " << endl;
	 FSOUND_SetHWND(wnd);

#endif /* win32 *//


	if(! FSOUND_Init(44100, 32, 0) )
	{
	  cout << "Could not init sound, disabling sound and music" << endl;
	   Settings::getSettings()->forceSoundDisable();
	   disableall = true;
	}
	else
	{
	  int drivernumber = FSOUND_GetDriver();
	  cout << "Sound driver:" << FSOUND_GetDriverName(drivernumber) << endl;
	  int output = FSOUND_GetOutput();
	  if(output & FSOUND_OUTPUT_OSS)
	  {
  	    cout << "Sound driver: OSS" << endl;
	  }
	  else if(output & FSOUND_OUTPUT_ESD)
	  {
  	    cout << "Sound driver: ESD" << endl;
	  }
	  else if(output & FSOUND_OUTPUT_ALSA)
	  {
  	    cout << "Sound driver: ALSA" << endl;
	  }
	  unsigned int caps = 0;
	  FSOUND_GetDriverCaps(drivernumber, &caps);
	  if(caps & FSOUND_CAPS_HARDWARE)
	  {
	    cout << "3d sound acceleration available" << endl;
	  }
	  else
	  {
	    cout << "3d sound acceleration not found" << endl;
	  }
	}
	sound3d = false;
	
#endif

   }


   SoundsXmlAdapter xmladapter;

#ifdef HAS_BASS 
   afterburnersound = new SoundSample3d("afterburner.wav", sound3d);
   armorsound = new SoundSample3d((char *) xmladapter.getArmorSound().c_str(), sound3d );
   shieldsound = new SoundSample3d((char *) xmladapter.getShieldSound().c_str() , sound3d);
   explosionsound = new SoundSample3d((char *) xmladapter.getExplosionSound().c_str() , sound3d);
   decoysound = new SoundSample3d("decoy.wav", sound3d);
#endif

#ifdef HAS_FMOD
   afterburnersound = new SoundSample3d("afterburner.wav", sound3d);
   armorsound = new SoundSample3d((char *) xmladapter.getArmorSound().c_str(), sound3d );
   shieldsound = new SoundSample3d((char *) xmladapter.getShieldSound().c_str() , sound3d);
   explosionsound = new SoundSample3d((char *) xmladapter.getExplosionSound().c_str() , sound3d);
   decoysound = new SoundSample3d("decoy.wav", sound3d);
#endif

   autopilotsound  = new SoundEffect("autopilot.wav");
   locksound = new SoundEffect("missile lock.wav");
   landingsound = new SoundEffect("landing.wav");
   takeoffsound = new SoundEffect("takeoff.wav");
   gunchange = new SoundEffect("changeguns.wav");
   missilechange = new SoundEffect("changemissiles.wav");
   missilealert = new SoundEffect("missilealert.wav");
   ejectalert = new SoundEffect("ejectsound.wav");
   beep = new SoundEffect("beep.wav");
   tractorbeam = new SoundEffect("tractorbeam.wav");
   missile_explode = new SoundSample3d("missileexplosion.wav", sound3d);

   numweaponsounds = xmladapter.getNumWeaponSounds();
   weaponsounds = new SoundSample3d *[numweaponsounds];

   nummissilesounds = xmladapter.getNumMissileSounds();
   missilesounds = new SoundSample3d *[nummissilesounds];

   chargesounds[0] = new SoundEffect("charge1.wav");
   chargesounds[1] = new SoundEffect("charge2.wav");
   chargesounds[2] = new SoundEffect("charge3.wav");
   chargesounds[3] = new SoundEffect("charge4.wav");

#ifdef HAS_BASS 
   for(i=0; i < numweaponsounds; i++)
   {
      weaponsounds[i] = new SoundSample3d((char *) xmladapter.getWeaponSound(i).c_str() , sound3d);
   }
   for(i=0; i < nummissilesounds; i++)
   {
      missilesounds[i] = new SoundSample3d((char *) xmladapter.getMissileSound(i).c_str() , sound3d);
   }
#endif

#ifdef HAS_FMOD
   for(i=0; i < numweaponsounds; i++)
   {
      weaponsounds[i] = new SoundSample3d((char *) xmladapter.getWeaponSound(i).c_str() , sound3d);
   }
   for(i=0; i < nummissilesounds; i++)
   {
      missilesounds[i] = new SoundSample3d((char *) xmladapter.getMissileSound(i).c_str() , sound3d);
   }
#endif

   if(sound3d)
   {
     listener = new Listener();
   }
   else
   {
     listener = new NullListener();
   }
}

SoundEffect * SoundFactory::getGunChangeSound()
{
  return gunchange;
}

SoundEffect * SoundFactory::getMissileChangeSound()
{
  return missilechange;
}




SoundEffect3d * SoundFactory::getMissileExplodeSound()
{
  return regsiterEffect(createSoundEffect(missile_explode));
}

SoundObject3d * SoundFactory::getListener()
{
  return listener;
}

SoundEffect * SoundFactory::getBeepSound()
{
  return beep;
}

SoundEffect3d * SoundFactory::regsiterEffect(SoundEffect3d * effect)
{
  garbage->push_back(effect);
  return effect;
}

void SoundFactory::advance()
{
  std::list<SoundEffect3d *>::iterator iter;

  iter = garbage->begin();
  while(iter != garbage->end() )
  {
	  SoundEffect3d * effect = *iter;
	  if(effect != NULL && ! effect->isPlaying() )
	  {
		 iter = garbage->erase(iter);
		 delete effect;
		 // hmm this looks like it should be done before the erase??
		 *iter = NULL;
	  }
	  else
	  {
        iter++;
	  }
  }

}

SoundEffect3d * SoundFactory::getAfterburnerSound()
{
  return regsiterEffect(createSoundEffect(afterburnersound));
}

SoundEffect * SoundFactory::getTakeoffSound()
{
  return takeoffsound;
}

void SoundFactory::clear()
{
  std::list<SoundEffect3d *>::iterator iter;

  iter = garbage->begin();
  while(iter != garbage->end() )
  {
	  SoundEffect3d * effect = *iter;
	  if(effect != NULL)
        delete effect;
	  iter++;
  }
  garbage->clear();
}

SoundEffect3d * SoundFactory::createSoundEffect(SoundSample3d * soundsample)
{
#ifdef HAS_BASS
  if(! Settings::getSettings()->getSound() )
  {
    return new NullSoundEffect3d();
  }
  else if(sound3d)
  {
     return new SurroundSoundEffect3d(soundsample);
  }
  else
  {
    return new StereoSoundEffect3d(soundsample);
  }
#elif defined(HAS_FMOD)
  if(! Settings::getSettings()->getSound() )
  {
    return new NullSoundEffect3d();
  }
  else
  {
    return new StereoSoundEffect3d(soundsample);
  }
#else
  return new NullSoundEffect3d();
#endif
}

SoundEffect * SoundFactory::getChargeSound(int index)
{
  assert(index < NUM_CHARGESOUNDS);
  assert(index >= 0);

  return chargesounds[index];
}

SoundEffect3d * SoundFactory::getDecoySound()
{
  return regsiterEffect(createSoundEffect(decoysound));
}

SoundEffect * SoundFactory::getMissileAlertSound()
{
  return missilealert;
}

SoundEffect * SoundFactory::getTractorBeamSound()
{
  return tractorbeam;
}

SoundEffect * SoundFactory::getEjectAlertSound()
{
  return ejectalert;
}
