// StarField.cpp: implementation of the StarField class.
//
//////////////////////////////////////////////////////////////////////

#include "GL/glew.h"
#include <glut.h>
#include "StarField.h"
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "Time.h"
#include "TextureFactory.h"
#include "Camera.h"
#include "Settings.h"
#include "glerr.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

static GLfloat linear[3] = { 0.0, 1/5.0, 0.0 };
static GLfloat constant[3] = { 1.0, 0.0, 0.0 };

StarField::StarField(int nnumstars, Texture * backgroundtexture, bool new_show_stars)
  :Object3d(3.0f, 3.0f, 3.0f), show_stars(new_show_stars)
//   camera(newcamera)   
{
  DisplayMode displaymode;
  const float defaultwidth = 800.0f;

  numstars = 6 * nnumstars ;
  viewradius = 10.0f;
  coords = new Coordinate[numstars];

  advancez = 0;
  fillrandom();
  ////////////////////////////////////// nebula
  nebula = backgroundtexture;
  displaymode = Settings::getSettings()->getDisplayMode();
  sizemultiplier = ((float)displaymode.width) / defaultwidth;

}

StarField::~StarField()
{
  delete [] coords; 

}

/* don't create the bottom and top because the texture seams will show otherwise */
static void CreateSkyBox(float x, float y, float z, float width, float height, float length)
{
	// This is the most important function of this tutorial.  This function
	// used to just create a silly colored cube in the RotateCube tutorial,
	// but now it creates something beautiful.  You'll notice we added
	// some more parameters to the function.  This way we can change the perspective
	// of the sky box.  It doesn't really look good if it's a perfect cube.  Some
	// textures look better at different ratios.  We assign the sky box textures
	// to each side of the box creating the illusion of a detailed 3D world.
	// You will notice I had to change the texture coordinates for every one
	// to be flipped correctly.  Also, depending on your culling, the vertex
	// order might need to be changed around.  I don't use culling in this tutorial
	// so it will work fine here, but be sure to remember this if you do.

	// Bind the BACK texture of the sky map to the BACK side of the cube
//	glBindTexture(GL_TEXTURE_2D, g_Texture[BACK_ID]);

	// Since we want the sky box to be centered around X, Y, and Z for ease,
	// we do a little math to accomplish this.  We just change the X, Y and Z
	// to perform this task.  If we just minus half the width, height and length
	// from x, y and z it will give us the desired result.  Now when we create the
	// box it will center it around (x, y, z)

	// This centers the sky box around (x, y, z)
	x = x - width  / 2;
	y = y - height / 2;
	z = z - length / 2;

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);		
      glNormal3f(0.0f, 0.0f, -1.0f);		
		// Assign the texture coordinates and vertices for the BACK Side
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,			y,			z);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,			y + height, z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z); 
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,			z);

		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,			z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,			y + height, z);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,			y,			z);

//	glEnd();

	// Bind the FRONT texture of the sky map to the FRONT side of the box
//	glBindTexture(GL_TEXTURE_2D, g_Texture[FRONT_ID]);

	// Start drawing the side as a QUAD
//	glBegin(GL_QUADS);	
      glNormal3f(0.0f, 0.0f, 1.0f);		
		// Assign the texture coordinates and vertices for the FRONT Side
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,			y,			z + length);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,			y + height, z + length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z + length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,			z + length);

		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,			z + length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z + length); 
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,			y + height, z + length);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,			y,			z + length);

//	glEnd();

	// Bind the LEFT texture of the sky map to the LEFT side of the box
//	glBindTexture(GL_TEXTURE_2D, g_Texture[LEFT_ID]);

	// Start drawing the side as a QUAD
//	glBegin(GL_QUADS);		
      glNormal3f(1.0f, 0.0f, 0.0f);		
		// Assign the texture coordinates and vertices for the LEFT Side
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,			y,			z);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,			y,			z + length);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,			y + height,	z + length); 
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,			y + height,	z);

		glTexCoord2f(1.0f, 1.0f); glVertex3f(x,			y + height,	z);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x,			y + height,	z + length); 
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x,			y,			z + length);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x,			y,			z);


//	glEnd();

	// Bind the RIGHT texture of the sky map to the RIGHT side of the box
//	glBindTexture(GL_TEXTURE_2D, g_Texture[RIGHT_ID]);

	// Start drawing the side as a QUAD
//	glBegin(GL_QUADS);		
      glNormal3f(-1.0f, 0.0f, 0.0f);
		// Assign the texture coordinates and vertices for the RIGHT Side
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,			z);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,			z + length);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height,	z + length); 
		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height,	z);

		glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height,	z);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height,	z + length); 
		glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,			z + length);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,			z);

	glEnd();
}


void StarField::render()
{
//  GLfloat oldptsize;
  int i;
  const int half = numstars / 6;

//  glPushAttrib(GL_ALL_ATTRIB_BITS);
 glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST); 
 glDisable(GL_TEXTURE_2D);

//  glDisable(GL_CULL_FACE);
// 
//  glDisable(GL_LIGHTING);
//  glDisable(GL_LIGHT0);

  glPushMatrix();

  glLoadIdentity ();

  Camera * camera = Camera::getCamera();
  Coordinate heading = camera->getHeading();
  heading.normalize();
  Coordinate up = camera->getUp();


  gluLookAt (0.0, 0.0, 0.0, 
             heading.x * viewradius/2.0f, heading.y * viewradius/2.0f, heading.z * viewradius/2.0f,
             up.x, up.y, up.z);
/////////////////////////////////////////////
//////// Nebula
  if(Settings::getSettings()->getNebula() )
  {
    glColor3f (1.0f, 1.0f, 1.0f); // set color to white
    glEnable(GL_TEXTURE_2D);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    nebula->render();
    CreateSkyBox(0.0f,0.0f, 0.0f, 50.0f, 50.0f, 50.0f);
    glDisable(GL_TEXTURE_2D);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  }

//////////////////////////////////////////////

if(show_stars)
{

  glEnable(GL_BLEND);
  if(GLEW_EXT_point_parameters)
  {
    glPointParameterfvEXT(GL_DISTANCE_ATTENUATION_EXT, linear);
    glPointSize(2.75f * sizemultiplier);
  }
  else
  {
    glPointSize(2.0f);
  }
  glBegin(GL_POINTS);
    const int half_5=half*5, half_4=half*4, half_3=half*3, half_2=half*2;
    for(i=0; i < numstars; i++)
    {
      if(i > half_5 )
      {
         glColor3f (.9f, .9f, .9f); // set color to purple
      }
      else if(i > half_4)
      {
         glColor3f (.7f, 0.0f, .7f); // set color to purple
      }
      else if(i > half_3)
      {
         glColor3f (.8f, 0.0f, .8f); // set color to purple
      }
      else if(i > half_2)
      {
         glColor3f (.9f, 0.0f, .9f); // set color to purple
      }
      else if(i > half)
      {
         glColor3f (.7f, .7f, .1f); // set color to purple
      }
      if(! ((i < half) && (! Camera::getCamera()->showMovingStars()  )) ) 
        glVertex3f(coords[i].x, coords[i].y, coords[i].z);
    }
  glEnd();

  glDisable(GL_BLEND);
  if(GLEW_EXT_point_parameters)
  {
    glPointParameterfvEXT(GL_DISTANCE_ATTENUATION_EXT, constant);
  }

} /* show_stars */
  glPopMatrix();
  glColor3f (1.0, 1.0, 1.0); // set color to white

  glEnable(GL_DEPTH_TEST); 
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_LIGHTING);

  GLERRORCHECK
}

void StarField::fillrandom()
{
  float a ,b ,c; // random numbers
  int i;

  for(i=0; i < numstars; i++)
  {
    a = rand();
    b = rand();
    c = rand();

    coords[i].x = a * 2.0 * viewradius/  RAND_MAX - viewradius;
    coords[i].y = b * 2.0 * viewradius/  RAND_MAX - viewradius;
    coords[i].z = c * 2.0 * viewradius/  RAND_MAX - viewradius;
  }
} 

void StarField::advance()
{
  Coordinate newcoord;
//  float length;
  int i;


  int half = numstars /6 ;

  Coordinate heading;
  Coordinate offset;
  Coordinate temp;

//  heading = getHeading();
  Object3d * obj = Camera::getCamera()->getFocusObject();
  heading= obj->getHeading();
  advancez = (obj->getVelocity() / 6.0f) * Time::getTimeScale();

  offset.x = -heading.x * advancez;
  offset.y = -heading.y * advancez;
  offset.z = -heading.z * advancez;
  for(i=0; i < half; i++)
  {
    temp =   coords[i] + offset;
    if(temp.x > viewradius )
    {
      temp.x -= 2.0 * viewradius;
    }
    if(temp.y > viewradius )
    {
      temp.y -= 2.0 * viewradius;
    }
    if(temp.z > viewradius )
    {
      temp.z -= 2.0 * viewradius;
    }

    if(temp.x < -viewradius )
    {
      temp.x += 2.0 * viewradius;
    }
    if(temp.y < -viewradius )
    {
      temp.y += 2.0 * viewradius;
    }
    if(temp.z < -viewradius )
    {
      temp.z += 2.0 * viewradius;
    }
    coords[i] = temp;
  }

}

