// StereoSoundEffect3d.cpp: implementation of the StereoSoundEffect3d class.
//
//////////////////////////////////////////////////////////////////////

#include "StereoSoundEffect3d.h"
#include "Settings.h"
#include <iostream.h>
#ifdef HAS_BASS
#include "bass.h"
#endif
#include "SoundSample3d.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

StereoSoundEffect3d::StereoSoundEffect3d(SoundSample3d * newsample)
{
#ifdef HAS_BASS
   channel = NULL;
#endif
#ifdef HAS_FMOD
   channel = -1;
#endif
   sample = newsample;
   loop = false;
}

StereoSoundEffect3d::~StereoSoundEffect3d()
{
  stop();
}

void StereoSoundEffect3d::render()
{
}

void StereoSoundEffect3d::play()
{
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }

#ifdef HAS_BASS
   if(loop)
   {
     channel = BASS_SamplePlayEx(sample->getHSAMPLE(), 
		 0, 
		 -1, 
		 -1, 
		 -101,
		 TRUE);
   }
   else
   {
     channel = BASS_SamplePlay(sample->getHSAMPLE() );
   }

   if(channel == NULL)
   {
      cout << "error in 3d sound play" << endl;
   }
#endif
#ifdef HAS_FMOD
   channel = FSOUND_PlaySound(FSOUND_FREE, sample->getSample() );
#endif

}


bool StereoSoundEffect3d::isPlaying()
{
   if(! Settings::getSettings()->getSound() )
   {
      return false;
   }
#ifdef HAS_BASS
  return BASS_ChannelIsActive(channel) == BASS_ACTIVE_PLAYING;
#elif defined(HAS_FMOD)
	return FSOUND_IsPlaying(channel);
#else
  return false;
#endif
}

void StereoSoundEffect3d::setLoop(bool newloop)
{
  loop = newloop;
#ifdef HAS_FMOD
  if(loop)
  {
     FSOUND_Sample_SetMode(sample->getSample(), FSOUND_LOOP_NORMAL);
  }
  else
  {
     FSOUND_Sample_SetMode(sample->getSample(), FSOUND_LOOP_OFF);
  }
#endif
}

void StereoSoundEffect3d::stop()
{
#ifdef HAS_BASS
  if(channel != NULL)
  {
    BASS_ChannelStop(channel);
  }
  channel = NULL;
#endif
#ifdef HAS_FMOD
  if(channel != -1)
  {
    FSOUND_StopSound(channel);
  }
  channel = -1;
#endif
}
