// SurroundSoundEffect3d.cpp: implementation of the SurroundSoundEffect3d class.
//
//////////////////////////////////////////////////////////////////////


#include "SurroundSoundEffect3d.h"
#include "Settings.h"
#include <iostream.h>
#include "bass.h"
#include "SoundSample3d.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SurroundSoundEffect3d::SurroundSoundEffect3d(SoundSample3d * newsample)
{
#ifdef HAS_BASS
   channel = NULL;
#endif
   sample = newsample;
   loop = false;
}

SurroundSoundEffect3d::~SurroundSoundEffect3d()
{
  stop();
}

void SurroundSoundEffect3d::render()
{
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }

#ifdef HAS_BASS
   if(channel == NULL)
	   return;
   BASS_3DVECTOR basspos, bassheading, bassvel;
   fillInBass(basspos, bassheading, bassvel);
   BASS_ChannelSet3DPosition(channel, &basspos, &bassheading, &bassvel );
   BASS_Apply3D();
#endif
}

void SurroundSoundEffect3d::play()
{
   if(! Settings::getSettings()->getSound() )
   {
      return;
   }

#ifdef HAS_BASS
   BASS_3DVECTOR basspos, bassheading, bassvel;

   fillInBass(basspos, bassheading, bassvel);

   if(loop)
   {
     channel = BASS_SamplePlay3DEx(sample->getHSAMPLE() , &basspos, NULL, &bassvel, 0, -1, -1, TRUE);
   }
   else
   {
     channel = BASS_SamplePlay3D(sample->getHSAMPLE() , &basspos, NULL, &bassvel);
   }
   BASS_ChannelSet3DAttributes(channel, BASS_3DMODE_NORMAL, 0.0f, 100.0f, 360, 360, 100);
   if(channel == NULL)
   {
      cout << "error in 3d sound play" << endl;
   }
#endif
}


bool SurroundSoundEffect3d::isPlaying()
{
   if(! Settings::getSettings()->getSound() )
   {
      return false;
   }
#ifdef HAS_BASS
  return BASS_ChannelIsActive(channel) == BASS_ACTIVE_PLAYING;
#else
  return false;
#endif
}

void SurroundSoundEffect3d::setLoop(bool newloop)
{
  loop = newloop;
}

void SurroundSoundEffect3d::stop()
{
#ifdef HAS_BASS
  if(channel != NULL)
  {
    BASS_ChannelStop(channel);
  }
  channel = NULL;
#endif
}

