// TakeOffCutScene.cpp: implementation of the TakeOffCutScene class.
//
//////////////////////////////////////////////////////////////////////

#include "TakeOffCutScene.h"
#include "World.h"
#include "Camera.h"
#include "ChaseCamera.h"
#include "Constants.h"
#include "MusicManager.h"
#include "SoundFactory.h"
#include "LightManager.h"
#include <iostream.h>
#include "Settings.h"
#include "glerr.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TakeOffCutScene::TakeOffCutScene()
{
  hangar = new Object3d( new mesh("hangerc.mesh.bin"), 0.0f, 0.0f, 0.0f , 0.0f, 0.0f, 0.0f);
  hangar->setVelocity(0.0f);
  player = NULL;
  duration = 9000;
  starttime = 0;
  inited = false;
  if(Settings::getSettings()->getDynamicLights() )
  {
    light.setLocation(Coordinate(0.0f, 0.0f, 0.0f));
    LightManager::getInstance()->addLocalLight(&light);
  }
  else
  {
	old_ambientlight = LightManager::getInstance()->getAmbientLight();
	new_ambientlight = new AmbientLight(Color(1.0f, 1.0f, 1.0f));
	LightManager::getInstance()->setAmbientLight(new_ambientlight );
  }
}

TakeOffCutScene::~TakeOffCutScene()
{
  delete hangar;//???
  player->removeThrusterLights();
}

bool TakeOffCutScene::isDone()
{
  TimeValue currenttime = Time::getCurrentTime();
  bool done;

   if(starttime == 0)
   {
     return true;
   }
   done = currenttime - starttime > duration;
   if(done)
   {
       World::getWorld().remove(hangar);
       World::getWorld().remove(player);
       hangar->setDestroyed();
       if(Settings::getSettings()->getDynamicLights() )
	   {
         LightManager::getInstance()->removeLocalLight(&light);
	   }
	   else
	   {	
         LightManager::getInstance()->setAmbientLight(old_ambientlight);		  
		 delete new_ambientlight;
	   }
       player->disableShadow();
   }
   return done;
}

void TakeOffCutScene::start()
{

  starttime = Time::getCurrentTime();

  World::getWorld().add(hangar);

  Camera::setCamera(ChaseCamera::getInstance() );
  player->setOrientation(Coordinate(0.0f, 0.0f, 1.0f), 
                         Coordinate(0.0f, 1.0f, 0.0f),
                         Coordinate(1.0f, 0.0f, 0.0f) );
  

  float y = player->getExtents().y * -.5f;

  player->setLocation(Coordinate(0.0f, y, -11.0f) );
  player->setShadowLocation(hangar->getExtents().y * -.5f);
  player->setVelocity(0.0f);
  player->setDesiredVelocity(.1f);
  MusicManager::getInstance()->takeoff();
  SoundEffect * effect = SoundFactory::getSoundFactory()->getTakeoffSound();
  effect->play();
}


void TakeOffCutScene::setPlayer(Object3d * newplayer)
{
  player = (ship *) newplayer;
}

bool TakeOffCutScene::instanceof(char classname[ ])
{
  return  strcmp(classname, "TakeOffCutScene") == 0;
}

void TakeOffCutScene::advance()
{
  float percent;
  TimeValue currenttime = Time::getCurrentTime();
  Color red(.65f, 0.0f, 0.0f);
  Color white(.65f, 0.65f, 0.65f);
  Color c;
  int side;
  int intpart;

  if(!inited && currenttime > starttime+500 )
  {
    inited =  true;
  }

  if(Settings::getSettings()->getDynamicLights() )
  {

    percent = (float)(currenttime - starttime) / (float)duration;
    intpart = 7.0f * percent;
    if(side % 2 == 1)
	{
       c = red.interpolate(white, percent* 7 - intpart);    
	}
    else 
	{
       c = white.interpolate(red, percent* 7 - intpart) ;    
	}
    light.setAmbient(c);
  }
//  cout << "play" << player->getcenter().z + player->getExtents().z * -.5f
//	   << "hang" << hangar->getExtents().z * .5f << endl;
  //if(player->getcenter().z + player->getExtents().z * .5f > hangar->getExtents().z * .5f)
  {
//    player->disableShadow();
//	light.setAmbient(white);
  }

}
 

void TakeOffCutScene::render()
{
  // setup stencil buffer 

  glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
  GLERRORCHECK
  glDepthMask(GL_FALSE);
  GLERRORCHECK

  // enable stencil buffer
  glEnable(GL_STENCIL_TEST);
  GLERRORCHECK
  glClear(GL_STENCIL_BUFFER_BIT);
  GLERRORCHECK
  // write a one to the stencil buffer everywhere we are about to draw
  glStencilFunc(GL_ALWAYS, 1, 0xFFFFFFFF);
  GLERRORCHECK
  // this is to always pass a one to the stencil buffer where we draw
  glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
  GLERRORCHECK
				
// hangar render
  glBegin(GL_QUADS);
    float x = 17.5;
	float y = -4.65;
	float z = 27.04 - 10.5;
    glVertex3f(x, y, z);
	glVertex3f(x, y, -z);
    glVertex3f(-x, y, -z);
    glVertex3f(-x, y, z);	
  glEnd();

	 
	  // turn the color and depth buffers back on
  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
  GLERRORCHECK
  glDepthMask(GL_TRUE);
  GLERRORCHECK
  glDisable(GL_STENCIL_TEST);
  GLERRORCHECK
}
