// Targetable.cpp: implementation of the Targetable class.
//
//////////////////////////////////////////////////////////////////////
#include <GL/glew.h>
#include "glut.h"

#include "Targetable.h"
#include "AIBehavior.h"
#include "ship.h"
#include "Mission.h"
#include <iostream.h>
#include "Settings.h"
#include "Circle.h"
#include "glerr.h"
#include "PositionHistory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Targetable::Targetable(Targetable & that)
  :Object3d(that),
    image(that.image),
    name(that.name), 
    armor(that.armor),
    shields(that.shields),
    ai(that.ai),
	target(NULL),
	targeted_component(-1)
 
{
  initHistory();
}


Targetable::Targetable(mesh * newmesh,  
        shipareas newshields, shipareas newarmor,
        float newvelocity, float newroll, float newpitch, float newyaw, 
		float newacceleration, float newangular_acceleration,
        Image * newimage, string newname)
  : Object3d(newmesh, newvelocity, newroll, newpitch, newyaw,
      newacceleration, newangular_acceleration),
    image(newimage),
    name(newname), 
    armor(newarmor),
    shields(newshields),
    ai(NULL),
	target(NULL),
	targeted_component(-1)
{
  initHistory();
}

Targetable::Targetable(mesh * newmesh,  Coordinate location, Coordinate heading, Coordinate up, Coordinate left,
        shipareas newshields, shipareas newarmor,
        float newvelocity, float newroll, float newpitch, float newyaw, 
		float newacceleration, float newangular_acceleration,
        Image * newimage, string newname)
  : Object3d(newmesh, location, heading, up, left, newvelocity, newroll, newpitch, 
      newyaw, newacceleration, newangular_acceleration),
    image(newimage),
    name(newname), 
    armor(newarmor),
    shields(newshields),
    ai(NULL),
	target(NULL),
	targeted_component(-1)

{
  initHistory();

}


Targetable::~Targetable()
{

}

shipareas Targetable::getArmor()
{
  return armor;
}

shipareas Targetable::getShields()
{
  return shields;
}

Image * Targetable::getImage()
{
  return image;
}

string Targetable::getName()
{
  return name;
}


/*bool Targetable::isShip()
{
  return true;
}
*/

void Targetable::advance()
{
  Targetable * target = getTarget();

  shields.recharge();
  if(ai != NULL && target == NULL )
  {
    if( instanceof(INST_SHIP) )
    {
       ((ship *)this)->targetClosest();
    }
  }
  // this is the case for a player who has not locked a target
  if(ai == NULL )
  {
    if( instanceof(INST_SHIP) )
	{

      ship  * s = (ship *)this;
	  if(! s->isTargetLockSelected() )
	  {
        s->targetClosestNoIff();
	  }
	}
  }
  if(ai != NULL && target != NULL)
  {
    ai->move(this, target);
  }
  Object3d::advance();
  Coordinate recordlocation = getcenter() + getHeading() * (getExtents().z * -.7) ;
  history->update(recordlocation );
}

bool Targetable::instanceof(char classname [ ])
{
  if(0 == strcmp(classname, "Targetable")) 
    return true;
  return Object3d::instanceof(classname);
}	


void Targetable::setAI(AIBehavior * newai)
{
  ai = newai;
}

bool Targetable::hasAI()
{
  return ai != NULL;
}


void Targetable::renderBox()
{
 /*
            A                      B
            /--------------------/
           /                    / |
          C--------------------D
          | E                   | |
          |                     |F|
          G--------------------H/

 */
  ship * player = Mission::getCurrentMission()->getPlayer();
  if(player == NULL)
    return;
  Color color = getRadarColor(player);
  Coordinate center = getMeshCenter();

    Coordinate extents = getExtents();

  extents.x /= 2.0f;
  extents.y /= 2.0f;
  extents.z /= 2.0f;

  Coordinate a(-extents.x, extents.y, extents.z);
  Coordinate b(extents.x, extents.y, extents.z);
  Coordinate c(-extents.x, extents.y, -extents.z);
  Coordinate d(extents.x, extents.y, -extents.z);
  Coordinate e(-extents.x, -extents.y, extents.z);
  Coordinate f(extents.x, -extents.y, extents.z);
  Coordinate g(-extents.x, -extents.y, -extents.z);
  Coordinate h(extents.x, -extents.y, -extents.z);

  a = a + center;
  b = b + center;
  c = c + center;
  d = d + center;
  e = e + center;
  f = f + center;
  g = g + center;
  h = h + center;

  glColor3f(color.r, color.g, color.b);

  glBegin(GL_LINES);
    glVertex3f(a.x, a.y, a.z);
    glVertex3f(b.x, b.y, b.z);

    glVertex3f(b.x, b.y, b.z);
    glVertex3f(d.x, d.y, d.z);

    glVertex3f(d.x, d.y, d.z);
    glVertex3f(c.x, c.y, c.z);

    glVertex3f(c.x, c.y, c.z);
    glVertex3f(a.x, a.y, a.z);

    glVertex3f(e.x, e.y, e.z);
    glVertex3f(f.x, f.y, f.z);

    glVertex3f(f.x, f.y, f.z);
    glVertex3f(h.x, h.y, h.z);

    glVertex3f(h.x, h.y, h.z);
    glVertex3f(g.x, g.y, g.z);

    glVertex3f(g.x, g.y, g.z);
    glVertex3f(e.x, e.y, e.z);

    glVertex3f(a.x, a.y, a.z);
    glVertex3f(e.x, e.y, e.z);

    glVertex3f(c.x, c.y, c.z);
    glVertex3f(g.x, g.y, g.z);

    glVertex3f(b.x, b.y, b.z);
    glVertex3f(f.x, f.y, f.z);

    glVertex3f(d.x, d.y, d.z);
    glVertex3f(h.x, h.y, h.z);

  glEnd();
  glColor3f(1.0f, 1.0f, 1.0f);
}

void Targetable::render()
{
  Coordinate location; 
  Coordinate heading; 
  Coordinate up; 
  
  // force a render for capship
  if(isdestroyed() && !instanceof(INST_CAPITALSHIP) )
    return;

  Object3d::render();

  ship * player = Mission::getCurrentMission()->getPlayer();
  if(Camera::getCamera()->shouldRenderCockpit() )
  {
    /* don't move this above, because under certain cameras player could
       be NULL. But since we're rendering the cockpit, player
       cannot be null */
    if(player->getTarget() == this)
    {
//       glDisable(GL_LIGHTING);
       location = getLocation();
       heading = getHeading();
       up = getUp();
  
       glPushMatrix();
         glTranslatef(location.x, location.y, location.z);
         matchDOF(heading, up);    
		 if(Settings::getSettings()->getShowCollisionSpheres() )
		 {
		   glColor3f(1.0f, 1.0f, 1.0f);
		   glDisable(GL_LIGHTING);
           renderBox();
		   glEnable(GL_LIGHTING);
		 }
         calculateRectangle();
       glPopMatrix();
       if(Settings::getSettings()->getShowCollisionSpheres() )
	   {
         glPushMatrix();
		   glColor3f(1.0f, 1.0f, 1.0f);
		   glDisable(GL_LIGHTING);

	        location = getcenter();
            glTranslatef(location.x, location.y, location.z);
            renderSphere();
            glPopMatrix();
		   glEnable(GL_LIGHTING);
	   }
       glColor3f(1.0f, 1.0f, 1.0f);
//       glEnable(GL_LIGHTING);
    }
  }
  GLERRORCHECK
//  drawHistory();
}


// should not be deleted by the outside world
ReticleSquare Targetable::getReticleSquare()
{
  return square;
}

void Targetable::calculateRectangle()
{
  square = calculateRectangle(getMeshCenter(), getExtents() );
}


ReticleSquare Targetable::calculateRectangle(Coordinate center, Coordinate extents)
{
  ReticleSquare returnsquare;
  GLdouble objx;
  GLdouble objy;
  GLdouble objz;
  GLdouble modelMatrix[16];
  GLdouble projMatrix[16];
  GLdouble centerz;

  GLint viewport[4];  
  GLdouble winx; 
  GLdouble winy;
  GLdouble winz;
  Rect rectangle;
 
  objx = 0.0f;
  objy = 0.0f;
  objz = 0.0f;

  float minx, miny, maxx, maxy;
  GLdouble centerx, centery;
  maxx = maxy = -999999999;
  minx = miny = 999999999;
  int i;
  Coordinate current;


  extents.x /= 2.0f;
  extents.y /= 2.0f;
  extents.z /= 2.0f;

  Coordinate a(-extents.x, extents.y, extents.z);
  Coordinate b(extents.x, extents.y, extents.z);
  Coordinate c(-extents.x, extents.y, -extents.z);
  Coordinate d(extents.x, extents.y, -extents.z);
  Coordinate e(-extents.x, -extents.y, extents.z);
  Coordinate f(extents.x, -extents.y, extents.z);
  Coordinate g(-extents.x, -extents.y, -extents.z);
  Coordinate h(extents.x, -extents.y, -extents.z);

  a = a + center;
  b = b + center;
  c = c + center;
  d = d + center;
  e = e + center;
  f = f + center;
  g = g + center;
  h = h + center;

  Coordinate coordinates[8] = {a, b, c, d, e, f, g, h};

  int outsidecount = 0;

  glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
  glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
  glGetIntegerv(GL_VIEWPORT, viewport);



// get center location
  current = center;
  objx = current.x;
  objy = current.y;
  objz = current.z;  
  int result = gluProject(objx, objy, objz, modelMatrix, projMatrix, viewport, & centerx, & centery, & centerz);




  double zvalues[4];
  for(i=0; i < 8; i++)
  {
     current = coordinates[i];
    objx = current.x;
    objy = current.y;
    objz = current.z;
    int result = gluProject(objx, objy, objz, modelMatrix, projMatrix, viewport, & winx, & winy, & winz);
	if(result == GL_FALSE)
	{
      rectangle.width = 0;
	  rectangle.height =0;
	  rectangle.x = 0;
	  rectangle.y = 0;
//	  returnsquare.setInvalid();
	  return returnsquare;
	}
    if(winx > maxx)
    {
      maxx = winx;
	  zvalues[0] = winz;
    }
    else if(winx < minx)
    {
      minx = winx;
	  zvalues[1] = winz;
    }  
    if(winy > maxy)
    {
      maxy = winy;
	  zvalues[2] = winz;
    }
    else if(winy < miny)
    {
      miny = winy;
	  zvalues[3] = winz;
    }  
  }
  for(i=0; i < 4; i++)
  {
    if(zvalues[i] >= 1.0f )
	{
      outsidecount++;
	}
  }
//  if(outsidecount >= 1)
  if(centerz > 1.0f)
  {
	returnsquare.setInvalid();
	return returnsquare;

  }
  else
  {
//    cout << "outsidecount " << outsidecount << endl;
  }
  rectangle.x = minx;
  rectangle.y = miny;
  rectangle.width = maxx-minx;
  rectangle.height = maxy-miny;
//  rectangle.centerx = centerx;
//  rectangle.centery = centery;
  returnsquare.set(centerx, centery, rectangle.width, rectangle.height);
  return returnsquare;
}

int Targetable::getTargetedCapshipComponentIndex()
{
  if(target->instanceof(INST_CAPITALSHIP) )
  {
    return targeted_component;
  }
  return -1;
}

void Targetable::setTarget(Targetable * targetable)
{
  target = targetable;
}

void Targetable::setTargetedComponent(int index)
{
  targeted_component = index;
}

void Targetable::renderSphere()
{
  glutWireSphere(getradius(), 10, 10);
}


bool Targetable::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_TARGETABLE);
}

void Targetable::advancePositionRecord()
{

}

void Targetable::initHistory()
{
  history = new PositionHistory(1.0f, 100);
}

void Targetable::drawHistory()
{
  int i;
  Coordinate coord;

  glDisable(GL_LIGHTING);
  glColor3f(1.0f, 1.0f, 1.0f);
  glBegin(GL_LINE_STRIP );
  for(i=0; i < history->getNumElements() ; i++)
  {
	 coord = history->getElement(i);
	 glVertex3f(coord.x, coord.y, coord.z);
  }
  glEnd();
  glEnable(GL_LIGHTING);
}
