// TextureFactory.cpp: implementation of the TextureFactory class.
//
//////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include "TextureFactory.h"
#include "TextureImpl.h"
#include "TransparentTexture.h"
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include "ImageFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HashTable TextureFactory::hashtable(400);

int TextureFactory::numtextures = 0;
int TextureFactory::maxcompression = -1;
int TextureFactory::maxanisotropy = -1;


Texture * TextureFactory::garbage[800];

Texture * TextureFactory::getTexture(char filename [ ])
{
  TextureImpl * newtexture;

   if(hashtable.hasKey(filename) )
   {
     return (Texture *) hashtable.getItem(filename);
   }


  ImageFile imagefile(filename, false);
  newtexture = new TextureImpl(imagefile.getData(), imagefile.getWidth(), 
	  imagefile.getHeight()  );

//  the bits are freed in the texture constructor
  garbage[numtextures] = newtexture;
  numtextures++;
  hashtable.addItem(filename, newtexture);
  return newtexture;
}

void TextureFactory::clear()
{
  int i;

  for(i=0; i < numtextures; i++)
  {
     delete garbage[i];
  }
//  cout << numtextures << endl;
  numtextures = 0;
  hashtable.clear();
}


/* if the image already has an alpha channel, ignore transparentcolor */
Texture * TextureFactory::getTransparentTexture(char filename [ ], Color transparentcolor)
{
  Texture * newtexture;

   if(hashtable.hasKey(filename) )
   {
     return (Texture *) hashtable.getItem(filename);
   }

  ImageFile imagefile(filename, false);
  if(imagefile.hasAlpha() )
  {
    ImageFile imagefile(filename, true);
    newtexture = new TransparentTexture(imagefile.getData(), imagefile.getWidth(), imagefile.getHeight() );    
  }
  else
  {
    newtexture = new TransparentTexture(imagefile.getData(), imagefile.getWidth(), imagefile.getHeight(), transparentcolor );
  }

  garbage[numtextures] = newtexture;
  numtextures++;
  hashtable.addItem(filename, newtexture);
  return newtexture;
}

Texture * TextureFactory::getTransparentTexture(char filename [ ])
{
   if(hashtable.hasKey(filename) )
   {
     return (Texture *) hashtable.getItem(filename);
   }

  ImageFile imagefile(filename, true);
  Texture * newtexture = new TransparentTexture(imagefile.getData(), imagefile.getWidth(), imagefile.getHeight());
  garbage[numtextures] = newtexture;
      numtextures++;
    hashtable.addItem(filename, newtexture);
  return newtexture;
}


Texture * TextureFactory::getTransparentTextureAveraged(char filename [ ])
{
   if(hashtable.hasKey(filename) )
   {
     return (Texture *) hashtable.getItem(filename);
   }

  ImageFile imagefile(filename, false);
  Texture * newtexture = new TransparentTexture(imagefile.getData(), imagefile.getWidth(), imagefile.getHeight(), 1.0f);
  garbage[numtextures] = newtexture;
      numtextures++;
    hashtable.addItem(filename, newtexture);
  return newtexture;
}

int TextureFactory::getMaxTextureCompression()
{
  if(maxcompression == -1)
  {
	if(GLEW_ARB_texture_compression)
	{
		if(GLEW_EXT_texture_compression_s3tc)
		{
           maxcompression = 2;
		}
		else
		{
           maxcompression = 1;
		}
	}
	else
	{
           maxcompression = 0;
	}
  }
  return maxcompression;
}

int TextureFactory::getMaxAnisotropy()
{
  if(maxanisotropy == -1)
  {
    if(GLEW_EXT_texture_filter_anisotropic)
	{
	  glGetIntegerv( GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &maxanisotropy); 
	}
    else
	{
       maxanisotropy = 1;
	}
  }
  return maxanisotropy;

}
