// TextureFont.cpp: implementation of the TextureFont class.
//
//////////////////////////////////////////////////////////////////////

#include "TextureFont.h"
#include "TextureFactory.h"
#include "Texture.h"
#include <stdarg.h>
#include "FontMetrics.h"

#define NUMCHARS 256
#define MAX_STRING 80

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TextureFont::TextureFont(char bmpname[], char datname[])
: metric(datname)
{
	float	cx;										// Holds Our X Character Coord
	float	cy;										// Holds Our Y Character Coord
	int		loop;
	float translation;
	int width;
	float fwidth;


	texture = TextureFactory::getTransparentTexture(bmpname, Color(0.0f, 0.0f, 0.0f));
	width = 16;
	fwidth = width;
    mydisplaylist = glGenLists(NUMCHARS);

	texture->render();
	for (loop=0; loop<NUMCHARS; loop++)
	{
		cx=((float)(loop%width))/fwidth;				// X Position Of Current Character
		cy=((float)(loop/width))/fwidth;				// Y Position Of Current Character

        int mywidth = 32;
		int height = 32;
		glNewList(mydisplaylist+loop, GL_COMPILE);	// Start Building A List
		    if(loop != 0)
			{
		    translation = metric.a[loop];
		    glTranslatef(translation,0,0);
			}
	        texture->render();
			glBegin(GL_QUADS);						// Use A Quad For Each Character
				glTexCoord2f(cx,1-cy-0.0625f);		// Texture Coord (Bottom Left)
				glVertex2i(0,0);					// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,1-cy-0.0625f);// Texture Coord (Bottom Right)
				glVertex2i(mywidth,0);					// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,1-cy);		// Texture Coord (Top Right)
				glVertex2i(mywidth, height);					// Vertex Coord (Top Right)
				glTexCoord2f(cx,1-cy);				// Texture Coord (Top Left)
				glVertex2i(0,height);					// Vertex Coord (Top Left)
			glEnd();								// Done Building Our Quad (Character)
//			glTranslatef(10.0f,0,0);				// Move To The Right Of The Character
			translation = metric.b[loop] + metric.c[loop];
		    glTranslatef(translation,0,0);
		glEndList();								// Done Building The Display List
	}
}

TextureFont::~TextureFont()
{
  glDeleteLists(mydisplaylist, NUMCHARS);
}

void TextureFont::printf(float x, float y, char format [ ], ...)
{
  va_list	ap;			/* Argument pointer */
  char		s[MAX_STRING + 1];	/* Output string */


  if (format == NULL)
    return;

  va_start(ap, format);
  vsprintf(s, format, ap);
  va_end(ap);
  print(x, y, s);
}

void TextureFont::print(float x, float y, char text [ ])
{
    int ix = x + .5f;
	int iy = y + .5f;
	// round, so we don't draw fonts in between true pixels 
	x = ix;
	y = iy;

    int oldmode;
    glGetTexEnviv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, &oldmode);


    glListBase(mydisplaylist);
	glEnable(GL_TEXTURE_2D );
	texture->render();
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	texture->render();
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glPushMatrix();
      glTranslatef(x, y - 12.0f, 0);	
//	  glScalef(2.0f, 2.0f, 0.0f);
	  glCallLists(strlen(text), GL_BYTE, text);	
	glPopMatrix();
	glDisable(GL_TEXTURE_2D );
	glDisable(GL_BLEND);
	glListBase(0);
//    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, oldmode);
}

int TextureFont::getWidth(char text [ ])
{
	int length;
	int i;
	int size;

	length = strlen(text);
	size = 0;

	for(i=0; i < length; i++)
	{
		if(i != 0)
		{
		  size += metric.c[text[i - 1]];
		}
		size += metric.a[text[i] ];
		size += metric.b[text[i] ];
	}
    return size;
}

int TextureFont::getHeight(char text [ ])
{
  return getWidth("W") * 1.2f;
}

