// TextureImpl.cpp: implementation of the Texture class.
//
//////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include "TextureImpl.h"
#include <malloc.h>
#include "Settings.h"
#include "FileNotFoundException.h"
#include "glerr.h"
#include "Settings.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TextureImpl::TextureImpl(GLuint display_list)
  : mydisplaylist(display_list)
{

}

void TextureImpl::create(GLubyte * bits, int newwidth, int newheight)
{
  create(bits, newwidth, newheight, GL_RGB, 
	  GL_COMPRESSED_RGB_ARB, GL_COMPRESSED_RGB_S3TC_DXT1_EXT, 3);
}

void TextureImpl::create(GLubyte * bits, int newwidth, int newheight, 
						 GLenum format,  GLenum compressedformat, GLenum s3tccompressedformat, 
						 int channels)
{
	width = newwidth;
	height = newheight;

    checkPowerOfTwo(width);
	checkPowerOfTwo(height);


	if(GLEW_EXT_texture_filter_anisotropic)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 
			Settings::getSettings()->getAnisotropy());
	}
    
	glGenTextures(1, &mydisplaylist);
    glBindTexture(GL_TEXTURE_2D, mydisplaylist);

    glPixelStorei(GL_UNPACK_ALIGNMENT, 4);	/* Force 4-byte alignment */
    glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
    glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
    glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);

	GLenum newformat = selectInternalFormat(format, compressedformat, s3tccompressedformat);
	if(newformat != format)
	{
	  channels = newformat;
	}

    if(Settings::getSettings()->getMipMap() )
    {
       glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
       glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
       if(GLEW_SGIS_generate_mipmap)
	   {
          glTexParameteri( GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_TRUE );
		  GLERRORCHECK
          glTexImage2D(GL_TEXTURE_2D, 0, channels, width, height, 0,
                 format, GL_UNSIGNED_BYTE, bits);
          GLERRORCHECK
	   }
	   else
	   {
         gluBuild2DMipmaps(GL_TEXTURE_2D, channels, width, height, format, GL_UNSIGNED_BYTE, bits); 
	   }
    }
    else
    {

       glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
       GLERRORCHECK
       glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
       GLERRORCHECK
       glTexImage2D(GL_TEXTURE_2D, 0, channels, width, height, 0,
                 format, GL_UNSIGNED_BYTE, bits);
	   GLERRORCHECK

    }
///    free(bits);
	GLERRORCHECK


}

TextureImpl::TextureImpl(GLubyte * bits, int width, int height)
{
	create(bits, width, height);
}

/*TextureImpl::TextureImpl(GLubyte * bits, int newwidth, int newheight, int target)
{ 
}
*/

TextureImpl::~TextureImpl()
{
  if(mydisplaylist != -1)
  {
    glDeleteTextures(1, &mydisplaylist);
  }
  GLERRORCHECK
}

void TextureImpl::render()
{
  glBindTexture(GL_TEXTURE_2D, mydisplaylist);
  GLERRORCHECK
}



void TextureImpl::checkPowerOfTwo(int number)
{
  if(number != 1 && 
	 number != 2 && 
	 number != 4 && 
	 number != 8 &&
	 number != 16 &&
	 number != 32 &&
	 number != 64 &&
	 number != 128 &&
	 number != 256 &&
	 number != 512 &&
	 number != 1024 &&
	 number != 2048 &&
	 number != 4096)
  {
    printf("Illegal image, not a power of 2\n");
  }

}

void TextureImpl::recycle(GLubyte * bits, int width, int height)
{
  render();
  glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE, bits);

}

TextureImpl::TextureImpl(int nwidth, int nheight)
: width(nwidth), height(nheight)
{
  

}

int TextureImpl::getWidth()
{
  return width;
}

int TextureImpl::getHeight()
{
  return height;
}

GLenum TextureImpl::selectInternalFormat(GLenum format, GLenum compressedformat, GLenum s3tccompressedformat)
{
	int texcompression = Settings::getSettings()->getTextureCompression();
	if(texcompression == 2)
	{
	   return s3tccompressedformat;
	}
	else if(texcompression == 1)
	{
		return compressedformat;
	}
	else
	{
	   return format;
	}
}
