// TransparentTexture.cpp: implementation of the TransparentTexture class.
//
//////////////////////////////////////////////////////////////////////

#include <GL/glew.h>
#include "Settings.h"
#include "TransparentTexture.h"
#include <stdio.h>
#include "glerr.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TransparentTexture::TransparentTexture(GLubyte * bits, int width, int height, Color transparent)
  : TextureImpl(width, height)
{
   int length, i;
   GLubyte * newbuffer;
   GLubyte * bmpbuffer = bits;
   length = width * height;
   newbuffer = new GLubyte[length * 4 ];

   checkPowerOfTwo(width);
   checkPowerOfTwo(height);

  
   for(i=0; i < length;i++)
   {
      GLubyte & r = bmpbuffer[i * 3];
      GLubyte & g = bmpbuffer[i * 3 + 1];
      GLubyte & b = bmpbuffer[i * 3 + 2];


      newbuffer[i*4] = r;
      newbuffer[i*4 + 1] = g;
      newbuffer[i*4 + 2] = b;
    
      float rf, gf, bf;

      rf = (float)r/(float)0xff;
      gf = (float)g/(float)0xff;
      bf = (float)b/(float)0xff;
      
      if(rf == transparent.r && gf == transparent.g &&  bf == transparent.b)
      {
         newbuffer[i*4 + 3] = 0x00;
      }
      else
      {
         newbuffer[i*4 + 3] = 0xFF;
      }          
   }     
   create(newbuffer, width, height, 
	   GL_RGBA, GL_COMPRESSED_RGBA_ARB, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT , 4);
   delete [] newbuffer;
}

TransparentTexture::TransparentTexture(GLubyte * bits, int width, int height, float alphascale)
  : TextureImpl(width, height)
{
   int length, i;
   GLubyte * newbuffer;
   GLubyte * bmpbuffer = bits;
   length = width * height;
   newbuffer = new GLubyte[length * 4 ];

   checkPowerOfTwo(width);
   checkPowerOfTwo(height);
  
   for(i=0; i < length;i++)
   {
      GLubyte & r = bmpbuffer[i * 3];
      GLubyte & g = bmpbuffer[i * 3 + 1];
      GLubyte & b = bmpbuffer[i * 3 + 2];


      newbuffer[i*4] = r;
      newbuffer[i*4 + 1] = g;
      newbuffer[i*4 + 2] = b;
    
      float rf, gf, bf;

      rf = (float)r/(float)0xff;
      gf = (float)g/(float)0xff;
      bf = (float)b/(float)0xff;

      
      newbuffer[i*4 + 3] = (rf + gf + bf)* 255.0f/3.0f * alphascale;
   }     
   create(newbuffer, width, height, 
	   GL_RGBA, GL_COMPRESSED_RGBA_ARB, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, 4);
   delete [] newbuffer;

}


TransparentTexture::~TransparentTexture()
{

}

// must have an alpha channel
TransparentTexture::TransparentTexture(GLubyte * bits, int width, int height)
  : TextureImpl(width, height)
{
  create(bits, width, height, 
	  GL_RGBA, GL_COMPRESSED_RGBA_ARB, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, 4);
}
