// Turret.cpp: implementation of the Turret class.
//
//////////////////////////////////////////////////////////////////////

#include "Turret.h"
#include "glut.h"
#include "coldet.h"
#include "Constants.h"
#include "Time.h"
#include "Math.h"
#include "weaponfactory.h"
#include <vector>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Turret::Turret(std::string name, mesh * undamaged, mesh * damaged, mesh * newbarrel, int hitpoints,
			   Coordinate translation, Coordinate newheading, Coordinate newup, Coordinate newside,
			   Coordinate bartranslation, Coordinate barnewheading, Coordinate barnewup, Coordinate barnewside,
			   std::vector<mount *> * turretmounts)
: Component(name, undamaged, damaged, hitpoints),
  barrel(newbarrel),
  mounts(turretmounts)
{
  setLocation(translation);
  setOrientation(newheading, newup, newside);
  getCollisionModel();
  int i;

  for(i=0; i < 16; i++)
    matrix[i] = 0.0;

  matrix[0] = matrix[4] = matrix[8] = matrix[12] = 1.0;
  initialized = false;
/*mesh * newmesh, float velocity, float newroll, float newpitch, float newyaw, float new_acceleration, float new_angular_acceleration
*/

  float omega = 400.0f * Constants::getAngleFactor();
  barrelobj = new Object3d(newbarrel, 0.0 /* velocity */,
	  omega, omega, omega, 0.0f/* accel */, 0.0f /* alpha*/);
  barrelobj->setLocation(bartranslation);
  barheadingo = barnewheading;
  barupo = barnewup;
  barsideo = barnewside;

  barrelobj->setOrientation(barnewheading, barnewup, barnewside);
  firetime = 0;
  if(mounts->size() != 0)
  {
    mount * mnt = (*mounts)[0];
    weapondescription * description=  weaponfactory::getweaponfactory().
	     getweapondescription(mnt->getWeaponNumber() );
    delay = description->getRefireDelay();
  }
}

Turret::~Turret()
{
  delete barrelobj;
}

void Turret::render()
{
  Component::render();
  if((barrel != NULL) && (! isDestroyed() ) )
  {
	glPushMatrix();
      pushBarrelTransform();
      barrel->render();
    glPopMatrix();
  }
}

Turret::Turret(Turret & copy)
: Component(copy),
  barrel(copy.barrel),
  barrelobj(copy.barrelobj)
{
  setLocation(copy.getLocation() );
  setOrientation(copy.getHeading(), copy.getUp(), copy.getleft() );
  int i;
  for(i=0; i < 16; i++)
  matrix[i] = 0.0;
  matrix[0] = matrix[4] = matrix[8] = matrix[12] = 1.0;
  initialized = false;
  barrelobj->setLocation(copy.barrelobj->getLocation() );
  barrelobj->setOrientation(copy.barheadingo,
						copy.barupo,
						copy.barsideo );
  mounts = new std::vector<mount *>();
  for(i=0; i < copy.mounts->size(); i++)
  {
	mount * m = (*(copy.mounts))[i];
    mounts->push_back( m );
  }
  firetime = 0;
  delay = copy.delay;
}


bool Turret::isInited()
{
  return   initialized ;
}

bool Turret::instanceof(char name [ ])
{
  if(strcmp(name, "Turret") == 0)
	  return true;
  return Component::instanceof(name);
}

void Turret::setBarrelMatrix(float newmatrix [ 16 ])
{
  int i;
   
  for(i=0; i < 16; i++)
  {
	barrelmatrix[i] = newmatrix[i];
  }
}

void Turret::getBarrelMatrix(float newmatrix [ 16 ])
{
  int i;
   
  for(i=0; i < 16; i++)
  {
	newmatrix[i] = barrelmatrix[i];
  }
  
}

CollisionModel3D * Turret::getBarrelCollisionModel()
{
  return barrelobj->getCollisionModel();
}

void Turret::fire()
{
  int i;
  Coordinate location(0.0f, 0.0f, 0.0f);
  Coordinate heading(0.0f, 0.0f, 1.0f);
  Coordinate up(0.0f, 1.0f, 0.0f);
  Coordinate side(1.0f, 0.0f, 0.0f);
  float rmatrix[16];
  mount * mnt;
  TimeValue now;
  
  now = Time::getCurrentTime();
  if(now - firetime <= delay)
  {
    return;
  }
  firetime = now;

  for(i=0; i < 16; i++)
  {
    rmatrix[i] = barrelmatrix[i];
  }
  rmatrix[12] = 0.0f;
  rmatrix[13] = 0.0f;
  rmatrix[14] = 0.0f;


  heading = heading.multiplybyglmatrix(rmatrix);
  up = up.multiplybyglmatrix(rmatrix);
  side = side.multiplybyglmatrix(rmatrix);

  heading.normalize();
  up.normalize();
  side.normalize();
  for(i=0; i < mounts->size(); i++)
  {
    mnt = (*mounts)[i];
	location = mnt->getLocation();
    location = location.multiplybyglmatrix(barrelmatrix);
    weapon * weapon = weaponfactory::getweaponfactory().addweapon(mnt->getWeaponNumber(),
		location, heading, up , side);
    weapon->setcanthit(this);
  }

}

Object3d * Turret::getBarrelObject()
{
	return barrelobj;
}


bool Turret::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_COMPONENT | INST_TURRET);
}

void Turret::setMatrix(float newmatrix [ 16 ])
{
	Component::setMatrix(newmatrix);
	initialized = true;
}

void Turret::advance()
{
	Component::advance();
}