// TurretBallSocket.cpp: implementation of the TurretBallSocket class.
//
//////////////////////////////////////////////////////////////////////

#include "TurretBallSocket.h"
#include "Math.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TurretBallSocket::TurretBallSocket(std::string name, mesh * undamaged, mesh * damaged, mesh * newbarrel, int hitpoints,
			   Coordinate translation, Coordinate newheading, Coordinate newup, Coordinate newside,
			   Coordinate bartranslation, Coordinate barnewheading, Coordinate barnewup, Coordinate barnewside,
			   std::vector<mount *> * turretmounts, float newcone)
 : Turret(name, undamaged, damaged, newbarrel, hitpoints, translation, newheading, newup, newside,
	bartranslation, barnewheading, barnewup, barnewside, turretmounts)
{
  cone = newcone;
  origheading = barnewheading;
}

TurretBallSocket::~TurretBallSocket()
{

}

void TurretBallSocket::pushLocalTransform()
{
  Coordinate location = getLocation() ;

  glTranslatef(location.x, location.y, location.z);
  matchDOF(getHeading(), getUp() );  
  // for game conventions
  glRotatef(-90.0f, 0.0, 1.0, 0.0);
}

void TurretBallSocket::pushBarrelTransform()
{
  Object3d * barrelobj =getBarrelObject(); 

  Coordinate location = barrelobj->getLocation() ;

  glTranslatef(location.x, location.y, location.z);

  matchDOF(barrelobj->getHeading(), barrelobj->getUp() );  
  // for game conventions
  glRotatef(-90.0f, 0.0, 1.0, 0.0);
}

void TurretBallSocket::rotateSide(float percent)
{
  Coordinate oldup, oldside, oldheading;

  oldup = getBarrelObject()->getUp();
  oldside = getBarrelObject()->getleft();
  oldheading = getBarrelObject()->getHeading();

  /*
     cos(theta) = (a dot b) / (|a| |b|)
   

  */
  getBarrelObject()->rotateRightPercent(percent);

  Coordinate a = origheading;
  Coordinate b = getBarrelObject()->getHeading();

//  float lengthproduct = a.getlength() * b.getlength();
//  float costheta = (a.dot(b) ) / lengthproduct;
  float costheta = a.dot(b) ;
  if(costheta > 1.0f)
  {
	costheta = 1.0f;
  }
  if(costheta < 0 || costheta > 1)
  {
	assert(false);
  }
  float theta_radians = acos(costheta);
  float theta_degrees = Math::radtodeg(theta_radians);
  if(fabs_f(theta_degrees) >= cone * .5f)
  {
    // reset original values
	getBarrelObject()->setOrientation(oldheading, oldup, oldside);
  }

}

void TurretBallSocket::rotateUp(float percent)
{
  Coordinate oldup, oldside, oldheading;

  oldup = getBarrelObject()->getUp();
  oldside = getBarrelObject()->getleft();
  oldheading = getBarrelObject()->getHeading();


  /*
     cos(theta) = (a dot b) / (|a| |b|)
   
  */
  getBarrelObject()->bankRightPercent(percent);

  Coordinate a = origheading;
  Coordinate b = getBarrelObject()->getHeading();

//  float lengthproduct = a.getlength() * b.getlength();
  //float costheta = (a.dot(b) ) / lengthproduct;
  float costheta = a.dot(b) ;
  if(costheta > 1.0f)
  {
	costheta = 1.0f;
  }
  if(costheta < 0 || costheta > 1)
  {
	assert(false);
  }
  float theta_radians = acos(costheta);
  float theta_degrees = Math::radtodeg(theta_radians);
  if(fabs_f(theta_degrees) >= cone * .5f)
  {
    // reset original values
	getBarrelObject()->setOrientation(oldheading, oldup, oldside);
  }

}

bool TurretBallSocket::instanceof(char name [ ])
{
  if(strcmp(name, "TurretBallSocket") == 0)
	  return true;
  return Turret::instanceof(name);

}

TurretBallSocket::TurretBallSocket(TurretBallSocket & that)
: Turret(that)
{
  cone = that.cone;
  origheading = that.origheading;
}

void TurretBallSocket::advance()
{
	Turret::advance();
}
