// TurretJointed.cpp: implementation of the TurretJointed class.
//
//////////////////////////////////////////////////////////////////////

#include "TurretJointed.h"
#include "Constants.h"
#include "Targetable.h"
#include "Mission.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TurretJointed::TurretJointed(std::string name, mesh * undamaged, mesh * damaged, mesh * newbarrel, int hitpoints,
			   Coordinate translation, Coordinate newheading, Coordinate newup, Coordinate newside,
			   Coordinate bartranslation, Coordinate barnewheading, Coordinate barnewup, Coordinate barnewside,
			   std::vector<mount *> * turretmounts, Coordinate newconstraint)
 : Turret(name, undamaged, damaged, newbarrel, hitpoints, translation, newheading, newup, newside,
	bartranslation, barnewheading, barnewup, barnewside, turretmounts),
	constraint(newconstraint)

{
  turret_theta = 0.0f;
  barrel_theta = 0.0f;

}

TurretJointed::~TurretJointed()
{

}

void TurretJointed::rotateUp(float percent)
{
  angularvelocity = 400.0f * Constants::getAngleFactor()* Time::getTimeScale() ;
  barrel_theta += angularvelocity * percent;
  if(barrel_theta > 0.0f)
  {
     barrel_theta = 0.0f;
  }
  if(barrel_theta < -constraint.z)
  {
     barrel_theta = -constraint.z;
  }
}

void TurretJointed::rotateSide(float percent)
{
  angularvelocity = 400.0f * Constants::getAngleFactor()* Time::getTimeScale() ;
  turret_theta += angularvelocity * percent;
  if(turret_theta > constraint.x)
  {
	turret_theta = constraint.x;
  }
  if(turret_theta < -constraint.y)
  {
	turret_theta = -constraint.y;
  } 
}

void TurretJointed::pushBarrelTransform()
{
  Object3d * barrelobj =getBarrelObject(); 

  Coordinate location = barrelobj->getLocation() ;

  glTranslatef(location.x, location.y, location.z);

  matchDOF(barrelobj->getHeading(), barrelobj->getUp() );  
  // for game conventions
  glRotatef(-90.0f, 0.0, 1.0, 0.0);
  // the turrets movement turn
  glRotatef(barrel_theta, 1.0, 0.0, 0.0);
}

void TurretJointed::pushLocalTransform()
{
  Coordinate location = getLocation() ;

  glTranslatef(location.x, location.y, location.z);
  matchDOF(getHeading(), getUp() );  
  // for game conventions
  glRotatef(-90.0f, 0.0, 1.0, 0.0);
  // the turrets movement turn
  glRotatef(turret_theta, 0.0, 1.0, 0.0);

}

bool TurretJointed::instanceof(char name [ ])
{
  if(strcmp(name, "TurretJointed") == 0)
	  return true;
  return Turret::instanceof(name);
}

TurretJointed::TurretJointed(TurretJointed & that):
  Turret(that)
{
  turret_theta = 0.0f;
  barrel_theta = 0.0f;
  constraint = that.constraint;
}

void TurretJointed::advance()
{
	Turret::advance();
	ai();
}

void TurretJointed::ai()
{
	Coordinate myworldlocation;
    Coordinate tovector;    
    Coordinate heading(0.0f, 0.0f, 1.0f);
    Coordinate up(0.0f, 1.0f, 0.0f);
    Coordinate side(1.0f, 0.0f, 0.0f);
    Coordinate bheading(0.0f, 0.0f, 1.0f);
    Coordinate bup(0.0f, 1.0f, 0.0f);
    Coordinate bside(1.0f, 0.0f, 0.0f);

    float rmatrix[16];
	float bmatrix[16];
	float headingdotproduct;
	float sidedotproduct;
	float updotproduct;


	Targetable * target = Mission::getCurrentMission()->getPlayer();
	if(target == NULL)
	  {
	    return;
	  }

	myworldlocation = getWorldLocation();
	tovector = target->getLocation() - myworldlocation;

	// base
    getMatrix(rmatrix);
    rmatrix[12] = 0.0f;
    rmatrix[13] = 0.0f;
    rmatrix[14] = 0.0f;

    heading = heading.multiplybyglmatrix(rmatrix);
    up = up.multiplybyglmatrix(rmatrix);
    side = side.multiplybyglmatrix(rmatrix);

    tovector.normalize();
	sidedotproduct = tovector.dot(side);
	headingdotproduct = tovector.dot(heading);
	headingdotproduct = fabs_f(headingdotproduct);

    if( headingdotproduct > .9f)
	{
	   //deadband one axis, aim the barrel
/*	   getBarrelMatrix(bmatrix);
	   bmatrix[12] = 0.0f;
	   bmatrix[13] = 0.0f;
	   bmatrix[14] = 0.0f;

       bheading = heading.multiplybyglmatrix(rmatrix);
       bup = up.multiplybyglmatrix(rmatrix);
       bside = side.multiplybyglmatrix(rmatrix);
	   headingdotproduct = bheading.dot(tovector);
	   updotproduct = bup.dot(tovector);
*/
	   if(headingdotproduct > .9f)
	   {
		  // good heading, deadband
	     //		   fire();

	   }
/*	   else if(updotproduct > 0.0f)
	   {
          rotateUp(1.0f * (1.0f - headingdotproduct) );
	   }
	   else
	   {
          rotateUp(-1.0f * (1.0f - headingdotproduct) );
	   }	   
*/
	}
	else if(sidedotproduct > 0.0f)
	{
	   rotateUp(1.0f);
       rotateSide(1.0f * (1.0f - headingdotproduct) );
	}
	else
	{
	   rotateUp(1.0f);
       rotateSide(-1.0f * (1.0f - headingdotproduct)  );
	}

}
