// ViewingCone.cpp: implementation of the ViewingCone class.
//
//////////////////////////////////////////////////////////////////////

#include "ViewingCone.h"
#include "Math.h"
#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ViewingCone::ViewingCone(float screenwidth, float screenheight, float fieldofview)
{
// set the properties of the frustum cone vLookVector is the look vector from the view matrix in the 
// camera.  Position() returns the position of the camera.  
// fWidth is half the width of the screen (in pixels).  
// fHeight is half the height of the screen in pixels. 
// m_fFovRadians is the FOV of the frustum.

// calculate the length of the fov triangle
  float fHeight = .5f * screenheight;
  float fWidth = .5f * screenwidth;
  float m_fFovRadians = Math::degtorad(fieldofview );
  float fDepth  = fHeight / tan(m_fFovRadians * 0.5f);

// calculate the corner of the screen
  float fCorner = sqrt(fWidth * fWidth + fHeight * fHeight);

// now calculate the new fov
  fov = atan(fCorner / fDepth);
  float sine = sin(fov);
  sinreciprocal = 1.0f / sine;
  sinsquared = sine * sine;
  float cosine = cos(fov);
  cossquared = cosine * cosine;
}

ViewingCone::~ViewingCone()
{

}

void ViewingCone::update(Coordinate newcameraposition, Coordinate newcameraheading)
{
  cameraposition = newcameraposition;
  cameraheading = newcameraheading;
}

bool ViewingCone::isSphereVisible(Coordinate spherecenter, float radius, float radiussquared)
{
//  return true;

  Coordinate U = cameraposition - cameraheading * radius * sinreciprocal;
  Coordinate D = spherecenter - U;

  float dsquared = D.dot(D);
  float e = cameraheading.dot(D);
  if(e > 0.0f && e * e >= dsquared * cossquared)
  {
	Coordinate D2 = spherecenter - cameraposition;	
	float d2squared = D2.dot(D2);
	float e2 = - cameraheading.dot(D2);
	if(e2 > 0.0f && e2 * e2 >= d2squared * sinsquared)
	{
      return d2squared <= radiussquared;
	}
	return true;
  }
  return false;
}
