// World.cpp: implementation of the World class.
//
//////////////////////////////////////////////////////////////////////

#include "World.h"
#include "Object3d.h"
#include <math.h>
#include <iostream.h>
#include "WorldObjectWalker.h"
#include "Camera.h"
#include "Physics.h"
#include "ship.h"
#include "CapitalShip.h"
#include "Component.h"
#include "Turret.h"
#include "Asteroid.h"
#include "Explosion.h"
#include "Math.h"
#include "Decoy.h"
#include "Missile.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

World * World::theworld = NULL;

World::World()
   :front(), shiplist(), collision_enabled(true),
    explosionlist(),
	viewcone(640.0f, 480.0f, 45.0f)
{
  asteroidmodel = NULL;
}

World::~World()
{

}

World & World::getWorld()
{
  if(theworld != NULL)
     return *theworld;
  else
     return *(theworld = new World());
}

void World::add(Object3d * newobject)
{
   if(newobject == NULL)
   {
     return;
   }

   front.push_front(newobject);

   if(newobject->instanceof(INST_TARGETABLE) )
   {
	   shiplist.push_back((Targetable *)newobject);
   }
}

void World::remove(Object3d * object)
{
   if(object != NULL)
   {
	  front.remove(object);
   }
}

void World::advance()
{
  std::list<Object3d *>::iterator i;
  int index;
  Object3d * currentobject;

  notifyTargetedByMissiles();

  if(collision_enabled)
     collisionDetection();

  for(index = 0; index < explosionlist.size() ; index++)
  {
     currentobject = explosionlist[index];
     if(currentobject != NULL && (! currentobject->isdestroyed() ) )
        currentobject->advance();
  }

  for(i = front.begin(); i != front.end(); i++)
  {
     currentobject = *i;
     if(currentobject != NULL)
	 {
       currentobject->advance();
	 }
  }
  removedestroyed();
}

void World::render()
{
  std::list<Object3d *>::iterator i;
  Object3d * currentobject;
  bool visible;
  int index;



  Camera * camera = Camera::getCamera();

  viewcone.update(camera->getLocation(), camera->getHeading() );

  // render missiles after, since they have transparency 
  for(i = front.begin(); i != front.end() ; i++)
  {
     currentobject = *i;

	 visible = viewcone.isSphereVisible( 
		  currentobject->getLocation(),
		  currentobject->getradius(),
		  currentobject->getRadiusSquared() );


     if((visible && currentobject->shouldRender()) || currentobject->instanceof(INST_CAPITALSHIP ) )
	 {
		if( (! currentobject->instanceof(INST_MISSILE))  && (! currentobject->instanceof(INST_WEAPON) ) )
		{
          currentobject->render();
		}
	 }
  }
  int j;

  Coordinate cameralocation = camera->getLocation();

  for(j=0; j < asteroids.size(); j++)
  {
    Asteroid * ast = asteroids[j];
		
		
	float distance = cameralocation.distanceSquared(ast->getLocation() );
	
	const float viewsquared = 10000.0f * 10000.0f;
	if(distance < viewsquared )
	{
       visible = viewcone.isSphereVisible( 
		  ast->getLocation(),
		  ast->getradius(),
		  ast->getRadiusSquared() );
       if(visible)
	   {
   		  ast->render();
	   }
	}

  }


  for(i = front.begin(); i != front.end() ; i++)
  {
     currentobject = *i;

	if(currentobject->instanceof(INST_WEAPON) )
	{
         currentobject->render();
	}
  }

  for(i = front.begin(); i != front.end() ; i++)
  {
     currentobject = *i;

	if(currentobject->instanceof(INST_MISSILE) )
	{
         currentobject->render();
	}
  }


  for(index = 0; index < explosionlist.size(); index++)
  {
    currentobject = explosionlist[index];
	 if(( currentobject != NULL) && (! currentobject->isdestroyed() ) )
       currentobject->render();
  }

}

void World::removedestroyed()
{

  // decoys
  std::vector<Decoy *>::iterator theIterator;
  std::vector<Object3d *>::iterator expiterator;
  std::list<Object3d *>::iterator i;

   for (theIterator = decoys.begin(); theIterator != decoys.end();
         )
  {
	  Decoy * decoy =  *theIterator;
	  if(decoy->isdestroyed() )
	  {
        theIterator = decoys.erase(theIterator);
	  }
	  else
	  {
	    theIterator++;
	  }
  }

  Object3d * currentobject;


  for(i = front.begin(); i != front.end(); )
  {
     currentobject = *i;
     if(currentobject == NULL)
     {
		  i = front.erase(i);
     }
     else
     {
       if( currentobject->isdestroyed() && !(currentobject->instanceof(INST_CAPITALSHIP) ) )
	   {
         if(currentobject->instanceof(INST_TARGETABLE) )
            removeShipFromShipList((Targetable *) currentobject);
		 i = front.erase(i);
         // don't delete now, we could be using as a prototype if its targetable, plus the time factor
         if(! currentobject->instanceof(INST_TARGETABLE))
           delete currentobject;

	   }
       else
	   {
		   i++;
	   }
     }
  }

   for (expiterator = explosionlist.begin(); expiterator != explosionlist.end();
         )
  {
	  Object3d * myexp =  *expiterator;
	  if(myexp->isdestroyed() )
	  {
		delete myexp;
        expiterator = explosionlist.erase(expiterator);
	  }
	  else
	  {
	    expiterator++;
	  }
  }

}

void World::collisionDetection()
{
  std::list<Object3d *>::iterator i;
  Object3d * currentobject1;
  std::list<Object3d *>::iterator j;
  Object3d * currentobject2;
  Object3d * testcurrentobject1;
  Object3d * testcurrentobject2;

  for(i = front.begin(); i != front.end(); i++)
  {
     currentobject1 = *i;

     CollisionResult result = asteroidCollisionDetection(currentobject1);
	 if(result.collision_occured)
	 {
        // handled by the asteroid
	 }
	 else
	 {
		j = i;
		if(j != front.end() )
		{
		  j++; // skip self
		}
        for(; j != front.end(); j++)
		{
            currentobject2 = *j;
            if(currentobject1->instanceof(INST_WEAPON) && currentobject2->instanceof(INST_WEAPON) )
			{

			}
		    else if(currentobject1->instanceof(INST_EXPLOSION) || currentobject2->instanceof(INST_EXPLOSION) )
			{
			
			}
			else if(currentobject1->instanceof(INST_COMPONENT) || currentobject2->instanceof(INST_COMPONENT) )
			{
				assert(false);
/* 				cout << "Checking a component" << endl;
				cout << "Front size is" << front.size() << endl;
				dump();
*/
			}
            else
			{

			   testcurrentobject1 = currentobject1;
			   testcurrentobject2 = currentobject2;
               CollisionResult result = collisionBetween(currentobject1, currentobject2);
               if(result.collision_occured)
			   {
      			   bool ship1, ship2;
				   bool eject1, eject2;
			       ship1 = currentobject1->instanceof(INST_SHIP);
			       ship2 = currentobject2->instanceof(INST_SHIP);
				   eject1 = currentobject1->instanceof(INST_EJECTEDPOD);
				   eject2 = currentobject2->instanceof(INST_EJECTEDPOD);

                  if(ship1 && ship2 )
				  {
                     if(result.locationvalid)
					 {
                        doCollisionResponse((ship *) currentobject1, (ship *) currentobject2, result.location);
					 }
                     else
					 {
                        doCollisionResponse((ship *) currentobject1, (ship *) currentobject2);
					 }
				  }
			      else if(result.component != NULL)
				  {
				      // find which one is the capship, and replace it by the component
				      if(currentobject1->instanceof(INST_CAPITALSHIP) )
					  {
    					  testcurrentobject1 = result.component;
					  }
				      else
					  {
    					  testcurrentobject2 = result.component;
					  }
				  }
				  else if(ship1 && eject2)
				  {
					
				    doCollisionResponse((ship *)currentobject1, currentobject2->getLocation(), 4000.0f);
				  }					  
				  else if(ship2 && eject1)
				  {
				    doCollisionResponse((ship *)currentobject2, currentobject1->getLocation(), 4000.0f);
				  }
                  testcurrentobject1->onCollisionWith(testcurrentobject2);
                  testcurrentobject2->onCollisionWith(testcurrentobject1);
			   }
            }
        }
     }
  }
  removedestroyed();
}

World::CollisionResult World::collisionBetween(Object3d * obj1, Object3d * obj2)
{
   World::CollisionResult result;
   bool obj1capship;
   bool obj2capship;

   result.collision_occured = false;
   result.locationvalid = false;
   result.component = NULL;



   if(obj1 == obj2)
     return result;
   if(obj1->cantHit(obj2) || obj2->cantHit(obj1) )
     return result;

	 bool dummyflag;
	 obj1capship = obj1->instanceof(INST_CAPITALSHIP);
	 obj2capship = obj2->instanceof(INST_CAPITALSHIP);

	 if( obj1capship || obj2capship
		 )//|| obj1asteroid || obj2asteroid)
	 {
        dummyflag = true;
		 //dummyflag = false;
	 }
	 else
	 {
		dummyflag = false;
	 }

   bool spherecollision = hasSphereCollision(obj1, obj2);
   if(spherecollision)
   {

     // skip obb box if 
	 if(dummyflag )
	 {
	 }
	 else
	 {
		Coordinate basisa[3];
		Coordinate basisb[3];

	     basisa[0] = obj1->getleft();
	 	basisa[1] = obj1->getUp();
		basisa[2] = obj1->getHeading();

	     basisb[0] = obj2->getleft();
	     basisb[1] = obj2->getUp();
		basisb[2] = obj2->getHeading();

		result.collision_occured = OBBOverlap( obj1->getExtents()*.5f, obj1->getcenter(), basisa,
                        obj2->getExtents()*.5f, obj2->getcenter(), basisb
		);
	 }


     if(result.collision_occured || dummyflag ) 
     {
        // capships handle their own collision response in a separate function
        if(obj1capship  )
        {
          result = capshipCollision((CapitalShip *) obj1, obj2);
        }
        else if(obj2capship )
        {
          result  = capshipCollision((CapitalShip *) obj2, obj1);
        }
     }
     return result;
   }
   return result;
}




bool World::OBBOverlap(
   //A
   Coordinate a, //extents
   Coordinate Pa, //position
   Coordinate A[3], //orthonormal basis

   //B
   Coordinate b, //extents
   Coordinate Pb, //position
   Coordinate B[3] //orthonormal basis
)
{
   //translation, in parent frame
   Coordinate v = Pb - Pa;
   //translation, in A's frame
   Coordinate T( v.dot(A[0]), v.dot(A[1]), v.dot(A[2]) );
   //B's basis with respect to A's local frame
   float R[3][3];
   float ra, rb, t;
   long i, k;
   //calculate rotation matrix
   for( i=0 ; i<3 ; i++ )
      for( k=0 ; k<3 ; k++ )
         R[i][k] = A[i].dot(B[k]);
   /*ALGORITHM: Use the
   separating axis test for all
   15 potential
   separating axes. If a
   separating axis could not be
   found, the two
   boxes overlap. */
   //A's basis vectors
    ra = a.x;
    rb = b.x*fabs_f(R[0][0]) + b.y*fabs_f(R[0][1]) + b.z*fabs_f(R[0][2]);
    t = fabs_f( T.x );
    if( t > ra + rb )
       return false;
    ra = a.y;
    rb = b.x*fabs_f(R[1][0]) + b.y*fabs_f(R[1][1]) + b.z*fabs_f(R[1][2]);
    t = fabs_f( T.y );
    if( t > ra + rb )
       return false;

    ra = a.z;
    rb = b.x*fabs_f(R[2][0]) + b.y*fabs_f(R[2][1]) + b.z*fabs_f(R[2][2]);
    t = fabs_f( T.z );
    if( t > ra + rb )
       return false;


   //B's basis vectors
      ra = a.x*fabs_f(R[0][0]) + a.y*fabs_f(R[1][1]) + a.z*fabs_f(R[2][2]);
      rb = b.x;
      t = fabs_f( T.x*R[0][0] + T.y*R[1][0] + T.z*R[2][0] );
      if( t > ra + rb )
         return false;

      ra = a.x*fabs_f(R[0][1]) + a.y*fabs_f(R[1][1]) + a.z*fabs_f(R[2][1]);
      rb = b.y;
      t = fabs_f( T.x*R[0][1] + T.y*R[1][k] + T.z*R[2][1] );
      if( t > ra + rb )
         return false;


      ra = a.x*fabs_f(R[0][2]) + a.y*fabs_f(R[1][2]) + a.z*fabs_f(R[2][2]);
      rb = b.z;
      t = fabs_f( T.x*R[0][2] + T.y*R[1][2] + T.z*R[2][2] );
      if( t > ra + rb )
         return false;


   //9 cross products
   //L = A0 x B0
   ra =  a.y*fabs_f(R[2][0]) + a.z*fabs_f(R[1][0]);
   rb =  b.y*fabs_f(R[0][2]) + b.z*fabs_f(R[0][1]);
   t = fabs_f( T.z*R[1][0] - T.y*R[2][0] );
   if( t > ra + rb )
      return false;

   //L = A0 x B1
   ra =a.y*fabs_f(R[2][1]) + a.z*fabs_f(R[1][1]);
   rb =b.x*fabs_f(R[0][2]) + b.z*fabs_f(R[0][0]);
   t =  fabs_f( T.z*R[1][1] - T.y*R[2][1] );
   if( t > ra + rb )
      return false;

   //L = A0 x B2
   ra = a.y*fabs_f(R[2][2]) + a.z*fabs_f(R[1][2]);
   rb = b.x*fabs_f(R[0][1]) + b.y*fabs_f(R[0][0]);
   t = fabs_f( T.z*R[1][2] - T.y*R[2][2] );
   if( t > ra + rb )
      return false;
   //L = A1 x B0

   ra = a.x*fabs_f(R[2][0]) + a.z*fabs_f(R[0][0]);
   rb = b.y*fabs_f(R[1][2]) + b.z*fabs_f(R[1][1]);

   t = fabs_f( T.x*R[2][0] - T.z*R[0][0] );
   if( t > ra + rb )
      return false;

   //L = A1 x B1
   ra = a.x*fabs_f(R[2][1]) + a.z*fabs_f(R[0][1]);
   rb = b.x*fabs_f(R[1][2]) + b.z*fabs_f(R[1][0]);
   t = fabs_f( T.x*R[2][1] - T.z*R[0][1] );
   if( t > ra + rb )
      return false;

   //L = A1 x B2
   ra = a.x*fabs_f(R[2][2]) + a.z*fabs_f(R[0][2]);
   rb = b.x*fabs_f(R[1][1]) + b.y*fabs_f(R[1][0]);
   t = fabs_f( T.x*R[2][2] - T.z*R[0][2] );
   if( t > ra + rb )
      return false;

   //L = A2 x B0
   ra = a.x*fabs_f(R[1][0]) + a.y*fabs_f(R[0][0]);
   rb = b.y*fabs_f(R[2][2]) + b.z*fabs_f(R[2][1]);
   t = fabs_f( T.y*R[0][0] - T.x*R[1][0] );
   if( t > ra + rb )
      return false;
   //L = A2 x B1
   ra =  a.x*fabs_f(R[1][1]) + a.y*fabs_f(R[0][1]);
   rb =  b.x *fabs_f(R[2][2]) + b.z*fabs_f(R[2][0]);
   t = fabs_f( T.y*R[0][1] - T.x*R[1][1] );
   if( t > ra + rb )
      return false;

//L = A2 x B2
   ra = a.x*fabs_f(R[1][2]) + a.y*fabs_f(R[0][2]);
   rb = b.x*fabs_f(R[2][1]) + b.y*fabs_f(R[2][0]);
   t = fabs( T.y*R[0][2] - T.x*R[1][2] );
   if( t > ra + rb )
      return false;
   /*no separating axis found,
     the two boxes overlap */
   return true;
}

bool World::formsSeparatingAxis(Coordinate t, Coordinate l, float ra, float rb)
{
  return fabs(t.dot(l) ) > ra + rb;
}

float World::getRadiusOfProjection(Coordinate L, Coordinate extents, Coordinate heading, Coordinate left, Coordinate up)
{
  return extents.x * fabs_f(left.dot(L)) + extents.y * fabs_f(up.dot(L) ) + extents.z * fabs_f(heading.dot(L) );
}

bool World::bouningBoxOverlap(Object3d * obj1, Object3d * obj2)
{
   const int numaxis = 15;
   Coordinate axes[15];
   int i;

   axes[0] = obj1->getHeading();
   axes[1] = obj2->getHeading();
   axes[2] = obj1->getUp();
   axes[3] = obj2->getUp();
   axes[4] = obj1->getleft();
   axes[5] = obj2->getleft();
   axes[6] = obj1->getHeading().cross(obj2->getHeading());
   axes[7] = obj1->getHeading().cross(obj2->getUp());
   axes[8] = obj1->getHeading().cross(obj2->getleft());
   axes[9] = obj1->getUp().cross(obj2->getHeading());
   axes[10] = obj1->getUp().cross(obj2->getUp());
   axes[11] = obj1->getUp().cross(obj2->getleft());
   axes[12] = obj1->getleft().cross(obj2->getHeading());
   axes[13] = obj1->getleft().cross(obj2->getUp());
   axes[14] = obj1->getleft().cross(obj2->getleft());

   Coordinate t = obj1->getcenter() - obj2->getcenter();
   for(i=0; i < numaxis; i++)
   {
      Coordinate l = axes[i];
      float ra, rb;

      ra =  getRadiusOfProjection(l, obj1->getExtents(), obj1->getHeading(), obj1->getleft(), obj1->getUp() );
      rb =  getRadiusOfProjection(l, obj2->getExtents(), obj2->getHeading(), obj2->getleft(), obj2->getUp() );
      if(formsSeparatingAxis(t, l, ra, rb)     )
        return false;

   }
   return true;
}

Object3d * World::getNext(Object3d * current)
{
  int i;

  if(current == NULL)
  {
	if(shiplist.size() != 0)
	{
      return shiplist[0];
	}
	else
	{
	  return NULL;
	}
  }

  for(i = 0; i != shiplist.size(); i++)
  {
    if(shiplist[i] == current)
    {
	  return shiplist[(i + 1) % shiplist.size()] ;
    }
  }

  return NULL;
}

void World::enumerateShips(WorldObjectWalker * walker)
{
  int i;

  for(i = 0; i <shiplist.size() ; i++)
  {
     if(shiplist[i] != NULL)
      walker->objectIs(shiplist[i] );
  }
}

void World::removeShipFromShipList(Targetable * deadship)
{ 
  std::vector<Targetable *>::iterator theIterator;

  notifyTargetDestroyed(deadship);

  for (theIterator = shiplist.begin(); theIterator != shiplist.end();
         )
  {
	  Targetable * targetable =  *theIterator;
	  if(targetable == deadship )
	  {
        theIterator = shiplist.erase(theIterator);
	  }
	  else
	  {
	    theIterator++;
	  }
  }


}

void World::notifyTargetDestroyed(Targetable * target)
{
  int i;

  for(i = 0; i < shiplist.size(); i++)
  {
     Targetable * currentship = (Targetable *) shiplist[i];
     if( currentship != target)
     {
       if(currentship->getTarget() == target)
         currentship->targetDestroyed();

     }
  }

}

/*void World::setCamera(Camera * newcamera)
{
  camera = newcamera;
}
*/

void World::doCollisionResponse(ship * ship1, ship * ship2)
{
   Coordinate newvelocity1, newvelocity2;
   bool result;

   result = Physics::collisionResponse(ship1->getMass(), ship2->getMass(),
                              ship1->getLocation(), ship2->getLocation(),
                              ship1->getVelocityVector(),ship2->getVelocityVector(),
                              newvelocity1, newvelocity2);

   if(result)
   {
     ship1->setVelocityVector(newvelocity1);
     ship2->setVelocityVector(newvelocity2);
   }
}

void World::clear()
{
  front.clear();
  shiplist.clear();
  explosionlist.clear();
  asteroids.clear();
}

ship * World::getNearestTarget(ship * body, bool ignore_iff)
{
  int i;
  ship * nearest = NULL;
  float distance;


  for(i = 0; i < shiplist.size(); i++)
  {
     if(shiplist[i]->instanceof(INST_SHIP) )
     {
       ship * currentship = (ship *) shiplist[i];
       if( (ignore_iff && (body != currentship    ))
		   || areEnemies(body->getIffCode(), currentship->getIffCode() ) )
       {
          if(nearest == NULL)
          {
            nearest = currentship;
            distance = currentship->getLocation().distanceSquared(body->getLocation() );
          }
          else
          {
            float mydistance = currentship->getLocation().distanceSquared(body->getLocation() );
            if(mydistance < distance)
            {
               distance = mydistance;
               nearest = currentship;
            }
          }
       }
     }
  }
  return nearest;
}

World::CollisionResult World::capshipCollision(CapitalShip * capship, Object3d * object)
{
   float matrix1[16];
   float matrix2[16];
   CollisionModel3D * collisionmodel1 = capship->getCollisionModel();
   CollisionModel3D * collisionmodel2 = object->getCollisionModel();
   float origin[3];
   Coordinate location = object->getcenter();
   World::CollisionResult result;

   result.collision_occured = false;
   result.locationvalid = false;
   result.component = NULL;

   origin[0] = location.x;
   origin[1] = location.y;
   origin[2] = location.z;
 
   capship->getMatrix(matrix1);
   collisionmodel1->setTransform(matrix1);
   object->getMatrix(matrix2);
   collisionmodel2->setTransform(matrix2);
    

//   result.collision_occured = collisionmodel->sphereCollision(origin, object->getradius() );
   result.collision_occured = collisionmodel1->collision(collisionmodel2 );
   if(result.collision_occured)
   {
      float point[3];
      result.locationvalid = true;
      collisionmodel1->getCollisionPoint(point, false);
      result.location.x = point[0];
      result.location.y = point[1];
      result.location.z = point[2];      
   }
   // else try components
   else
   {
	    int component_index;

		for(component_index=0; component_index < capship->getNumComponentsOnly(); component_index++)
		{
			Component * component= capship->getComponent(component_index);
			CollisionModel3D * component_coll_model = component->getCollisionModel();
			if(component_coll_model != NULL)
			{
                component->getMatrix(matrix1);
				component_coll_model->setTransform(matrix1);
				result.collision_occured = component_coll_model->collision(collisionmodel2 );
				if(result.collision_occured)
				{
					float point[3];
	                result.locationvalid = true;
					component_coll_model->getCollisionPoint(point, false);
					result.location.x = point[0];
					result.location.y = point[1];
					result.location.z = point[2];   
					result.component = component;
					return result;
				}
			}

		}

		// no component collision,
		// if destroyed, ignore turrets
        if(capship->isdestroyed() )
		{
	      return result;
		}

		for(component_index=0; component_index < capship->getNumTurrets(); component_index++)
		{
			Turret * turr = (Turret *) capship->getTurret(component_index);
			if(turr->isInited() )
			{
				// ignore destroyed turrets with no mesh
				if(turr->isDestroyed() && (! turr->hasDamagedMesh() ) )
				{

				}
				else if(object->cantHit(turr) )
				{
					// ignore if this turret fired it
				}
				else
				{
					CollisionModel3D * component_coll_model = turr->getCollisionModel();
					if(component_coll_model != NULL)
					{
						turr->getMatrix(matrix1);
						component_coll_model->setTransform(matrix1);
						result.collision_occured = component_coll_model->collision(collisionmodel2 );
						if(result.collision_occured)
						{
							float point[3];
							result.locationvalid = true;
							component_coll_model->getCollisionPoint(point, false);
							result.location.x = point[0];
							result.location.y = point[1];
							result.location.z = point[2];   
							result.component = turr;
							return result;
						}
						else // try turret barrel
						{
							CollisionModel3D * barrel_model = turr->getBarrelCollisionModel();
							turr->getBarrelMatrix(matrix1);
							barrel_model->setTransform(matrix1);
							result.collision_occured = barrel_model->collision(collisionmodel2 );
			
	
							if(result.collision_occured)
							{
								float point[3];
								result.locationvalid = true;
								barrel_model->getCollisionPoint(point, false);
								result.location.x = point[0];
								result.location.y = point[1];
								result.location.z = point[2];   
								result.component = turr;
								return result;
							}
						}
					}
				}
			}
		}

   }
   return result;
}

void World::doCollisionResponse(ship * ship1, ship * ship2, Coordinate location)
{
   Coordinate newvelocity1, newvelocity2;
   bool result;

   Coordinate location1, location2;
   if(ship1->instanceof(INST_CAPITALSHIP) )
   {
      location1 = location;
      location2 = ship2->getLocation();
   }     
   else
   {
      location1 = ship1->getLocation();
      location2 = location;
   }

   result = Physics::collisionResponse(ship1->getMass(), ship2->getMass(),
                              location1, location2,
                              ship1->getVelocityVector(),ship2->getVelocityVector(),
                              newvelocity1, newvelocity2);

   if(result)
   {
     ship1->setVelocityVector(newvelocity1);
     ship2->setVelocityVector(newvelocity2);
   }
}

void World::setCollisionEnabled(bool value)
{
  collision_enabled = value;
}


/* find the nearest target who is targeting defendee.  
   If no such target exists, target the nearest enemy */
ship * World::getNearestTargetDefender(ship * defender, ship * defendee)
{
  int i;
  ship * nearest = NULL;
  ship * nearest_attacker = NULL;
  float distance;
  float mydistance;
  float distance_defend;


  for(i = 0; i < shiplist.size(); i++)
  {
     if(shiplist[i]->instanceof(INST_SHIP) && !(shiplist[i]->isdestroyed() ) )
     {
       ship * currentship = (ship *) shiplist[i];       
       if(areEnemies(defender->getIffCode(), currentship->getIffCode() ) )
       {
          mydistance = currentship->getLocation().distance(defender->getLocation() );
          if(nearest == NULL)
          {
            nearest = currentship;
            distance = mydistance;
          }
          else
          {
            if(mydistance < distance)
            {
               distance = mydistance;
               nearest = currentship;
            }
          }
          if(currentship->getTarget() == defendee) 
          {
             if(nearest_attacker == NULL)
             {
                nearest_attacker = currentship;
                distance_defend = mydistance;
             }
             else
             {
                if(mydistance < distance_defend)
                {
                  distance_defend = mydistance;
                  nearest_attacker = currentship;
                }
             }
          }
       }
     }
  }
  if(nearest_attacker != NULL)
    return nearest_attacker;
  else
    return nearest;

}

void World::doCollisionResponse(ship * ship1, Asteroid * asteroid)
{
   Coordinate newvelocity1, newvelocity2;
   bool result;
   Coordinate zero;

   result = Physics::collisionResponse(ship1->getMass(), asteroid->getMass(),
                              ship1->getLocation(), asteroid->getLocation(),
                              ship1->getVelocityVector(), zero,
                              newvelocity1, newvelocity2);

   if(result)
   {
     ship1->setVelocityVector(newvelocity1);
	 ship1->onCollisionWith(asteroid);
   }
}

void World::doCollisionResponse(ship * ship1, Coordinate location, float mass)
{
   Coordinate newvelocity1, newvelocity2;
   bool result;
   Coordinate zero;

   result = Physics::collisionResponse(ship1->getMass(), mass,
                              ship1->getLocation(), location,
                              ship1->getVelocityVector(), zero,
                              newvelocity1, newvelocity2);

   if(result)
   {
     ship1->setVelocityVector(newvelocity1);
   }
}



World::CollisionResult World::detailedCollisionDetection(Object3d * obj1, Object3d * obj2)
{
   float matrix1[16];
   float matrix2[16];
   CollisionModel3D * collisionmodel1 = obj1->getCollisionModel();
   CollisionModel3D * collisionmodel2 = obj2->getCollisionModel();
   float origin[3];
   Coordinate location = obj1->getcenter();
   World::CollisionResult result;

   result.collision_occured = false;
   result.locationvalid = false;
   result.component = NULL;


   origin[0] = location.x;
   origin[1] = location.y;
   origin[2] = location.z;
 
   
   obj1->getMatrix(matrix1);
   collisionmodel1->setTransform(matrix1);
   obj2->getMatrix(matrix2);
   collisionmodel2->setTransform(matrix2);
    

//   result.collision_occured = collisionmodel->sphereCollision(origin, object->getradius() );
   result.collision_occured = collisionmodel1->collision(collisionmodel2 );
   if(result.collision_occured)
   {
      float point[3];
      result.locationvalid = true;
      collisionmodel1->getCollisionPoint(point, false);
      result.location.x = point[0];
      result.location.y = point[1];
      result.location.z = point[2];      
   }
  return result;
}

World::CollisionResult World::asteroidCollisionDetection(Object3d * obj1)
{
   World::CollisionResult result;
   int i;
   Asteroid * ast;

   result.collision_occured = false;
   result.locationvalid = false;
   result.component = NULL;
   // 

   if(! obj1->instanceof(INST_SHIP) )
	   return result;

   for(i=0; i < asteroids.size(); i++)
   {
	   ast = asteroids[i];
       if(hasSphereCollision(obj1, ast) )
	   {
		   doCollisionResponse((ship *) obj1, ast);
		   result.collision_occured = true;
		   return result;
	   }
   }   
   return result;


}

bool World::hasSphereCollision(Object3d * obj1, Object3d * obj2)
{
   float distance = obj1->getcenter().distanceSquared(obj2->getcenter() );
   return distance < obj1->getRadiusSquared() || distance < obj2->getRadiusSquared() ;
}

void World::setAsteroidModel(CollisionModel3D * astmodel)
{
  asteroidmodel = astmodel;
}

void World::addAsteroid(Asteroid * newasteroid)
{
   asteroids.push_back( newasteroid);
}

void World::addExplosion(Explosion * newexplosion)
{
	explosionlist.push_back(newexplosion);
}

void World::addMissile(Missile * missile)
{
  int i;

  add(missile);

  for(i=0; i < decoys.size(); i++)
  {
	Decoy * decoy = decoys[i];
	if(! decoy->isdestroyed() )
	{
		missile->attemptSpoof(decoy->getLocation() );
	}
  }
}

void World::addDecoy(Decoy * newdecoy)
{
  int i;

  addVisualOnly(newdecoy);
  decoys.push_back(newdecoy);
  // for each missile attempt spoof

  for(i = 0; i < shiplist.size(); i++)
  {
    if(shiplist[i]->instanceof(INST_MISSILE) )
    {
		Missile * m = (Missile *) shiplist[i];
		m->attemptSpoof(newdecoy->getLocation() );
    }
  }

}

void World::addVisualOnly(Object3d * visual)
{
	explosionlist.push_back(visual);
}

void World::notifyTargetedByMissiles()
{
  int i;


  // first clear all
  for(i = 0; i < shiplist.size(); i++)
  {
    if(shiplist[i]->instanceof(INST_SHIP) )
    {
		ship * myship = (ship *)shiplist[i];
		myship->setMissileAttackingMe(false);
    }
  }
  // then notify those targeted

  for(i = 0; i < shiplist.size(); i++)
  {
    if(shiplist[i]->instanceof(INST_MISSILE) )
    {
		Missile * m = (Missile *) shiplist[i];
		Targetable * mtarget = m->getTarget();
		if(mtarget != NULL)
		{
		  if(mtarget->instanceof(INST_SHIP) )
		  {
     		  ship * myship = (ship *)mtarget;
		    myship->setMissileAttackingMe(true);
		  }
		}
    }
  }

}

void World::dump()
{
  std::list<Object3d *>::iterator i;
  Object3d * currentobject;

  cout << "---------------------------" << endl;
  cout << "World size is " << front.size() << endl;

  for(i = front.begin(); i != front.end(); i++)
  {
     currentobject = *i;
	 if(currentobject != NULL)
	 {
		if(currentobject->instanceof(INST_CAPITALSHIP) )
		{
		  cout << "Capship" << endl;
		}
		else if(currentobject->instanceof(INST_SHIP) )
		{
		  ship * myship = (ship *) currentobject;
		  cout << "Ship:" << myship->getPilot() << " : " << myship->getName() << endl;
		}
		else if(currentobject->instanceof(INST_MISSILE) )
		{
		  cout << "Missile" << endl;
		}
		else if(currentobject->instanceof(INST_WEAPON))
		{
		  cout << "Weapon" << endl;
		}
		else if(currentobject->instanceof(INST_EXPLOSION) )
		{
		  cout << "Explosion" << endl;
		}
		else if(currentobject->instanceof(INST_COMPONENT) )
		{
		  cout << "Component" << endl;
		}
		else 
		{
		  cout << "unknown object" << endl;
		}		
	 }
	 else
	 {
		cout << "NULL object" << endl;
	 }
  }
  cout << "---------------------------" << endl;
}
