// World.h: interface for the World class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WORLD_H__66CC534F_78AB_4232_B4C1_89239918651C__INCLUDED_)
#define AFX_WORLD_H__66CC534F_78AB_4232_B4C1_89239918651C__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <GL/glew.h>

#include <vector>
#include <list>
#include "coord.h"	// Added by ClassView
#include "ViewingCone.h"
#include "Asteroid.h"
#include "Decoy.h"
#include "Targetable.h"
class Object3d;
class WorldObjectWalker;
class ship;
class Camera;
class Ship;
class Targetable;
class CapitalShip;
class Component;
class Asteroid;
class Explosion;
class Missile;

class World  
{
public:
	void dump();
	void addVisualOnly(Object3d * visual);
	void addDecoy(Decoy * newdecoy);
	void addMissile(Missile * missile);
	void addExplosion(Explosion * newexplosion);
	void addAsteroid(Asteroid * newasteroid);
	void setAsteroidModel(CollisionModel3D * astmodel);

	ship * getNearestTargetDefender(ship *  defender, ship * defendee);
	void setCollisionEnabled(bool value);
	ship * getNearestTarget(ship * body, bool ignore_iff=false);
	void clear();
//	void setCamera(Camera * camera);
	void enumerateShips(WorldObjectWalker * walker);
	Object3d * getNext(Object3d * current);
	void collisionDetection();
	void render();
	void advance();
	void remove(Object3d * object);
	void add(Object3d * newobject);
	static World & getWorld();
 	virtual ~World();
private:

	void notifyTargetedByMissiles();
	bool hasSphereCollision(Object3d * obj1, Object3d * obj2);

    ViewingCone viewcone;
	void doCollisionResponse(ship * ship1, Coordinate location, float mass);
	
	void doCollisionResponse(ship * ship1, Asteroid * asteroid);
   bool collision_enabled;
	void doCollisionResponse(ship * ship1, ship * ship2, Coordinate location);

	void doCollisionResponse(ship * ship1, ship * ship2);
//   Camera * camera;
	void notifyTargetDestroyed(Targetable * target);
	void removeShipFromShipList(Targetable * deadship);
	bool bouningBoxOverlap(Object3d * obj1, Object3d * obj2);
	float getRadiusOfProjection(Coordinate L, Coordinate extents, Coordinate heading, Coordinate left, Coordinate up);
	bool formsSeparatingAxis(Coordinate t, Coordinate l, float ra, float rb);
   class CollisionResult
   {
     public:
        bool collision_occured;
        bool locationvalid;
        Coordinate location;   
		Component * component;
   };
	CollisionResult capshipCollision(CapitalShip * capship, Object3d * object);
	CollisionResult asteroidCollisionDetection(Object3d * obj1);
	std::list<Object3d *> front;
   std::vector<Targetable *> shiplist;
   std::vector<Object3d *> explosionlist;
	CollisionResult collisionBetween(Object3d * obj1, Object3d * obj2);
	CollisionResult detailedCollisionDetection(Object3d * obj1, Object3d * obj2);
	void removedestroyed();
	static World * theworld;
	World();
   bool OBBOverlap(
   //A
   Coordinate a, //extents
   Coordinate Pa, //position
   Coordinate A[3], //orthonormal basis
   
   //B
   Coordinate b, //extents
   Coordinate Pb, //position
   Coordinate B[3] //orthonormal basis
   );
   std::vector<Asteroid *> asteroids;
   CollisionModel3D * asteroidmodel;

   std::vector<Decoy *> decoys;

};

#endif // !defined(AFX_WORLD_H__66CC534F_78AB_4232_B4C1_89239918651C__INCLUDED_)
