// WorldOrientedBillboard.cpp: implementation of the WorldOrientedBillboard class.
//
//////////////////////////////////////////////////////////////////////

#include "WorldOrientedBillboard.h"
#include "TextureFactory.h"
#include "Camera.h"
#include <iostream.h>
#include "AbstractBillboard.h"
#include "glerr.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WorldOrientedBillboard::WorldOrientedBillboard(char filename[], 
		Coordinate newrelative_location, Coordinate newdesired_up,
		float newwidth, float newheight)     
:AbstractBillboard(filename, newwidth, newheight)
{
	desired_up = newdesired_up;

	// repick desired up
    if( fabs(newrelative_location.x) < fabs(newrelative_location.y) &&
		fabs(newrelative_location.x) < fabs(newrelative_location.z) )
	{
	   desired_up = Coordinate(1.0f, 0.0f, 0.0f);

	}
    else if( fabs(newrelative_location.y) < fabs(newrelative_location.z) &&
		fabs(newrelative_location.y) < fabs(newrelative_location.x) )
	{
	   desired_up = Coordinate(0.0f, 1.0f, 0.0f);
	}
	else
	{
	   desired_up = Coordinate(0.0f, 0.0f, 1.0f);
	}

	setLocation(newrelative_location);
	camerarelative = true;
}

WorldOrientedBillboard::WorldOrientedBillboard(Texture * newtexture, 
		float newwidth, float newheight)
:AbstractBillboard(newtexture, newwidth, newheight)
{
//	assert(false);
}


WorldOrientedBillboard::~WorldOrientedBillboard()
{

}

void WorldOrientedBillboard::render()
{
   Coordinate cameraheading, minuscameraheading;
   Coordinate right;
   Coordinate up;
   Coordinate location;

   if(camerarelative)
   {
     location = getLocation() + Camera::getCamera()->getLocation();
   }
   else
   {
	 location = getLocation();
   }


   cameraheading = Camera::getCamera()->getHeading();
//
   minuscameraheading =   cameraheading * -1.0f;

   //
   minuscameraheading = cameraheading * -1.0f;
   minuscameraheading.normalize();

   desired_up.normalize();
   right = desired_up.cross(minuscameraheading);
   right.normalize();
   up = minuscameraheading.cross(right);
   up.normalize();

   
   glNormal3f(minuscameraheading.x, minuscameraheading.y, minuscameraheading.z);
   glPushMatrix();
     glTranslatef(location.x, location.y, location.z);
     matchDOF(minuscameraheading, up);    
     renderRect();
   glPopMatrix();
   GLERRORCHECK
}


void WorldOrientedBillboard::setDesiredUp(Coordinate newdesiredup)
{
  desired_up = newdesiredup;
}

void WorldOrientedBillboard::setCameraRelative(bool relative)
{
  camerarelative = relative;
}
