// XmlMeshFile.cpp: implementation of the XmlMeshFile class.
//
//////////////////////////////////////////////////////////////////////

#include "XmlMeshFile.h"
#include "saxexpat.h"
#include "InputSource.h"
#include "AttributeList.h"
#include "TextureFactory.h"
#include <glut.h>
#include "ErrorHandlerE.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XmlMeshFile::XmlMeshFile(char filename[])
{
	expat myParser;
   int i;

   ErrorHandlerE::checkFile(filename);
   currentpoint = 0;
   currentface = 0;
	myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(SAX::InputSource(filename));
   for(i=0; i < numtextures; i++)
     textures[i] = TextureFactory::getTexture((char *) texturefiles[i].c_str() );
   delete [] texturefiles;
}

XmlMeshFile::~XmlMeshFile()
{
  int i;

//   for(i=0; i < numtextures; i++)
   //  delete textures[i];
   delete [] textures;
   delete [] texels;
   delete [] points;
   delete [] pointsinface;
   delete [] normals;
   delete [] whichtexture;
}

Coordinate XmlMeshFile::getExtents()
{
  return extents;
}

Coordinate XmlMeshFile::getcenter()
{
  return center;
}


float XmlMeshFile::getradius()
{
 return radius;
}

void XmlMeshFile::render()
{
  int currentpoint;
  int currentface;
  bool lastwastriangle=true;
  int lasttexture = 0;
  Coordinate location; 
  Coordinate normal;
  Texel texel;

  glEnable(GL_TEXTURE_2D);
  glDisable(GL_COLOR_MATERIAL);
  glColor3f(1.0f, 1.0f, 1.0f);
  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
  textures[0]->render();
//  glColor3f(1.0f, 0.0f, 0.0f);
  glBegin(GL_TRIANGLES);  
  currentpoint = 0;

  for(currentface =0; currentface < numfaces; currentface++)
  {
     if(pointsinface[currentface] == 3)
     {
        if(whichtexture[currentface] != lasttexture)
        {
           glEnd();
           textures[ whichtexture[currentface]]->render();
           glBegin(GL_TRIANGLES);                  
        }                
        else if(lastwastriangle == false)
        {
           glEnd();
           glBegin(GL_TRIANGLES);  
        }
           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;
           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;
           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;

       lastwastriangle = true;
     }
     else if(pointsinface[currentface] ==4)
     {
        if(whichtexture[currentface] != lasttexture)
        {
           glEnd();
           textures[ whichtexture[currentface]]->render();
           glBegin(GL_QUADS);  
        }                
        else if(lastwastriangle == true)
        {
           glEnd();
           glBegin(GL_QUADS);  
        }

           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;
           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;
           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;
           location = points[currentpoint];
           normal = normals[currentpoint];
           texel = texels[currentpoint];
           glTexCoord2f(texel.s, texel.t);
  	        glNormal3f(normal.x, normal.y, normal.z);
           glVertex3f(location.x , location.y , location.z);
           currentpoint++;
          lastwastriangle = false;
     }
  }
  glEnd();
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_COLOR_MATERIAL);
  
}

void XmlMeshFile::startDocument()
{


}

void XmlMeshFile::endDocument()
{

}

void XmlMeshFile::startElement(const std::string& name, const SAX::AttributeList* atts)
{
   int i;
   //cout << name.c_str() << endl;
  
   if(name == "point")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "x")
         {
            coord.x = atof(currentvalue.c_str() );
         }
         else if (currentname == "y")
         {
            coord.y = atof(currentvalue.c_str() );
         }
         else if(currentname == "z")
         {
            coord.z = atof(currentvalue.c_str() );
         }
      }
   }
   else if(name == "texel")
   {
     for(int i=0; i < 2; i++)
     {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "s")
         {
            texels[currentpoint].s = atof(currentvalue.c_str() );
         }
         else if (currentname == "t")
         {
            texels[currentpoint].t = atof(currentvalue.c_str() );            
         }
     }
   }
   else if(name == "faces")
   {
      std::string  currentname = atts->getName(0);
      std::string  currentvalue = atts->getValue(0);
      if(currentname == "numfaces")
      {
         numfaces = atoi(currentvalue.c_str());
         texels = new Texel[4 * numfaces];
         points = new Coordinate[4 *numfaces];
         pointsinface = new int[numfaces];
         normals = new Coordinate[4 * numfaces];
         whichtexture = new int[numfaces];
      }
      currenttexture = 0;
   }
   else if(name == "texture")
   {
      std::string  currentname = atts->getName(0);
      std::string  currentvalue = atts->getValue(0);
      if(currentname == "filename")
      {
         texturefiles[currenttexture] = currentvalue;
         currenttexture++;
      }
   }
   else if(name == "bounds")
   {
      std::string  currentname = atts->getName(0);
      std::string  currentvalue = atts->getValue(0);
      if(currentname == "radius")
      {
         radius = atof(currentvalue.c_str() );
      }
   }
   else if(name == "textures")
   {
      std::string  currentname = atts->getName(0);
      std::string  currentvalue = atts->getValue(0);
      if(currentname == "numtextures")
      {
         numtextures = atof(currentvalue.c_str() );
         texturefiles = new string[numtextures];
         textures = new Texture *[numtextures];
         currenttexture = 0;
      }  
   }
   else if(name == "triangle")
   {
      if(atts != NULL)
      {
        std::string  currentname = atts->getName(0);
        std::string  currentvalue = atts->getValue(0);

        currenttexture = atoi(currentvalue.c_str() );
      }
   }   
   else if(name == "quad")
   {
      if(atts != NULL)
      {
        std::string  currentname = atts->getName(0);
        std::string  currentvalue = atts->getValue(0);

        currenttexture = atoi(currentvalue.c_str() );
      }
   }   

}


void XmlMeshFile::endElement(const std::string& name)
{
   if(name == "center")
   {
      center = coord;
   }
   else if(name == "extents")
   {
      extents = coord;
   }
   else if(name == "normal")
   {
     normals[currentpoint] = coord;
   }
   else if(name == "location")
   {
     points[currentpoint] = coord;
   }
   else if(name =="triangle")
   {
     pointsinface[currentface] = 3;
     whichtexture[currentface] = currenttexture;
     currentface++;
   }
   else if(name =="quad")
   {
     pointsinface[currentface] = 4;
     whichtexture[currentface] = currenttexture;
     currentface++;
   }
   else if(name == "facepoint")
   {
     currentpoint++;
   }
}

void XmlMeshFile::characters(const std::string& ch)
{


}
