#include "coord.h"
#include "quaternion.h"
#include <iostream.h>
#include "Math.h"

  float Coordinate::dot(Coordinate right)
  {
    return x * right.x + y * right.y + z * right.z;
  }

  Coordinate Coordinate::cross(Coordinate right)
  {
    Coordinate newcoord;

    newcoord.x = y * right.z - z * right.y;
    newcoord.y = z * right.x - x * right.z;
    newcoord.z = x * right.y - y * right.x;
    return newcoord;
  }


  Coordinate Coordinate::abs()
  {

    Coordinate returnvalue;

	returnvalue.x = fabs_f( x);
	returnvalue.y = fabs_f(y);
	returnvalue.z = fabs_f(z);

	return returnvalue;
  }

  float Coordinate::angle(Coordinate right)
  {
    float length;

    length = fastsqrt( getlength() * right.getlength() );
  
    return acos(dot(right) / length);

  }
  float Coordinate::distance(Coordinate right)
  {
     Coordinate deltas;
     

     deltas.x = x - right.x;
     deltas.y = y - right.y;
     deltas.z = z - right.z;
     return deltas.getlength();
  }

Coordinate Coordinate::rotatearoundaxis(Coordinate axis, float theta)
{
  quaternion  q(axis, theta);
  float m[4][4];
  
  q.tomatrix(m);
  return matrixmult(m, *this); 
}

Coordinate Coordinate::matrixmult(float m [ 4 ] [ 4 ], Coordinate coord)
{
  Coordinate newcoord;

  newcoord.x = m[0][0] * coord.x + m[0][1] * coord.y + m[0][2] * coord.z;
  newcoord.y = m[1][0] * coord.x + m[1][1] * coord.y + m[1][2] * coord.z;
  newcoord.z = m[2][0] * coord.x + m[2][1] * coord.y + m[2][2] * coord.z;
  return newcoord;

}

Coordinate Coordinate::operator +(Coordinate coord)
{
  Coordinate newcoord;

  newcoord.x = x + coord.x;
  newcoord.y = y + coord.y;
  newcoord.z = z + coord.z;

  return newcoord;
}

/*Coordinate Coordinate::operator -(Coordinate & coord)
{
  Coordinate newcoord;

  newcoord.x = x - coord.x;
  newcoord.y = y - coord.y;
  newcoord.z = z - coord.z;

  return newcoord;
}
*/

Coordinate Coordinate::operator -(Coordinate coord)
{
  Coordinate newcoord;

  newcoord.x = x - coord.x;
  newcoord.y = y - coord.y;
  newcoord.z = z - coord.z;

  return newcoord;
}


Coordinate Coordinate::multiplybyglmatrix(float matrix [ 16 ])
{
  Coordinate output;

  output.x = x * matrix[0] + y * matrix[4] + z * matrix[8] + matrix[12];
  output.y = x * matrix[1] + y * matrix[5] + z * matrix[9] + matrix[13];
  output.z = x * matrix[2] + y * matrix[6] + z * matrix[10] + matrix[14];
  //w = matrix[12] +  matrix[13] + matrix[14] + matrix[15];
/*  output.x /= w;
  output.y /= w;
  output.z /= w;
*/
  return output;
}

/*Coordinate::Coordinate(float newx, float newy, float newz)
{
  x = newx;
  y = newy;
  z = newz;
}

Coordinate::Coordinate()
{
   x = 0.0f;
   y = 0.0f;
   z = 0.0f;
}
*/

Coordinate Coordinate::operator *(float value)
{
  return Coordinate(x * value, y * value, z * value);
}

Coordinate Coordinate::average(Coordinate other)
{
  return Coordinate( (x + other.x)*.5f, (y + other.y)*.5f, (z + other.z)*.5f );
}

Coordinate Coordinate::maxPairwise(Coordinate other)
{
   Coordinate returncoord;

   returncoord.x = (x > other.x ) ? x : other.x;  
   returncoord.y = (y > other.y ) ? y : other.y;  
   returncoord.z = (z > other.z ) ? z : other.z;  
   return returncoord;
}


Coordinate Coordinate::minPairwise(Coordinate other)
{
   Coordinate returncoord;

   returncoord.x = (x < other.x ) ? x : other.x;  
   returncoord.y = (y < other.y ) ? y : other.y;  
   returncoord.z = (z < other.z ) ? z : other.z;  
   return returncoord;
}

Coordinate Coordinate::interpolate(Coordinate to, double t)
{
  Coordinate returnvalue;
  double oneminust = 1.0 - t;

  returnvalue.x = oneminust * x + t * to.x;
  returnvalue.y = oneminust * y + t * to.y;
  returnvalue.z = oneminust * z + t * to.z;

  return returnvalue;
}

void Coordinate::print()
{
  cout << "x=" << x  << " y=" << y << " z=" << z << endl;
}

/*bool Coordinate::operator==(Coordinate & coord)
{
  return x == coord.x && y == coord.y && z == coord.z;
}
*/

bool operator==(const Coordinate lhs, const Coordinate rhs)
{
 return lhs.x == rhs.x && lhs.y == rhs.y && lhs.z == rhs.z;
}

bool Coordinate::operator<(Coordinate coord)
{
  return x < coord.x && y < coord.y && z < coord.z;
}


float Coordinate::distanceSquared(Coordinate right)
{
  float newx = x - right.x;
  float newy = y - right.y;
  float newz = z - right.z;

  return newx * newx + newy * newy + newz * newz;
}
