#ifndef FASTMATH_H
#define FASTMATH_H


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef WIN32
#include <wtypes.h>
#else
#define DWORD unsigned int
#endif


#define FP_BITS(fp) (*(DWORD *)&(fp))
#define FP_ABS_BITS(fp) (FP_BITS(fp)&0x7FFFFFFF)
#define FP_SIGN_BIT(fp) (FP_BITS(fp)&0x80000000)
#define FP_ONE_BITS 0x3F800000

extern unsigned int fast_sqrt_table[0x10000];  // declare table of square roots

void  build_sqrt_table();

inline float fastsqrt(float n)
{

  if (FP_BITS(n) == 0)
    return 0.0;                 // check for square root of 0

  FP_BITS(n) = fast_sqrt_table[(FP_BITS(n) >> 8) & 0xFFFF] | ((((FP_BITS(n) - 0x3F800000) >> 1) + 0x3F800000) & 0x7F800000);

  return n;
}


#endif