// mesh.cpp: implementation of the mesh class.
//
//////////////////////////////////////////////////////////////////////

#include "mesh.h"
#include <glut.h>
#include <iostream.h>
#include "MeshFileFactory.h"
#include "BinaryMeshFile.h"
#include "glerr.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
std::vector<mesh *> mesh::garbage;

mesh::mesh(char filename [ ])
: defaultcolor(1.0f, 1.0f, 1.0f)
{
//  meshfile file(filename);
  file = MeshFileFactory::getMeshFile(filename);
  mydisplaylist = glGenLists(1); 
  glNewList(mydisplaylist, GL_COMPILE);
    file->render();
//    ((BinaryMeshFile *)file)->renderWireFrame(Color(0.0f, 0.0f, 1.0f));
  glEndList();

   radius = file->getradius();
  extents = file->getExtents();
  center = file->getcenter();
  collision_model = ((BinaryMeshFile*) file)->getCollisionModel();
  max = file->getMaxs();
  min = file->getMins();
  delete file;
  garbage.push_back(this);
  istransparent = false;
  GLERRORCHECK
}

mesh::mesh(char filename [ ], Color transparent)
: defaultcolor(1.0f, 1.0f, 1.0f)
{
  file = MeshFileFactory::getMeshFile(filename, transparent);
  mydisplaylist = glGenLists(1); 
  glNewList(mydisplaylist, GL_COMPILE);
    file->render();
  glEndList();
  radius = file->getradius();
  extents = file->getExtents();
  center = file->getcenter();
  collision_model = ((BinaryMeshFile*) file)->getCollisionModel();
  max = file->getMaxs();
  min = file->getMins();
  delete file;
  garbage.push_back(this);
  istransparent = true;
  GLERRORCHECK
}

mesh::~mesh()
{
 glDeleteLists(mydisplaylist, 1);
 GLERRORCHECK
}


void mesh::render()
{
  defaultcolor.render();
  GLERRORCHECK
//  glDisable(GL_TEXTURE_2D);
  if(istransparent)
  {
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
	GLERRORCHECK
  }

  glCallList(mydisplaylist);
  GLERRORCHECK
  if(istransparent)
  {
    glDisable(GL_BLEND);
	GLERRORCHECK
  }
//  glEnable(GL_TEXTURE_2D);
//  file.render();
  GLERRORCHECK
}

float mesh::getradius()
{
  return radius;
}

Coordinate mesh::getcenter()
{
  return center;
}

Coordinate mesh::getExtents()
{
  return extents;
}

void mesh::clear()
{
  int i;
  for(i=0 ; i < garbage.size(); i++)
  {
     delete garbage[i];
  }
  garbage.clear();
}


CollisionModel3D * mesh::getCollisionModel()
{
  return collision_model;  
}

Coordinate mesh::getMaxs()
{
  return max;
}

Coordinate mesh::getMins()
{
  return min;
}


// force a wireframe mesh
mesh::mesh(char filename [ ], Color enemycolor, bool wireframe)
: defaultcolor(1.0f, 1.0f, 1.0f)
{
  file = MeshFileFactory::getMeshFile(filename);
  mydisplaylist = glGenLists(1); 
  glNewList(mydisplaylist, GL_COMPILE);
    ((BinaryMeshFile *)file)->renderWireFrame(enemycolor);
  glEndList();

   radius = file->getradius();
  extents = file->getExtents();
  center = file->getcenter();
  collision_model = ((BinaryMeshFile*) file)->getCollisionModel();
  max = file->getMaxs();
  min = file->getMins();
  delete file;
  garbage.push_back(this);
  istransparent = false;
  GLERRORCHECK
}

void mesh::setDefaultColor(Color newcolor)
{
  defaultcolor = newcolor;
}
