// mount.cpp: implementation of the mount class.
//
//////////////////////////////////////////////////////////////////////

#include "mount.h"
#include "weapondescription.h"
#include "weaponfactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

mount::mount(Coordinate newlocation, int newweaponnumber)
  : location(newlocation), weaponnumber(newweaponnumber), num_shots_total(-1)
{

}

mount::~mount()
{

}

Coordinate mount::getLocation()
{
   return location;
}

int mount::getWeaponNumber()
{
  return weaponnumber;
}


mount::mount(mount * newmount)
{
  location = newmount->location;
  weaponnumber = newmount->weaponnumber;
  num_shots_total = num_shots_remaining = newmount->num_shots_total;
}

mount::mount(Coordinate newlocation, int newweaponnumber, int numshots)
  : location(newlocation), weaponnumber(newweaponnumber),
    num_shots_total(numshots), num_shots_remaining(numshots)
{

}

bool mount::hasShotRemaining()
{
  if(num_shots_total == -1)
    return true;
  if(num_shots_remaining > 0)
    return true;
  return false;
}

void mount::useShot()
{
  num_shots_remaining --;
}

int mount::getNumShotsRemaining()
{
  return num_shots_remaining;
}

bool mount::hasUnlimitedShots()
{
  return num_shots_total == -1;
}

bool mount::isMissile()
{
  weapondescription * description = weaponfactory::getweaponfactory().getweapondescription(weaponnumber);

  return description->isMissile();
}

weapondescription * mount::getWeaponDescription()
{
  return weaponfactory::getweaponfactory().getweapondescription(weaponnumber);
}
