// quaternion.cpp: implementation of the quaternion class.
//
//////////////////////////////////////////////////////////////////////

#include "quaternion.h"
#define DELTA 1e-6

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


quaternion::~quaternion()
{

}

quaternion::quaternion(float newx, float newy, float newz, float neww)
  : x(newx), y(newy), z(newz), w(neww)
{

}

quaternion::quaternion(Coordinate axis, float angle)
{
  float sine, cosine;


  sine = sin(angle * .5f);
  cosine = cos(angle * .5f);

  x = axis.x * sine;
  y = axis.y * sine;
  z = axis.z * sine;
  w = cosine;

}

void quaternion::tomatrix(float m[4][4])
{
  float wx, wy, wz, xx, yy, yz, xy, xz, zz, x2, y2, z2;
  // calculate coefficients

  x2 = x + x; y2 = y + y; 
  z2 = z + z;
  xx = x * x2;   xy = x * y2;   xz = x * z2;
  yy = y * y2;   yz = y * z2;   zz = z * z2;
  wx = w * x2;   wy = w * y2;   wz = w * z2;

  m[0][0] = 1.0 - (yy + zz);    m[0][1] = xy - wz;
  m[0][2] = xz + wy;            m[0][3] = 0.0;
                    
  m[1][0] = xy + wz;            m[1][1] = 1.0 - (xx + zz);
  m[1][2] = yz - wx;            m[1][3] = 0.0;

  m[2][0] = xz - wy;            m[2][1] = yz + wx;
  m[2][2] = 1.0 - (xx + yy);    m[2][3] = 0.0;

  m[3][0] = 0;                  m[3][1] = 0;
  m[3][2] = 0;                  m[3][3] = 1;
}


quaternion::quaternion()
: x(0), y(0), z(0), w(0)
{

}

