#include "rotate.h"
#include "quaternion.h"
#include "coord.h"
#include "glut.h"

float degtorad(float);

Coordinate rotz(Coordinate c, float theta)
{
  Coordinate newc;
  float thetar, sintheta, costheta;


  thetar = degtorad(theta);
  sintheta = sin(thetar);
  costheta = cos(thetar);

  newc.x = c.x * costheta - c.y * sintheta;
  newc.y = c.x * sintheta + costheta * c.y;
  newc.z = c.z;
  return newc;


}

Coordinate rotx(Coordinate c, float theta)
{
  Coordinate newc;
  float thetar, sintheta, costheta;

  thetar = degtorad(theta);
  sintheta = sin(thetar);
  costheta = cos(thetar);

  newc.x = c.x;
  newc.y = c.y * costheta - sintheta * c.z;
  newc.z = c.y * sintheta + costheta * c.z;
  return newc;

}

Coordinate roty(Coordinate c, float theta)
{
  Coordinate newc;
  float thetar, sintheta, costheta;

  thetar = degtorad(theta);
  sintheta = sin(thetar);
  costheta = cos(thetar);
  
  newc.x = c.x * costheta + c.z * sintheta;
  newc.y = c.y ;
  newc.z = -c.x * sintheta + costheta * c.z;
  return newc;

}


Coordinate matrixmult(float m[4][4], Coordinate coord)
{
  Coordinate newcoord;

  newcoord.x = m[0][0] * coord.x + m[0][1] * coord.y + m[0][2] * coord.z;
  newcoord.y = m[1][0] * coord.x + m[1][1] * coord.y + m[1][2] * coord.z;
  newcoord.z = m[2][0] * coord.x + m[2][1] * coord.y + m[2][2] * coord.z;
  return newcoord;
}

Coordinate rotatearoundaxis(Coordinate axis, Coordinate point, float theta)
{
  QUAT q;
  float m[4][4];
  Coordinate newcoord;

  QuatRotAxis(&q, axis.x, axis.y, axis.z, theta);
  QuatToMatrix(&q, m);
  newcoord = matrixmult(m, point);
  return newcoord;
}

void dorotate(Coordinate dof)
{

  float array[4][4], glarray[16];
  int i;
  int j;
  Coordinate y , ycrossdof, ycrossdofnorm, dofnorm, dofcycdof;

  y.x = 0.0;
  y.y = 1.0;
  y.z = 0.0;

  ycrossdof = y.cross(dof);
  ycrossdofnorm = ycrossdof;
  ycrossdofnorm.normalize();
  dofnorm = dof;
  dofnorm.normalize();
  dofcycdof = dof.cross( ycrossdof);
  dofcycdof.normalize();
  
  glarray [0] =  array[0][0] = ycrossdofnorm.x;
  glarray [1] = array[1][0] = ycrossdofnorm.y;
  glarray [2] = array[2][0] = ycrossdofnorm.z;
  glarray [3] = array[3][0] = 0;
  glarray [4] = array[0][1] = dofcycdof.x;
  glarray [5] = array[1][1] = dofcycdof.y;
  glarray [6] = array[2][1] = dofcycdof.z;
  glarray [7] = array[3][1] = 0;
  glarray [8] = array[0][2] = dofnorm.x;
  glarray [9] = array[1][2] = dofnorm.y;
  glarray [10] = array[2][2] = dofnorm.z;
  glarray [11] = array[3][2] = 0;
  glarray [12] = array[0][3] = 0;
  glarray [13] = array[1][3] = 0;
  glarray [14] = array[2][3] = 0;
  glarray [15] = array[3][3] = 1;

/*  for(i=0; i < 4; i++)
  {
    for(j=0; j < 4; j++)
    {
      glarray[j * 4 +i]  = array[i][j];

    }
  } */
  glMultMatrixf(glarray);
}
