// ship.cpp: implementation of the ship class.
//
//////////////////////////////////////////////////////////////////////

#include <assert.h>
#include <GL/glew.h>
#include "ship.h"
#include "weaponfactory.h"
#include <iostream.h>
#include "World.h"
#include "Missile.h"
#include "GunDescription.h"
#include "Explosion.h"
#include "World.h"
#include "SoundFactory.h"
#include "SoundEffect.h"
#include "Mission.h"
#include "MissileDescription.h"
#include "Time.h"
#include "CapitalShip.h"
#include "Component.h"
#include "SoundEffect3d.h"
#include "JoystickFactory.h"
#include "ForceEffectFactory.h"
#include "ForceEffectSet.h"
#include "Reticle.h"
#include "Pod.h"
#include "glerr.h"
#include "Turret.h"
#include "DiffuseLight.h"
#include "LightManager.h"
//#include "Shader.h"
#include "CubeMap.h"
#include "TurretJointed.h"
#include "TurretBallSocket.h"
#include "Math.h"
#include "Settings.h"
#include "Decoy.h"
#include "EjectionPod.h"

#define INTERPOLATETIME 2500

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//Shader * shader = NULL;




const int shieldtime = 500;

ship::ship(mesh * newmesh, int newnummounts, mount * mounts[], 
           Coordinate newcockpit, shipareas newshields, shipareas newarmor,
           float newvelocity, float newroll, float newpitch, float newyaw, 
		   float newacceleration, float newangularacceleration,
		   float newafterburnervelocity, float newfterburneracceleration, float newfuel,
           int newnumthrusters, Thruster * newthrusters[], Image * newimage, string newname,
           int newtotalenergy, int newenergyrecharge, float newmass, string newcockpitmesh,
     	   int newmaxdecoys,
		   std::vector<Turret *> * newturrets,
		   std::vector<Coordinate> newenginelocations,
		   int newmaxhitpoints,
		   Pod ** newpods, int newnumpods)
  : Targetable(newmesh, newshields, newarmor, newvelocity, 
     newroll, newpitch, newyaw, 
	 newacceleration, newangularacceleration,
	 newimage, newname),
//    nummounts(newnummounts), 
    cockpit(newcockpit),
    thrusters(newthrusters),
    numthrusters(newnumthrusters),
    maxenergy(newtotalenergy),
    currentenergy(newtotalenergy),
    energy_recharge_rate(newenergyrecharge),
    mass(newmass),
    newvelocity(false),
    startvelocityinterpolate(0),
    shieldstarttime(0),
    lockendtime(0),
    autopilots(true),
    mustsurvive(false),
    lastmissilefiretime(0),
    last_missile_fired_at(NULL),
    is_player(false),
    normalacceleration(newacceleration),
	normalvelocity(newvelocity),
	afterburneracceleration(newfterburneracceleration),
	afterburnervelocity(newafterburnervelocity),
    currentfuel(newfuel),
	maxfuel(newfuel),
   afterburn_on(false),
   prev_afterburn_on(false),
   cockpitmesh(newcockpitmesh),
   target_lock(false),
   pods(newpods),
   numpods(newnumpods),
   turrets(newturrets),
   enginelocations(newenginelocations),
   maxdecoys(newmaxdecoys),
   numdecoys(newmaxdecoys),
   maxhitpoints(newmaxhitpoints),
   currenthitpoints(newmaxhitpoints)
{
  int i;
  int nummounts = newnummounts;
  mount * * mymounts = NULL;
   

/*  if(shader == NULL)
  {
	shader = new Shader("perfragment.vert", "perfragment.frag");
  }
*/




  mymounts = new mount *[nummounts];
  for(i=0; i < nummounts; i++)
     mymounts[i] = new mount(mounts[i]);
  quadric = gluNewQuadric();
  gluQuadricDrawStyle(quadric, GLU_FILL);
  gluQuadricNormals(quadric, GLU_SMOOTH);
  gluQuadricOrientation(quadric, GLU_OUTSIDE);
  gluQuadricTexture(quadric, GL_FALSE);
  target = NULL;
  initGunsAndMissiles(mymounts, nummounts);
  delete [] mymounts;
  gunfiretime = new TimeValue[numguns];
  missilefiretime = new TimeValue[nummissiles];
  for(i=0; i < numguns; i++)
  {
    gunfiretime[i] = 0;
  }       
  for(i=0; i < nummissiles; i++)
  {
    missilefiretime[i] = 0;
  }       
  charge_start_time = 0;
  currentchargesound = NULL;
  playedcharge1 = false;
  playedcharge2 = false;
  buildGunTypes();   
  targeted_component = 0;
   missile_sound_played = false;
  computeFullShipExtents();
//  createThrusterLights();
  shadow = false;
  comm = NULL;
  missileattackingme = false;
  invulnerable = false;
  lastdecoytime  = 0;
  float frand = (float) rand()  / RAND_MAX;
  ejectallowed = (frand > .5f);

}


ship::~ship()
{
  int i;

  for(i=0; i < numguns; i++)
     delete guns[i];
  for(i=0; i < nummissiles; i++)
     delete missiles[i];
  delete [] guns;
  delete [] missiles;
  delete [] gunfiretime;
  delete [] missilefiretime;
  delete [] differentguntypes;
  if(numpods != 0)
  {
	  for(i=0; i < numpods; i++)
	  {
        delete pods[i];
	  }
	  delete [] pods;
  }
/*
  LightManager * lightmanager = LightManager::getInstance();
  for(i=0; i < numthrusters;i ++)
  {
     lightmanager->removeLocalLight(thrusterlights[i]);
     delete thrusterlights[i];   
  }
  delete [] thrusterlights;
*/
  if(is_player)
  {
#ifdef WIN32
    afterburnersound = SoundFactory::getSoundFactory()->getAfterburnerSound();
	afterburnersound->stop();
#endif
  }
}


void ship::fire()
{  
  int i;
  mount * mnt;

  if(numguns == 0)
    return;
  if(isFullGuns() ) 
  {


    for(i=0; i < numguns ; i++)
    {
       mnt = guns[i];
       /* game rules are that in full guns mode, only energy weapons
          those that don't need ammo, can fire */
	   /* also, charging weapons can't fire in full guns mode */
	   GunDescription * gundesc = (GunDescription *) mnt->getWeaponDescription();
       bool cancharge = gundesc->isChargeable();

       if(mnt->hasUnlimitedShots() && (! cancharge) )
         fireGun(i);
    }
  }
  else
  {
     for(i=0 ; i < numguns; i++)
     {
        if(guns[i]->getWeaponNumber() == differentguntypes[currentgun] )
           fireGun(i);
     }
  }
}


Coordinate ship::getCockpitLocation()
{
  return toWorldLocation(cockpit);
}


void ship::damage(int amount, Coordinate fromlocation)
{
  Coordinate us_to_them;
  Coordinate heading = getHeading();
  Coordinate left = getleft();
  Coordinate upleft = left.average(heading);
  Coordinate right(-left.x, -left.y, -left.z);
  Coordinate upright = right.average(heading); 
  int remaining;
  bool armorflag = false;

  if(isdestroyed() )
    return;
  if(invulnerable)
	return;

  us_to_them = fromlocation - getLocation();

  if(upleft.dot(us_to_them) > 0.0f)
  {
     if(upright.dot(us_to_them) > 0.0f)
     {
      //  cout << "front" << endl;
        remaining = shields.damage(FRONT, amount);
        if(remaining > 0)
        {
           remaining = armor.damage(FRONT, remaining);
		   remaining = damageHull(remaining);
           if(remaining > 0)
             destroy();
           armorflag = true;
        }
     }
     else
     {
       // cout << "left" << endl;
        remaining = shields.damage(LEFT, amount);
        if(remaining > 0)
        {
           remaining = armor.damage(LEFT, remaining);
		   remaining = damageHull(remaining);
           if(remaining > 0)
             destroy();
           armorflag = true;
        }
     }
  }
  else
  {
     if(upright.dot(us_to_them) > 0.0f)
     {
       // cout << "right" << endl;
        remaining = shields.damage(RIGHT, amount);
        if(remaining > 0)
        {
           remaining = armor.damage(RIGHT, remaining);
		   remaining = damageHull(remaining);
           if(remaining > 0)
             destroy();
           armorflag = true;
        }
     }
     else
     {
       // cout << "back" << endl;
        remaining = shields.damage(BACK, amount);
        if(remaining > 0)
        {
           remaining = armor.damage(BACK, remaining);
		   remaining = damageHull(remaining);
           if(remaining > 0)
             destroy();
           armorflag = true;
        }
     }
  }



  if(armorflag)
  {
    SoundEffect3d * effect = SoundFactory::getSoundFactory()->getArmorSound();
    effect->setPosition(fromlocation);
    effect->play();
  }
  else
  {
    shieldstarttime = Time::getCurrentTime();
    shieldendtime = shieldstarttime + shieldtime;
    SoundEffect3d * effect = SoundFactory::getSoundFactory()->getShieldSound();
    effect->setPosition(fromlocation);
    effect->play();
  }
  if(is_player)
  {
	  if(JoystickFactory::getInstance()->getForceFeedbackJoystick() != NULL)
	  {

        us_to_them = fromlocation - getCockpitLocation();
		float x = us_to_them.dot(heading);
		float y = us_to_them.dot(left);

		float theta_radians = atan2(y, x );
		if(theta_radians < 0.0f)
		{
			const float twopi = 6.28318530718f;
			theta_radians += twopi;
		}
		const float radians2deg = 57.2957795131f;

		int direction = theta_radians * radians2deg * 100.0f;
        
        ForceEffectSet * effect;
		if(armorflag)
		{
    	    effect = ForceEffectFactory::getInstance()->getArmorForce();
		}
		else
		{
    	    effect = ForceEffectFactory::getInstance()->getShieldForce();
		}
		effect->setDirection(direction);
        effect->play();

	  }
  }

}

void ship::onCollisionWith(Object3d * otherobject)
{
  if(is_player && (otherobject->instanceof(INST_SHIP | INST_ASTEROID | INST_EJECTEDPOD) ))
  {
	  if(otherobject != Mission::getCurrentMission()->getLandingShip() )
	  {
    	  if(JoystickFactory::getInstance()->getForceFeedbackJoystick() != NULL)
		  {
		    ForceEffectSet * effect = ForceEffectFactory::getInstance()->getCollisionForce();
		    effect->play();
		  }
	  }
  }
}

void ship::render()
{
//   shader->begin();
   Targetable::render();
//   shader->end();

   Coordinate location = getLocation();
   Coordinate heading = getHeading();
   Coordinate up = getUp();
   

   int i;

   if(numpods != 0)
   {
        for(i=0; i < numpods; i++)
		{     
          glPushMatrix();
            location = getLocation();
            glTranslatef(location.x, location.y, location.z);
            matchDOF(heading, up);    
		    Coordinate podlocation = pods[i]->getTranslation();
            glTranslatef(podlocation.x, podlocation.y, podlocation.z);
            pods[i]->getMesh()->render();
          glPopMatrix();
		}
   }


   // what is this for ??
   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); 
   float velocitypercent =   getVelocity()/normalvelocity;




   for(i=0; i < numthrusters; i++)
   {
/*      glPushMatrix();
	      Coordinate thrloc = thrusterlights[i]->getLocation() ;
          glTranslatef(thrloc.x, thrloc.y, thrloc.z);
          glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
          gluSphere(quadric, 1.0f, 21, 21);
       glPopMatrix();
*/


       glPushMatrix();
          location = getLocation();
          glTranslatef(location.x, location.y, location.z);
          matchDOF(heading, up);    
          thrusters[i]->render(velocitypercent, false);
       glPopMatrix();
   }

   if(shieldstarttime != 0)
   {
	  if(Time::getCurrentTime() < shieldendtime )
      {
		  renderShield();
      }
      else
      {
        shieldstarttime = 0;
      }      
   }
   GLERRORCHECK

   if(shadow)
   {
	 renderShadow();
   }

}


void ship::nextTarget()
{
  target = (Targetable *) World::getWorld().getNext(target);
  // infinite recursion if no other targets and use a while
  if(target == this)
  {
    target = (Targetable *) World::getWorld().getNext(target);
  }
  if(target == this)
  {
    target = NULL;
    lockendtime = 0;
  }
  else
  {
    onNewTarget();
  }
  targeted_component = 0;
  missile_sound_played = false;
}

bool ship::isShip()
{
  return true;
}

Targetable * ship::getTarget()
{
  return target;
}

void ship::targetDestroyed()
{
  target = NULL;
  lockendtime = 0;
  targeted_component = 0;
}

Missile * ship::fireMissile()
{
   if(nummissiles == 0)
   {
     return NULL;
   }
   if(! isCurrentMissileAppropriateForTarget() )
   {
      return NULL;
   }

   mount * mnt = missiles[currentmissile];
   MissileDescription * missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );


   if(lockRequired() && 
          (! hasLock() ) 
     )
   {
       return NULL;
   }

   TimeValue  currenttime = Time::getCurrentTime();
   if(currenttime - missilefiretime[currentmissile] < missiledescription->getRefireDelay() )
   {
     return NULL;
   }


   if(!mnt->hasUnlimitedShots() )
   {
      if(mnt->hasShotRemaining() )
      {
        mnt->useShot();
      }
      else
      {
         return NULL;
      }
   }
   Coordinate mountlocation = mnt->getLocation();
   Coordinate worldpoint;

   worldpoint = toWorldLocation(mountlocation);

   Missile * missile = weaponfactory::getweaponfactory().addMissile(mnt->getWeaponNumber(), worldpoint, getHeading(), getUp(), getleft());
   missile->setCantHit(this);
   missile->setTarget(getTarget() );
   if(getTarget()->instanceof(INST_CAPITALSHIP) )
   {
     missile->setTargetedComponent(targeted_component);
   }

   if(lockRequired() )
   {
     TimeValue  locktime = missiledescription->getLockTime();
     lockendtime  = Time::getCurrentTime() + locktime;  
   }
   missilefiretime[currentmissile] = Time::getCurrentTime();
   if(is_player)
   {
      missile->setFiredByPlayer();
      Mission::getCurrentMission()->incrementPlayerMissileFired();
	  if(JoystickFactory::getInstance()->getForceFeedbackJoystick() != NULL)
	  {
		  ForceEffectSet * effect = ForceEffectFactory::getInstance()->getMissileLaunchForce();
		  effect->play();
	  }
   }
   missile_sound_played = false;
   return missile;
}

Color ship::getRadarColor(ship * player)
{
  if(iffcode == FRIEND)
  {
    return Color(0.0f, 0.0f, 1.0f);
  }
  else if(iffcode == ENEMY)
  {
    return Color(1.0f, 0.0f, 0.0f);
  }
  else if(iffcode == NEUTRAL)
  {
    return Color(.5f, 0.5f, 0.5f);
  }
  return Color(1.0f, 0.0f, 0.0f);
}

ship::ship(ship & that)
  :Targetable(that),
   cockpit(that.cockpit),
//   nummounts(that.nummounts),
   numthrusters(that.numthrusters),
   quadric(that.quadric),
//   showshield(false),   
   thrusters(that.thrusters),
   energy_recharge_rate(that.energy_recharge_rate),
   maxenergy(that.maxenergy),
   currentenergy(that.maxenergy),
   mass(that.mass),
   newvelocity(false),
   gunfiretime(0),
   startvelocityinterpolate(0),
   shieldstarttime(0),
   lockendtime(0),
   autopilots(true),
   mustsurvive(false),
   currentgun(0),
   currentmissile(0),
   numguns(that.numguns),
   nummissiles(that.nummissiles),
   numdifferentguntypes(that.numdifferentguntypes),
   lastmissilefiretime(0),
   last_missile_fired_at(NULL),
   is_player(false),
   normalacceleration(that.normalacceleration),
   normalvelocity(that.normalvelocity),
   afterburneracceleration(that.afterburneracceleration),
   afterburnervelocity(that.afterburnervelocity),
   currentfuel(that.maxfuel),
   maxfuel(that.maxfuel),
   afterburn_on(false),
   prev_afterburn_on(false),
   cockpitmesh(that.cockpitmesh),
   target_lock(false),
   enginelocations(that.enginelocations),
   maxdecoys(that.maxdecoys),
   numdecoys(that.maxdecoys),
   maxhitpoints(that.maxhitpoints),
   currenthitpoints(that.maxhitpoints)


{
  int i;

  guns = new mount *[numguns];
  missiles = new mount *[nummissiles];

  for(i=0; i < numguns; i++)
  {
     guns[i] = new mount(that.guns[i]);
  }

  for(i=0; i < nummissiles; i++)
  {
     missiles[i] = new mount(that.missiles[i] );
  }

  gunfiretime = new TimeValue[numguns];
  missilefiretime = new TimeValue[nummissiles];
  for(i=0; i < numguns; i++)
  {
    gunfiretime[i] = 0;
  }        
  for(i=0; i < nummissiles; i++)
  {
    missilefiretime[i] = 0;
  }
  charge_start_time = 0;
  currentchargesound = NULL;
  playedcharge1 = false;
  playedcharge2 = false;
  differentguntypes = new int[numdifferentguntypes];
  for(i=0; i < numdifferentguntypes; i++)
  {
     differentguntypes[i] = that.differentguntypes[i];
  }
  targeted_component = 0;
  missile_sound_played = false;
  numpods = that.numpods;
  if(numpods == 0)
  {
    pods = NULL;
  }
  else
  {
    pods = new Pod *[numpods];
    for(i=0; i < numpods; i++)
	{
      pods[i] = new Pod(*that.pods[i]);
	}
  }
  turrets = new std::vector<Turret *>();
  for(i=0; i < (*that.turrets).size() ; i++)
  {
    Turret * thatturret;

	thatturret = (*that.turrets)[i];
	if(thatturret->instanceof("TurretJointed") )
	{
		TurretJointed * tj = (TurretJointed *) thatturret;
	    turrets->push_back(new TurretJointed(*tj) );
	}
	else
	{
		TurretBallSocket * tj = (TurretBallSocket *) thatturret;
	    turrets->push_back(new TurretBallSocket(*tj) );
	}

  }
  computeFullShipExtents();
//  createThrusterLights();
  shadow = false;
  comm = NULL;
  missileattackingme = false;
  invulnerable = false;
  lastdecoytime  = 0;

  float frand = (float) rand()  / RAND_MAX;
  ejectallowed = (frand > .5f);

}

void ship::nextGun()
{
  charge_start_time = 0;
  if(numguns == 0)
    return;
  if(numdifferentguntypes == 1)
    return;
  currentgun = (currentgun + 1) % (numdifferentguntypes + 1);
}

void ship::nextMissile()
{
  if(nummissiles == 0)
    return;

  currentmissile = (currentmissile + 1) % nummissiles;
  if(getTarget() != NULL)
  {
     MissileDescription * missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );
     TimeValue  locktime = missiledescription->getLockTime();
     lockendtime  = Time::getCurrentTime() + locktime;  
  }
  missile_sound_played = false;
}

void ship::initGunsAndMissiles(mount ** mymounts, int nummounts)
{
  //first count how many guns to make
  int i;
  int currentgunindex;
  int currentmissileindex;

  currentgun = 0;
  currentmissile = 0;

  currentgunindex = 0;
  currentmissileindex = 0;
  
  numguns = 0;
  nummissiles = 0;
  for(i=0; i < nummounts; i++)
  {
    if(mymounts[i]->isMissile() )
      nummissiles++;
    else
      numguns++;
  }
  guns = new mount *[numguns];
  missiles = new mount *[nummissiles];
  for(i=0; i < nummounts; i++)
  {
    if(mymounts[i]->isMissile() )
    {
      missiles[currentmissileindex] = mymounts[i];
      currentmissileindex++;
    }
    else
    {
      guns[currentgunindex] = mymounts[i];
      currentgunindex++;
    }
  }
}

int ship::getNumCurrentMissile()
{
  if(nummissiles == 0)
    return 0;
  mount * missilemount = missiles[currentmissile];
  return missilemount->getNumShotsRemaining();
}

string ship::getCurrentMissileName()
{
  if(nummissiles == 0)
  {
     return string("No missiles");
  }
  else
  {
  int currentweapontype = missiles[currentmissile]->getWeaponNumber();

  weapondescription * description = weaponfactory::getweaponfactory().getweapondescription(currentweapontype);
  return description->getName();
  }
}

string ship::getCurrentGunName()
{
  if(numguns == 0)
  {
     return string("No guns");
  }
  else
  {
//    int currentweapontype = guns[currentgun]->getWeaponNumber();
    int currentweapontype = differentguntypes[currentgun];

    weapondescription * description = weaponfactory::getweaponfactory().getweapondescription(currentweapontype);
    return description->getName();
  }
}

int ship::getMaxEnergy()
{
  return maxenergy;
}

int ship::getCurrentEnergy()
{
  return currentenergy;
}

void ship::advance()
{
  int i;
  Coordinate worldpoint;

  missileAI();
  ejectAI();
  fireAI();
  decoyAI();
  

  if(afterburn_on)
  {
    currentfuel -= 1.0f * Time::getTimeScale();
  }
#ifdef WIN32
  if(afterburn_on && ! prev_afterburn_on)
  {
    afterburnersound = SoundFactory::getSoundFactory()->getAfterburnerSound();
	afterburnersound->setLoop(true);
	afterburnersound->setPosition(getLocation() );
	afterburnersound->play();
  }
  else if(prev_afterburn_on && ! afterburn_on)
  {
    if(afterburnersound != NULL)
    {
      afterburnersound->stop();
      afterburnersound = NULL;
    }

  }
#endif

  if(newvelocity)
  {

    if(startvelocityinterpolate == 0)
    {
       startvelocityinterpolate = Time::getCurrentTime();
       setLocation(getLocation() + velocityvector );
    }
    else
    {      

      TimeValue  currenttime = Time::getCurrentTime();
      if(currenttime < startvelocityinterpolate + INTERPOLATETIME)
      {

        float progress = (float)(currenttime - startvelocityinterpolate) / (float) INTERPOLATETIME;

        setLocation(getLocation() + velocityvector * (1.0f - progress) + getHeading() * (progress * getVelocity() * Time::getTimeScale()  )  );
      }
      else
      {
        startvelocityinterpolate = 0;
        newvelocity = false;
        Targetable::advance();        
      }
    }
  }
  else
  {
    Targetable::advance();
  }

  float rate = energy_recharge_rate * Time::getTimeScale();
  if(currentenergy + rate > maxenergy)
    currentenergy = maxenergy;
  else
    currentenergy += rate;

  missileLockedPlaySound();
  advanceChargeSounds();
  prev_afterburn_on = afterburn_on;

/*
  for(i=0; i <  numthrusters; i++)
  {
	   // thruster light location
       matchDOF(getHeading(), getUp(), matrix);
       worldpoint = thrusters[i]->getLocation().multiplybyglmatrix(matrix);
       worldpoint = worldpoint  + getLocation() - getHeading() * 2.0f;
	   thrusterlights[i]->setLocation(worldpoint);
  }
*/
  advancePredictor();

}

float ship::getMass()
{
  return mass;
}

bool ship::instanceof(char classname [ ])
{
  if(0 == strcmp(classname, "ship")) 
    return true;
  return Targetable::instanceof(classname);
}	

Coordinate ship::getVelocityVector()
{
  if(newvelocity)
  {
   TimeValue  currenttime = Time::getCurrentTime();
   float progress = (float)(currenttime - startvelocityinterpolate) / (float) INTERPOLATETIME;

     Coordinate myvelocitynow = velocityvector * (1.0f - progress) + getHeading() * (progress * getVelocity() * Time::getTimeScale() );
    return myvelocitynow;
  }
  else
    return getHeading() * getVelocity() * Time::getTimeScale();
}

void ship::setVelocityVector(Coordinate newvelocityp)
{
  newvelocity = true;
  startvelocityinterpolate = 0;
  velocityvector = newvelocityp;
//  velocityvector = newvelocityp* Time::getTimeScale();
}

void ship::setFullGuns(bool newfullguns)
{
  if(newfullguns)
    currentgun = numdifferentguntypes;
  else
    currentgun = 0;
}

bool ship::isFullGuns()
{
  return currentgun == numdifferentguntypes;
}

weapon * ship::fireGun(int gunnumber)
{
   mount * mnt = guns[gunnumber];
   GunDescription * gundescription = 
        (GunDescription *) weaponfactory::getweaponfactory().getweapondescription(mnt->getWeaponNumber());

   int gunenergy = gundescription->getEnergy();

   TimeValue  currenttime = Time::getCurrentTime();
   if(currenttime - gunfiretime[gunnumber] < gundescription->getRefireDelay() )
   {
     return NULL;
   }

   if(currentenergy - gunenergy < 0)
     return NULL;

   if(! mnt->hasShotRemaining() )
     return NULL;

   mnt->useShot();
   currentenergy -= gunenergy;
   Coordinate mountlocation = mnt->getLocation();
   Coordinate worldpoint;
   worldpoint = toWorldLocation(mountlocation);
   weapon * weap = weaponfactory::getweaponfactory().addweapon(mnt->getWeaponNumber(), worldpoint, getHeading(), getUp(), getleft());
   gunfiretime[gunnumber] = currenttime;
   weap->setcanthit(this);
   if(is_player)
   {
      weap->setFiredByPlayer();
      Mission::getCurrentMission()->incrementPlayerGunFired();
   }
   return weap;
}


void ship::setTarget(Targetable * newtarget)
{
  Targetable::setTarget(newtarget);
  onNewTarget();

}

void ship::destroy()
{
  Mission::getCurrentMission()->onDestroyed(this);
  setDestroyed();
  World::getWorld().addExplosion(new Explosion (getLocation() ) );
  SoundEffect3d * effect = SoundFactory::getSoundFactory()->getExplosionSound();
  effect->setPosition(getLocation() );
  effect->play();
  if(is_player)
  {
	  if(JoystickFactory::getInstance()->getForceFeedbackJoystick() != NULL)
	  {
		  ForceEffectSet * effect = ForceEffectFactory::getInstance()->getExplodeForce();
		  effect->play();
	  }
	    SoundEffect * locksound = SoundFactory::getSoundFactory()->getMissileAlertSound();
		locksound->stop();
	    SoundEffect * ejectsound = SoundFactory::getSoundFactory()->getEjectAlertSound();
		ejectsound->stop();
  }
  comm->dead();
}

void ship::fireAI()
{
  if(! hasAI() )
    return;

  Targetable * target = getTarget();
  Coordinate location = getLocation();
  if(target == NULL)
    return;
  Coordinate targetlocation = target->getLocation();

  if(currentenergy < .15f * maxenergy)
    return;

  GunDescription * gundescription = 
      (GunDescription *) weaponfactory::getweaponfactory().getweapondescription(currentgun);
  float range = gundescription->getrange();

  if(targetlocation.distance(location) > range)
    return;

  Coordinate totargetvector = targetlocation - location;
  Coordinate heading = getHeading();
  
  totargetvector.normalize();

  float dotproduct = totargetvector.dot(heading);
  if(dotproduct < .95f || dotproduct > 1.05f)
    return;

  fire();
}

IffCode ship::getIffCode()
{
  return iffcode;
}

void ship::setIffCode(IffCode newcode)
{
  iffcode = newcode;
}

void ship::setInitialPosition(Coordinate location)
{
  initialposition = location;
}

Coordinate ship::getInitialPosition()
{
  return initialposition;
}

void ship::targetClosest()
{
  target = World::getWorld().getNearestTarget(this);
  onNewTarget(); 
}

bool ship::hasLock()
{
  if(target == NULL)
    return false;
  if(lockendtime == 0)
    return false;
  if(! isCurrentMissileAppropriateForTarget() )
  {
     return false;
  }

  return Time::getCurrentTime() >= lockendtime;
}

bool ship::lockRequired()
{
  if(nummissiles == 0)
  {
    return false;
  }
  else
  {
    MissileDescription * missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );
    TimeValue  locktime = missiledescription->getLockTime();

    return locktime != 0;
  }
}


void ship::setAutopilot(bool newautopilots)
{
  autopilots = newautopilots;
}

bool ship::doesAutoPilot(int nextnav)
{
  if(autopilots)
  {
	if(lastautopilotnav < 0)
	{
      return true; /* no specified value */
	}
    return nextnav <= lastautopilotnav ;
  }
  else
  {
	return false;	  
  }
}

void ship::setMustSurvive(bool newvalue)
{
  mustsurvive = newvalue;
}

bool ship::getMustSurvive()
{
  return mustsurvive;
}

void ship::buildGunTypes()
{
  int guntypes[50]; // 
  int numguntypes=0;
  int i;
  int currenttype;

  for(i=0; i < numguns; i++)
  {
     currenttype = guns[i]->getWeaponNumber();
     // check to see if we already have the current type
     if(! haveGunTypeAlready(guntypes, currenttype, numguntypes) )
     {
        guntypes[numguntypes] = currenttype;
        numguntypes++;
     }
  }
  numdifferentguntypes = numguntypes;
  differentguntypes = new int[numguntypes];
  for(i=0; i < numguntypes; i++)
  {
     differentguntypes[i] = guntypes[i];
  }
}

bool ship::haveGunTypeAlready(int guntypes [ ], int newtype, int numguntypes)
{
  int i;

  for(i=0; i < numguntypes; i++)
  {
     if(guntypes[i] == newtype)
       return true;        
  }
  return false;
}

void ship::clearVelocityVector()
{
  newvelocity = false;
}

void ship::missileAI()
{
  TimeValue  now;
  if(! hasAI() )
    return;


  if(nummissiles == 0)
    return;

  Targetable * target = getTarget();
  const int waittime = 55000;
  const int waittime2 = 35000;
  if(target == NULL)
     return;
  now = Time::getCurrentTime();
  if(target->instanceof(INST_CAPITALSHIP) )
  {

     if(
       selectTorpedo() && 
       hasLock() &&
       (now > lastmissilefiretime + waittime ) &&
       (target->getLocation().distance(getLocation() ) < 300 )
     )
     {
        fireMissile();
        lastmissilefiretime = now;
     }
  }
  else
  {
     if(
         selectMissile() && 
         (now > lastmissilefiretime + waittime2 ) &&
         last_missile_fired_at != target && 
         (target->getLocation().distance(getLocation() ) < 225 ) &&
         hasLock() && 
         target->getShields().lessThanPercent(.15f)
     )
     {
        fireMissile();
        lastmissilefiretime = now;
        last_missile_fired_at = target;
     }
  }
}


bool ship::selectTorpedo()
{
  int i;
  MissileDescription * missiledescription;

  /* check if we already have a torpedo */
  missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );
  if(missiledescription->canDamageCapships() )
     return true;

  for(i=0; i< nummissiles; i++)
  {
     if(missiles[i]->getNumShotsRemaining() != 0)
     {
        missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[i]->getWeaponNumber() );
        if(missiledescription->canDamageCapships() )
        {
            currentmissile = i;
            TimeValue  locktime = missiledescription->getLockTime();
            lockendtime  = Time::getCurrentTime() + locktime;  
            return true;
        }  
     }
  }
  return false;
}

bool ship::selectMissile()
{
  int i;
  MissileDescription * missiledescription;

  /* check if we already have a torpedo */
  missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );
  if(! missiledescription->canDamageCapships() &&  missiledescription->isTracking() )
     return true;

  for(i=0; i< nummissiles; i++)
  {
     if(missiles[i]->getNumShotsRemaining() != 0)
     {
        missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[i]->getWeaponNumber() );
        if(! missiledescription->canDamageCapships() && missiledescription->isTracking())
        {
            currentmissile = i;
            TimeValue  locktime = missiledescription->getLockTime();
            lockendtime  = Time::getCurrentTime() + locktime;  
            return true;
        }  
     }
  }
  return false;

}

string ship::getPilot()
{
  return pilot;
}

void ship::setPilot(string newpilot)
{
  pilot = newpilot;
}

void ship::defend(ship * defendee)
{
  target = World::getWorld().getNearestTargetDefender(this, defendee);
}

void ship::setComm(Comm * newcomm)
{
  comm = newcomm;
}

Comm * ship::getComm()
{
  return comm;
}

bool ship::isCurrentGunUnlimitedAmmo()
{
  int i;
  if(isFullGuns() ) 
  {
     return true;
  }
  else
  {
     for(i=0 ; i < numguns; i++)
     {
        if(guns[i]->getWeaponNumber() == differentguntypes[currentgun] )
        {
          return guns[i]->hasUnlimitedShots();
        }           
     }
  }
  return true;
}

int ship::getCurrentGunAmmoRemaining()
{
  int remaining=0;
  int i;

  for(i=0 ; i < numguns; i++)
  {
     if(guns[i]->getWeaponNumber() == differentguntypes[currentgun] && (! guns[i]->hasUnlimitedShots() ) )
        remaining += guns[i]->getNumShotsRemaining() ;
  }
  return remaining;
}

void ship::onNewTarget()
{
  if(nummissiles != 0)
  {
     MissileDescription * missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );
     TimeValue  locktime = missiledescription->getLockTime();
     lockendtime  = Time::getCurrentTime() + locktime;
  }
}

void ship::setPlayer()
{
  is_player = true;
}

void ship::targetNextComponent()
{
  if(target == NULL)
	  return;

  if(target->instanceof(INST_CAPITALSHIP) )
  {
    int num_components = ((CapitalShip *) target)->getNumComponents();
    targeted_component = (targeted_component + 1) % num_components; 
  }
}

Component * ship::getTargetedCapshipComponent()
{
  if(target->instanceof(INST_CAPITALSHIP) )
  {
    CapitalShip * cap_target = (CapitalShip *) target;
    return cap_target->getComponent(targeted_component);
  }
  return NULL;
}

float ship::getMaximumFuel()
{
  return maxfuel;
}

float ship::getCurrentFuel()
{
  return currentfuel;
}

void ship::setAfterburner(bool on)
{
  afterburn_on = on;

  if(afterburn_on)
  {
    setMaxVelocity(afterburnervelocity);
    setAcceleration(afterburneracceleration);
	setDesiredVelocity(1.0f);
  }
  else
  {
    setMaxVelocity(normalvelocity);
    setAcceleration(normalacceleration);
	setDesiredVelocity(1.0f);
  }

}

string ship::getCockpitMesh()
{
  return cockpitmesh;
}

void ship::setTargetLock(bool newtargetlock)
{
  target_lock = newtargetlock;
}

bool ship::isTargetLockSelected()
{
  return target_lock;
}

void ship::targetClosestNoIff()
{
  target = World::getWorld().getNearestTarget(this, true);
  onNewTarget(); 
}

void ship::missileLockedPlaySound()
{
  if(is_player && (!missile_sound_played) && hasLock() && target_lock)
  {
     SoundEffect * effect = SoundFactory::getSoundFactory()->getLockSound();
	 effect->play();
     missile_sound_played = true;
  }
}

Reticle * ship::getCurrentReticle()
{
  if(nummissiles == 0)
  {
     return NULL;
  }
  else
  {
    int currentweapontype = missiles[currentmissile]->getWeaponNumber();

    MissileDescription * description = (MissileDescription *) 
		weaponfactory::getweaponfactory().getweapondescription(currentweapontype);
    return description->getReticle();
  }

}


bool ship::isCurrentMissileAppropriateForTarget()
{
  Targetable * target = getTarget();
  if(target == NULL)
	  return false;
  bool targetcapship = target->instanceof(INST_CAPITALSHIP);
  
  MissileDescription * missiledescription = (MissileDescription *) weaponfactory::getweaponfactory().getweapondescription(missiles[currentmissile]->getWeaponNumber() );

  if(targetcapship)
  {
	 CapitalShip * capship = (CapitalShip *) target;
     Component * comp = capship->getComponent(getTargetedCapshipComponentIndex() );
	 if(comp->instanceof(INST_TURRET) )
	 {
       return missiledescription->canDamageTurrets();
	 }
	 else
	 {
       return missiledescription->canDamageCapships();
	 }
  }
  else
  {
	return true;
  }
}

Image * ship::getCurrentGunIcon()
{
  if(numguns == 0)
  {
     return NULL;
  }
  else
  {
//    int currentweapontype = guns[currentgun]->getWeaponNumber();
    int currentweapontype = differentguntypes[currentgun];

    weapondescription * description = weaponfactory::getweaponfactory().getweapondescription(currentweapontype);
    return description->getIcon();
  }

}

Image * ship::getCurrentMissileIcon()
{
  if(nummissiles == 0)
  {
     return NULL;
  }
  else
  {
  int currentweapontype = missiles[currentmissile]->getWeaponNumber();

  weapondescription * description = weaponfactory::getweaponfactory().getweapondescription(currentweapontype);
  return description->getIcon();
  }
}

float ship::getradius()
{
  return fullradius;
}

Coordinate  ship::getMeshCenter()
{
  return fullcenter;
}


Coordinate ship::getExtents()
{
  return fullextents;
}


void ship::computeFullShipExtents()
{
  Coordinate currentmax, currentmin;
  Coordinate delta_extents;
  Pod * current_component;
  int i;


  currentmax = getMeshMaxs();
  currentmin = getMeshMins();

  for(i=0; i < numpods; i++)
  {
	current_component = pods[i];
    currentmax = currentmax.maxPairwise(current_component->getMaxs() );
    currentmin = currentmin.minPairwise(current_component->getMins() );
  }

  fullextents = currentmax - currentmin;
  fullcenter = (currentmax + currentmin) * .5f;
  delta_extents = fullextents * .5f;
  fullradius = fastsqrt( fabs_f(delta_extents.x * delta_extents.x) + 
					 fabs_f(delta_extents.y * delta_extents.y) +
					 fabs_f(delta_extents.z * delta_extents.z) );

}


int ship::getNumTurrets()
{
  return turrets->size();
}

Turret * ship::getTurret(int index)
{
  return (*turrets)[index];
}

void ship::createThrusterLights()
{
/*  int i;
  LightManager * lightmanager = LightManager::getInstance();

  thrusterlights = new DiffuseLight *[numthrusters];
  for(i=0; i < numthrusters; i++)
  {
    thrusterlights[i] = new DiffuseLight();
	thrusterlights[i]->setDiffuse(Color(0.0f, .7f, 1.0f) );
	thrusterlights[i]->setAttenuation(1.0f, .005f * numthrusters, 0.0f);
	// lower priority so weapons shine through
	thrusterlights[i]->setPriority(5);
    lightmanager->addLocalLight(thrusterlights[i] );
  }
*/
}

void ship::removeThrusterLights()
{
/*  int i;

  LightManager * lightmanager = LightManager::getInstance();
  for(i=0; i < numthrusters; i++)
  {
    lightmanager->removeLocalLight(thrusterlights[i] );
  }
*/
}

void ship::setShadowLocation(float newwy)
{
  shadowy = newwy;
  shadow = true;
}

// assumes stencil buffer is already setup
void ship::renderShadow()
{
	Coordinate location;
	Coordinate heading,up;
    int i;

// shadow 
       float matrix[] = 
	   { 1.0f, 0.0f, 0.0f, 0.0f, 
	     0.0f, 0.0f, 0.0f, 0.0f,
		 0.0f, 0.0f, 1.0f, 0.0f,
		 0.0f, 0.0f, 0.0f, 1.0f};

// end shadow
   glEnable(GL_STENCIL_TEST);
   GLERRORCHECK
  // until stencil test is diabled, only write to areas where the
  // stencil buffer has a one. This is to draw the shadow only on
  // the floor.
  glStencilFunc(GL_EQUAL, 1, 0xFFFFFFFF);
   GLERRORCHECK
  // don't modify the contents of the stencil buffer
  glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
   GLERRORCHECK

   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   GLERRORCHECK
   glEnable(GL_BLEND);
   GLERRORCHECK


  setMeshDefaultColor(Color(0.0f, 0.0f, 0.0f, .25f));

  glPushMatrix();
	glTranslatef(0.0f, shadowy, 0.0f);
    glMultMatrixf(matrix);
	Object3d::render();
	for(i=0; i < numpods; i++)
	{
	  glPushMatrix();	  
            location = getLocation();
            glTranslatef(location.x, location.y, location.z);
		    Coordinate podlocation = pods[i]->getTranslation();
            glTranslatef(podlocation.x, podlocation.y, podlocation.z);
       pods[i]->getMesh()->setDefaultColor(Color(0.0f, 0.0f, 0.0f, .25f));
       pods[i]->getMesh()->render();
	   pods[i]->getMesh()->setDefaultColor(Color(1.0f, 1.0f, 1.0f));
    glPopMatrix();
	}
  glPopMatrix();
  GLERRORCHECK

  setMeshDefaultColor(Color(1.0f, 1.0f, 1.0f));
  glDisable(GL_BLEND);
  GLERRORCHECK
  glDisable(GL_STENCIL_TEST);
  GLERRORCHECK
}

void ship::disableShadow()
{
  shadow = false;
}

extern CubeMap * cube2;

void ship::renderShield()
{
         Coordinate location = getcenter();

		  glPushMatrix();
          glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
          glEnable(GL_BLEND);
		  glDisable(GL_LIGHTING);

          glTranslatef(location.x, location.y, location.z);
		  Camera * camera = Camera::getCamera();
//		  matchDOF(camera->getHeading(), camera->getUp() );    
          //glColor4f(0.0f, 0.0f, 1.0f, .2f);
		  glColor4f(0.0f, 0.0f, 1.0f, 0.25f);
          gluSphere(quadric, getradius() - .01, 30, 30);
  if(GLEW_EXT_texture_cube_map && Settings::getSettings()->getReflections() )
  {
    CubeMap * cube2 = CubeMap::getEnvironmentMap();
	cube2->enable();

	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
    matchDOF(camera->getHeading() 
		   , camera->getUp());    

    glMatrixMode(GL_MODELVIEW);
		

	cube2->render();

   //       Coordinate location = getLocation();

//          glTranslatef(location.x, location.y, location.z);
          ///glColor4f(0.0f, 0.0f, 1.0f, .2f);
		  glColor4f(1.0f, 1.0f, 1.0f, 0.30f);
//			  glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
          gluSphere(quadric, getradius(), 30, 30);
          glDisable(GL_BLEND);
		  glEnable(GL_LIGHTING);

         glPopMatrix();
	cube2->disable();
	glMatrixMode(GL_TEXTURE);
	glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
  }
  else
  {
	glPopMatrix();
  }

}


bool ship::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_TARGETABLE | INST_SHIP);
}

bool ship::isCurrentGunChargeable()
{
    if(isFullGuns() )
	{
	   return false;
	}

	int currentweapontype = differentguntypes[currentgun];

    GunDescription * description = 
		(GunDescription *) weaponfactory::getweaponfactory().getweapondescription(currentweapontype);

	return description->isChargeable();
}

void ship::beginCharging()
{
	if(charge_start_time ==  0)
	{
	  charge_start_time = Time::getCurrentTime();
	}
}

void ship::endCharging()
{
  int i;
  weapon * chargedweapon;

  if(charge_start_time ==  0)
  {
     return;
  }

  float percent = getChargePercent();

  for(i=0 ; i < numguns; i++)
  {
     if(guns[i]->getWeaponNumber() == differentguntypes[currentgun] )
	 {
        chargedweapon = fireGun(i);
		if(chargedweapon != NULL) // like if we ran out of energy
		{
			chargedweapon->setDamagePercent(percent);
		}
	 }
  }
  charge_start_time = 0;

}

void ship::advanceChargeSounds()
{
  if(charge_start_time == 0)
  {
      if(currentchargesound != NULL)
	  {
		currentchargesound->stop();
		currentchargesound = NULL;
		playedcharge1= false;
		playedcharge2= false;
	  }
	  return;
  }
  
  if(currentchargesound == NULL)
  {
	  currentchargesound = SoundFactory::getSoundFactory()->getChargeSound(0);
	  currentchargesound->play();
  }
  else
  {
	  if(! currentchargesound->isPlaying() )
	  {
		  // pick between [1, 3]
         float percent = getChargePercent();
		 if(percent >= 1.0f && playedcharge1 && playedcharge2)
		 {
		   currentchargesound = SoundFactory::getSoundFactory()->getChargeSound(3);
		 }
		 else if(percent >= .5f && playedcharge1)
		 {
		   currentchargesound = SoundFactory::getSoundFactory()->getChargeSound(2);
		   playedcharge2 = true;
		 }
		 else
		 {
		   currentchargesound = SoundFactory::getSoundFactory()->getChargeSound(1);
		   playedcharge1 = true;
		 }
         currentchargesound->play();
	  }
  }
}

float ship::getChargePercent()
{
  int currentweapontype = differentguntypes[currentgun];
  GunDescription * description = 
		(GunDescription *) weaponfactory::getweaponfactory().getweapondescription(currentweapontype);
  float numerator = ((float)(Time::getCurrentTime() - charge_start_time)) ;
  float denominator = (float) description->getChargeTime();

  float percent = numerator / denominator;
  percent = MIN(percent, 1.0f);
  return percent;
}

int ship::getNumEngineLocations()
{
  return enginelocations.size();
}

// return in world space
Coordinate ship::getEngineLocation(int index)
{
  Coordinate enginelocation = enginelocations[index];

  return toWorldLocation(enginelocation);
}

Coordinate ship::toWorldLocation(Coordinate & input)
{
  GLfloat matrix[16];
  Coordinate worldpoint;

  matchDOF(getHeading(), getUp(), matrix);
  worldpoint = input.multiplybyglmatrix(matrix);
  worldpoint = worldpoint  + getLocation();
  return worldpoint;  
}

void ship::advancePredictor()
{
  int i;
  mount * mnt;
  float gunvelocity;

  if(getTarget() == NULL)
  {
	return;
  }
/* pick out current gun */
  for(i=0 ; i < numguns; i++)
  {
     if(guns[i]->getWeaponNumber() == differentguntypes[currentgun] )
	 {
       mnt = guns[i];
	   GunDescription * gundesc = (GunDescription *) mnt->getWeaponDescription();
	   gunvelocity = gundesc->getVelocity();
	   break;

	 }
  }

  Coordinate desiredheading = 
	  ittspredictor.getDesiredHeading(getLocation(), gunvelocity,
	    target->getLocation(), target->getVelocity(),
		target->getHeading(), .5f);

 if(Mission::getCurrentMission()->getPlayer() == this )
 {
    Coordinate worldlocation = desiredheading * 10.0f + getCockpitLocation() ;
    // project
    GLdouble objx = worldlocation.x;
    GLdouble objy = worldlocation.y;
    GLdouble objz = worldlocation.z;
    GLdouble modelMatrix[16];
    GLdouble projMatrix[16];
    GLdouble winx;
    GLdouble winy;
    GLdouble winz;
    GLint viewport[4];  

    glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
    glGetIntegerv(GL_VIEWPORT, viewport);
  
	int result = gluProject(objx, objy, objz, modelMatrix, projMatrix, viewport, & winx, & winy, & winz);
	ittsprojected.x = winx;
	ittsprojected.y = winy;
	ittsprojected.z = winz;

 }
}

Coordinate ship::getProjectedItts()
{
  return ittsprojected;
}

int ship::getNumDecoys()
{
  return numdecoys;
}

void ship::fireDecoy()
{
  if(numdecoys != 0)
  {
    numdecoys --;
    SoundEffect3d * effect = SoundFactory::getSoundFactory()->getDecoySound();

    effect->play();
	Decoy * newdecoy = new Decoy();
	Coordinate location = getcenter() - getHeading() * getExtents().z * .5f;
	newdecoy->setLocation( location);
    effect->setPosition(location );
	World::getWorld().addDecoy(newdecoy);
  }
}

void ship::setLastAutopilotNav(int lastnav)
{
  lastautopilotnav = lastnav;
}

int ship::damageHull(int remaining)
{
    int result;

	if(remaining > currenthitpoints)
	{
		result =  remaining - currenthitpoints;
		currenthitpoints = 0;
		return result;
	}
	else
	{
		currenthitpoints -= remaining;
		return 0;
	}
}

float ship::getCoreDamagePercent()
{
  float percentok;

  percentok = (float) currenthitpoints / (float) maxhitpoints;

  return 1.0f - percentok;
}

bool ship::isMissileAttackingMe()
{
  return   missileattackingme ;
}

void ship::setMissileAttackingMe(bool newvalue)
{
  missileattackingme = newvalue;
}

void ship::eject()
{
  EjectionPod * ejected = new EjectionPod(string() + "Ejected Pilot: " + getPilot(), getLocation(), 
	  getUp(), getHeading() * -1.0f, getleft() );
  World::getWorld().add(ejected);

//

  Mission::getCurrentMission()->onDestroyed(this, true);
  setDestroyed();
  World::getWorld().addExplosion(new Explosion (getLocation() ) );
  SoundEffect3d * effect = SoundFactory::getSoundFactory()->getExplosionSound();
  effect->setPosition(getLocation() );
  effect->play();
  if(is_player)
  {
	  if(JoystickFactory::getInstance()->getForceFeedbackJoystick() != NULL)
	  {
		  ForceEffectSet * effect = ForceEffectFactory::getInstance()->getExplodeForce();
		  effect->play();
	  }
  }
  else 
  {
    WingmanComm * wccomm = (WingmanComm *) comm;
	wccomm->eject();
  }
/// should have ejection comm here
///  comm->dead();
}

void ship::setInvulnerable(bool newvalue)
{
  invulnerable = newvalue;
}

void ship::decoyAI()
{
  if(! hasAI() )
    return;
  if(missileattackingme)
  {
	  TimeValue now = Time::getCurrentTime();
	  if(now - lastdecoytime > 500)
	  {
		fireDecoy();
		lastdecoytime = now;
	  }
  }

}

bool ship::shouldEject()
{
  if(getCoreDamagePercent() >= .75f)
  {
	 if(shields.lessThanPercent(.25f) )
	 {
	    return true;
	 }
  }
  return false;
}

void ship::ejectAI()
{
  if(! hasAI() )
    return;

  if(shouldEject() && getIffCode() != ENEMY && ejectallowed)
  {
	eject();
  }
}
