// ship.h: interface for the ship class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SHIP_H__EF21EA99_AFF2_453E_BBD5_11492802AE04__INCLUDED_)
#define AFX_SHIP_H__EF21EA99_AFF2_453E_BBD5_11492802AE04__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//#include "Object3d.h"
#include "Targetable.h"
#include "mount.h"
#include "shipareas.h"
#include "coord.h"
#include "Thruster.h"
#include <string>
#include "Color.h"	// Added by ClassView
#include "IffCode.h"	// Added by ClassView
#include "coldet.h"
#include "Time.h"
#include <vector>
#include "Predictor.h"

using std::string;

class Image;
class Comm;
class Reticle;
class Pod;
class Missile;
class Component;
class SoundEffect3d;
class Turret;
class DiffuseLight;
class weapon;
class SoundEffect;

class ship : public Targetable
{
public:
	bool shouldEject();
	void setInvulnerable(bool newvalue);
	void eject();
	void setMissileAttackingMe(bool newvalue);
	bool isMissileAttackingMe();
	float getCoreDamagePercent();
	void setLastAutopilotNav(int lastnav);
	void fireDecoy();
	int getNumDecoys();
	Coordinate getProjectedItts();
	Coordinate getEngineLocation(int index);
	int getNumEngineLocations();
	void endCharging();
	void beginCharging();
	bool isCurrentGunChargeable();
	virtual bool instanceof(InstanceT it);
	void disableShadow();
	void setShadowLocation(float y);
	void removeThrusterLights();
	Turret * getTurret(int index);
	int getNumTurrets();
	virtual float getradius();
	virtual Coordinate  getMeshCenter();
	virtual  Coordinate getExtents();
	Image * getCurrentMissileIcon();
	Image * getCurrentGunIcon();
	bool isCurrentMissileAppropriateForTarget();
	Reticle * getCurrentReticle();
	void targetClosestNoIff();
	bool isTargetLockSelected();
	void setTargetLock(bool  newtargetlock);
	string getCockpitMesh();
	void setAfterburner(bool on);
	float getCurrentFuel();
	float getMaximumFuel();
	Component * getTargetedCapshipComponent();
	void targetNextComponent();
	void setPlayer();
	int getCurrentGunAmmoRemaining();
	bool isCurrentGunUnlimitedAmmo();
	Comm * getComm();
	void setComm(Comm * newcomm);
	void defend(ship * defendee);
	void setPilot(string pilot);
	string getPilot();
	void clearVelocityVector();
	bool getMustSurvive();
	void setMustSurvive(bool mustsurvive);
	bool doesAutoPilot(int nextnav);
	void setAutopilot(bool autopilots);
	bool hasLock();
	void targetClosest();
	Coordinate getInitialPosition();
	void setInitialPosition(Coordinate location);
	void setIffCode(IffCode newcode);
	IffCode getIffCode();
	virtual void setTarget(Targetable * newtarget);
	bool isFullGuns();
	void setFullGuns(bool fullguns);
	void setVelocityVector(Coordinate newvelocity);
	virtual Coordinate getVelocityVector();
	virtual bool instanceof(char classname[]);
	float getMass();
	virtual void advance();
	int getCurrentEnergy();
	int getMaxEnergy();
	string getCurrentGunName();
	string getCurrentMissileName();
	int getNumCurrentMissile();
	void nextMissile();
	void nextGun();
	 ship(ship & newship);
	virtual Color getRadarColor(ship * player);
	Missile * fireMissile();
	virtual void targetDestroyed();
	Targetable * getTarget();
	virtual bool isShip();
	void nextTarget();
	virtual void  render();
	virtual void onCollisionWith(Object3d * otherobject);
	virtual void damage(int amount, Coordinate fromlocation);
	Coordinate getCockpitLocation();
	void fire();
   ship(mesh * newmesh, int nummounts, mount * mounts[], 
        Coordinate cockpit, shipareas newshields, shipareas newarmor,
        float newvelocity, float newroll, float newpitch, float newyaw, 
 	    float newacceleration, float newangularacceleration,
	    float newafterburnervelocity, float newfterburneracceleration, float newfuel,
        int numthrusters, Thruster * thrusters[], Image * newimage, string newname, int totalenergy, int energyrecharge,
        float mass, string cockpitmesh,
		int maxdecoys,
		std::vector<Turret *> * turrets,
        std::vector<Coordinate> newenginelocations,
		int newmaxhitpoints,
		Pod ** newpods=NULL, int newnumpods=0);
   virtual ~ship();
  	bool lockRequired();
	// for debugging use only
	virtual void destroy();
protected:
   IffCode iffcode;
	std::vector<Turret *> * turrets;
	Coordinate toWorldLocation(Coordinate & input);
private:
	void ejectAI();
	void decoyAI();
	int damageHull(int remaining);
	Coordinate ittsprojected;
	void advancePredictor();
	float getChargePercent();
	void advanceChargeSounds();
	void renderShield();
	void renderShadow();
	void createThrusterLights();
	void computeFullShipExtents();
	void missileLockedPlaySound();
	bool missile_sound_played;
	void onNewTarget();
	bool selectMissile();
   /* returns true if it worked, false if no torpedos left */
   bool selectTorpedo();
	void missileAI();
	bool haveGunTypeAlready(int guntypes[], int newtype, int numguntypes);
	void buildGunTypes();
	void fireAI();
	weapon * fireGun(int gunnumber);
	void initGunsAndMissiles(mount ** mymounts, int nummounts);
    mount ** guns;
    mount ** missiles;
    int nummounts;
    int numguns;
    int nummissiles;
    int currentgun; // an index into different gun type.  
    int currentmissile;
    int * differentguntypes;
    int numdifferentguntypes;
    Coordinate cockpit;
    GLUquadricObj * quadric;
    TimeValue shieldstarttime;
    TimeValue shieldendtime;
    Thruster * * thrusters;
//	DiffuseLight * * thrusterlights;
    int numthrusters;
    float maxenergy;
    float currentenergy;
    float energy_recharge_rate;
    float mass;
	int maxhitpoints;
	int currenthitpoints;
    Coordinate velocityvector;
    bool newvelocity;
    TimeValue * gunfiretime;
    TimeValue * missilefiretime;
    TimeValue startvelocityinterpolate;
    TimeValue lockendtime;
    Coordinate initialposition;
    bool autopilots;
    bool mustsurvive;
    TimeValue lastmissilefiretime;
    Targetable * last_missile_fired_at;
    string pilot;
    Comm * comm;
    bool is_player;
    float normalvelocity;
	float normalacceleration;
    float afterburnervelocity;
	float afterburneracceleration;
	float maxfuel;
	float currentfuel;
	bool afterburn_on;
	bool prev_afterburn_on;
	string cockpitmesh;
	/* player or ai has locked the selected target.  This is
	   different from the missile lock. */
	bool target_lock;
	Pod ** pods;
	int numpods;
	float fullradius;
	Coordinate fullcenter;
	Coordinate fullextents;
	SoundEffect3d * afterburnersound;
	SoundEffect * currentchargesound;
	bool playedcharge1;
	bool playedcharge2;
	bool shadow;
	float shadowy;
    TimeValue charge_start_time;
    std::vector<Coordinate> enginelocations;
	Predictor ittspredictor;
	int maxdecoys;
	int numdecoys;
	int lastautopilotnav;
	bool missileattackingme;
	bool invulnerable;
    TimeValue lastdecoytime;
	// won't always succeed in ejecting
	bool ejectallowed;
};

#endif // !defined(AFX_SHIP_H__EF21EA99_AFF2_453E_BBD5_11492802AE04__INCLUDED_)
