// shipfactory.cpp: implementation of the shipfactory class.
//
//////////////////////////////////////////////////////////////////////

#include "shipfactory.h"
#include "shipxmladapter.h"
#include "World.h"
#include "CapitalShip.h"
#include "CommFactory.h"
#include "LandingComm.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

shipfactory::shipfactory()
  : ships(50), numships(0)
{

}

shipfactory::~shipfactory()
{

}

shipfactory shipfactory::theshipfactory;

shipfactory & shipfactory::getshipfactory()
{
  return theshipfactory;
}

ship * shipfactory::getship(char  shipname [])
{
   ship * oldship;
   ship * newship;

   if(ships.hasKey(shipname))
   {
     oldship = (ship *) ships.getItem(shipname);
     if(oldship->instanceof(INST_CAPITALSHIP) )
     {
        newship  = new CapitalShip(*(CapitalShip *)oldship);    
     }
     else
     {
        newship  = new ship(*oldship);    
     }
   }
   else
   {
      shipxmladapter * adapter = new shipxmladapter(shipname);      
      newship = adapter->getship();
      delete adapter;
      ships.addItem(shipname, newship);
   }
   World::getWorld().add(newship);
   garbage[numships] = newship;
   numships++;
   return newship;
}

ship * shipfactory::getShip(ShipInfo & info)
{
  ship * newship;

  newship = getship((char *) info.getFileName().c_str() );
  newship->setIffCode(info.getIffCode() );
  newship->setLocation(info.getStartingLocation() );
  newship->setInitialPosition(info.getStartingLocation() );
  newship->setAutopilot(info.getAutoPilot() );
  newship->setLastAutopilotNav(info.getLastAutopilot() );
  newship->setPilot( info.getPilot() );
  if(info.getPilot() == "player")
  {
    newship->setComm( (Comm *) 
		CommFactory::getInstance()->getPlayerComm( 
		  (char *) info.getComm().c_str() 		  
		) 
	);
  }
  else if(info.getIffCode() == FRIEND)
  {
    if(info.getLand() )
    {
      newship->setComm(
		CommFactory::getInstance()->getLandingComm( 
		  (char *) info.getComm().c_str() ,		  
		  (char *) info.getVideo().c_str(),
		  (char *) info.getPilot().c_str()
		)		   
	  );
    }
    else
    {
      newship->setComm( (Comm *) 
		  CommFactory::getInstance()->getWingmanComm( 
		    (char *) info.getComm().c_str(),
			(char *) info.getVideo().c_str(),
    		(char *) info.getPilot().c_str()
		  ) 
	  );
    }
  }
  else
  {
    newship->setComm( 
		CommFactory::getInstance()->getComm( 
			(char *) info.getComm().c_str() ,
			(char *) info.getVideo().c_str(),
    		(char *) info.getPilot().c_str(),
			false
		) 

	);
  }
  if(info.getLand() )
  {
    if(newship->instanceof("CapitalShip") )
    {
      CapitalShip * capship = (CapitalShip *) newship;
      capship->setLandOn(info.getLand() );
    }
  }
  if(info.getMustSurvive() )
  {
    newship->setMustSurvive(true);
  }
  return newship;
}

void shipfactory::clear()
{
  int i;

  for(i=0; i < numships; i++)
  {
     delete garbage[i];
  }
  numships = 0;
  ships.clear();
}

void shipfactory::preloadShip(char shipname [ ])
{
   ship * newship;

   if(ships.hasKey(shipname))
   {
     return;
   }
   else
   {
      shipxmladapter * adapter = new shipxmladapter(shipname);      
      newship = adapter->getship();
      delete adapter;
      ships.addItem(shipname, newship);
   }
   garbage[numships] = newship;
   numships++;
}
