// shipxmladapter.cpp: implementation of the shipxmladapter class.
//
//////////////////////////////////////////////////////////////////////

#include "shipxmladapter.h"
#include "saxexpat.h"
#include "InputSource.h"
#include "AttributeList.h"
#include "CapitalShip.h"
#include "ErrorHandlerE.h"
#include "OutOfBoundsException.h"
#include "Constants.h"
#include "ImageFactory.h"
#include "Image.h"
#include "Component.h"
#include "ship.h"
#include "Pod.h"
#include "TurretJointed.h"
#include "TurretBallSocket.h"
#include <sstream>
#include <vector>
#include <iostream.h>


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

shipxmladapter::shipxmladapter(char xmlfilename[])
{
	expat myParser;

   ErrorHandlerE::checkFile(xmlfilename);
   iscapship = false;
   totalenergy = 0;
   energy_recharge_rate = 0;
   numshots = -1;
   mounts = NULL;
   thrusters = NULL;
   components = NULL;
   currentmount = 0;
   currentthruster = 0;
   nummounts = 0;
   length = 1.0f;
   numdecoys = 0;
   num_components = 0;
   current_component = 0;
   currentpod = 0;
   numpods = 0;
   turrets = new std::vector<Turret *>();
   cockpit_mesh = "co_tig.mesh.bin";
	myParser.setDocumentHandler(*this);
   myParser.setErrorHandler(ErrorHandlerE::getInstance() );
   myParser.parse(* (new SAX::InputSource(xmlfilename)));

}

shipxmladapter::~shipxmladapter()
{

}

void shipxmladapter::startDocument()
{

}

void shipxmladapter::endDocument()
{

}

void shipxmladapter::startElement(const std::string& name, const SAX::AttributeList* atts)
{
   int i;
   //cout << name.c_str() << endl;
  
   if(name == "point")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "x")
         {
            currentcoordinate.x = atof(currentvalue.c_str() );
         }
         else if (currentname == "y")
         {
            currentcoordinate.y = atof(currentvalue.c_str() );
         }
         else if(currentname == "z")
         {
            currentcoordinate.z = atof(currentvalue.c_str() );
         }
      }
   }
   else if(name == "meshfile")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "name")
         {
            meshfilename = currentvalue;
         }
         else if (currentname == "numpods")
         {
		   numpods = atoi(currentvalue.c_str() );
		   if(numpods == 0)
		   {
			 pods = NULL;
		   }
		   else
		   {
             pods = new Pod *[numpods];
		   }
         }
      }
   }
   else if(name == "mounts")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);

         if(currentname == "number")
         {
             nummounts = atoi(currentvalue.c_str());
             mounts = new mount * [nummounts];
         }
		 else if(currentname == "decoys")
		 {
			numdecoys = atoi(currentvalue.c_str());
		 }
	  }
   }
   else if(name == "turretmounts")
   {
	   // pick arbitrary number
      nummounts = 5;
	  currentmount = 0;
      mounts = new mount * [nummounts];
   }
   else if(name == "mount")
   {
      if(atts != NULL)
      {
         for(i=0; i < atts->getLength(); i++)
         {
            std::string  currentname = atts->getName(i);
            std::string  currentvalue = atts->getValue(i);
  
            if(currentname == "number_of_shots")
            {
               numshots = atoi(currentvalue.c_str() );
            }
         } 
      }
   }
   else if(name == "thrusters")
   {
      numthrusters = atoi(atts->getValue(0).c_str());
      thrusters = new Thruster * [numthrusters];
   }
   else if(name == "weapon")
   {    
      weaponnumber = atoi(atts->getValue(0).c_str() );
   }
   else if(name == "speeds")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "velocity")
         {
            velocity = atof(currentvalue.c_str() ) * Constants::getDistanceFactor();
         }
         else if (currentname == "roll")
         {
            roll = atof(currentvalue.c_str() )* Constants::getAngleFactor();
         }
         else if(currentname == "pitch")
         {
            pitch = atof(currentvalue.c_str() )* Constants::getAngleFactor();
         }
         else if(currentname == "yaw")
         {
            yaw = atof(currentvalue.c_str() )* Constants::getAngleFactor();
         }
         else if(currentname == "angular_acceleration")
         {
			 // check this later
            angular_acceleration = atof(currentvalue.c_str() )* Constants::getAngleFactor();
         }
         else if(currentname == "acceleration")
         {
			 // check this later
            acceleration = atof(currentvalue.c_str() ) * Constants::getDistanceFactor();
         }

      }
   }
   else if(name == "afterburner")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "velocity")
		 {
			afterburner_velocity = atof((char *) currentvalue.c_str() )  * Constants::getDistanceFactor();
		 }
		 else if(currentname == "acceleration")
		 {
			afterburner_acceleration = atof((char *) currentvalue.c_str() )  * Constants::getDistanceFactor();
		 }
		 else if(currentname == "fuel")
		 {
			fuel = atof((char *) currentvalue.c_str() );
		 }
	  }
   }
   else if(name == "pod")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "mesh")
		 {
            mesh * podmesh = new mesh((char *) currentvalue.c_str() );
            pods[currentpod] = new Pod(podmesh);
		 }
	  }
   }
   else if(name == "armor")
   {  
      int front, back, left, right;

      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "front")
         {
            front = atoi(currentvalue.c_str() );
         }
         else if (currentname == "rear")
         {
            back = atoi(currentvalue.c_str() );
         }
         else if(currentname == "left")
         {
            left = atoi(currentvalue.c_str() );
         }
         else if(currentname == "right")
         {
            right = atoi(currentvalue.c_str() );
         }
         armor = shipareas(front, back, left, right);
      }
   }
   else if(name == "shields")
   {  
      int front, back, left, right, recharge;

      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "front")
         {
            front = atoi(currentvalue.c_str() );
         }
         else if (currentname == "rear")
         {
            back = atoi(currentvalue.c_str() );
         }
         else if(currentname == "left")
         {
            left = atoi(currentvalue.c_str() );
         }
         else if(currentname == "right")
         {
            right = atoi(currentvalue.c_str() );
         }
         else if(currentname == "recharge_rate" || currentname == "regeneration_rate")
         {
            recharge = atoi(currentvalue.c_str() );
         }
         shields = shipareas(front, back, left, right, recharge);
      }

   }
   else if(name == "thruster")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         if(currentname == "radius")
         {
            radius = atof(currentvalue.c_str() );
         }
         else if(currentname == "length")
         {
            length = atof(currentvalue.c_str() );   
         }
      }
   }
   else if(name == "shipfile")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         
         if(currentname == "name")
         {
            shipname = currentvalue;
         }
         else if(currentname == "image")
         {
            imagename = currentvalue;
         }
         else if(currentname == "mass")
         {
           mass = atof((char *) currentvalue.c_str() );
         }
         else if(currentname == "hitpoints")
         {
           hitpoints = atoi((char *) currentvalue.c_str() );
         }

      }
   }
   else if(name == "energy")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         
         if(currentname == "total")
         {
            totalenergy = atoi(currentvalue.c_str() );
         }
         else if(currentname == "recharge_rate")
         {
            energy_recharge_rate = atoi(currentvalue.c_str() );
         }
      }
   }
   else if(name == "capship")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         
         if(currentname == "num_components")
         {
            num_components = atoi(currentvalue.c_str() );
            components = new Component *[num_components];
         }
      }
   }
   else if(name == "component")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         
         if(currentname == "type")
         {
            component_name = currentvalue;
         }
         else if(currentname == "mesh")
         {
            component_mesh = currentvalue;
         }
         else if(currentname == "damaged_mesh")
         {
            component_damaged_mesh = currentvalue;
         }
         else if(currentname == "hit_points")
         {
            component_hitpoints = atoi(currentvalue.c_str() );
         }
      }
   }
   else if(name == "turret")
   {
      for(i=0; i < atts->getLength(); i++)
      {
         std::string  currentname = atts->getName(i);
         std::string  currentvalue = atts->getValue(i);
         
         if(currentname == "type")
         {
            component_name = currentvalue;
         }
         else if(currentname == "mesh")
         {
            component_mesh = currentvalue;
         }
         else if(currentname == "damaged_mesh")
         {
            component_damaged_mesh = currentvalue;
         }
         else if(currentname == "hit_points")
         {
            component_hitpoints = atoi(currentvalue.c_str() );
         }
      }
   }
   else if(name == "cockpit")
   {
	  if(atts != NULL)
      {
        for(i=0; i < atts->getLength(); i++)
		{
           std::string  currentname = atts->getName(i);
           std::string  currentvalue = atts->getValue(i);
           if(currentname == "mesh")
		   {
			  cockpit_mesh = currentvalue;
		   }
		}
	  }
   }
   else if(name == "rotation")
   {
	  if(atts != NULL)
      {
        for(i=0; i < atts->getLength(); i++)
		{
           std::string  currentname = atts->getName(i);
           std::string  currentvalue = atts->getValue(i);
           if(currentname == "matrix")
		   {
			   int j;
			   std::istringstream is(currentvalue);
			   for(j=0; j < 9; j++)
			   {
                  is >> rotation[j] ;
			   }

		   }
		}
	  }
   }
   else if(name == "barrel")
   {
	  if(atts != NULL)
      {
        for(i=0; i < atts->getLength(); i++)
		{
           std::string  currentname = atts->getName(i);
           std::string  currentvalue = atts->getValue(i);

		    if(currentname == "barrel_mesh")
			{
				barrelmesh = currentvalue;
			}
		}
	  }
   }
   else if(name == "jointconstraint")
   {
	  jointconstraint = true;
   }
   else if(name == "ballsocketconstraint")
   {
      jointconstraint = false;
	  if(atts != NULL)
      {
        for(i=0; i < atts->getLength(); i++)
		{
           std::string  currentname = atts->getName(i);
           std::string  currentvalue = atts->getValue(i);
           if(currentname == "cone")
		   {
              cone = atof(currentvalue.c_str() );
		   }
		}
	  }
   }


}

void shipxmladapter::endElement(const std::string& name)
{
   int i;

   if(name == "mount")
   {
      if(currentmount >= nummounts)
      {  
         throw OutOfBoundsException("mounts in a ship file");
         return;
      }

      if(numshots != -1)
      {
        mounts[currentmount] = new mount(currentcoordinate, weaponnumber, numshots);
      }
      else
      {
        mounts[currentmount] = new mount(currentcoordinate, weaponnumber);
      }
      currentmount++;
      numshots = -1;
   }
   else if(name == "meshfile")
   {
	  if(numpods != 0)
	  {
        if(currentpod >= numpods + 1)
		{  
           throw OutOfBoundsException("pods in a ship file");
           return;
		}
	  }
   }
   else if(name == "thruster")
   {
      if(currentthruster >= numthrusters)
      {  
         throw OutOfBoundsException("thrusters in a ship file");
         return;
      }

      thrusters[currentthruster] = new Thruster(currentcoordinate, radius, length);
      currentthruster++;
   }
   else if(name == "component")
   {
      if(current_component >= num_components)
      {  
         throw OutOfBoundsException("components in a ship file");
         return;
      }

      mesh * damagedmesh;
	  if(component_damaged_mesh == "")
	  {
        damagedmesh = NULL;
	  }
	  else
	  {
        damagedmesh = new mesh((char *) component_damaged_mesh.c_str() );
	  }
      components[current_component] = new Component(component_name, 
        new mesh((char *) component_mesh.c_str()),
	    damagedmesh, 
        component_hitpoints);
	  components[current_component]->setLocation(currentcoordinate);
//	  components[current_component]->debuglocation = currentcoordinate;
//	  cout << "loading comp coord" << endl;
//	  currentcoordinate.print();
      current_component++;
   }
   else if(name == "turret")
   {
      mesh * turdamagedmesh;
	  mesh * barmesh;
	  if(component_damaged_mesh == "")
	  {
        turdamagedmesh = NULL;
	  }
	  else
	  {
        turdamagedmesh = new mesh((char *) component_damaged_mesh.c_str() );
	  }
	  if(barrelmesh == "")
	  {
		barmesh = NULL;
	  }
	  else
	  {
        barmesh = new mesh((char *) barrelmesh.c_str() );
	  }

	  std::vector<mount *> * turretmounts = new std::vector<mount *>();
	  for(i=0; i < currentmount; i++)
	  {
         turretmounts->push_back(mounts[i]);
	  }
      Turret * turret ;
	  if(jointconstraint)
	  {
          turret = new TurretJointed(component_name, 
          new mesh((char *) component_mesh.c_str()),
          turdamagedmesh, 
		  barmesh,
          component_hitpoints,
		  turretlocation,
		  Coordinate(turretrotation[0], turretrotation[3], turretrotation[6]), 
		  Coordinate(turretrotation[1], turretrotation[4], turretrotation[7]), 
		  Coordinate(turretrotation[2], turretrotation[5], turretrotation[8]),
		  barrellocation,
		  Coordinate(barrelrotation[0], barrelrotation[3], barrelrotation[6]), 
		  Coordinate(barrelrotation[1], barrelrotation[4], barrelrotation[7]), 
		  Coordinate(barrelrotation[2], barrelrotation[5], barrelrotation[8]),
		  turretmounts,
		  constraint
		);
	  }
	  else
	  {
          turret = new TurretBallSocket(component_name, 
          new mesh((char *) component_mesh.c_str()),
          turdamagedmesh, 
		  barmesh,
          component_hitpoints,
		  turretlocation,
		  Coordinate(turretrotation[0], turretrotation[3], turretrotation[6]), 
		  Coordinate(turretrotation[1], turretrotation[4], turretrotation[7]), 
		  Coordinate(turretrotation[2], turretrotation[5], turretrotation[8]),
		  barrellocation,
		  Coordinate(barrelrotation[0], barrelrotation[3], barrelrotation[6]), 
		  Coordinate(barrelrotation[1], barrelrotation[4], barrelrotation[7]), 
		  Coordinate(barrelrotation[2], barrelrotation[5], barrelrotation[8]),
		  turretmounts,
		  cone);

	  }
	  turrets->push_back(turret);
   }

   else if(name == "cockpit")
   {
      cockpitlocation = currentcoordinate;
   }
   else if(name == "engine")
   {
      enginelocations.push_back(currentcoordinate);
   }
   else if(name == "explosion")
   {
      explosionlocations.push_back(currentcoordinate);
   }
   else if(name == "jointconstraint")
   {
      constraint = currentcoordinate;
   }
   else if(name == "capship")
   {
     iscapship = true;
   }
   else if(name == "pod")
   {
     pods[currentpod]->setTranslation(currentcoordinate);
	 currentpod++;
   }
   else if(name == "turretlocation")
   {
	   turretlocation = currentcoordinate;
	   int i;
	   for(i=0; i <9; i++)
	   {
			turretrotation[i] = rotation[i];
	   }
   }
   else if(name == "barrel")
   {
	   barrellocation = currentcoordinate;
	   int i;
	   for(i=0; i <9; i++)
	   {
			barrelrotation[i] = rotation[i];
	   }
   }
}

void shipxmladapter::characters(const std::string& ch)
{

}


ship * shipxmladapter::getship()
{
  ship * returnship;
  if(iscapship)
  {
   returnship = new CapitalShip(new mesh((char *) meshfilename.c_str()), 
                   currentmount, mounts, cockpitlocation, 
                   shields, armor, velocity, roll, pitch, yaw,
				   acceleration, angular_acceleration, 
				   afterburner_velocity, afterburner_acceleration, 
				   fuel,				   
				   numthrusters, thrusters,
                   ImageFactory::getImage((char *) imagename.c_str() ),  shipname,
                   totalenergy, energy_recharge_rate, mass, 
				   turrets,
				   enginelocations,
				   components, num_components,
				   explosionlocations);

  }
  else
  {

     returnship = new ship(new mesh((char *) meshfilename.c_str()),  
                   currentmount, mounts, cockpitlocation, 
                   shields, armor, velocity, roll, pitch, yaw, 
				   acceleration, angular_acceleration, 
				   afterburner_velocity, afterburner_acceleration, 
				   fuel,

				   numthrusters, thrusters,
                   ImageFactory::getImage((char *) imagename.c_str() ),  shipname,
                   totalenergy, energy_recharge_rate, mass, cockpit_mesh,
				   numdecoys,
				   turrets,
				   enginelocations,
				   hitpoints,
				   pods, numpods
				   );
  }
  return returnship;
}

