#include "GL/glew.h"
#ifdef WIN32
#include "gl/wglew.h"
#endif
#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <math.h>
#include "EventListener.h"

#include "FileNotFoundException.h"
#include "OutOfBoundsException.h"
#include "Settings.h"
#include "SoundFactory.h"
#include "GameEngine.h"
#include "GameFlow.h"
#include "SplashScreen.h"
#include "Math.h"
#include "JoystickFactory.h"
#include "Joystick.h"
#include "PositionHistory.h"

#ifdef WIN32
#include <float.h>
#endif

void IdleFunction();
void ChangeDisplayMode();
void RestoreDisplayMode();
void setEventListener(EventListener * el);

EventListener * listener = NULL;

void setEventListener(EventListener * el)
{
  listener = el;
}


void init(void)  {

   SoundFactory::getSoundFactory();

/*   GLfloat ambientlight[] = {.5f, .5f, .5f, 1.0f};
   GLfloat diffuselight[] = {.7f, .7f, .7f, 1.0f};
   GLfloat lightpos [] = {-20.0f, 20.0f, 20.0f, 1.0f};

   glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
   glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
   glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
   glDisable(GL_DITHER);
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_CULL_FACE);
   
   glEnable(GL_LIGHTING);
   glLightfv(GL_LIGHT0, GL_AMBIENT, ambientlight);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuselight);
   glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
   glEnable(GL_LIGHT0);
   glEnable(GL_TEXTURE_2D);
   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);



   glEnable(GL_COLOR_MATERIAL);
   glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
  // glEnable(GL_TEXTURE_2D);
  // glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

*/


}

void display(void) {
  listener->onDisplay();
}

void reshape (int w, int h) {
  listener->onReshape(w, h);
}

void keyboardUp(unsigned char key, int x, int y) 
{
  listener->onKeyboardUp(key, x, y);
}

void keyboard(unsigned char key, int x, int y) 
{
  listener->onKeyboard(key, x, y);
}

void specialUpKeyboard(int key, int x, int y)
{
  listener->onSpecialUpKeyboard(key, x, y);
}

void specialKeyboard(int key, int x, int y) {
  listener->onSpecialKeyboard(key, x, y);
}

void IdleFunction() {
  listener->onIdle();
  glutReportErrors();
}


void RestoreDisplayMode()
{
#ifdef GAMEMODE
  glutLeaveGameMode();
#endif
}

void mousefunction(int button, int state, int x, int y) 
{
  if(state == GLUT_DOWN)
  {
    listener->onMouseDown(button, state, x, y);
  }
}


void mousemove(int x, int y)
{
  listener->onMouseMove(x, y);
}

void registerlisteners()
{
   glutDisplayFunc(display); 
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutKeyboardUpFunc(keyboardUp);
   glutSpecialFunc(specialKeyboard);
   glutSpecialUpFunc(specialUpKeyboard);
   glutIdleFunc(IdleFunction);
   glutMouseFunc(mousefunction);
   glutPassiveMotionFunc(mousemove);
}

void printAvailableExtensions()
{
    cout << "Graphics: " << 
		glGetString(GL_VENDOR) << " " << 
		glGetString(GL_RENDERER) << endl;


    if (!GLEW_EXT_point_parameters)
	{
		cout << "no point parameters" << endl;
	}
	else
	{
		cout << "point parameters extension found" << endl;
	}

    if (!GLEW_NV_point_sprite)
	{
		cout << "no point sprites" << endl;
	}
	else
	{
		cout << "point sprites extension found" << endl;
	}
    if( GLEW_ARB_shader_objects  && GLEW_ARB_fragment_shader && GLEW_ARB_vertex_shader)
	{
		cout << "Shaders found" << endl;
	}
	else
	{
		cout << "No Shaders found" << endl;
	}

	if(GLEW_SGIS_generate_mipmap)
	{
		cout << "Generate mipmap found" << endl;
	}
	else
	{
		cout << "Generate mipmap not found" << endl;
	}

	if(GLEW_ARB_texture_compression)
	{
		cout << "Texture compression found" << endl;
		if(GLEW_EXT_texture_compression_s3tc)
		{
           cout << "S3TC texture compression found" << endl;
		}
		else
		{
           cout << "No S3TC texture compression found" << endl;
		}
	}
	else
	{
		cout << "No texture compression found" << endl;
	}
	if(GLEW_EXT_texture_filter_anisotropic)
	{

		cout << "Anisotropic texture filtering found" << endl;
	}
	else
	{
		cout << "Anisotropic texture filtering  not found" << endl;
	}
#ifdef WIN32
	//EXT_swap_control
	if(WGLEW_EXT_swap_control)
	{
	  cout << "Swap control found" << endl;
	}
	else
	{
	  cout << "Swap control not found" << endl;
	}
#endif
	if(GLEW_EXT_texture_cube_map)
	{
	  cout << "Cube mapping found" << endl;
	}
	else
	{
	  cout << "No cube mapping found" << endl;
	}
}

void printPos(PositionHistory & hist)
{
  int i;

  for(i=0; i < hist.getNumElements(); i++)
  {

	Coordinate c = hist.getElement(i);
	c.print();
  }
  cout << "--" << endl;

}

void testPositionHistory()
{
  PositionHistory poshist(10.0f, 3);

  poshist.update(Coordinate(1.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(5.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(10.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(11.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(16.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(25.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(29.0f, 0.0f, 0.0f));
  printPos(poshist);
  poshist.update(Coordinate(39.0f, 0.0f, 0.0f));
  printPos(poshist);

}

#define MAIN main


int MAIN(int argc, char** argv) {

try
{
   char version[]="Flight Commander 1.1 alpha";

/*   testPositionHistory();
   return;
*/

#ifdef WIN32
   // use lower precision for better performance hopefully
   _control87( _PC_24, MCW_PC );

#endif
   build_sqrt_table();

   cout << version << endl;
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL );
#ifdef GAMEMODE
    // 640x480, 16bit pixel depth, 60Hz refresh rate
    glutGameModeString( "640x480:16" );
    // start fullscreen game mode

    int value = glutEnterGameMode();
#else
    int window = glutCreateWindow(version);
    glutSetWindow(window);
#endif
	

	/////////////
GLenum err = glewInit();
if (GLEW_OK != err)
{
  /* problem: glewInit failed, something is seriously wrong */
  fprintf(stderr, "Error: %s\n", glewGetErrorString(err));
}

     printAvailableExtensions();
     JoystickFactory * factory = JoystickFactory::getInstance();
     Joystick * joy = factory->getJoystick();
	 if(joy == NULL)
	 {
		cout << "Joystick not found" << endl;
	 }
	 else
	 {
	   joy->printInfo();
	 }



   init ();
//   setEventListener(new GameFlow() );
   setEventListener(new SplashScreen() );

   registerlisteners();
   glutReportErrors();
   ///
#ifndef GAMEMODE
    glutReshapeWindow(640, 480);
   glutPositionWindow(0, 0);
    glutShowWindow();
#endif
   //
   glutMainLoop();

}
catch(FileNotFoundException e)
{
  e.print(cout);
  cout.flush();
  RestoreDisplayMode();
}
catch(OutOfBoundsException e)
{
  cout << "You used too many elements in " << e.file.c_str() << " , please declare more" << endl;
  cout.flush();
  RestoreDisplayMode();
}
  delete SoundFactory::getSoundFactory();
return 0;
}

