// weapon.cpp: implementation of the weapon class.
//
//////////////////////////////////////////////////////////////////////

#include "weapon.h"
#include "World.h"
#include <string.h>
#include "Mission.h"
#include "AxisAlignedBillboard.h"
#include "LightManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

weapon::weapon(GunDescription * newdescription, Coordinate position, Coordinate newheading, Coordinate newup, Coordinate newleft, SoundEffect3d * neweffect)
   : Object3d(newdescription->getmesh(), 
        position /*+ getoffset(newdescription->getmesh(), newheading) */,
        newheading, newup, newleft, newdescription->getVelocity(), 
        newdescription->getRoll(), newdescription->getPitch(), newdescription->getYaw(),
		0.0f, 0.0f // no accelerations for weapons
		),
     description(newdescription),
     fired_by_player(false),
	 effect(neweffect)
{
  setMaxDistance(newdescription->getrange() );
  light.setDiffuse(newdescription->getLightColor()  );
  light.setAttenuation(1.0f, .005f, 0.0f);
//  light.setAttenuation(1.0f, .05f, 0.0f);
  LightManager::getInstance()->addLocalLight(&light);
  damagepercent = 1.0f;
}


weapon::~weapon()
{
  LightManager::getInstance()->removeLocalLight(&light);
}

Coordinate weapon::getoffset(mesh * newmesh, Coordinate pheading)
{
   float radius = newmesh->getExtents().z / 2.0f;
   return pheading * radius;
}

void weapon::onCollisionWith(Object3d * newobject)
{
  if(canDamage(newobject) )
  {
    newobject->damage(description->getdamage() * damagepercent, getLocation() );
    if(fired_by_player)
    {
       Mission * mission = Mission::getCurrentMission();
       if( newobject->isdestroyed() && newobject->instanceof(INST_SHIP))
       {
          // we have a kill
          mission->incrementPlayerKills();

       }
       // player had a hit
       mission->incrementPlayerGunHits();        
    }
  }
  setDestroyed();
}

void weapon::damage(int amount, Coordinate fromlocation)
{
  setDestroyed();
}

bool weapon::cantHit(Object3d * other)
{
  if(other == canthit)
    return true;
  return false;
}

void weapon::setcanthit(Object3d * obj)
{
  canthit = obj;
}

bool weapon::instanceof(char classname [ ])
{
  if(strcmp(classname, "weapon") == 0)
    return true;
  return Object3d::instanceof(classname);
}

void weapon::setFiredByPlayer()
{
  fired_by_player =  true;
}


void weapon::advance()
{
	Object3d::advance();
	if(effect != NULL && effect->isPlaying() )
	{
      effect->setPosition(getLocation() );
	  effect->render();
	}
	else
	{
      delete effect;
	  effect = NULL;
	}
	light.setLocation(getLocation() );
}

void weapon::render()
{
   AbstractBillboard * billboard = description->getBillboard();

   if(billboard != NULL)
   {
     /////////////////////////
 
     /////////////////////////
     billboard->setLocation(getLocation() );
     billboard->setOrientation(getHeading(), getUp(), getleft() );
     billboard->render();
   }
   else
   {
     glDisable(GL_LIGHTING);
     Object3d::render();
     glEnable(GL_LIGHTING);
   }
}

bool weapon::canDamage(Object3d * newobject)
{
  if(newobject->instanceof(INST_TURRET) && description->canDamageTurrets() )
  {
    return true;
  }
  else if(newobject->instanceof(INST_COMPONENT) && description->canDamageCapships() )
  {
    return true;
  }
  else if(newobject->instanceof(INST_TARGETABLE) && !(newobject->instanceof("CapitalShip")) )
  {
    return true;	  
  }
  else
  {
    return false;
  }
}


bool weapon::instanceof(InstanceT it)
{
  return it & (INST_OBJECT | INST_WEAPON);
}

void weapon::setDamagePercent(float percent)
{
  damagepercent = percent;
}
