// weapondescription.cpp: implementation of the weapondescription class.
//
//////////////////////////////////////////////////////////////////////

#include "weapondescription.h"
#include "mesh.h"
#include "ImageFactory.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

weapondescription::weapondescription(float newrange, int newdamage, string newmesh,
   float newvelocity, float newroll, float newpitch, float newyaw, string newname,
   int new_refire_delay, float new_acceleration, string newicon, bool newtransparentmesh,
   bool newdamagecapships, bool newdamageturrets, int newsoundindex)
   : range(newrange),
     damage(newdamage),
     mymesh(NULL),
     velocity(newvelocity),
     roll(newroll),
     pitch(newpitch),
     yaw(newyaw),
     name(newname),
     refiredelay(new_refire_delay),
	 acceleration(new_acceleration),
	 image(NULL),
	 imagename(newicon),
	 meshname(newmesh),
	 transparentmesh(newtransparentmesh),
	 loaded(false),
	 damage_capships(newdamagecapships),
	 damage_turrets(newdamageturrets),
	 soundindex(newsoundindex)
{
  rangesquared = range * range;
}

weapondescription::~weapondescription()
{
 //  delete mymesh;
}


float weapondescription::getrange()
{
   return range;
}

int weapondescription::getdamage()
{
   return damage;
}

mesh * weapondescription::getmesh()
{
   return mymesh;
}

string weapondescription::getName()
{
  return name;
}

bool weapondescription::isMissile()
{
  return false;
}

int weapondescription::getRefireDelay()
{
  return refiredelay;
}

float weapondescription::getAcceleration()
{
  return acceleration;
}


Image * weapondescription::getIcon()
{
  return image;
}

void weapondescription::load()
{
  if(loaded)
  {
    return;
  }


  if(imagename == "")
  {
    image = NULL;
  }
  else
  {
    image = ImageFactory::getTransparentImage((char *) imagename.c_str(), Color(1.0f, 0.0, 1.0f) );
  }
  if(transparentmesh)
  {
    mymesh = new mesh((char *) meshname.c_str(), Color(0,0,0)  );
  }
  else
  {
    mymesh = new mesh((char *) meshname.c_str() );
  }
  loaded = true;
}

bool weapondescription::canDamageCapships()
{
  return damage_capships;
}

bool weapondescription::canDamageTurrets()
{
  return damage_turrets;
}

int weapondescription::getSoundIndex()
{
  return soundindex;
}

float weapondescription::getRangeSquared()
{
  return rangesquared;
}
