// weaponfactory.cpp: implementation of the weaponfactory class.
//
//////////////////////////////////////////////////////////////////////

#include "weaponfactory.h"
#include "World.h"
#include "weaponsxmladapter.h"
#include "MissileDescription.h"
#include "SoundFactory.h"
#include "SoundEffect3d.h"
#include "GunDescription.h"
#include "TextureFactory.h"
#include "OutOfBoundsException.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
weaponfactory * weaponfactory::factory = NULL;

weaponfactory::weaponfactory()
{
   weaponsxmladapter xmladapter("weapons.xml");
   numweapons = xmladapter.getnumweapons();
   descriptions = xmladapter.getweapondescriptions();
   missileenginetexture = NULL;
}

weaponfactory::~weaponfactory()
{
   for(int i=0; i < numweapons; i++)
      delete descriptions[i];

   delete [] descriptions;
}

weaponfactory & weaponfactory::getweaponfactory()
{
  if(factory == NULL)
  {
    factory = new weaponfactory();
  }
  return *factory;
}


weapon * weaponfactory::addweapon(int weapontype, Coordinate location, Coordinate heading, Coordinate up, Coordinate left)
{
   if(weapontype >= numweapons)
   {
	  throw OutOfBoundsException("weaponsfactory.cpp");
      return NULL;
   }

   GunDescription * gundescription = (GunDescription *)descriptions[weapontype];
   gundescription->load();
   int soundindex = gundescription->getSoundIndex();
   SoundEffect3d * sound = SoundFactory::getSoundFactory()->getWeaponSound(soundindex);
   sound->setPosition(location);
   sound->play();

   weapon * newweapon = new weapon((GunDescription *) descriptions[weapontype], location, heading, up, 
	   left, sound );

   World::getWorld().add(newweapon );


   return newweapon;
}


weapondescription * weaponfactory::getweapondescription(int weaponnumber)
{
  if(weaponnumber < numweapons)
  {
	descriptions[weaponnumber]->load();
    return descriptions[weaponnumber];
  }
  return NULL;
}

Missile * weaponfactory::addMissile(int weapontype, Coordinate location, Coordinate heading, Coordinate up, Coordinate left)
{
   if(weapontype >= numweapons)
   {
	  throw OutOfBoundsException("weaponsfactory.cpp");
      return NULL;
   }


   MissileDescription * description = (MissileDescription *) descriptions[weapontype];
   description->load();
   Missile * newmissile;

   int soundindex = description->getSoundIndex();
   SoundEffect3d * missileeffect = SoundFactory::getSoundFactory()->getMissileSound(soundindex);
   missileeffect->setPosition(location);
   missileeffect->play();


   if(missileenginetexture == NULL)
   {
	   missileenginetexture = TextureFactory::getTransparentTexture("missileengineglow.png");
   }
   newmissile = new Missile(description->getmesh(),
                            location, heading, up, left,
                            description->getArmor(),
                            description->getVelocity(),
                            description->getRoll(),
                            description->getPitch(),
                            description->getYaw(),
							description->getAcceleration(),
                            description->getImage(),
                            description->getName(),
                            description->getdamage(),
                            description->getrange(),
                            description->isTracking(),
                            description->canDamageCapships(),
							description->canDamageTurrets(),
							missileeffect, 
							missileenginetexture,
							description->getExhaustTexture(),
							description->getSpoof() );

   World::getWorld().addMissile(newmissile );  
   return newmissile;
}

void weaponfactory::clear()
{
  if(factory != NULL)
  {
    delete factory;
  }
  factory = NULL;
}
